@@ -4,6 +4,7 @@
 #include "UmlFormalParameter.h"
 
 #include "JavaSettings.h"
+#include "PhpSettings.h"
 #include "UmlArtifact.h"
 #include "UmlComponent.h"
 #include "UmlClassDiagram.h"
@@ -56,7 +57,7 @@ void UmlClass::html() {
   
   UmlCom::message(name());
   
-  start_file("class" + s.setNum(refId), "Class " + name(), TRUE);
+  start_file("class" + s.setNum((unsigned) getIdentifier()), "Class " + name(), TRUE);
   define();
   gen_html("", 0, 0);
   end_file();
@@ -68,6 +69,14 @@ void UmlClass::gen_html(QCString pfix, unsigned int rank, unsigned int level) {
   UmlCom::message(name());
   
   QCString s;
+ 
+  s = description();
+  
+  if (!s.isEmpty()) {
+    fw.write("<p>");
+    gen_cpp_decl(s, TRUE);
+    fw.write("<br /></p>");
+  }
   
   fw.write("<p>Declaration :</p><ul>");
   
@@ -87,6 +96,14 @@ void UmlClass::gen_html(QCString pfix, unsigned int rank, unsigned int level) {
     fw.write("</li>");
   }
 
+  s = phpDecl();
+
+  if (!s.isEmpty()) {
+    fw.write("<li>Php : ");
+    gen_php_decl(s);
+    fw.write("</li>");
+  }
+
   fw.write("</ul>");
   
   if (subClasses.size() != 0) {
@@ -103,14 +120,6 @@ void UmlClass::gen_html(QCString pfix, unsigned int rank, unsigned int level) {
   write_dependencies();
 
   annotation_constraint();
- 
-  s = description();
-  
-  if (!s.isEmpty()) {
-    fw.write("<p>");
-    gen_cpp_decl(s, TRUE);
-    fw.write("<br /></p>");
-  }
   
   bool p = FALSE;
   UmlItem * x;
@@ -156,6 +165,8 @@ void UmlClass::gen_html(QCString pfix, unsigned int rank, unsigned int level) {
     fw.write("</p>\n");
   }
 
+  write_properties();
+
   //
 
   const QVector<UmlItem> ch = children();
@@ -229,17 +240,17 @@ void UmlClass::gen_html(QCString pfix, unsigned int rank, unsigned int level) {
 }
 
 void UmlClass::write() {
-  if (refId == 0)
+  if (!known)
     writeq(name());
   else {
     if (flat)
       fw.write("<a href=\"index");
     else {
       fw.write("<a href=\"class");
-      fw.write(refId);
+      fw.write((unsigned) getIdentifier());
     }
-    fw.write(".html#ref");
-    fw.write(refId);
+    fw.write(".html#refclass");
+    fw.write((unsigned) getIdentifier());
     fw.write("\"><b>");
     writeq(name());
     fw.write("</b></a>");
@@ -247,15 +258,15 @@ void UmlClass::write() {
 }
 
 void UmlClass::write(QCString target) {
-  if (refId != 0) {
+  if (known) {
     if (flat)
       fw.write("<a href=\"index");
     else {
       fw.write("<a href=\"class");
-      fw.write(refId);
+      fw.write((unsigned) getIdentifier());
     }
-    fw.write(".html#ref");
-    fw.write(refId);
+    fw.write(".html#refclass");
+    fw.write((unsigned) getIdentifier());
     fw.write("\" target = \"");
     fw.write(target);
     fw.write("\"><b>");
@@ -369,7 +380,7 @@ void UmlClass::gen_cpp_decl(QCString s, bool descr) {
 	}
       }
       if (! find)
-	UmlItem::write(baseType(), TRUE);
+	UmlItem::write(baseType(), cppLanguage);
     }
     else if (!strncmp(p, "${template}", 11)) {
       p += 11;
@@ -397,7 +408,7 @@ void UmlClass::gen_cpp_decl(QCString s, bool descr) {
 	    // UmlItem::write else G++ call UmlClass::write(QCString) !
 	    UmlItem::write((rel->cppVisibility() == DefaultVisibility)
 		           ? rel->visibility() : rel->cppVisibility(),
-			   TRUE);
+			   cppLanguage);
 	    fw.write((rel->cppVirtualInheritance()) ? " virtual " : " ");
 	    rel->roleType()->write();
 	    generate(actuals(), rel->roleType(), TRUE);
@@ -447,7 +458,7 @@ void UmlClass::gen_java_decl(QCString s) {
     }
     else if (!strncmp(p, "${visibility}", 13)) {
       p += 13;
-      UmlItem::write(visibility(), FALSE);
+      UmlItem::write(visibility(), javaLanguage);
       fw.write(' ');
     }
     else if (!strncmp(p, "${final}", 8)) {
@@ -532,6 +543,103 @@ void UmlClass::gen_java_decl(QCString s) {
   }
 }
 
+void UmlClass::gen_php_decl(QCString s) {
+  QCString st = PhpSettings::classStereotype(stereotype());
+  
+  if (st == "ignored")
+    return;
+    
+  const char * p = bypass_comment(s);
+  UmlRelation * extend = 0;
+
+  while (*p != 0) {
+    if (!strncmp(p, "${comment}", 10))
+      p += 10;
+    else if (!strncmp(p, "${description}", 14))
+      p += 14;
+    else if (!strncmp(p, "${visibility}", 13)) {
+      p += 13;
+      UmlItem::write(visibility(), phpLanguage);
+      fw.write(' ');
+    }
+    else if (!strncmp(p, "${final}", 8)) {
+      p += 8;
+      if (isPhpFinal())
+	fw.write("final ");
+    }
+    else if (!strncmp(p, "${abstract}", 11)) {
+      p += 11;
+      if (isAbstract())
+	fw.write("abstract ");
+    }
+    else if (!strncmp(p, "${name}", 7)) {
+      p += 7;
+      writeq(name());
+      generics();
+    }
+    else if (!strncmp(p, "${extends}", 10)) {
+      p += 10;
+
+      const QVector<UmlItem> ch = children();
+
+      for (unsigned i = 0; i != ch.size(); i += 1) {
+	if (ch[i]->kind() == aRelation) {
+	  UmlRelation * rel = (UmlRelation *) ch[i];
+	  aRelationKind k = rel->relationKind();
+	  
+	  if (((k == aGeneralisation) ||
+	       (k == aRealization)) &&
+	      (!rel->phpDecl().isEmpty()) &&
+	      ((st == "interface") ||
+	       (PhpSettings::classStereotype(rel->roleType()->stereotype()) != "interface"))) {
+	    extend = rel;
+	    fw.write(" extends ");
+	    rel->roleType()->write();
+	    break;
+	  }
+	}
+      }
+    }
+    else if (!strncmp(p, "${implements}", 13)) {
+      p += 13;
+
+      const QVector<UmlItem> ch = children();
+      const char * sep = " implements ";
+
+      for (unsigned i = 0; i != ch.size(); i += 1) {
+	if (ch[i]->kind() == aRelation) {
+	  UmlRelation * rel = (UmlRelation *) ch[i];
+	  aRelationKind k = rel->relationKind();
+	  
+	  if ((rel != extend) &&
+	      ((k == aGeneralisation) ||
+	       (k == aRealization)) &&
+	      (!rel->phpDecl().isEmpty())) {
+	    fw.write(sep);
+	    sep = ", ";
+	    rel->roleType()->write();
+	  }
+	}
+      }
+    }
+    else if (*p == '\r')
+      p += 1;
+    else if (*p == '\n') {
+      fw.write(' ');
+
+      do
+	p += 1;
+      while ((*p != 0) && (*p <= ' '));
+    }
+    else if ((*p == '{') || (*p == ';'))
+      break;
+    else if (*p == '@')
+      manage_alias(p);
+    else
+      writeq(*p++);
+  }
+}
+
 void UmlClass::generate(const QValueList<UmlActualParameter> a, UmlClass * mother, bool cpp) {
   unsigned i;
   unsigned n = a.count();
@@ -545,10 +653,11 @@ void UmlClass::generate(const QValueList<UmlActualParameter> a, UmlClass * mothe
   }
     
   const char * sep = "<";
+  aLanguage lang = (cpp) ? cppLanguage : javaLanguage;
   
   do {
     writeq(sep);
-    UmlItem::write(a[i].value(), cpp);
+    UmlItem::write(a[i].value(), lang);
     sep = ", ";
   }
   while ((++i != n) && (a[i].superClass() == mother));
@@ -594,7 +703,7 @@ void UmlClass::generics() {
       
       if ((t.type != 0) || ! t.explicit_type.isEmpty()) {
 	fw.write(" extends ");
-	UmlItem::write(t, FALSE);
+	UmlItem::write(t, javaLanguage);
       }
     }
     
