@@ -9,7 +9,7 @@ void UmlDiagram::html(QCString, unsigned int, unsigned int) {
 
   QCString s;
   
-  s.setNum(++fig_rank);
+  s.setNum(getIdentifier());
   s = "fig" + s + ((svg) ? ".svg" : ".png");
   saveIn(directory + s);
   
@@ -59,8 +59,8 @@ void UmlDiagram::html(QCString, unsigned int, unsigned int) {
     fw.write("<br /></p>");
   }
 
+  write_properties();
+
   unload(FALSE, FALSE);
 }
 
-int UmlDiagram::fig_rank;
-
