@@ -162,7 +162,7 @@ void UmlItem::set_dir(int argc, char ** argv) {
       fw.write("\
 div.title { font-size: 150%; background: #87ceff; text-align: center; font-weight: bold; }\n\
 \n\
-div.sub { margin-left : 20; }\n\
+div.sub { margin-left : 20px; }\n\
 div.element { background: #d3d3d3; }\n\
 \n\
 h1.package { background: #ffe4c4; }\n\
@@ -197,8 +197,7 @@ h4.activity { background: #87ceff; }\n");
 
 void UmlItem::memo_ref() {
   all.addElement(this);
-  
-  refId = ++nrefs;
+  known = TRUE;
   
   const QVector<UmlItem> ch = children();
   
@@ -208,7 +207,8 @@ void UmlItem::memo_ref() {
 
 void UmlItem::define() {
   fw.write("<a name=\"ref");
-  fw.write(refId);
+  fw.write(sKind());
+  fw.write((unsigned) getIdentifier());
   fw.write("\"></a>\n");
 }
 
@@ -322,20 +322,17 @@ void UmlItem::generate_indexes()
   previous = 0;
   for (i = 0; i != n; i += 1) {
     UmlItem * x = all.elementAt(i);
+    QCString s = x->pretty_name();
     
-    if (x->refId != 0) {
-      QCString s = x->pretty_name();
+    if (! s.isEmpty()) {
+      char c = *((const char *) s);
       
-      if (! s.isEmpty()) {
-	char c = *((const char *) s);
-	
-	if ((c >= 'a') && (c <= 'z'))
-	  c += 'A' - 'a';
-	
-	if (c != previous) {
-	  previous = c;
-	  letters += c;
-	}
+      if ((c >= 'a') && (c <= 'z'))
+	c += 'A' - 'a';
+      
+      if (c != previous) {
+	previous = c;
+	letters += c;
       }
     }
   }
@@ -343,42 +340,39 @@ void UmlItem::generate_indexes()
   previous = 0;
   for (i = 0; i != n; i += 1) {
     UmlItem * x = all.elementAt(i);
+    QCString s = x->pretty_name();
     
-    if (x->refId != 0) {
-      QCString s = x->pretty_name();
+    if (! s.isEmpty()) {
+      char c = *((const char *) s);
       
-      if (! s.isEmpty()) {
-	char c = *((const char *) s);
-	
-	if ((c >= 'a') && (c <= 'z'))
-	  c += 'A' - 'a';
-	
-	if (c != previous) {
-	  if (previous != 0) {
-	    fw.write("</table>\n");
-	    end_file();
-	  }
-	  
-	  previous = c;
-	  
-	  QCString sn;
-	  
-	  sn.setNum((int) (c & 255));
-	  
-	  start_file(QCString("index_") + sn, QCString("") + c, TRUE);
-	  
-	  fw.write("<table>\n");
-	  fw.write("<tr bgcolor=#f0f0f0><td align=center><b>Name</b></td><td align=center><b>Kind</b></td><td align=center><b>Description</b></td></tr>\n");
+      if ((c >= 'a') && (c <= 'z'))
+	c += 'A' - 'a';
+      
+      if (c != previous) {
+	if (previous != 0) {
+	  fw.write("</table>\n");
+	  end_file();
 	}
 	
-	fw.write("<tr bgcolor=#f0f0f0><td>");
-	x->write("projectFrame");
-	fw.write("</td><td>");
-	fw.write(x->sKind());
-	fw.write("</td><td>");
-	writeq(x->description());
-	fw.write("</td></tr>\n");
+	previous = c;
+	
+	QCString sn;
+	
+	sn.setNum((int) (c & 255));
+	
+	start_file(QCString("index_") + sn, QCString("") + c, TRUE);
+	
+	fw.write("<table>\n");
+	fw.write("<tr bgcolor=#f0f0f0><td align=center><b>Name</b></td><td align=center><b>Kind</b></td><td align=center><b>Description</b></td></tr>\n");
       }
+      
+      fw.write("<tr bgcolor=#f0f0f0><td>");
+      x->write("projectFrame");
+      fw.write("</td><td>");
+      fw.write(x->sKind());
+      fw.write("</td><td>");
+      writeq(x->description());
+      fw.write("</td></tr>\n");
     }
   }
 
@@ -430,6 +424,8 @@ void UmlItem::html(QCString pfix, unsigned int rank, QCString what, unsigned int
     writeq(description());
     fw.write("<br /></p>\n");
   }
+
+  write_properties();
   
   const QVector<UmlItem> ch = children();
   unsigned n = ch.size();
@@ -463,8 +459,6 @@ void UmlItem::html(const char * what, UmlDiagram * diagram) {
   writeq(name());
   fw.write("</b></div></td></tr></table>\n");
   
-  write_dependencies();
-
   QCString d = description();
   
   if (!d.isEmpty()) {
@@ -473,11 +467,15 @@ void UmlItem::html(const char * what, UmlDiagram * diagram) {
     fw.write("<br /></p>\n");
   }
   
+  write_dependencies();
+
   if (diagram != 0) {
     fw.write("<p>Diagram : ");
     diagram->write();
     fw.write("</p>\n");
   }
+
+  write_properties();
 }
 
 void UmlItem::write_children(QCString pfix, unsigned int rank, unsigned int level) {
@@ -521,6 +519,27 @@ void UmlItem::write_dependencies() {
   }
 }
 
+void UmlItem::write_properties() {
+  const QDict<QCString> d = properties();
+  
+  if (! d.isEmpty()) {
+    fw.write("<p>Properties:<ul>\n");
+    
+    QDictIterator<QCString> it(d);
+    
+    while (it.current()) {
+      fw.write("<li>");
+      writeq(it.currentKey().latin1());
+      fw.write(":<br /><div class=\"sub\">");
+      writeq(*(it.current()));
+      fw.write("</div></p></li>\n");
+      ++it;
+    }
+    
+    fw.write("</ul></p>\n");
+  }
+}
+
 void UmlItem::chapter(QCString k, QCString pfix, unsigned int rank, QCString kind, unsigned int level) {
   if (rank != 0) {
     if (level > 4)
@@ -620,16 +639,17 @@ void UmlItem::manage_alias(const char *& p) {
 }
 
 void UmlItem::write() {
-  if (refId != 0) {
+  if (known) {
     fw.write("<a href=\"");
     if (!flat && (parent() != 0) && (parent()->kind() == aClass)){
       fw.write("class");
-      fw.write(parent()->refId);
+      fw.write((unsigned) parent()->getIdentifier());
     }
     else
       fw.write("index");
     fw.write(".html#ref");
-    fw.write(refId);
+    fw.write(sKind());
+    fw.write((unsigned) getIdentifier());
     fw.write("\"><b>");
     writeq(pretty_name());
     fw.write("</b></a>");
@@ -639,9 +659,10 @@ void UmlItem::write() {
 }
 
 void UmlItem::write(QCString target) {
-  if (refId != 0) {
+  if (known) {
     fw.write("<a href=\"index.html#ref");
-    fw.write(refId);
+    fw.write(sKind());
+    fw.write((unsigned) getIdentifier());
     fw.write("\" target = \"");
     fw.write(target);
     fw.write("\"><b>");
@@ -683,14 +704,21 @@ void UmlItem::writeq(char c)
  }
 }
 
-void UmlItem::write(const UmlTypeSpec & t, bool cpp)
+void UmlItem::write(const UmlTypeSpec & t, aLanguage lang)
 {
   if (t.type != 0)
     t.type->write();
   else
-    writeq((cpp) ? CppSettings::type(t.toString())
-		 : JavaSettings::type(t.toString()));
-
+    switch (lang) {
+    case cppLanguage:
+      writeq(CppSettings::type(t.toString()));
+      break;
+    case javaLanguage:
+      writeq(JavaSettings::type(t.toString()));
+      break;
+    default:
+      writeq(t.toString());
+  }
 }
 
 void UmlItem::write(const UmlTypeSpec & t)
@@ -701,7 +729,7 @@ void UmlItem::write(const UmlTypeSpec & t)
     writeq(t.toString());
 }
 
-void UmlItem::write(aVisibility v, bool cpp)
+void UmlItem::write(aVisibility v, aLanguage lang)
 {
   switch (v) {
   case PublicVisibility:
@@ -714,7 +742,15 @@ void UmlItem::write(aVisibility v, bool cpp)
     fw.write("private");
     break;
   case PackageVisibility:
-    fw.write((cpp) ? "public" : "package");
+    switch (lang) {
+    case cppLanguage:
+      fw.write("public");
+      break;
+    case javaLanguage:
+      fw.write("package");
+    default:
+      break;
+    }
     break;
   default:
     fw.write("???");
