@@ -2,7 +2,6 @@
 #include "Association.h"
 #include "FileIn.h"
 #include "Token.h"
-#include "UmlItem.h"
 
 #include "UmlCom.h"
 #include "UmlRelation.h"
@@ -93,10 +92,10 @@ void Association::solveThem()
     (*it).solve(it.key());
     
   All.clear();
-
-  AssociationClass::solveThem();
 }
 
+QMap<QCString, Association> Association::All;
+
 void Association::solve(QCString id) {
   if (roles[0].id.isEmpty() || roles[0].idref.isEmpty() ||
       roles[1].id.isEmpty() || roles[1].idref.isEmpty())
@@ -156,7 +155,6 @@ void Association::solve(QCString id) {
 	  ra->set_Name(name);
 	
 	UmlItem::All.insert(a.id, ra);
-	UmlItem::All.insert(id, ra);	// for association classes
 	if (a.name.isEmpty()) {
 	  s.sprintf("anonymous_role_%u", ++n);
 	  ra->set_RoleName(s);
@@ -171,6 +169,13 @@ void Association::solve(QCString id) {
 	if (! a.multiplicity.isEmpty())
 	  ra->set_Multiplicity(a.multiplicity);
 	
+ 	if (is_class_association) {
+ 	  UmlTypeSpec t;
+ 	  
+ 	  t.type = (UmlClass *) UmlItem::All[id];	// exist !
+ 	  ra->set_Association(t);
+ 	}
+	
 	if (rb != 0) {
 	  UmlItem::All.insert(b.id, rb);
 	  if (b.name.isEmpty()) {
@@ -192,85 +197,3 @@ void Association::solve(QCString id) {
   }
 }
 
-QMap<QCString, Association> Association::All;
-
-AssociationClass::AssociationClass() {
-}
-
-void AssociationClass::init()
-{
-  UmlItem::declareFct("ownedelement", "uml:AssociationClass", &importIt);
-  UmlItem::declareFct("ownedmember", "uml:AssociationClass", &importIt);
-  UmlItem::declareFct("packagedelement", "uml:AssociationClass", &importIt);
-}
-
-void AssociationClass::importIt(FileIn & in, Token & token, UmlItem *)
-{
-  QCString ref1;
-  QCString ref2;
-  
-  if (! token.closed()) {
-    QCString k = token.what();
-    const char * kstr = k;
-    
-    while (in.read(), !token.close(kstr)) {
-      QCString s = token.what();
-      
-      if (s == "memberend") {
-	if (ref1.isEmpty())
-	  ref1 = token.xmiIdref();
-	else
-	  ref2 = token.xmiIdref();
-      }
-      if (! token.closed())
-	in.finish(s);
-    }
-  }
-      
-  if (ref1.isEmpty() || ref2.isEmpty())
-    in.warning("association class not fully defined");
-  else
-    All.append(AssociationClass(ref1, ref2));
-}
-
-void AssociationClass::solveThem()
-{
-  QValueList<AssociationClass>::Iterator it;
-  
-  for (it = All.begin(); it != All.end(); ++it)
-    (*it).solve();
-    
-  All.clear();
-}
-
-void AssociationClass::solve() {
-  QMap<QCString, UmlItem*>::Iterator it1 = UmlItem::All.find(ref1);
-  QMap<QCString, UmlItem*>::Iterator it2 = UmlItem::All.find(ref2);
-  UmlTypeSpec t;
-    
-  if (it1 == UmlItem::All.end())
-    UmlCom::trace("association class : unknown reference '" + ref1 + "'<br>");
-  else if (it2 == UmlItem::All.end())
-    UmlCom::trace("association class : unknown reference '" + ref2 + "'<br>");
-  else if ((*it1)->kind() == aRelation) {
-    if ((*it2)->kind() != aClass)
-      UmlCom::trace("'" + ref2 + "' must be a class<br>");
-    else {
-      t.type = (UmlClass *) *it2;
-      ((UmlRelation *) *it1)->set_Association(t);
-    }
-  }
-  else if ((*it1)->kind() == aClass) {
-    if ((*it2)->kind() != aRelation)
-      UmlCom::trace("'" + ref2 + "' must be an association<br>");
-    else {
-      t.type = (UmlClass *) *it1;
-      ((UmlRelation *) *it2)->set_Association(t);
-    }
-  }
-  else
-    UmlCom::trace("'" + ref1 + "' must be a class or an association<br>");
-}
-
-QValueList<AssociationClass> AssociationClass::All;
-
