@@ -50,6 +50,8 @@ class FileIn {
 
     FILE * _fp;
 
+    bool _utf8;
+
     int _linenum;
 
     int _length;
@@ -66,6 +68,10 @@ class FileIn {
 
     char read_special_char();
 
+
+  public:
+    void setEncoding(QCString s);
+
 };
 
 #endif
