@@ -7,7 +7,10 @@
 UmlItem * UmlUseCaseView::container(anItemKind kind, const Token & token, FileIn & in) {
   switch (kind) {
   case aClass:
+  case aClassInstance:
   case anUseCase:
+  case aState:
+  case anActivity:
     return this;
   default:
     return parent()->container(kind, token, in);
