@@ -41,7 +41,11 @@ void UmlArtifact::generate() {
   if (! managed) {
     managed = TRUE;
     
-    if (stereotype() != "source")
+    if (stereotype() == "text") {
+      generate_text();
+      return;
+    }
+    else if (stereotype() != "source")
       return;
     
     package_of_generated_artifact = package();
@@ -64,7 +68,7 @@ void UmlArtifact::generate() {
     QCString nasp_end;
     QCString nasp = pack->cppNamespace();
     
-    if (! nasp.isEmpty()) {
+    if (!nasp.isEmpty()) {
       int index = 0;
       int index2;
       QCString closed = "\n} // namespace ";
@@ -243,6 +247,7 @@ void UmlArtifact::generate() {
 			+ name + "</i> : cannot open <i> "
 			+ h_path + "</i>, edit the <i> generation settings</i> (tab directory) or the <i>"
 			+ pack->name() + "</i> C++ directory specification</b></font><br>");
+	  incr_error();
 	}
 	else {
 	  fputs((const char *) file, fp_h);
@@ -347,6 +352,7 @@ void UmlArtifact::generate() {
 			+ name + " : </i> cannot open <i> " 
 			+ src_path + "</i>, edit the <i> generation settings</i> (tab directory) or the <i>"
 			+ pack->name() + "</i> C++ directory specification</b></font><br>");
+	  incr_error();
 	}
 	else {
 	  fputs((const char *) file, fp_src);
@@ -360,6 +366,53 @@ void UmlArtifact::generate() {
   }
 }
 
+void UmlArtifact::generate_text() {
+  const QCString srcdef = cppSource();
+  
+  if (srcdef.isEmpty()) {
+    if (verbose())
+      UmlCom::trace(QCString("<hr><font face=helvetica><i>")
+		    + name() + "</i> has an empty C++ definition</font><br>");
+    return;
+  }
+    
+  UmlPackage * pack = package();
+  const QCString & name = UmlArtifact::name();    
+  QCString src_path = pack->text_path(name);
+  
+  QCString s = " in <i> " + src_path + "</i>";
+      
+  UmlCom::message(name);
+  if (verbose())
+    UmlCom::trace(QCString("<hr><font face=helvetica>Generate code for <i> ")
+		  + name + "</i>" + s + "</font><br>");
+  else
+    set_trace_header(QCString("<font face=helvetica>Generate code for <i> ")
+		     + name + "</i>" + s + "</font><br>");
+      
+  if (must_be_saved(src_path, (const char *) srcdef)) {
+    write_trace_header();
+    
+    FILE * fp_src;
+    
+    if ((fp_src = fopen((const char *) src_path, "wb")) == 0) {
+      write_trace_header();
+      UmlCom::trace(QCString("<font color=\"red\"><b><i> ")
+		    + name + " : </i> cannot open <i> " 
+		    + src_path + "</i>, edit the <i> generation settings</i> (tab directory) or the <i>"
+		    + pack->name() + "</i> C++ directory specification</b></font><br>");
+      incr_error();
+    }
+    else {
+      fputs((const char *) srcdef, fp_src);
+      fclose(fp_src);
+    }
+  }
+  else if (get_trace_header().isEmpty())
+    UmlCom::trace(QCString("<font face=helvetica><i> ")
+		  + src_path + "</i> not modified</font><br>");
+}
+
 UmlPackage * UmlArtifact::generation_package()
 {
   return package_of_generated_artifact;
