@@ -101,10 +101,12 @@ void UmlClass::compute_dependency(QList<CppRefType> & dependencies,
   if (an_enum && (!formals.isEmpty() || !actuals.isEmpty())) {
     write_trace_header();
     UmlCom::trace("<tt>        </tt><font color=\"red\"><b><i>template enum</i></b></font><br>");
+    incr_warning();
   }
   else if (a_typedef && !formals.isEmpty()) {
     write_trace_header();
     UmlCom::trace("<tt>        </tt><font color=\"red\"><b><i>template typedef</i></b></font><br>");
+    incr_warning();
   }
   else {
     QValueList<UmlFormalParameter>::ConstIterator itf;
@@ -181,7 +183,9 @@ void UmlClass::generate_decl(QTextOStream & f_h, QCString indent) {
 
     if (*p == '\n') {
       f_h << *p++;
-      if (*p && (*p != '#') && strncmp(p, "${members}", 10))
+      if (*p && (*p != '#') &&
+	  strncmp(p, "${members}", 10) &&
+	  strncmp(p, "${items}", 8))
 	f_h << indent;
     }
     else if (*p == '@')
@@ -517,21 +521,17 @@ void UmlClass::write(QTextOStream & f, bool with_formals,
     QCString s = cppDecl();
     int index = s.find('\n');
     
-    if (index == -1)
-      // invalid, write the name
-      f << name();
-    else {
-      s = s.left(index).stripWhiteSpace();
+    s = (index == -1) ? s.stripWhiteSpace()
+		      : s.left(index).stripWhiteSpace();
       
-      if ((index = s.find("${name}")) != -1)
-	s.replace(index, 7, name());
-      else if ((index = s.find("${Name}")) != -1)
-	s.replace(index, 7, capitalize(name()));
-      else if ((index = s.find("${NAME}")) != -1)
-	s.replace(index, 7, name().upper());
-      
-      f << s;
-    }
+    if ((index = s.find("${name}")) != -1)
+      s.replace(index, 7, name());
+    else if ((index = s.find("${Name}")) != -1)
+      s.replace(index, 7, capitalize(name()));
+    else if ((index = s.find("${NAME}")) != -1)
+      s.replace(index, 7, name().upper());
+    
+    f << s;
   }
   else 
     f << name();	// true_name
