@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -163,7 +163,8 @@ bool UmlClassMember::compute_dependency(QList<CppRefType> & dependencies,
       else if (dontsubstituteuntil != 0) {
 	if (p >= dontsubstituteuntil)
 	  dontsubstituteuntil = 0;
-	p += 1;
+	else
+	  p += 1;
       }
       else if (c == '=')
 	// init, all is done
@@ -265,14 +266,15 @@ bool UmlClassMember::compute_dependency(QList<CppRefType> & dependencies,
   return have_type;
 }
 
-void UmlClassMember::insert_template(const char *& p, QTextOStream & fs,
+// return TRUE if stop on comment/description
+bool UmlClassMember::insert_template(const char *& p, QTextOStream & fs,
 				     const QCString & indent,
 				     const QCString & templ)
 {
   // search the beginning of the definition/declaration in p;
   for (;;) {
     if (*p == 0)
-      return;
+      return FALSE;
     
     if (*p == '\n') {
       fs << *p++;
@@ -314,9 +316,13 @@ void UmlClassMember::insert_template(const char *& p, QTextOStream & fs,
 	}
       } while (*p && (*p != '\n'));
     }
+    else if ((strncmp(p, "${comment}", 10) == 0) ||
+	     (strncmp(p, "${description}", 14) == 0))
+      return TRUE;
     else
       break;
   }
   
   fs << templ;
+  return FALSE;
 }
