@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -30,40 +30,58 @@
 UmlItem::~UmlItem() {
 }
 
-void UmlItem::manage_comment(const char *& p, const char *& pp) {
+bool UmlItem::manage_comment(const char *& p, const char *& pp,
+			     bool javadoc) {
   static QString the_comment;
   
   p += 10;
   
   if ((pp != 0) || // comment contains ${comment} !
       description().isEmpty())
-    return;
+    return FALSE;
   
   const char * comment = description();
   
-  the_comment = "//";
-  
-  do {
-    the_comment += *comment;
-    if ((*comment++ == '\n') && *comment)
-      the_comment += "//";
-  } while (*comment);
-  
-  if (*p != '\n')
-    the_comment += '\n';
+  if (javadoc) {
+    the_comment = "/**\n * ";
+    
+    do {
+      the_comment += *comment;
+      if ((*comment++ == '\n') && *comment)
+	the_comment += " * ";
+    } while (*comment);
+    
+    if (*p != '\n')
+      the_comment += (comment[-1] != '\n') ? "\n */\n" : " */\n";
+    else
+      the_comment += (comment[-1] != '\n') ? "\n */" : " */";
+  }
+  else {
+    the_comment = "//";
+    
+    do {
+      the_comment += *comment;
+      if ((*comment++ == '\n') && *comment)
+	the_comment += "//";
+    } while (*comment);
+    
+    if (*p != '\n')
+      the_comment += '\n';
+  }
     
   pp = p;
   p = the_comment;
+  return TRUE;
 }
 
-void UmlItem::manage_description(const char *& p, const char *& pp) {
+bool UmlItem::manage_description(const char *& p, const char *& pp) {
   static QCString the_comment;
   
   p += 14;
   
   if ((pp != 0) || // comment contains ${description} !
       description().isEmpty())
-    return;
+    return FALSE;
   
   the_comment = description();
   if (*p != '\n')
@@ -71,6 +89,7 @@ void UmlItem::manage_description(const char *& p, const char *& pp) {
     
   pp = p;
   p = the_comment;
+  return TRUE;
 }
 
 void UmlItem::replace_alias(QCString & s) {
