@@ -109,7 +109,7 @@ void UmlOperation::compute_dependency(QList<CppRefType> & dependencies,
     decl.remove((unsigned) index, 7);
   if ((index = decl.find("${stereotype}")) != -1)
     decl.replace((unsigned) index, 13,
-		 CppSettings::relationStereotype(stereotype()));
+		 CppSettings::relationAttributeStereotype(stereotype()));
   if ((index = decl.find("${association}")) != -1)
     // dependency computed for the relation
     decl.remove((unsigned) index, 14);
@@ -298,6 +298,7 @@ static void param_error(const QCString & parent, const QCString & name,
 		+ parent + "::" + name + "</i> " + where
 		+ ", parameter rank " + QCString().setNum(rank)
 		+ " does not exist</font></b><br>");
+  incr_error();
 }
 
 QCString UmlOperation::compute_name() {
@@ -334,11 +335,13 @@ void UmlOperation::generate_decl(aVisibility & current_visibility, QTextOStream
     if (cl_stereotype == "enum") {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>an <i>enum</i> cannot have operation</b></font><br>");
+      incr_warning();
       return;
     }
     if (cl_stereotype == "typedef") {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>a <i>typedef</i> cannot have operation</b></font><br>");
+      incr_warning();
       return;
     }
     
@@ -410,7 +413,7 @@ void UmlOperation::generate_decl(aVisibility & current_visibility, QTextOStream
       else if (!strncmp(p, "${stereotype}", 13)) {
 	p += 13;
 	// get/set relation with multiplicity > 1
-	f_h << CppSettings::relationStereotype(stereotype());
+	f_h << CppSettings::relationAttributeStereotype(stereotype());
       }
       else if (!strncmp(p, "${name}", 7)) {
 	p += 7;
@@ -506,7 +509,7 @@ const char * UmlOperation::generate_body(QTextOStream & fs,
   bool no_indent;
   char s_id[9];
   
-  if (preserve()) {
+  if (preserve() && !isBodyGenerationForced()) {
     unsigned id = get_id();
     
     sprintf(s_id, "%08X", id);
@@ -526,7 +529,7 @@ const char * UmlOperation::generate_body(QTextOStream & fs,
   while (*p != '$')
     indent += *p++;
 
-  if (preserve())
+  if (preserve() && !isBodyGenerationForced())
     fs << indent << BodyPrefix << s_id << '\n';
     
   if ((body != 0) && (*body != 0)) {
@@ -562,7 +565,7 @@ const char * UmlOperation::generate_body(QTextOStream & fs,
     add_nl = body[-1] != '\n';
   }
   
-  if (preserve()) {
+  if (preserve() && !isBodyGenerationForced()) {
     if (add_nl)
       fs << '\n';
     
@@ -684,7 +687,7 @@ void UmlOperation::generate_def(QTextOStream & fs, QCString indent, bool h,
 	else if (!strncmp(p, "${stereotype}", 13)) {
 	  p += 13;
 	  // get/set relation with multiplicity > 1
-	  fs << CppSettings::relationStereotype(stereotype());
+	  fs << CppSettings::relationAttributeStereotype(stereotype());
 	}
 	else if (!strncmp(p, "${class}", 8)) {
 	  p += 8;
