@@ -71,7 +71,7 @@ void UmlRelation::compute_dependency(QList<CppRefType> & dependencies,
       decl.remove((unsigned) index, 10);
     if ((index = decl.find("${stereotype}")) != -1)
       decl.replace((unsigned) index, 13,
-		   CppSettings::relationStereotype(stereotype()));
+		   CppSettings::relationAttributeStereotype(stereotype()));
     if ((index = decl.find("${association}")) != -1) {
       decl.replace((unsigned) index, 14,
 		   association().toString());
@@ -97,11 +97,13 @@ void UmlRelation::generate_inherit(const char *& sep, QTextOStream & f_h,
       write_trace_header();
       UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>an <i>")
 		    + cl_stereotype + "</i> cannot inherits</b></font><br>");
+      incr_warning();
       return;
     }
     else if (cl_stereotype == "typedef") {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>a <i>typedef</i> cannot inherits</b></font><br>");
+      incr_warning();
       return;
     }
     
@@ -113,6 +115,7 @@ void UmlRelation::generate_inherit(const char *& sep, QTextOStream & f_h,
       write_trace_header();
       UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>cannot inherits an <i>")
 		    + other_stereotype + "</i></b></font><br>");
+      incr_warning();
       return;
     }
     
@@ -186,11 +189,13 @@ void UmlRelation::generate_decl(aVisibility & current_visibility, QTextOStream &
       if (cl_stereotype == "enum") {
 	write_trace_header();
 	UmlCom::trace("<tt>        </tt><font color=\"red\"><b>an <i>enum</i> cannot have relation</b></font><br>");
+	incr_warning();
 	return;
       }
       if (cl_stereotype == "typedef") {
 	write_trace_header();
 	UmlCom::trace("<tt>        </tt><font color=\"red\"><b>a <i>typedef</i> cannot have relation</b></font><br>");
+	incr_warning();
 	return;
       }
       generate_visibility(current_visibility, f_h, first, indent);
@@ -271,7 +276,7 @@ void UmlRelation::generate_decl(aVisibility & current_visibility, QTextOStream &
 	}
 	else if (!strncmp(p, "${stereotype}", 13)) {
 	  p += 13;
-	  f_h << CppSettings::relationStereotype(stereotype());
+	  f_h << CppSettings::relationAttributeStereotype(stereotype());
 	}
 	else if (!strncmp(p, "${value}", 8))
 	  p += 8;
@@ -396,7 +401,7 @@ void UmlRelation::generate_def(QTextOStream & f, QCString indent, bool h,
 	}
 	else if (!strncmp(p, "${stereotype}", 13)) {
 	  p += 13;
-	  f << CppSettings::relationStereotype(stereotype());
+	  f << CppSettings::relationAttributeStereotype(stereotype());
 	}
 	else if (!strncmp(p, "${value}", 8)) {
 	  p += 8;
