@@ -31,6 +31,7 @@ using namespace std;
 
 #include <qfiledialog.h> 
 #include <qapplication.h>
+#include <qmessagebox.h>
 #include <qdir.h>
 
 #include "Package.h"
@@ -47,14 +48,17 @@ using namespace std;
 #include "Statistic.h"
 #include "Progress.h"
 
+// create all packages under it
+Package * Package::Root;
+
+// the packages selected by the user to reverse them
+QList<Package> Package::Choozen;
+
 // all the classes presents in the environment, the
 // key is the full name including namespaces
 NDict<Class> Package::Declared(511);
 NDict<Class> Package::Defined(511);
 
-// create all packages under it
-Package * Package::Root;
-
 // to place unknown classes
 //Package * Package::Unknown;
 
@@ -65,9 +69,9 @@ bool Package::Scan;
 QValueList<FormalParameterList> Package::Formals;
 
 // to show a progress bar
-Progress * Package::progress;
+Progress * Package::ProgressBar;
 QApplication * Package::app;
-int Package::nfiles;
+int Package::Nfiles;
 
 // memorise the name of the header currently reversed
 // to create artifact having the right name and respect
@@ -82,7 +86,7 @@ Package::Package(BrowserView * parent, UmlPackage * u)
 }
 #endif
 
-Package::Package(Package * parent, const char * p, const char * n)
+Package::Package(Package * parent, QString p, QString n)
     : BrowserNode(parent, n) {
   path = p;
   uml = 0;
@@ -113,46 +117,111 @@ void Package::init(UmlPackage * r, QApplication * a)
   app = a;
 }
 
-Package * Package::scan_dir() {
-  // get input C++ source dir
-  
-  QString path = QFileDialog::getExistingDirectory();
+bool Package::scan_dirs() {
+  QStringList dirs;
+  QString path = QDir::currentDirPath();
 
-  if (! path.isEmpty()) {
-    QDir d(path);
-    Package * p = new Package(Root, path, d.dirName());
+  // get input C++ source dirs
+  while (!(path = QFileDialog::getExistingDirectory(path, 0, 0,
+						   "select a directory to reverse, press cancel to finish"))
+						   .isEmpty()) {
+
+    int pathlen = path.length();
     
-    // scanning phase
+    if (((const char *) path)[pathlen - 1] != '/') {
+      path += "/";
+      pathlen += 1;
+    }
+    
+    QString err = 0;
+    
+    for (QStringList::Iterator it = dirs.begin(); it != dirs.end(); ++it) {
+      int dlen = (*it).length();
+      
+      if (dlen >= pathlen) {
+	if ((*it).left(pathlen) == path) {
+	  err = (dlen == pathlen)
+	    ? path + " is already selected\n"
+	    : "An already selected directory is a subdirectory of " + path;
+	  break;
+	}
+      }
+      else if (path.left(dlen) == *it) {
+	err = path + " is a subdirectory of an already selected directory\n";
+	break;
+      }
+    }
+		   
+    if (!err.isEmpty())
+      QMessageBox::warning(0, "C++ reverse",
+			    err + "Press 'cancel' to reverse selected directories");
+    else {
+      QDir d(path);
+      QCString s;
+      
+      dirs.append(path);
+      s.sprintf("<font face=helvetica>%dth directory to reverse : <b>", dirs.count());
+      s += QCString(path) + "</b><br></font>\n";
+      UmlCom::trace(s);
+
+      Choozen.append(new Package(Root, path, d.dirName()));
+      
+      d.cdUp();
+      path = d.absPath();
+    }
+  }
+  
+  if (dirs.isEmpty())
+    return FALSE;
+
+  UmlCom::trace("<hr>\n");
+  
+  dirs.clear();
+
 #ifndef REVERSE
-    QApplication::setOverrideCursor(Qt::waitCursor);
-    CppCatWindow::clear_trace();
+  QApplication::setOverrideCursor(Qt::waitCursor);
+  CppCatWindow::clear_trace();
 #endif
-    nfiles = file_number(d, TRUE, 
-			 CppSettings::headerExtension(),
-			 CppSettings::sourceExtension());
+
+  // count files
+  Package * p;
+
+  Nfiles = 0;
+
+  for (p = Choozen.first(); p != 0; p = Choozen.next()) {
+    QDir d(p->path);
+
+    Nfiles += file_number(d, TRUE, 
+			  CppSettings::headerExtension(),
+			  CppSettings::sourceExtension());
+  }
     
-    progress = new Progress(nfiles, "Scanning in progress, please wait ...");
-    Scan = TRUE;
+  // scanning phase
+  ProgressBar = new Progress(Nfiles, "Scanning in progress, please wait ...");
+  Scan = TRUE;
+
+  for (p = Choozen.first(); p != 0; p = Choozen.next()) {
+    QDir d(p->path);
+
     p->reverse_directory(d, TRUE, CppSettings::headerExtension(), TRUE);
     p->reverse_directory(d, TRUE, CppSettings::sourceExtension(), FALSE);
-    Scan = FALSE;
-    if (progress != 0)
-      delete progress;
+  }
+  Scan = FALSE;
+
+  if (ProgressBar != 0)
+    delete ProgressBar;
 #ifndef REVERSE
-    QApplication::restoreOverrideCursor();
+  QApplication::restoreOverrideCursor();
     
-    Root->setOpen(TRUE);
+  Root->setOpen(TRUE);
 #endif
     
-    return p;
-  }
-  else
-    return 0;
+  return TRUE;
 }
     
 void Package::progress_closed()
 {
-  progress = 0;
+  ProgressBar = 0;
 }
 
 int Package::file_number(QDir & d, bool rec, const char * h, const char * cpp)
@@ -163,7 +232,7 @@ int Package::file_number(QDir & d, bool rec, const char * h, const char * cpp)
   QFileInfo * fi;
   
   while ((fi = it.current()) != 0) {
-    if ((fi->extension() == h) || (fi->extension() == cpp))
+    if ((fi->extension(FALSE) == h) || (fi->extension(FALSE) == cpp))
       result += 1;
     ++it;
   }
@@ -187,7 +256,7 @@ int Package::file_number(QDir & d, bool rec, const char * h, const char * cpp)
   return result;
 }
 
-void Package::send_dir(bool rec) {
+void Package::send_dirs(bool rec) {
   // reverse phase
 #ifdef REVERSE
   UmlItem * prj = Root->uml;
@@ -200,16 +269,23 @@ void Package::send_dir(bool rec) {
   
   prj->set_childrenVisible(FALSE);
   
-  QDir d(path);
-  
 #ifndef REVERSE
   QApplication::setOverrideCursor(Qt::waitCursor);
 #endif
-  progress = new Progress(nfiles, "Reverse in progress, please wait ...");
-  reverse_directory(d, rec, CppSettings::headerExtension(), TRUE);
-  reverse_directory(d, rec, CppSettings::sourceExtension(), FALSE);
-  if (progress != 0)
-    delete progress;
+  ProgressBar = new Progress(Nfiles, "Reverse in progress, please wait ...");
+
+  Package * p;
+
+  for (p = Choozen.first(); p != 0; p = Choozen.next()) {
+    QDir d(p->path);
+  
+    p->reverse_directory(d, rec, CppSettings::headerExtension(), TRUE);
+    p->reverse_directory(d, rec, CppSettings::sourceExtension(), FALSE);
+  }
+
+  if (ProgressBar != 0)
+    delete ProgressBar;
+
 #ifndef REVERSE
   QApplication::restoreOverrideCursor();
 #endif
@@ -221,6 +297,16 @@ void Package::send_dir(bool rec) {
 #endif
 }
 
+QString my_baseName(QFileInfo * fi)
+{
+  QString fn = fi->fileName();
+  int index = fn.findRev('.');
+  
+  return (index == -1)
+    ? fn
+    : fn.left(index);
+}
+
 void Package::reverse_directory(QDir & d, bool rec, QString ext, bool h) {
   // reads files
   const QFileInfoList * list =
@@ -229,10 +315,10 @@ void Package::reverse_directory(QDir & d, bool rec, QString ext, bool h) {
   
   while (it.current() != 0) {
     if (h)
-      fname = it.current()->baseName();
+      fname = my_baseName(it.current());
     reverse_file(QCString(it.current()->filePath()));
-    if (progress)
-      progress->tic();
+    if (ProgressBar)
+      ProgressBar->tic();
     app->processEvents();
     ++it;
   }
