@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -27,22 +27,12 @@
 #define UMLCLASS_H
 
 #include "UmlBaseClass.h"
+#include "UmlFormalParameter.h"
 
 class ClassContainer;
 
-class UmlFormalParameter : public UmlBaseFormalParameter {
-  public:
-    UmlFormalParameter(){};
-    UmlFormalParameter(QCString t, QCString n, QCString v);
-};
-
 typedef QValueList<UmlFormalParameter> FormalParameterList;
 
-class UmlActualParameter : public UmlBaseActualParameter {
-  public:
-    UmlActualParameter(){};
-};
-
 class UmlClass : public UmlBaseClass {
   public:
     UmlClass(void * id, const QCString & n);
