@@ -60,8 +60,11 @@ bool UmlRelation::new_one(UmlClass * cl, const QCString & name,
 #endif
   
   UmlRelation * rel = 
-    UmlBaseRelation::create((modifier.isEmpty()) ? aDirectionalAggregationByValue
-						 : aDirectionalAssociation,
+    UmlBaseRelation::create((modifier.isEmpty() &&
+			     ((typeform == "${type}") ||
+			      (typeform[typeform.find("${type}") + 7] != '*')))
+			    ? aDirectionalAggregationByValue
+			    : aDirectionalAssociation,
 			    cl, dest);
   
   if (rel == 0) {
@@ -107,13 +110,16 @@ bool UmlRelation::new_one(UmlClass * cl, const QCString & name,
   else {
     if (!array.isEmpty())
       rel->set_Multiplicity(array);
-    decl = CppSettings::relationDecl(modifier.isEmpty(), array);
+    decl = CppSettings::relationDecl(modifier != "*", array);
     
     int index;
     
     if (!pretype.isEmpty() &&
 	((index = decl.find("${type}")) != 0))
       decl.insert(index, pretype + " ");
+    
+    if ((modifier == "&") && ((index = decl.find("${type}")) != 0))
+      decl.insert(index + 7, " &");
   }
   
   if (! value.isEmpty()) {
