@@ -41,7 +41,7 @@ int main(int argc, char ** argv)
   if (UmlCom::connect(QCString(argv[1]).toUInt())) {
     try {
       //UmlCom::with_ack(FALSE);
-      UmlCom::trace("<b>C++ reverse</b> release 2.5<br><hr>");
+      UmlCom::trace("<b>C++ reverse</b> release 2.7.1<br>");
       UmlCom::traceAutoRaise(FALSE);
       
       UmlItem * item = UmlCom::targetItem();
@@ -60,14 +60,11 @@ int main(int argc, char ** argv)
 	if (((UmlPackage *) item)->propertyValue("#file", f))
 	  Lex::defines(f);
 	
-	//..ccat
+	// add c++ catalog like java ?
 	
-	Package * p = Package::scan_dir();
-	
-	if (p != 0) {
+	if (Package::scan_dirs()) {
 	  CppSettings::set_UseDefaults(TRUE);
-	  
-	  p->send_dir(TRUE);
+	  Package::send_dirs(TRUE);
 	  Statistic::produce();
 	  
 	  UmlCom::message("");
