@@ -28,6 +28,7 @@
 #include "UmlAttribute.h"
 #include "UmlSettings.h"
 #include "IdlSettings.h"
+#include "UmlClass.h"
 #include "UmlCom.h"
 #include "util.h"
 
@@ -36,6 +37,7 @@ void UmlAttribute::generate_decl(QTextOStream & f, const QCString & cl_stereotyp
     if (cl_stereotype == "typedef") {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>a <i>typedef</i> cannot have attribute</b></font><br>");
+      incr_warning();
       return;
     }
     const char * p = idlDecl();
@@ -80,6 +82,14 @@ void UmlAttribute::generate_decl(QTextOStream & f, const QCString & cl_stereotyp
 	p += 7;
 	f << name();
       }
+      else if (!strncmp(p, "${stereotype}", 13)) {
+	p += 13;
+	f << IdlSettings::relationAttributeStereotype(stereotype());
+      }
+      else if (!strncmp(p, "${multiplicity}", 15)) {
+	p += 15;
+	f << multiplicity();
+      }
       else if (!strncmp(p, "${value}", 8)) {
 	p += 8;
 	if (!defaultValue().isEmpty() && isReadOnly() && isClassMember())
@@ -121,6 +131,7 @@ void UmlAttribute::generate_decl(QTextOStream & f, const QCString & cl_stereotyp
 	  write_trace_header();
 	  UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>unspecified <i>case</i> for <i>")
 			+ name() + "</b></font><br>");
+	  incr_error();
 	}
 	else
 	  f << idl_case;
@@ -132,7 +143,7 @@ void UmlAttribute::generate_decl(QTextOStream & f, const QCString & cl_stereotyp
       }
       else if (!strncmp(p, "${type}", 7)) {
 	p += 7;
-	f << IdlSettings::type(type().toString());
+	UmlClass::write(f, type());
       }
       else
 	// strange
