@@ -59,7 +59,7 @@ static bool generate_type(const QValueList<UmlParameter> & params,
   if (rank >= params.count())
     return FALSE;
   
-  f << IdlSettings::type(params[rank].type.toString());
+  UmlClass::write(f, params[rank].type);
   return TRUE;
 }
 
@@ -81,6 +81,7 @@ static void param_error(const QCString & parent, const QCString & name,
 		+ parent + "::" + name + "</i> " + where
 		+ ", parameter rank " + QCString().setNum(rank)
 		+ " does not exist</font></b><br>");
+  incr_error();
 }
 
 QCString UmlOperation::compute_name() {
@@ -117,6 +118,7 @@ void UmlOperation::generate_decl(QTextOStream & f,
 	(cl_stereotype != "valuetype")) {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>cannot have operation</b></font><br>");
+      incr_warning();
       return;
     }
     
@@ -167,7 +169,7 @@ void UmlOperation::generate_decl(QTextOStream & f,
       }
       else if (!strncmp(p, "${type}", 7)) {
 	p += 7;
-	f << IdlSettings::type(returnType().toString());
+	UmlClass::write(f, returnType());
       }
       else if (!strncmp(p, "${name}", 7)) {
 	p += 7;
@@ -198,7 +200,7 @@ void UmlOperation::generate_decl(QTextOStream & f,
       }
       else if (!strncmp(p, "${stereotype}", 13)) {
 	p += 13;
-	f << IdlSettings::relationStereotype(stereotype());
+	f << IdlSettings::relationAttributeStereotype(stereotype());
       }
       else if (!strncmp(p, "${raisesnl}", 11)) {
 	if (exceptions.count() != 0) {
