@@ -46,6 +46,7 @@ void UmlRelation::generate_inherit(const char *& sep, QTextOStream & f,
       write_trace_header();
       UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>an <i>")
 		    + cl_stereotype + "</i> cannot inherits</b></font><br>");
+      incr_warning();
       return;
     }
     
@@ -56,6 +57,7 @@ void UmlRelation::generate_inherit(const char *& sep, QTextOStream & f,
       write_trace_header();
       UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>cannot inherits an <i>")
 		    + other_stereotype + "</i></b></font><br>");
+      incr_warning();
       return;
     }
     
@@ -83,7 +85,7 @@ void UmlRelation::generate_inherit(const char *& sep, QTextOStream & f,
     
     while (*p) {
       if (!strncmp(p, "${type}", 7)) {
-	f << role_type->name();
+	role_type->write(f);
 	p += 7;
       }
       else
@@ -102,11 +104,13 @@ void UmlRelation::generate_decl(QTextOStream & f,
     if (cl_stereotype == "enum") {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>an <i>enum</i> cannot have relation</b></font><br>");
+      incr_warning();
       return;
     }
     if (cl_stereotype == "typedef") {
       write_trace_header();
       UmlCom::trace("<tt>        </tt><font color=\"red\"><b>a <i>typedef</i> cannot have relation</b></font><br>");
+      incr_warning();
       return;
     }
     if (!idlDecl().isEmpty()) {
@@ -186,13 +190,14 @@ void UmlRelation::generate_decl(QTextOStream & f,
 	    write_trace_header();
 	    UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>unspecified <i>case</i> for <i>")
 			  + name() + "</b></font><br>");
+	    incr_error();
 	  }
 	  else
 	    f << idlCase();
 	}
 	else if (!strncmp(p, "${type}", 7)) {
 	  p += 7;
-	  f << roleType()->name();
+	  roleType()->write(f);
 	}
 	else if (!strncmp(p, "${name}", 7)) {
 	  p += 7;
@@ -200,7 +205,7 @@ void UmlRelation::generate_decl(QTextOStream & f,
 	}
 	else if (!strncmp(p, "${stereotype}", 13)) {
 	  p += 13;
-	  f << IdlSettings::relationStereotype(stereotype());
+	  f << IdlSettings::relationAttributeStereotype(stereotype());
 	}
 	else if (!strncmp(p, "${multiplicity}", 15)) {
 	  p += 15;
