@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -26,7 +26,10 @@
 #include "BrowserNode.h"
 #include "UmlClass.h"
 #include "Class.h"
-#include "api.h"
+#include "aRelationKind.h"
+#include "aDirection.h"
+#include "aVisibility.h"
+#include "anItemKind.h"
 #include "BrowserView.h"
 
 BrowserNode::BrowserNode(BrowserView * parent, const char * n)
