@@ -411,7 +411,8 @@ bool Class::add_inherit(aRelationKind k, UmlTypeSpec & typespec,
     actual_rank = uml->actuals().count();
     
   UmlRelation * rel =
-    UmlRelation::create(k, uml, typespec.type);
+    UmlRelation::create((actuals.isEmpty()) ? k : aRealization,
+			uml, typespec.type);
   
   if (rel == 0) {
 #ifdef TRACE
@@ -435,6 +436,8 @@ bool Class::add_inherit(aRelationKind k, UmlTypeSpec & typespec,
       uml->replaceActual(actual_rank++, *iter);
     
     rel->set_JavaDecl("${type}");
+    if (! actuals.isEmpty())
+      rel->set_Stereotype("bind");
   }
   
   return TRUE;
@@ -701,8 +704,9 @@ bool Class::manage_member(QCString s, QCString & path) {
 	}
       }
       
-      return UmlOperation::new_one(this, name, tmplts, oper_templ, type,
-				   str_actuals, visibility, m_finalp, m_abstractp,
+      return UmlOperation::new_one(this, name, tmplts, oper_templ,
+				   type, str_actuals, first_actual_class, type_def,
+				   visibility, m_finalp, m_abstractp,
 				   m_staticp, m_nativep, m_strictfp, m_synchronizedp,
 				   array, comment, description, annotation);
     }
