@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -33,11 +33,6 @@
 #include "Class.h"
 #endif
 
-UmlFormalParameter::UmlFormalParameter(QCString n, const UmlTypeSpec & e) {
-  _name = n;
-  _extends = e;
-}
-
 UmlClass::UmlClass(void * id, const QCString & n) 
     : UmlBaseClass(id, n) {
 }
@@ -45,6 +40,7 @@ UmlClass::UmlClass(void * id, const QCString & n)
 #ifdef REVERSE
 
 void UmlClass::need_artifact(const QStringList & imports,
+			     bool remove_java_lang,
 			     const QStringList & static_imports,
 			     UmlArtifact *& cp) {
   if (parent()->kind() == aClassView) {
@@ -74,11 +70,13 @@ void UmlClass::need_artifact(const QStringList & imports,
 	for (it = imports.begin(); *it; ++it) {
 	  QString import = *it;
 	  
-	  import += (((const char *) import)[import.length() - 1] == '.')
-	    ? "*;\n" : ";\n";
-	  
-	  s.insert(index, QString("import ") + import);
-	  index = s.find("${definition}", index);
+	  if (!remove_java_lang || (import != "java.lang.")) {
+	    import += (((const char *) import)[import.length() - 1] == '.')
+	      ? "*;\n" : ";\n";
+	    
+	    s.insert(index, QString("import ") + import);
+	    index = s.find("${definition}", index);
+	  }
 	}
 	
 	for (it = static_imports.begin(); *it; ++it) {
