@@ -30,6 +30,7 @@
 #include "JavaSettings.h"
 #include "UmlClass.h"
 #include "UmlCom.h"
+#include "util.h"
 
 void UmlAttribute::generate(QTextOStream & f, const QCString & st, QCString indent) {
   if (!javaDecl().isEmpty()) {
@@ -95,10 +96,35 @@ void UmlAttribute::generate(QTextOStream & f, const QCString & st, QCString inde
 	p += 7;
 	UmlClass::write(f, type());
       }
+      else if (!strncmp(p, "${stereotype}", 13)) {
+	p += 13;
+	f << JavaSettings::relationAttributeStereotype(stereotype());
+      }
       else if (!strncmp(p, "${name}", 7)) {
 	p += 7;
 	f << name();
       }
+      else if (!strncmp(p, "${multiplicity}", 15)) {
+	p += 15;
+	
+	QCString m = multiplicity();
+	
+	if (*m != '[')
+	  f << "[]";
+	else {
+	  for (unsigned index = 0; index != m.length(); index += 1) {
+	    switch (m.at(index)) {
+	    case '[':
+	      f << '[';
+	      break;
+	    case ']':
+	      f << ']';
+	    default:
+	      break;
+	    }
+	  }
+	}
+      }
       else if (!strncmp(p, "${value}", 8)) {
 	p += 8;
 	if (!defaultValue().isEmpty()) {
@@ -176,10 +202,12 @@ void UmlAttribute::generate_enum_pattern_item(QTextOStream & f, int & current_va
 	  int v;
 	  
 	  v = d.toInt(&ok);
-	  if (! ok) 
+	  if (! ok) {
 	    UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b><i>") +
 			  defaultValue() + "</i> wrong value for <i> "
 			  + name() + "</i></b></font><br>");
+	    incr_error();
+	  }
 	  else
 	    current_value = v;
 	}
