@@ -94,7 +94,7 @@ void UmlClass::generate(QTextOStream & f, QCString indent) {
 
     if (*p == '\n') {
       f << *p++;
-      if (*p && strncmp(p, "${members}", 10))
+      if (*p && strncmp(p, "${members}", 10) && strncmp(p, "${items}", 8))
 	f << indent;
     }
     else if (*p == '@')
@@ -219,7 +219,7 @@ void UmlClass::generate(QTextOStream & f, QCString indent) {
 	      f << '\n';
 	    }
 	  }
-	  else if ((ch[index]->kind() != aNcRelation) &&
+	  else if ((it->kind() != aNcRelation) &&
 		   !((UmlClassItem *) it)->javaDecl().isEmpty())
 	    ((UmlClassItem *) it)->generate(f, stereotype, indent);
 	}
@@ -317,6 +317,7 @@ void UmlClass::generate_enum_pattern_item(QTextOStream &, int &,
 					  const QCString &, QCString) {
   write_trace_header();
   UmlCom::trace("<tt>        </tt><font color=\"red\"><b>an <i>enum pattern</i> cannot have sub-class</b></font><br>");
+  incr_warning();
 }
 
 void UmlClass::generate_enum_pattern_case(QTextOStream &, QCString) {
