@@ -70,6 +70,7 @@ static void param_error(const QCString & parent, const QCString & name, unsigned
   UmlCom::trace(QCString("<tt>        </tt><font color=\"red\"><b>while compiling <i>")
 		+ parent + "::" + name + "</i> parameter rank " + QCString().setNum(rank)
 		+ " does not exist</font></b><br>");
+  incr_error();
 }
 
 QCString UmlOperation::compute_name() {
@@ -112,7 +113,7 @@ const char * UmlOperation::generate_body(QTextOStream & f,
   bool no_indent;
   char s_id[9];
   
-  if (preserve()) {
+  if (preserve() && !isBodyGenerationForced()) {
     unsigned id = get_id();
     
     sprintf(s_id, "%08X", id);
@@ -134,7 +135,7 @@ const char * UmlOperation::generate_body(QTextOStream & f,
   while (*p != '$')
     bindent += *p++;
 
-  if (preserve())
+  if (preserve() && !isBodyGenerationForced())
     f << bindent << BodyPrefix << s_id << '\n';
 
   if ((body != 0) && (*body != 0)) {
@@ -159,7 +160,7 @@ const char * UmlOperation::generate_body(QTextOStream & f,
     add_nl = body[-1] != '\n';
   }
   
-  if (preserve()) {
+  if (preserve() && !isBodyGenerationForced()) {
     if (add_nl)
       f << '\n';
     
@@ -278,7 +279,7 @@ void UmlOperation::generate(QTextOStream & f, const QCString & cl_stereotype,
       else if (!strncmp(p, "${stereotype}", 13)) {
 	p += 13;
 	// get/set relation with multiplicity > 1
-	f << JavaSettings::relationStereotype(stereotype());
+	f << JavaSettings::relationAttributeStereotype(stereotype());
       }
       else if (!strncmp(p, "${name}", 7)) {
 	p += 7;
@@ -360,6 +361,7 @@ void UmlOperation::generate_enum_pattern_item(QTextOStream &, int &,
 					      const QCString &, QCString) {
   write_trace_header();
   UmlCom::trace("<tt>        </tt><font color=\"red\"><b>an <i>enum</i> cannot have operation</b></font><br>");
+  incr_warning();
 }
 
 void UmlOperation::generate_enum_pattern_case(QTextOStream &, QCString) {
