@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -37,12 +37,13 @@ class UmlRelation : public UmlBaseRelation {
       : UmlBaseRelation(id, n) {
     };
   
-    //virtual void compute_dependency(QList<JavaRefType> & dependency, QCString cl_stereotype);
     virtual void generate(QTextOStream & f, const QCString & cl_stereotype,
 			  QCString indent);
-    virtual void generate_enum_pattern_item(QTextOStream &, int &, const QCString &, QCString);
+    virtual void generate_enum_pattern_item(QTextOStream &, int &,
+					    const QCString &, QCString);
     virtual void generate_enum_pattern_case(QTextOStream &, QCString);
     virtual void generate_enum_member(QTextOStream &, QCString);
+    virtual void generate_import(QTextOStream & f, const QCString & indent);
     
     void generate_extends(const char *& sep, QTextOStream & f,
 			  const QValueList<UmlActualParameter> & actuals,
