@@ -24,11 +24,29 @@
 // *************************************************************************
 
 #include "UmlOperation.h"
+#include "UmlClass.h"
 #include "UmlCom.h"
 #include "CppSettings.h"
 #include "JavaSettings.h"
 #include "util.h"
 
+static QCString remove_throw(QCString d)
+{
+  int index;
+  
+  index = d.find("${throw}");
+  if (index != -1)
+    d.remove(index, 8);
+  
+  return d;
+}
+
+
+void UmlOperation::remove_cpp_throw() {
+  set_CppDecl(remove_throw(CppSettings::operationDecl()));
+  set_CppDef(remove_throw(CppSettings::operationDef()));
+}
+
 void UmlOperation::add_param(int rank, aDirection dir,
 			     const char * name, const char * type) {
   UmlParameter p;
@@ -70,7 +88,7 @@ void UmlOperation::set_cpp(const char * return_form_or_inherit,
   if (*return_form_or_inherit == ':') {
     // inherit
     if (inlinep) {
-      QCString s = CppSettings::operationDecl();
+      QCString s = remove_throw(CppSettings::operationDecl());
       int index = s.find("${)}");
       
       s.resize(index + 5);
@@ -90,7 +108,7 @@ void UmlOperation::set_cpp(const char * return_form_or_inherit,
       set_CppDef("");
     }
     else {
-      QCString s = CppSettings::operationDecl();
+      QCString s = remove_throw(CppSettings::operationDecl());
       int index = s.find("${)}");
       
       s.resize(index + 5);
@@ -99,7 +117,7 @@ void UmlOperation::set_cpp(const char * return_form_or_inherit,
       conditional(s, if_def, end_if);
       set_CppDecl(s);
       
-      s = CppSettings::operationDef();
+      s = remove_throw(CppSettings::operationDef());
       index = s.find("${)}");
       s.resize(index + 5);
       s.insert(index, params);
@@ -119,12 +137,12 @@ void UmlOperation::set_cpp(const char * return_form_or_inherit,
   else {
     // return
     if (inlinep) {
-      QCString s = CppSettings::operationDecl();
+      QCString s = remove_throw(CppSettings::operationDecl());
       int index = s.find("${type}");
       
       s.replace(index, 7, return_form_or_inherit);
       s.insert(s.find("${)}", index), params);
-      s.resize(s.findRev(";"));
+      s.resize(s.findRev(";") + 1);
       if (*body) {
 	s.append(" {\n  ");
 	s.append(body);
@@ -138,7 +156,7 @@ void UmlOperation::set_cpp(const char * return_form_or_inherit,
       set_CppDef("");
     }
     else {
-      QCString s = CppSettings::operationDecl();
+      QCString s = remove_throw(CppSettings::operationDecl());
       int index = s.find("${type}");
       
       s.replace(index, 7, return_form_or_inherit);
@@ -146,7 +164,7 @@ void UmlOperation::set_cpp(const char * return_form_or_inherit,
       conditional(s, if_def, end_if);
       set_CppDecl(s);
       
-      s = CppSettings::operationDef();
+      s = remove_throw(CppSettings::operationDef());
       index = s.find("${type}");
       s.replace(index, 7, return_form_or_inherit);
       s.insert(s.find("${)}", index), params);
@@ -291,3 +309,64 @@ void UmlOperation::rename_jdk5() {
 		  " upgraded to take into account the rename of 'enum' to 'enum pattern'<br>\n");
   
 }
+
+
+UmlOperation * UmlOperation::java2Php(UmlClass * php, UmlClass * java,
+				      const char * javaname,
+				      const char * phpname)
+{
+  if (phpname == 0)
+    phpname = javaname;
+  
+  UmlOperation * from = java->get_operation(javaname);
+  
+  if (from == 0) {
+    QCString err = QCString("cannot find operation '") + 
+      javaname + QCString("' in class '") + java->name()
+	+ QCString("'<br>\n");
+    UmlCom::trace(err);
+    throw 0;
+  }
+  
+  UmlOperation * to = UmlBaseOperation::create(php, phpname);
+  
+  if (to == 0) {
+    QCString err = QCString("cannot create operation '") + 
+      phpname + QCString("' in class '") + php->name()
+	+ QCString("'<br>\n");
+    UmlCom::trace(err);
+    throw 0;
+  }
+  
+  UmlCom::trace("add operation " + php->name() + "::" + phpname + "<br>\n");
+
+  to->set_Description(::java2Php(from->description()));
+  to->set_ReturnType(from->returnType());
+  to->set_isClassMember(from->isClassMember());
+  to->set_Visibility(from->visibility());
+  to->set_CppVisibility(from->cppVisibility());
+  
+  const QValueList<UmlParameter> params = from->params();
+  unsigned index;
+  
+  for (index = 0; index != params.count(); index += 1)
+    to->addParameter(index, params[index]);
+  
+  const QValueList<UmlTypeSpec> exceptions = from->exceptions();
+  
+  for (index = 0; index != exceptions.count(); index += 1)
+    to->addException(index, exceptions[index]);
+  
+  to->set_isCppVirtual(from->isCppVirtual());
+  to->set_isCppConst(from->isCppConst());
+  to->set_isCppInline(from->isCppInline());
+  to->set_CppDecl(::java2Php(from->cppDecl()));
+  to->set_CppDef(::java2Php(from->cppDef()));
+  to->set_CppBody(::java2Php(from->cppBody()));
+  
+  to->set_isJavaFinal(from->isJavaFinal());
+  to->set_JavaDef(from->javaDef());
+  to->set_JavaBody(::java2Php(from->javaBody()));
+  
+  return to;
+}
