@@ -38,3 +38,10 @@ void UmlDeploymentView::roundtrip_java() {
   for (unsigned index = 0; index != ch.size(); index += 1)
     ch[index]->roundtrip_java();
 }
+
+void UmlDeploymentView::roundtrip_php() {
+  QVector<UmlItem> ch = UmlItem::children();
+  
+  for (unsigned index = 0; index != ch.size(); index += 1)
+    ch[index]->roundtrip_php();
+}
