@@ -28,7 +28,8 @@
 
 enum CmdFamily {
   onInstanceCmd, classGlobalCmd, packageGlobalCmd, miscGlobalCmd,
-  umlSettingsCmd, cppSettingsCmd, javaSettingsCmd, idlSettingsCmd
+  umlSettingsCmd, cppSettingsCmd, javaSettingsCmd, idlSettingsCmd,
+  phpSettingsCmd
 };
 
 enum MiscGlobalCmd {
@@ -80,6 +81,14 @@ enum OnInstanceCmd {
   
   sideCmd,
   
+  // api 31
+  attributesCmd,
+  relationsCmd,
+  
+  // api 34
+  getPhpDefCmd,
+  phpBodyCmd,
+  
   // set commands
   
   firstSetCmd = 50,	// just a mark
@@ -209,11 +218,33 @@ enum OnInstanceCmd {
   setJavaFrozenCmd,
   setIdlFrozenCmd,
   
-  // api30
+  // api 30
   setConstraintCmd,
   deleteCmd,
+  
+  // api 31
+  setAttributeCmd,
+  addRelationCmd,
+  removeRelationCmd,
+  
+  // api 34
+  setPhpDeclCmd,
+  setIsPhpExternalCmd,
+  setIsPhpFinalCmd,
+  setPhpSrcCmd,
+  setPhpFinalCmd,
+  setPhpBodyCmd,
+  setPhpNameSpecCmd,
+  setPhpDirCmd,
+  setPhpFrozenCmd,
+  
+  // api 35
+  setIsForceBodyGenCmd,
     
-  getIdCmd = 254, // not in plug-outs managed through bouml
+  setUserCmd = 252, // not in plug-outs managed through bouml
+  moveInCmd = 253,  // not in plug-outs managed through bouml
+
+  getIdCmd = 254, // not in old plug-outs managed through bouml
     
   // api 20
   old_deleteCmd = 255 // not in plug-outs managed through bouml
@@ -253,7 +284,7 @@ enum CppSettingsCmd {
   
   setCppUseDefaultsCmd,
   setCppTypeCmd,
-  setCppRelationStereotypeCmd,
+  setCppRelationAttributeStereotypeCmd,
   setCppClassStereotypeCmd,
   setCppIncludeCmd,
   setCppHeaderContentCmd,
@@ -309,7 +340,7 @@ enum JavaSettingsCmd {
   
   setJavaUseDefaultsCmd,
   setJavaTypeCmd,
-  setJavaRelationStereotypeCmd,
+  setJavaRelationAttributeStereotypeCmd,
   setJavaClassStereotypeCmd,
   setJavaImportCmd,
   setJavaRootdirCmd,
@@ -339,13 +370,38 @@ enum JavaSettingsCmd {
   setJavaJavadocStyleCmd
 };
 
+// api 34
+enum PhpSettingsCmd {
+  getPhpSettingsCmd,
+  getPhpUseDefaultsCmd,
+  
+  setPhpUseDefaultsCmd,
+  setPhpClassStereotypeCmd,
+  setPhpRootdirCmd,
+  setPhpSourceContentCmd,
+  setPhpSourceExtensionCmd,
+  setPhpClassDeclCmd,
+  setPhpEnumDeclCmd,
+  setPhpExternalClassDeclCmd,
+  setPhpInterfaceDeclCmd,
+  setPhpAttributeDeclCmd,
+  setPhpEnumItemDeclCmd,
+  setPhpRelationDeclCmd,
+  setPhpOperationDefCmd,
+  setPhpGetNameCmd,
+  setPhpSetNameCmd,
+  setPhpIsGetFinalCmd,
+  setPhpIsSetFinalCmd,
+  setPhpIsSetParamFinalCmd,
+};
+
 enum IdlSettingsCmd {
   getIdlSettingsCmd,
   getIdlUseDefaultsCmd,
   
   setIdlUseDefaultsCmd,
   setIdlTypeCmd,
-  setIdlRelationStereotypeCmd,
+  setIdlRelationAttributeStereotypeCmd,
   setIdlClassStereotypeCmd,
   setIdlIncludeCmd,
   setIdlRootDirCmd,
