@@ -183,6 +183,43 @@ bool UmlBaseClass::set_isJavaFinal(bool y) {
 }
 #endif
 
+#ifdef WITHPHP
+bool UmlBaseClass::isPhpExternal() {
+  read_if_needed_();
+  
+  return _php_external;
+}
+
+bool UmlBaseClass::set_isPhpExternal(bool y) {
+  bool r;
+  
+  if (set_it_(r, y, setIsPhpExternalCmd)) {
+    _php_external = y;
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+
+bool UmlBaseClass::isPhpFinal() {
+  read_if_needed_();
+  
+  return _php_final;
+}
+
+bool UmlBaseClass::set_isPhpFinal(bool y) {
+  bool r;
+  
+  if (set_it_(r, y, setIsPhpFinalCmd)) {
+    _php_final = y;
+    return TRUE;
+  }
+  else
+    return FALSE;
+
+}
+#endif
+
 #ifdef WITHIDL
 const UmlTypeSpec & UmlBaseClass::switchType() {
   read_if_needed_();
@@ -329,6 +366,14 @@ void UmlBaseClass::read_java_() {
 }
 #endif
 
+#ifdef WITHPHP
+void UmlBaseClass::read_php_() {
+  UmlBaseClassMember::read_php_();
+  _php_final = UmlCom::read_bool();
+  _php_external = UmlCom::read_bool();
+}
+#endif
+
 #ifdef WITHIDL
 void UmlBaseClass::read_idl_() {
   UmlBaseClassItem::read_idl_();
