@@ -9,6 +9,7 @@
 #include "UmlAttribute.h"
 #include "UmlOperation.h"
 #include "UmlClass.h"
+#include "UmlClassInstance.h"
 #include "UmlUseCase.h"
 #include "UmlNode.h"
 #include "UmlArtifact.h"
@@ -57,7 +58,15 @@
 #include "MiscGlobalCmd.h"
 
 bool UmlBaseItem::set_Name(const QCString & s) {
-  return set_it_(_name, s, setNameCmd);
+  // don't use _set_it else the name is not
+  // updated in case the item is not yet load (!_defined)
+  UmlCom::send_cmd(_identifier, setNameCmd, s);
+  if (UmlCom::read_bool()) {
+    _name = s;
+    return TRUE;
+  }
+  else
+    return FALSE;
 }
 
 const QCString & UmlBaseItem::stereotype() {
@@ -146,6 +155,12 @@ const QDict<QCString> UmlBaseItem::properties() {
   return _dict;
 }
 
+int UmlBaseItem::getIdentifier() {
+  read_if_needed_();
+
+  return _modeler_id;
+}
+
 bool UmlBaseItem::moveAfter(const UmlItem * x) {
   UmlCom::send_cmd(_identifier, moveAfterCmd, (x != 0) ? ((UmlBaseItem *)x)->_identifier : 0);
   if (UmlCom::read_bool()) {
@@ -230,43 +245,43 @@ QPtrDict<UmlItem> UmlBaseItem::_all(997);
 
 void UmlBaseItem::read_if_needed_() {
   if (!_defined) {
-#ifdef WITHCPP
-# ifdef WITHJAVA
-#  ifdef WITHIDL
+#if defined(WITHCPP) & defined(WITHJAVA) & defined(WITHPHP) & defined(WITHIDL)
     UmlCom::send_cmd(_identifier, getDefCmd);
     read_uml_();
     read_cpp_();
     read_java_();
+    read_php_();
     read_idl_();
-#  else
-    ... WITHIDL must be defined when WITHCPP and WITHJAVA are both defined
-#  endif
-# else
-#  ifdef WITHIDL
-    ... WITHJAVA must be defined when WITHCPP and WITHIDL are both defined
-#  else
+#else
+# if defined(WITHCPP) & !defined(WITHJAVA) & !defined(WITHPHP) & !defined(WITHIDL)
     UmlCom::send_cmd(_identifier, getCppDefCmd);
     read_uml_();
     read_cpp_();
-#  endif
-# endif
-#else
-# ifdef WITHJAVA
-#  ifdef WITHIDL
-    ... WITHCPP must be defined when WITHIDL and WITHJAVA are both defined
-#  else
+# else
+#  if !defined(WITHCPP) & defined(WITHJAVA) & !defined(WITHPHP) & !defined(WITHIDL)
     UmlCom::send_cmd(_identifier, getJavaDefCmd);
     read_uml_();
     read_java_();
-#  endif
-# else
-#  ifdef WITHIDL
+#  else
+#   if !defined(WITHCPP) & !defined(WITHJAVA) & defined(WITHPHP) & !defined(WITHIDL)
+    UmlCom::send_cmd(_identifier, getPhpDefCmd);
+    read_uml_();
+    read_php_();
+#   else
+#    if !defined(WITHCPP) & !defined(WITHJAVA) & !defined(WITHPHP) & defined(WITHIDL)
     UmlCom::send_cmd(_identifier, getIdlDefCmd);
     read_uml_();
     read_idl_();
-#  else
+#    else
+#     if !defined(WITHCPP) & !defined(WITHJAVA) & !defined(WITHPHP) & !defined(WITHIDL)
     UmlCom::send_cmd(_identifier, getUmlDefCmd);
     read_uml_();
+#     else
+    ... WITHCPP and WITHJAVA and WITHPHP and WITHIDL must be both defined or undefined
+    ... or only one of them must be defined
+#     endif
+#    endif
+#   endif
 #  endif
 # endif
 #endif
@@ -307,6 +322,8 @@ void UmlBaseItem::read_uml_() {
   _description = UmlCom::read_string();
   
   _marked = UmlCom::read_bool();
+  
+  _modeler_id = (int) UmlCom::read_unsigned();
 }
 
 #ifdef WITHCPP
@@ -319,6 +336,11 @@ void UmlBaseItem::read_java_() {
 }
 #endif
 
+#ifdef WITHPHP
+void UmlBaseItem::read_php_() {
+}
+#endif
+
 #ifdef WITHIDL
 void UmlBaseItem::read_idl_() {
 }
@@ -551,6 +573,8 @@ UmlItem * UmlBaseItem::read_()
       return new UmlJoinActivityNode(id, name);
     case aPartition:
       //return new UmlPartition(id, name);
+    case aClassInstance:
+      return new UmlClassInstance(id, name);
     default:
       UmlCom::bye();
       UmlCom::fatal_error(QCString("unknown item type ") + QCString().setNum(kind));
@@ -588,6 +612,25 @@ void UmlBaseItem::delete_it() {
   parent()->unload(TRUE);
 }
 
+// warning : operation bodies are lost
+bool UmlBaseItem::moveIn(UmlItem * x) {
+  UmlBaseItem * old_parent = (UmlBaseItem *)parent();
+
+  UmlCom::send_cmd(_identifier, moveInCmd, ((UmlBaseItem *)x)->_identifier);
+  if (UmlCom::read_bool()) {
+    old_parent->reread_children_if_needed_();
+    ((UmlBaseItem *)parent())->reread_children_if_needed_();
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+
+void UmlBaseItem::setUser(unsigned uid)
+{
+  UmlCom::send_cmd(_identifier, setUserCmd, uid);
+}
+
 UmlItem * UmlBaseItem::from_id(unsigned uid, anItemKind k)
 {
   UmlCom::send_cmd(miscGlobalCmd, fromIdCmd, uid, (char) k);
