@@ -153,6 +153,13 @@ class UmlBaseItem {
     //  UmlOperation (their bodies are not taken into account) , UmlClass
     //  and UmlComponents.
     const QVector<UmlItem> referencedBy();
+    
+    // return a constant identifier, it is unique within a given
+    // kind of element (two classes can't have the same identifiers, but
+    // a class and a component may have the same identifier) 
+    // except for the diagrams (a class diagram can't have the identifier
+    // of a component diagram)
+    int getIdentifier();
 
     // to unload the object to free memory, it will be reloaded automatically
     // if needed. Recursively done for the sub items if 'rec' is TRUE. 
@@ -172,6 +179,8 @@ class UmlBaseItem {
     
     // not in plug-outs managed through bouml
     void delete_it();
+    bool moveIn(UmlItem * x);
+    void setUser(unsigned uid);
     static UmlItem * from_id(unsigned uid, anItemKind);
 
     static const QVector<UmlItem> markedItems();
@@ -179,6 +188,7 @@ class UmlBaseItem {
   friend class UmlBaseClassItem;
   friend class UmlBaseAttribute;
   friend class UmlBaseClass;
+  friend class UmlBaseClassInstance;
   friend class UmlBaseClassDiagram;
   friend class UmlBaseClassView;
   friend class UmlBaseCollaborationDiagram;
@@ -211,6 +221,8 @@ class UmlBaseItem {
     bool _marked;
 
     void * _identifier;
+    
+    int _modeler_id;
 
     QCString _name;
 
@@ -254,6 +266,12 @@ class UmlBaseItem {
     virtual void read_java_();
 #endif
 
+#ifdef WITHPHP
+    //internal, do NOT use it
+    
+    virtual void read_php_();
+#endif
+
 #ifdef WITHIDL
     //internal, do NOT use it
     
