@@ -15,6 +15,23 @@ anItemKind UmlBaseOperation::kind() {
   return anOperation;
 }
 
+bool UmlBaseOperation::isBodyGenerationForced() {
+  read_if_needed_();
+    
+  return _force_body_generation;
+}
+
+bool UmlBaseOperation::set_isBodyGenerationForced(bool y) {
+  bool b;
+  
+  if (set_it_(b, y, setIsForceBodyGenCmd)) {
+    _force_body_generation = y;
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+
 bool UmlBaseOperation::isAbstract() {
   read_if_needed_();
     
@@ -340,6 +357,73 @@ bool UmlBaseOperation::set_JavaGetSetFrozen(bool v) {
 }
 #endif
 
+#ifdef WITHPHP
+bool UmlBaseOperation::isPhpFinal() {
+  read_if_needed_();
+    
+  return _php_final;
+}
+
+bool UmlBaseOperation::set_isPhpFinal(bool y) {
+  bool b;
+  
+  if (set_it_(b, y, setPhpFinalCmd)) {
+    _php_final = y;
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+
+const QCString & UmlBaseOperation::phpDef() {
+  return phpDecl();
+}
+
+bool UmlBaseOperation::set_PhpDef(const char * s) {
+  return set_PhpDecl(s);
+}
+
+QCString UmlBaseOperation::phpBody() {
+  // not memorized in the instance for memory size reason
+  UmlCom::send_cmd(_identifier, phpBodyCmd);
+  return UmlCom::read_string();
+}
+
+bool UmlBaseOperation::set_PhpBody(const char * s) {
+  // not memorized in the instance for memory size reason
+  UmlCom::send_cmd(_identifier, setPhpBodyCmd, s);
+  return UmlCom::read_bool();
+}
+
+const QCString & UmlBaseOperation::phpNameSpec() {
+  read_if_needed_();
+    
+  return _php_name_spec;
+}
+
+bool UmlBaseOperation::set_PhpNameSpec(const char * s) {
+  return set_it_(_php_name_spec, s, setPhpNameSpecCmd);
+}
+#endif
+
+#ifdef WITHPHP
+bool UmlBaseOperation::phpGetSetFrozen() {
+  read_if_needed_();
+  return _php_get_set_frozen;
+}
+
+bool UmlBaseOperation::set_PhpGetSetFrozen(bool v) {
+  bool vv;
+
+  if (set_it_(vv, v, setPhpFrozenCmd)) {
+    _php_get_set_frozen = v;
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+#endif
+
 #ifdef WITHIDL
 bool UmlBaseOperation::isIdlOneway() {
   read_if_needed_();
@@ -398,6 +482,9 @@ void UmlBaseOperation::unload(bool rec, bool del) {
 #ifdef WITHJAVA
   _java_name_spec = 0;
 #endif
+#ifdef WITHPHP
+  _php_name_spec = 0;
+#endif
 #ifdef WITHIDL
   _idl_name_spec = 0;
 #endif
@@ -411,6 +498,8 @@ void UmlBaseOperation::read_uml_() {
     _return_type.explicit_type = UmlCom::read_string();
   _abstract = UmlCom::read_bool();
   
+  _force_body_generation = UmlCom::read_bool();
+  
   unsigned n;
   
   for (n = UmlCom::read_unsigned(); n; n -= 1) {
@@ -460,6 +549,15 @@ void UmlBaseOperation::read_java_() {
 }
 #endif
 
+#ifdef WITHPHP
+void UmlBaseOperation::read_php_() {
+  UmlBaseClassMember::read_php_();
+  _php_final = UmlCom::read_bool();
+  _php_name_spec = UmlCom::read_string();
+  _php_get_set_frozen = UmlCom::read_bool();
+}
+#endif
+
 #ifdef WITHIDL
 void UmlBaseOperation::read_idl_() {
   UmlBaseClassMember::read_idl_();
