@@ -60,7 +60,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserActivity> BrowserActivity::all;
+IdDict<BrowserActivity> BrowserActivity::all(__FILE__);
 QStringList BrowserActivity::its_default_stereotypes;	// unicode
 
 BrowserActivity::BrowserActivity(QString s, BrowserNode * p, int id)
@@ -99,11 +99,25 @@ BrowserActivity::~BrowserActivity() {
 void BrowserActivity::clear(bool old)
 {
   all.clear(old);
+  BrowserActivityNode::clear(old);
+  BrowserParameter::clear(old);
+  BrowserActivityAction::clear(old);
+  BrowserInterruptibleActivityRegion::clear(old);
+  BrowserExpansionRegion::clear(old);
+  BrowserActivityObject::clear(old);
+  BrowserFlow::clear(old);
 }
 
 void BrowserActivity::update_idmax_for_root()
 {
   all.update_idmax_for_root();
+  BrowserActivityNode::update_idmax_for_root();
+  BrowserParameter::update_idmax_for_root();
+  BrowserActivityAction::update_idmax_for_root();
+  BrowserInterruptibleActivityRegion::update_idmax_for_root();
+  BrowserExpansionRegion::update_idmax_for_root();
+  BrowserActivityObject::update_idmax_for_root();
+  BrowserFlow::update_idmax_for_root();
 }
     
 void BrowserActivity::referenced_by(QList<BrowserNode> & l) {
@@ -190,10 +204,6 @@ BrowserNode * BrowserActivity::add_parameter(BrowserParameter * param) {
   return 0;
 }
 
-void BrowserActivity::add_activity_diagram() {
-  (new BrowserActivityDiagram(child_random_name("Diagram"), this))->select_in_browser();
-}
-
 void BrowserActivity::menu() {
   QPopupMenu m(0, name);
   QPopupMenu toolm(0);
@@ -263,7 +273,14 @@ Note that you can undelete it after");
 void BrowserActivity::exec_menu_choice(int rank) {
   switch (rank) {
   case 0:
-    add_activity_diagram();
+    {
+      BrowserActivityDiagram * d = 
+	BrowserActivityDiagram::add_activity_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 1:
     add_parameter(0);
@@ -408,6 +425,10 @@ UmlCode BrowserActivity::get_type() const {
   return UmlActivity;
 }
 
+int BrowserActivity::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserActivity::get_data() const {
   return def;
 }
@@ -793,11 +814,20 @@ BrowserActivity * BrowserActivity::read(char * & st, char * k,
     
     if (result == 0)
       result = new BrowserActivity(read_string(st), parent, id);
+    else if (result->is_defined) {
+      BrowserActivity * already_exist = result;
+
+      result = new BrowserActivity(read_string(st), parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("activity", result);
+    }
     else {
       result->set_parent(parent);
       result->set_name(read_string(st));
     }
     
+    result->is_defined = TRUE;
     k = read_keyword(st);
     result->def->read(st, k);
 
