@@ -54,7 +54,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserActivityAction> BrowserActivityAction::all(257);
+IdDict<BrowserActivityAction> BrowserActivityAction::all(257, __FILE__);
 QStringList BrowserActivityAction::its_default_stereotypes;	// unicode
 
 BrowserActivityAction::BrowserActivityAction(QString s, BrowserNode * p, int id)
@@ -100,11 +100,15 @@ BrowserActivityAction::~BrowserActivityAction() {
 void BrowserActivityAction::clear(bool old)
 {
   all.clear(old);
+  BrowserParameterSet::clear(old);
+  BrowserPin::clear(old);
 }
 
 void BrowserActivityAction::update_idmax_for_root()
 {
   all.update_idmax_for_root();
+  BrowserParameterSet::update_idmax_for_root();
+  BrowserPin::update_idmax_for_root();
 }
     
 void BrowserActivityAction::referenced_by(QList<BrowserNode> & l) {
@@ -189,8 +193,12 @@ BrowserActivityAction::add_parameter_set(BrowserParameterSet * paramset,
 
 // flow or dependency
 BasicData * BrowserActivityAction::add_relation(UmlCode l, BrowserNode * end) {
-  if (l == UmlFlow)
-    return (new BrowserFlow(this, end))->get_data();
+  if (l == UmlFlow) {
+    BasicData * result = (new BrowserFlow(this, end))->get_data();
+   
+    modified();	// to update diagrams
+    return result;
+  }
   else
     return BrowserNode::add_relation(l, end);
 }
@@ -489,7 +497,7 @@ BrowserActivityAction::add_remove_variable_value(BrowserNode * future_parent,
 void BrowserActivityAction::menu() {
   QString s = name;
   int index;
-  BrowserNode * who;
+  BrowserNode * who = 0;
   
   if (s.isEmpty()) {
     s = stringify(def->get_action_kind());
@@ -676,6 +684,10 @@ UmlCode BrowserActivityAction::get_type() const {
   return UmlActivityAction;
 }
 
+int BrowserActivityAction::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserActivityAction::get_data() const {
   return def;
 }
@@ -1051,11 +1063,20 @@ BrowserActivityAction * BrowserActivityAction::read(char * & st, char * k,
     
     if (result == 0)
       result = new BrowserActivityAction(read_string(st), parent, id);
+    else if (result->is_defined) {
+      BrowserActivityAction * already_exist = result;
+
+      result = new BrowserActivityAction(read_string(st), parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("activity action", result);
+    }
     else {
       result->set_parent(parent);
       result->set_name(read_string(st));
     }
     
+    result->is_defined = TRUE;
     k = read_keyword(st);
     result->def->read(st, k);
     
