@@ -56,7 +56,7 @@ BrowserActivityDiagram::BrowserActivityDiagram(QString s, BrowserNode * p, int i
 }
 
 BrowserActivityDiagram::BrowserActivityDiagram(BrowserActivityDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -125,6 +125,17 @@ void BrowserActivityDiagram::make() {
   package_color = UmlDefaultColor;
 }
 
+BrowserActivityDiagram *  BrowserActivityDiagram::add_activity_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter activity diagram's name : ",
+				      UmlActivityDiagram, TRUE, FALSE))
+    return new BrowserActivityDiagram(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserActivityDiagram::set_name(const char * s) {
   BrowserDiagram::set_name(s);
   if (window != 0)
@@ -236,11 +247,13 @@ void BrowserActivityDiagram::exec_menu_choice(int rank) {
     return;
   case 3:
     {
-      BrowserActivityDiagram * cd =
-	new BrowserActivityDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      cd->select_in_browser();
-      cd->edit("Activity diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter activity diagram's name : ",
+						      UmlActivityDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -334,6 +347,10 @@ UmlCode BrowserActivityDiagram::get_type() const {
   return UmlActivityDiagram;
 }
 
+int BrowserActivityDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserActivityDiagram::get_activitydiagramsettings(ActivityDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_activitydiagramsettings(r);
@@ -499,14 +516,6 @@ bool BrowserActivityDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
-void BrowserActivityDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserActivityDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserActivityDiagram::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
   if (after == 0)
     ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -624,11 +633,21 @@ BrowserActivityDiagram * BrowserActivityDiagram::read(char * & st, char * k,
     
     if ((r = (BrowserActivityDiagram *) all[id]) == 0)
       r = new BrowserActivityDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserActivityDiagram * already_exist = r;
+
+      r = new BrowserActivityDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("activity diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = (!in_import() && read_only_file()) || 
       (user_id() != 0) && r->is_api_base();
     
