@@ -53,14 +53,14 @@
 #include "BrowserView.h"
 #include "mu.h"
 
-IdDict<BrowserArtifact> BrowserArtifact::all(257);
+IdDict<BrowserArtifact> BrowserArtifact::all(257, __FILE__);
 QStringList BrowserArtifact::its_default_stereotypes;	// unicode
 QStringList BrowserArtifact::relation_default_stereotypes;	// unicode
 
 BrowserArtifact::BrowserArtifact(QString s, BrowserNode * p, int id)
     : BrowserNode(s, p), Labeled<BrowserArtifact>(all, id),
       cpp_h_edited(FALSE), cpp_src_edited(FALSE),
-      java_edited(FALSE), idl_edited(FALSE) {
+      java_edited(FALSE), php_edited(FALSE), idl_edited(FALSE) {
   def = new ArtifactData;
   def->set_browser_node(this);
   
@@ -71,7 +71,7 @@ BrowserArtifact::BrowserArtifact(const BrowserArtifact * model,
 				 BrowserNode * p)
     : BrowserNode(model->name, p), Labeled<BrowserArtifact>(all, 0),
       cpp_h_edited(FALSE), cpp_src_edited(FALSE),
-      java_edited(FALSE), idl_edited(FALSE) {
+      java_edited(FALSE), php_edited(FALSE), idl_edited(FALSE) {
   def = new ArtifactData(model->def, this);
   comment = model->comment;
   associated_diagram = 0;  
@@ -80,7 +80,7 @@ BrowserArtifact::BrowserArtifact(const BrowserArtifact * model,
 BrowserArtifact::BrowserArtifact(int id)
     : BrowserNode(), Labeled<BrowserArtifact>(all, id),
       cpp_h_edited(FALSE), cpp_src_edited(FALSE),
-      java_edited(FALSE), idl_edited(FALSE) {
+      java_edited(FALSE), php_edited(FALSE), idl_edited(FALSE) {
   // not yet read
   def = new ArtifactData;
   def->set_browser_node(this);
@@ -176,7 +176,7 @@ QString BrowserArtifact::full_name(bool rev, bool) const {
 }
 
 QString BrowserArtifact::get_path(QString path, QString root,
-				   const char * ext) const {
+				  const char * ext) const {
   if (QDir::isRelativePath(root))
     root = BrowserView::get_dir().filePath(root);
   
@@ -192,33 +192,44 @@ QString BrowserArtifact::get_path(QString path, QString root,
   
   QDir d(path);
     
-  path = d.filePath(name + "." + ext);
+  path = (ext != 0) ? d.filePath(name + "." + ext) : d.filePath(name);
   
   return (QFile::exists(path)) ? path : QString::null;
 }
 
 void BrowserArtifact::get_paths(QString & cpp_h_path, QString & cpp_src_path,
-				 QString & java_path, QString & idl_path) const {
-  if (!strcmp(def->get_stereotype(), "source")) {
+				QString & java_path, QString & php_path, 
+				QString & idl_path) const {
+  bool a_text = !strcmp(def->get_stereotype(), "text");
+  
+  if (a_text || !strcmp(def->get_stereotype(), "source")) {
     PackageData * pd = (PackageData *)
       ((BrowserNode *) parent()->parent())->get_data();
     
-    if (def->get_cpp_h()[0] != 0)
+    if (!a_text && (def->get_cpp_h()[0] != 0))
       cpp_h_path = get_path((const char *) pd->get_cpp_h_dir(),
 			    GenerationSettings::get_cpp_root_dir(),
 			    GenerationSettings::get_cpp_h_extension());
     if (def->get_cpp_src()[0] != 0)
       cpp_src_path = get_path((const char *) pd->get_cpp_src_dir(),
 			      GenerationSettings::get_cpp_root_dir(),
-			      GenerationSettings::get_cpp_src_extension());
+			      (a_text) ? (const char *) 0
+				       : (const char *) GenerationSettings::get_cpp_src_extension());
     if (def->get_java_src()[0] != 0)
       java_path = get_path((const char *) pd->get_java_dir(),
 			   GenerationSettings::get_java_root_dir(),
-			   GenerationSettings::get_java_extension());
+			   (a_text) ? (const char *) 0
+				    : (const char *) GenerationSettings::get_java_extension());
+    if (def->get_php_src()[0] != 0)
+      php_path = get_path((const char *) pd->get_php_dir(),
+			  GenerationSettings::get_php_root_dir(),
+			  (a_text) ? (const char *) 0
+				   : (const char *) GenerationSettings::get_php_extension());
     if (def->get_idl_src()[0] != 0)
       idl_path = get_path((const char *) pd->get_idl_dir(),
 			  GenerationSettings::get_idl_root_dir(),
-			  GenerationSettings::get_idl_extension());
+			  (a_text) ? (const char *) 0
+				   : (const char *) GenerationSettings::get_idl_extension());
   }
 }
 
@@ -249,6 +260,7 @@ void BrowserArtifact::menu() {
   QString cpp_h_path;
   QString cpp_src_path;
   QString java_path;
+  QString php_path;
   QString idl_path;
   
   m.insertItem(new MenuTitle(name, m.font()), -1);
@@ -265,22 +277,27 @@ a double click with the left mouse button does the same thing");
 Note that you can undelete it after");
       }
     }
-    if (!strcmp(def->get_stereotype(), "source")) {
+    
+    bool a_text = !strcmp(def->get_stereotype(), "text");
+    
+    if (a_text || !strcmp(def->get_stereotype(), "source")) {
       m.insertSeparator();
       m.insertItem("Generate", &gensubm);
       gensubm.insertItem("C++", 10);
       gensubm.insertItem("Java", 11);
+      gensubm.insertItem("Php", 22);
       gensubm.insertItem("Idl", 12);
       
-      if (preserve_bodies()) {
+      if (!a_text && preserve_bodies()) {
 	m.insertItem("Roundtrip body", &roundtripbodysubm);
 	
 	roundtripbodysubm.insertItem("C++", 30);
 	roundtripbodysubm.insertItem("Java", 31);
+	roundtripbodysubm.insertItem("Php", 32);
       }
       
-      get_paths(cpp_h_path, cpp_src_path, java_path, idl_path);
-      if (!cpp_h_edited && !cpp_h_path.isEmpty()) {
+      get_paths(cpp_h_path, cpp_src_path, java_path, php_path, idl_path);
+      if (!a_text && !cpp_h_edited && !cpp_h_path.isEmpty()) {
 	//if (! cpp_src_path.isEmpty())
 	  //roundtripsubm.insertItem("C++ header & source files", 13);
 	editsubm.insertItem("C++ header file", 14);
@@ -294,6 +311,10 @@ Note that you can undelete it after");
 	editsubm.insertItem("Java source file", 18);
 	//roundtripsubm.insertItem("Java source file", 19);
       }
+      if (!php_edited && !php_path.isEmpty()) {
+	editsubm.insertItem("Php source file", 23);
+	//roundtripsubm.insertItem("Php source file", 24);
+      }
       if (!idl_edited && !idl_path.isEmpty()) {
 	editsubm.insertItem("Idl source file", 20);
 	//roundtripsubm.insertItem("Idl source file", 21);
@@ -350,13 +371,15 @@ through a relation");
   }
   
   exec_menu_choice(m.exec(QCursor::pos()),
-		   cpp_h_path, cpp_src_path, java_path, idl_path);
+		   cpp_h_path, cpp_src_path,
+		   java_path, php_path, idl_path);
 }
 
 void BrowserArtifact::exec_menu_choice(int rank,
 				       QString cpp_h_path,
 				       QString cpp_src_path,
 				       QString java_path,
+				       QString php_path,
 				       QString idl_path) {
   switch (rank) {
   case 0:
@@ -391,6 +414,16 @@ void BrowserArtifact::exec_menu_choice(int rank,
 		   this);
     }
     return;
+  case 22:
+    {
+      bool preserve = preserve_bodies();
+      
+      ToolCom::run((verbose_generation()) 
+		   ? ((preserve) ? "php_generator -v -p" : "php_generator -v")
+		   : ((preserve) ? "php_generator -p" : "php_generator"), 
+		   this);
+    }
+    return;
   case 12:
     ToolCom::run((verbose_generation()) ? "idl_generator -v" : "idl_generator", this);
     return;
@@ -415,6 +448,12 @@ void BrowserArtifact::exec_menu_choice(int rank,
   case 19:
     //roundtrip Java source files
     return;
+  case 23:
+    (new SourceDialog(php_path, php_edited, edition_number))->show();
+    return;
+  case 24:
+    //roundtrip Php source files
+    return;
   case 20:
     (new SourceDialog(idl_path, idl_edited, edition_number))->show();
     return;
@@ -427,6 +466,9 @@ void BrowserArtifact::exec_menu_choice(int rank,
   case 31:
     ToolCom::run((verbose_generation()) ? "roundtrip_body -v java" : "roundtrip_body java", this);
     return;
+  case 32:
+    ToolCom::run((verbose_generation()) ? "roundtrip_body -v php" : "roundtrip_body php", this);
+    return;
   case 9999:
     {
       ClassListDialog dialog("Choose class", associated_classes);
@@ -463,6 +505,7 @@ void BrowserArtifact::apply_shortcut(QString s) {
   QString cpp_h_path;
   QString cpp_src_path;
   QString java_path;
+  QString php_path;
   QString idl_path;
   
   if (!deletedp()) {
@@ -474,50 +517,77 @@ void BrowserArtifact::apply_shortcut(QString s) {
 	  choice = 1;
       }
     }
-    if (!strcmp(def->get_stereotype(), "source")) {
+    
+    bool a_text = !strcmp(def->get_stereotype(), "text");
+    
+    if (a_text || !strcmp(def->get_stereotype(), "source")) {
       if (s == "Generate C++")
 	choice = 10;
       else if (s == "Generate Java")
 	choice = 11;
+      else if (s == "Generate Php")
+	choice = 22;
       else if (s == "Generate Idl")
 	choice = 12;
-      
-      get_paths(cpp_h_path, cpp_src_path, java_path, idl_path);
-      if (!cpp_h_edited && !cpp_h_path.isEmpty()) {
-	//if (! cpp_src_path.isEmpty())
-	  //roundtripsubm.insertItem("C++ header & source files", 13);
-	if (s == "See C++ header file")
-	  choice = 14;
-	//roundtripsubm.insertItem("C++ header file", 15);
+      else {
+	if (!a_text && preserve_bodies()) {
+	  if (s == "Roundtrip C++ operation body")
+	    choice = 30;
+	  else if (s == "Roundtrip Java operation body")
+	    choice = 31;
+	  else if (s == "Roundtrip Php operation body")
+	    choice = 32;
+	}
+	
+	if (choice == -1) {
+	  get_paths(cpp_h_path, cpp_src_path, java_path, php_path, idl_path);
+	  if (!a_text && !cpp_h_edited && !cpp_h_path.isEmpty()) {
+	    //if (! cpp_src_path.isEmpty())
+	    //roundtripsubm.insertItem("C++ header & source files", 13);
+	    if (s == "See C++ header file")
+	      choice = 14;
+	    //roundtripsubm.insertItem("C++ header file", 15);
+	  }
+	  if (!cpp_src_edited && !cpp_src_path.isEmpty()) {
+	    if (s == "See C++ source file")
+	      choice = 16;
+	    //roundtripsubm.insertItem("C++ source file", 17);
+	  }
+	  if (!java_edited && !java_path.isEmpty()) {
+	    if (s == "Java source file")
+	      choice = 18;
+	    //roundtripsubm.insertItem("Java source file", 19);
+	  }
+	  if (!php_edited && !php_path.isEmpty()) {
+	    if (s == "Php source file")
+	      choice = 23;
+	    //roundtripsubm.insertItem("Php source file", 24);
+	  }
+	  if (!idl_edited && !idl_path.isEmpty()) {
+	    if (s == "Idl source file")
+	      choice = 20;
+	    //roundtripsubm.insertItem("Idl source file", 21);
+	  }
+	}
       }
-      if (!cpp_src_edited && !cpp_src_path.isEmpty()) {
-	if (s == "See C++ source file")
-	  choice = 16;
-	//roundtripsubm.insertItem("C++ source file", 17);
-      }
-      if (!java_edited && !java_path.isEmpty()) {
-	if (s == "Java source file")
-	  choice = 18;
-	//roundtripsubm.insertItem("Java source file", 19);
-      }
-      if (!idl_edited && !idl_path.isEmpty()) {
-	if (s == "Idl source file")
-	  choice = 20;
-	//roundtripsubm.insertItem("Idl source file", 21);
+    }
+    
+    if (choice == -1) {
+      if (s == "Referenced by")
+	choice = 3;
+      else {
+	mark_shortcut(s, choice, 90);
+	if ((choice == -1) && (edition_number == 0))
+	  Tool::shortcut(s, choice, get_type(), 100);
       }
     }
-    if (s == "Referenced by")
-      choice = 3;
-    mark_shortcut(s, choice, 90);
-    if (edition_number == 0)
-      Tool::shortcut(s, choice, get_type(), 100);
   }
   else if (!is_read_only && (edition_number == 0))
     if (s == "Undelete")
       choice = 2;
   
-  exec_menu_choice(choice,
-		   cpp_h_path, cpp_src_path, java_path, idl_path);
+  exec_menu_choice(choice, cpp_h_path, cpp_src_path,
+		   java_path, php_path, idl_path);
 }
 
 void BrowserArtifact::open(bool force_edit) {
@@ -529,10 +599,21 @@ void BrowserArtifact::open(bool force_edit) {
     def->edit();
 }
 
+void BrowserArtifact::modified() {
+  repaint();
+  ((BrowserNode *) parent())->modified();
+  // to update regions drawing
+  def->modified();
+}
+
 UmlCode BrowserArtifact::get_type() const {
   return UmlArtifact;
 }
 
+int BrowserArtifact::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserArtifact::DragMoveEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, BrowserSimpleRelation::drag_key(this)))
     e->accept();
@@ -569,6 +650,7 @@ void BrowserArtifact::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
       if (old != this) {
 	old->modified();
 	old->package_modified();
+	bn->modified();
       }
       
       modified();
@@ -723,6 +805,10 @@ void BrowserArtifact::associate_class(BrowserClass * c, bool on_read) {
       if (*def->get_java_src() == 0)
 	def->use_default_java_src();
     }
+    if (*cd->get_phpdecl() != 0) {
+      if (*def->get_php_src() == 0)
+	def->use_default_php_src();
+    }
     if (*cd->get_idldecl() != 0) {
       if (*def->get_idl_src() == 0)
 	def->use_default_idl_src();
@@ -798,6 +884,7 @@ void BrowserArtifact::init()
   its_default_stereotypes.append("file");
   its_default_stereotypes.append("script");
   its_default_stereotypes.append("source");
+  its_default_stereotypes.append("text");
   its_default_stereotypes.append("library");
   its_default_stereotypes.append("executable");
   
@@ -813,7 +900,7 @@ const QStringList & BrowserArtifact::default_stereotypes()
 }
 
 // unicode
-const QStringList & BrowserArtifact::default_stereotypes(UmlCode) {
+const QStringList & BrowserArtifact::default_stereotypes(UmlCode, const BrowserNode *) const {
   return relation_default_stereotypes;
 }
 
@@ -857,6 +944,12 @@ bool BrowserArtifact::tool_cmd(ToolCom * com, const char * args) {
 	  com->write_ack(TRUE);
 	}
 	break;
+      case setUserCmd:
+	// plug-out upgrade
+	all.remove(get_ident());
+	new_ident(com->get_unsigned(args), all);      
+	package_modified();
+	break;
       case createCmd:
 	{
 	  bool ok = TRUE;
@@ -1053,12 +1146,23 @@ BrowserArtifact * BrowserArtifact::read(char * & st, char * k,
       result = new BrowserArtifact(s, parent, id);
       result->def->read(st, k);	// updates k
     }
+    else if (result->is_defined) {
+      BrowserArtifact * already_exist = result;
+
+      result = new BrowserArtifact(s, parent, id);
+      result->def->read(st, k);	// updates k
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("artifact", result);
+    }
     else {
       result->def->read(st, k);	// updates k
       result->set_parent(parent);
       result->set_name(s);
     }
     
+    result->is_defined = TRUE;
+
     result->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && result->is_api_base();
     
