@@ -37,6 +37,7 @@
 #include "AttributeData.h"
 #include "ClassData.h"
 #include "BrowserActivityAction.h"
+#include "BrowserClassInstance.h"
 #include "UmlPixmap.h"
 #include "UmlGlobal.h"
 #include "myio.h"
@@ -47,7 +48,7 @@
 #include "ReferenceDialog.h"
 #include "mu.h"
 
-IdDict<BrowserAttribute> BrowserAttribute::all(1023);
+IdDict<BrowserAttribute> BrowserAttribute::all(1023, __FILE__);
 QStringList BrowserAttribute::its_default_stereotypes;	// unicode
 
 BrowserAttribute::BrowserAttribute(QString s, BrowserNode * p, AttributeData * d, int id)
@@ -139,7 +140,8 @@ void BrowserAttribute::set_set_oper(BrowserOperation * op) {
 
 void BrowserAttribute::update_get_oper() {
   get_oper->update_get_of(name, def->get_cppdecl(),
-			  def->get_javadecl(), def->get_idldecl(), 
+			  def->get_javadecl(), def->get_phpdecl(), 
+			  def->get_idldecl(), 
 			  def->get_isa_const_attribute(),
 			  def->get_isa_class_attribute(),
 			  def->get_type());
@@ -147,7 +149,8 @@ void BrowserAttribute::update_get_oper() {
 
 void BrowserAttribute::update_set_oper() {
   set_oper->update_set_of(name, def->get_cppdecl(),
-			  def->get_javadecl(), def->get_idldecl(), 
+			  def->get_javadecl(), def->get_phpdecl(), 
+			  def->get_idldecl(), 
 			  def->get_isa_const_attribute(),
 			  def->get_isa_class_attribute(),
 			  def->get_type());
@@ -328,6 +331,8 @@ void BrowserAttribute::apply_shortcut(QString s) {
     if (!is_read_only && (edition_number == 0)) {
       if (s == "Delete")
 	choice = 1;
+      else if (s == "Duplicate")
+	choice = 6;
     }
     mark_shortcut(s, choice, 90);
     if (edition_number == 0)
@@ -359,6 +364,10 @@ UmlCode BrowserAttribute::get_type() const {
   return UmlAttribute;
 }
 
+int BrowserAttribute::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserAttribute::get_data() const {
   return def;
 }
@@ -410,6 +419,7 @@ void BrowserAttribute::compute_referenced_by(QList<BrowserNode> & l,
 void BrowserAttribute::referenced_by(QList<BrowserNode> & l) {
   BrowserNode::referenced_by(l);
   BrowserActivityAction::compute_referenced_by(l, this);
+  BrowserClassInstance::compute_referenced_by(l, this);
 }
 
 bool BrowserAttribute::tool_cmd(ToolCom * com, const char * args) {
@@ -417,14 +427,6 @@ bool BrowserAttribute::tool_cmd(ToolCom * com, const char * args) {
 	  BrowserNode::tool_cmd(com, args));
 }
 
-void BrowserAttribute::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserAttribute::DropEvent(QDropEvent * e) {
-  DropAfterEvent(e, 0);
-}
-
 void BrowserAttribute::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
   if (after == 0)
     ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -538,12 +540,23 @@ BrowserAttribute * BrowserAttribute::read(char * & st, char * k,
       result = new BrowserAttribute(s, parent, new AttributeData, id);
       result->def->read(st, k);	// updates k2
     }
+    else if (result->is_defined) {
+      BrowserAttribute * already_exist = result;
+
+      result = new BrowserAttribute(s, parent, new AttributeData, id);
+      result->def->read(st, k);	// updates k2
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("attribute", result);
+    }
     else {
       result->def->read(st, k);	// updates k2
       result->set_parent(parent);
       result->set_name(s);
     }
     
+    result->is_defined = TRUE;
+
     result->is_read_only = (!in_import() && read_only_file()) || 
       (user_id() != 0) && result->is_api_base();
     result->def->set_browser_node(result, FALSE, FALSE);
