@@ -75,6 +75,7 @@ class BrowserAttribute : public BrowserNode, public Labeled<BrowserAttribute> {
     virtual void apply_shortcut(QString s);
     virtual void open(bool);
     virtual UmlCode get_type() const;
+    virtual int get_identifier() const;
     virtual void modified();
     virtual BasicData * get_data() const;
     virtual bool allow_spaces() const;
@@ -101,8 +102,6 @@ class BrowserAttribute : public BrowserNode, public Labeled<BrowserAttribute> {
     
     virtual bool tool_cmd(ToolCom * com, const char * args);
     
-    virtual void DragMoveEvent(QDragMoveEvent * e);
-    virtual void DropEvent(QDropEvent * e);
     virtual void DropAfterEvent(QDropEvent * e, BrowserNode * after);
     
     static const QStringList & default_stereotypes();
