@@ -70,7 +70,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserClass> BrowserClass::all(257);
+IdDict<BrowserClass> BrowserClass::all(257, __FILE__);
 
 QStringList BrowserClass::its_default_stereotypes;	// unicode
 QStringList BrowserClass::relations_default_stereotypes[UmlRelations];	// unicode
@@ -371,10 +371,13 @@ the <em>class view</em>, this artifact will contain the generated code of the cl
 	}
       }
     }
-    else
+    else {
       m.setWhatsThis(m.insertItem("Edit", 3),
 		     "to edit the <em>class</em>, \
 a double click with the left mouse button does the same thing");
+      m.setWhatsThis(m.insertItem("Duplicate", 13),
+		     "to duplicate the <em>class</em>");
+    }
     
     bool have_sep = FALSE;
     
@@ -392,13 +395,13 @@ a double click with the left mouse button does the same thing");
       if (associated_components.first() == associated_components.last())
 	// only one component
 	m.setWhatsThis(m.insertItem("Select associated component", 100000),
-		       "to select a <em>component</em> realizing or providing the <em>class</em>");
+		       "to select the <em>component</em> providing the <em>class</em>");
       else {
 	compsubm.insertItem(new MenuTitle("Choose component", m.font()), -1);
 	compsubm.insertSeparator();
 	
 	m.setWhatsThis(m.insertItem("Select an associated component", &compsubm),
-		       "to select a <em>component</em> realizing or providing the <em>class</em>");
+		       "to select a <em>component</em> providing the <em>class</em>");
 	
 	index = 100000;
 	
@@ -420,6 +423,7 @@ a double click with the left mouse button does the same thing");
       m.insertItem("Generate", &gensubm);    
       gensubm.insertItem("C++", 10);
       gensubm.insertItem("Java", 11);
+      gensubm.insertItem("Php", 22);
       gensubm.insertItem("Idl", 12);
     }
     if ((edition_number == 0) && 
@@ -524,6 +528,16 @@ void BrowserClass::exec_menu_choice(int index,
 		   this);
     }
     return;
+  case 22:
+    {
+      bool preserve = preserve_bodies();
+      
+      ToolCom::run((verbose_generation()) 
+		   ? ((preserve) ? "php_generator -v -p" : "php_generator -v")
+		   : ((preserve) ? "php_generator -p" : "php_generator"), 
+		   this);
+    }
+    return;
   case 12:
     ToolCom::run((verbose_generation()) ? "idl_generator -v" : "idl_generator", this);
     return;
@@ -534,6 +548,8 @@ void BrowserClass::exec_menu_choice(int index,
       if (((BrowserNode *) parent())->enter_child_name(name, "enter class's name : ",
 						       UmlClass, FALSE, FALSE))
 	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 14: 
@@ -658,6 +674,8 @@ void BrowserClass::apply_shortcut(QString s) {
 	choice = 10;
       else if (s == "Generate Java")
 	choice = 11;
+      else if (s == "Generate Php")
+	choice = 22;
       else if (s == "Generate Idl")
 	choice = 12;
     }
@@ -940,6 +958,10 @@ UmlCode BrowserClass::get_type() const {
   return UmlClass;
 }
 
+int BrowserClass::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserClass::modified() {
   repaint();
   def->modified();
@@ -976,11 +998,33 @@ void BrowserClass::get_all_parents(QList<BrowserClass> & l) const {
   }
 }
 
+// returns all attributes included the inherited
+
+void BrowserClass::get_attrs(BrowserNodeList & attributes) const {
+  attributes.clear();
+  
+  QList<BrowserClass> lcl;
+  
+  get_all_parents(lcl);
+  lcl.append(this);
+  
+  QListIterator<BrowserClass> it(lcl);
+  
+  for (; it.current(); ++it) {
+    QListViewItem * child;
+    
+    for (child = it.current()->firstChild(); child; child = child->nextSibling())
+      if (!((BrowserNode *) child)->deletedp() &&
+	  (((BrowserNode *) child)->get_type() == UmlAttribute))
+	attributes.append((BrowserNode *) child);
+  }
+}
+
 // returns all operations included the inherited (except the redefined ones
 // and the contructor and destructor of parents)
 
 void BrowserClass::get_opers(QValueList<const OperationData *> & opers,
-			     QStringList & list) {
+			     QStringList & list) const {
   QList<BrowserClass> all_parents;
   
   get_all_parents(all_parents);
@@ -1026,6 +1070,118 @@ void BrowserClass::get_opers(QValueList<const OperationData *> & opers,
   }
 }
 
+// search for the relations from classes of 'from' having
+// target in 'to'
+
+static void get_assocs(QList<BrowserClass> & from,
+		       QList<BrowserClass> & to,
+		       QList<RelationData> & rels)
+{
+  QListIterator<BrowserClass> it(from);
+  
+  for (; it.current(); ++it) {
+    QListViewItem * child;
+    
+    for (child = it.current()->firstChild(); child; child = child->nextSibling()) {
+      if (!((BrowserNode *) child)->deletedp()) {
+	switch (((BrowserNode *) child)->get_type()) {
+	case UmlAssociation:
+	case UmlDirectionalAssociation:
+	case UmlAggregation:
+	case UmlAggregationByValue:
+	case UmlDirectionalAggregation:
+	case UmlDirectionalAggregationByValue:
+	  {
+	    RelationData * rd = (RelationData *)
+	      ((BrowserRelation *) child)->get_data();
+	    
+	    if ((rels.findRef(rd) == -1) &&
+		(to.findRef(rd->get_end_class()) != -1))
+	      rels.append(rd);
+	  }
+	default:
+	  break;
+	}
+      }
+    }
+  }
+}
+
+// returns all relation to target included the inherited
+// if 'rev' != 0 search also for relation in opposite direction
+// and set rev to the number of relation in the first direction
+void BrowserClass::get_rels(BrowserClass * target,
+			    QList<RelationData> & l, int * rev) const {
+  QList<BrowserClass> la;
+  QList<BrowserClass> lb;
+  
+  get_all_parents(la);  
+  target->get_all_parents(lb);
+  la.append((BrowserClass *) this);
+  lb.append(target);
+  get_assocs(la, lb, l);
+  if (rev != 0) {
+    *rev = l.count();
+    QList<RelationData> ll;
+    
+    get_assocs(lb, la, ll);
+    while (! ll.isEmpty())
+      l.append(ll.take(0));
+  }
+}
+
+// search for the relations from classes of 'from' having
+// target in 'to' (may be the reverse direction of a bidir)
+
+static void get_assocs(QList<BrowserClass> & from,
+		       QList<BrowserClass> & to,
+		       QList<BrowserRelation> & rels)
+{
+  QListIterator<BrowserClass> it(from);
+  
+  for (; it.current(); ++it) {
+    QListViewItem * child;
+    
+    for (child = it.current()->firstChild(); child; child = child->nextSibling()) {
+      if (!((BrowserNode *) child)->deletedp()) {
+	switch (((BrowserNode *) child)->get_type()) {
+	case UmlAssociation:
+	case UmlDirectionalAssociation:
+	case UmlAggregation:
+	case UmlAggregationByValue:
+	case UmlDirectionalAggregation:
+	case UmlDirectionalAggregationByValue:
+	  if (rels.findRef((BrowserRelation *) child) == -1) {
+	    BrowserRelation * r = (BrowserRelation *) child;
+	    RelationData * rd = (RelationData *) r->get_data();
+	    
+	    if ((rd->is_a(r))
+		? (to.findRef(rd->get_end_class()) != -1)
+		: (to.findRef(rd->get_start_class()) != -1))
+	      rels.append(r);
+	  }
+	default:
+	  break;
+	}
+      }
+    }
+  }
+}
+
+// returns all relation to target included the inherited
+// (may be the reverse direction of a bidir)
+void BrowserClass::get_rels(BrowserClass * target,
+			    QList<BrowserRelation> & l) const {
+  QList<BrowserClass> la;
+  QList<BrowserClass> lb;
+  
+  get_all_parents(la);  
+  target->get_all_parents(lb);
+  la.append((BrowserClass *) this);
+  lb.append(target);
+  get_assocs(la, lb, l);
+}
+
 // get the class and its members, and all parent and their members
 // except deleted
 void BrowserClass::get_tree(BrowserNodeList & l) {
@@ -1140,7 +1296,9 @@ bool BrowserClass::may_contains_them(const QList<BrowserNode> & l,
 }
 
 void BrowserClass::move(BrowserNode * bn, BrowserNode * after) {
-  if ((bn->get_type() == UmlClass) && !bn->nestedp()) {
+  UmlCode what = bn->get_type();
+  
+  if ((what == UmlClass) && !bn->nestedp()) {
     ((BrowserClass *) bn)->def->set_uml_visibility(UmlProtected);
     ((BrowserClass *) bn)->def->set_cpp_visibility(UmlDefaultVisibility);
     ((BrowserClass *) bn)->set_associated_artifact(0);
@@ -1149,16 +1307,18 @@ void BrowserClass::move(BrowserNode * bn, BrowserNode * after) {
   BrowserNode * old = ((BrowserNode *) bn->parent());
   char * cpp;
   char * java;
+  char * php;
   
-  if ((old != this) && (bn->get_type() == UmlOperation)) {
+  if ((old != this) && (what == UmlOperation)) {
     OperationData * d = (OperationData *) bn->get_data();
     
-    cpp = d->get_body(TRUE);
-    java = d->get_body(FALSE);
+    cpp = d->get_body('c');
+    java = d->get_body('j');
+    php = d->get_body('p');
     d->create_modified_body_file();
   }
   else
-    cpp = java = 0;
+    cpp = java = php = 0;
   
   if (after)
     bn->moveItem(after);
@@ -1171,21 +1331,27 @@ void BrowserClass::move(BrowserNode * bn, BrowserNode * after) {
     old->modified();
     old->package_modified();
     
-    if (bn->get_type() == UmlOperation) {
+    if (what == UmlOperation) {
       OperationData * d = (OperationData *) bn->get_data();
       
       if (cpp) {
-	d->new_body(cpp, TRUE);
+	d->new_body(cpp, 'c');
 	delete [] cpp;
       }
       if (java) {
-	d->new_body(java, FALSE);
+	d->new_body(java, 'j');
 	delete [] java;
       }
+      if (php) {
+	d->new_body(php, 'p');
+	delete [] php;
+      }
       
       if (d->get_is_abstract())
 	def->set_is_abstract(TRUE);
     }
+    
+    bn->modified();
   }
   
   modified();
@@ -1207,9 +1373,11 @@ void BrowserClass::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
        ((bn = UmlDrag::decode(e, BrowserRelation::drag_key(this))) != 0) ||
        ((bn = UmlDrag::decode(e, BrowserSimpleRelation::drag_key(this))) != 0)) &&
       (bn != after) && (bn != this)) {
-    if (may_contains(bn, bn->get_type() == UmlClass))  {
+    bool a_class = (bn->get_type() == UmlClass);
+    
+    if (may_contains(bn, a_class))  {
       if ((after == 0) &&
-	  (bn->get_type() == UmlClass) &&
+	  a_class &&
 	  ((BrowserNode *) parent())->may_contains(bn, TRUE)) {
 	// have choice
 	QPopupMenu m(0);
@@ -1414,13 +1582,28 @@ const QStringList & BrowserClass::default_stereotypes()
 }
 
 // unicode
-const QStringList & BrowserClass::default_stereotypes(UmlCode arrow) const {
-  if (IsaRelation(arrow))
-    return relations_default_stereotypes[arrow];
-  else {
-    static QStringList empty;
+const QStringList & BrowserClass::default_stereotypes(UmlCode arrow, const BrowserNode * target) const {
+  switch (target->get_type()) {
+  case UmlUseCase:
+    return target->default_stereotypes(arrow, this);
+  case UmlPackage:
+    {
+      static QStringList l;
   
-    return empty;
+      if (arrow == UmlDependOn)
+	l.append("import");
+      return l;
+    }
+  case UmlClass:
+    if (IsaRelation(arrow))
+      return relations_default_stereotypes[arrow];
+    // no break;
+  default:
+    {
+      static QStringList empty;
+      
+      return empty;
+    }
   }
 }
 
@@ -1507,7 +1690,11 @@ void BrowserClass::set_associated_artifact(BrowserArtifact * a,
     if ((associated_artifact = a) != 0)
       a->add_associated_class(this, on_read);
 
-    def->modified();	// to upgrade drawing context
+    if (! on_read)
+      // to upgrade drawing context
+      // not done on read to not remove actuals because
+      // inherited class is not yet read
+      def->modified();
   }
 }
 
@@ -1576,6 +1763,10 @@ void BrowserClass::init()
     
     switch (r) {
     case UmlGeneralisation: 
+      relations_default_stereotypes[r].append("{complete,disjoint}");
+      relations_default_stereotypes[r].append("{incomplete,disjoint}");
+      relations_default_stereotypes[r].append("{complete,overlapping}");
+      relations_default_stereotypes[r].append("{incomplete,overlapping}");
       break;
     case UmlDependency:
       relations_default_stereotypes[r].append("friend");
@@ -1589,6 +1780,7 @@ void BrowserClass::init()
       relations_default_stereotypes[r].append("list");
       relations_default_stereotypes[r].append("set");
       relations_default_stereotypes[r].append("vector");
+      relations_default_stereotypes[r].append("map");
     }
   }
 }
@@ -1709,6 +1901,14 @@ bool BrowserClass::tool_cmd(ToolCom * com, const char * args) {
 	(*it)->write_id(com);
     }
     return TRUE;
+  case setUserCmd:
+    // plug-out upgrade, limited checks
+    if (!is_read_only || root_permission()) {
+      all.remove(get_ident());
+      new_ident(com->get_unsigned(args), all);      
+      package_modified();
+    }
+    return TRUE;
   default:
     {
       QString s = name;
@@ -1831,8 +2031,9 @@ void BrowserClass::save(QTextStream & st, bool ref, QString & warning) {
 	    OperationData * od =
 	      (OperationData *) ((BrowserNode *) child)->get_data();
 	    
-	    od->save_body(qf, modified_bodies, TRUE);
-	    od->save_body(qf, modified_bodies, FALSE);
+	    od->save_body(qf, modified_bodies, 'c');
+	    od->save_body(qf, modified_bodies, 'j');
+	    od->save_body(qf, modified_bodies, 'p');
 	  }
 	  child = child->nextSibling();
 	  if (child != 0)
@@ -2047,12 +2248,12 @@ void BrowserClass::plug_out_conversion()
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
 	  op2 = (OperationData *) baseComponentCreation->get_data();
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op2->new_body(b, TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op2->new_body(b, 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op2->new_body(b, FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op2->new_body(b, 'j');
 	    delete [] b;
 	  }	  
 	  baseComponentCreation->set_comment(((BrowserNode *) child)->get_comment());
@@ -2070,14 +2271,14 @@ void BrowserClass::plug_out_conversion()
 	  op1 = (OperationData *) baseComponentCreate->get_data();
 	  op2 = (OperationData *) ((BrowserNode *) child)->get_data();
 
-	  if ((b = op2->get_body(TRUE)) != 0) {
-	    op1->new_body(b, TRUE);
-	    op2->new_body(component2artifact(b), TRUE);
+	  if ((b = op2->get_body('c')) != 0) {
+	    op1->new_body(b, 'c');
+	    op2->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op2->get_body(FALSE)) != 0) {
-	    op1->new_body(b, FALSE);
-	    op2->new_body(component2artifact(b), FALSE);
+	  if ((b = op2->get_body('j')) != 0) {
+	    op1->new_body(b, 'j');
+	    op2->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  baseComponentCreate->set_comment(((BrowserNode *) child)->get_comment());
@@ -2094,14 +2295,14 @@ void BrowserClass::plug_out_conversion()
 	  op1 = (OperationData *) baseComponentKind->get_data();
 	  op2 = (OperationData *) ((BrowserNode *) child)->get_data();
 
-	  if ((b = op2->get_body(TRUE)) != 0) {
-	    op1->new_body(b, TRUE);
-	    op2->new_body(component2artifact(b), TRUE);
+	  if ((b = op2->get_body('c')) != 0) {
+	    op1->new_body(b, 'c');
+	    op2->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op2->get_body(FALSE)) != 0) {
-	    op1->new_body(b, FALSE);
-	    op2->new_body(component2artifact(b), FALSE);
+	  if ((b = op2->get_body('j')) != 0) {
+	    op1->new_body(b, 'j');
+	    op2->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  baseComponentKind->set_comment(((BrowserNode *) child)->get_comment());
@@ -2115,19 +2316,19 @@ void BrowserClass::plug_out_conversion()
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) baseComponentRead_uml_->get_data();
 	  op1->new_body("  _assoc_diagram = (UmlComponentDiagram *) UmlBaseItem::read_();\n\
-  UmlBaseItem::read_uml_();\n", TRUE);
+  UmlBaseItem::read_uml_();\n", 'c');
 	  op1->new_body("  _assoc_diagram = (UmlComponentDiagram) UmlBaseItem.read_();\n\
-  super.read_uml_();\n", FALSE);
+  super.read_uml_();\n", 'j');
 	  baseComponentRead_uml_->set_comment(((BrowserNode *) child)->get_comment());
 	  
 	  // UmlBaseArtifact read_uml_ operation
 	  op2 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op2->get_body(TRUE)) != 0) {
-	    op2->new_body(component2artifact(b), TRUE);
+	  if ((b = op2->get_body('c')) != 0) {
+	    op2->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op2->get_body(FALSE)) != 0) {
-	    op2->new_body(component2artifact(b), FALSE);
+	  if ((b = op2->get_body('j')) != 0) {
+	    op2->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  set_user_id(0);
@@ -2140,12 +2341,12 @@ void BrowserClass::plug_out_conversion()
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) baseComponentAssociatedDiagram->get_data();
 	  op2 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op2->get_body(TRUE)) != 0) {
-	    op1->new_body(b, TRUE);
+	  if ((b = op2->get_body('c')) != 0) {
+	    op1->new_body(b, 'c');
 	    delete [] b;
 	  }
-	  if ((b = op2->get_body(FALSE)) != 0) {
-	    op1->new_body(b, FALSE);
+	  if ((b = op2->get_body('j')) != 0) {
+	    op1->new_body(b, 'j');
 	    delete [] b;
 	  }	  
 	  baseComponentAssociatedDiagram->set_comment(((BrowserNode *) child)->get_comment());
@@ -2161,12 +2362,12 @@ void BrowserClass::plug_out_conversion()
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) baseComponentSet_AssociatedDiagram->get_data();
 	  op2 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op2->get_body(TRUE)) != 0) {
-	    op1->new_body(b, TRUE);
+	  if ((b = op2->get_body('c')) != 0) {
+	    op1->new_body(b, 'c');
 	    delete [] b;
 	  }
-	  if ((b = op2->get_body(FALSE)) != 0) {
-	    op1->new_body(b, FALSE);
+	  if ((b = op2->get_body('j')) != 0) {
+	    op1->new_body(b, 'j');
 	    delete [] b;
 	  }	  
 	  baseComponentSet_AssociatedDiagram->set_comment(((BrowserNode *) child)->get_comment());
@@ -2178,12 +2379,12 @@ void BrowserClass::plug_out_conversion()
 	  ((BrowserNode *) child)->set_name("associatedArtifacts");
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op1->new_body(component2artifact(b), TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op1->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op1->new_body(component2artifact(b), FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op1->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  set_user_id(0);
@@ -2193,12 +2394,12 @@ void BrowserClass::plug_out_conversion()
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
 	  op1->set_param_type(0, t_artifact);
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op1->new_body(component2artifact(b), TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op1->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op1->new_body(component2artifact(b), FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op1->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  set_user_id(0);
@@ -2208,12 +2409,12 @@ void BrowserClass::plug_out_conversion()
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
 	  op1->set_param_type(0, t_artifact);
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op1->new_body(component2artifact(b), TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op1->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op1->new_body(component2artifact(b), FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op1->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  set_user_id(0);
@@ -2222,12 +2423,12 @@ void BrowserClass::plug_out_conversion()
 	  ((BrowserNode *) child)->set_name("removeAllAssociatedArtifacts");
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op1->new_body(component2artifact(b), TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op1->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op1->new_body(component2artifact(b), FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op1->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	  set_user_id(0);
@@ -2286,12 +2487,12 @@ void BrowserClass::plug_out_conversion()
 	  
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op1->new_body(component2artifact(b), TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op1->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op1->new_body(component2artifact(b), FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op1->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	}
@@ -2321,12 +2522,12 @@ void BrowserClass::plug_out_conversion()
 	else if (strcmp(((BrowserNode *) child)->get_name(), "read_uml_") == 0) {
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op1->get_body(TRUE)) != 0) {
-	    op1->new_body(component2artifact(b), TRUE);
+	  if ((b = op1->get_body('c')) != 0) {
+	    op1->new_body(component2artifact(b), 'c');
 	    delete [] b;
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
-	    op1->new_body(component2artifact(b), FALSE);
+	  if ((b = op1->get_body('j')) != 0) {
+	    op1->new_body(component2artifact(b), 'j');
 	    delete [] b;
 	  }	  
 	}
@@ -2346,22 +2547,22 @@ void BrowserClass::plug_out_conversion()
 	if (strcmp(((BrowserNode *) child)->get_name(), "read_") == 0) {
 	  set_user_id(uid);	// to create saved  *_<n>.b file name
 	  op1 = (OperationData *) ((BrowserNode *) child)->get_data();
-	  if ((b = op1->get_body(TRUE)) != 0) {
+	  if ((b = op1->get_body('c')) != 0) {
 	    s = b;
 	    delete [] b;
 	    s.insert(s.find("case aNode:"),
 		     "case anArtifact:\n\
       result = new UmlArtifact(id, name);\n\
       break;\n    ");
-	    op1->new_body(s, TRUE);
+	    op1->new_body(s, 'c');
 	  }
-	  if ((b = op1->get_body(FALSE)) != 0) {
+	  if ((b = op1->get_body('j')) != 0) {
 	    s = b;
 	    delete [] b;
 	    s.insert(s.find("case anItemKind._aNode:"),
 		   "case anItemKind._anArtifact:\n\
       return new UmlArtifact(id, name);\n    ");
-	    op1->new_body(s, FALSE);
+	    op1->new_body(s, 'j');
 	  }
 	  set_user_id(0);
 	}
@@ -2419,18 +2620,18 @@ void BrowserClass::plug_out_conversion()
 	  (strcmp(((BrowserNode *) child)->get_name(), "connect") == 0)) {
 	set_user_id(uid);	// to create saved  *_<n>.b file name
 	op1 = (OperationData *) ((BrowserNode *) child)->get_data();
-	if ((b = op1->get_body(TRUE)) != 0) {
+	if ((b = op1->get_body('c')) != 0) {
 	  s = b;
 	  delete [] b;
 	  s.insert(s.find("write_unsigned(")+15, "1");
-	  op1->new_body(s, TRUE);
+	  op1->new_body(s, 'c');
 	}
 	
-	if ((b = op1->get_body(FALSE)) != 0) {
+	if ((b = op1->get_body('j')) != 0) {
 	  s = b;
 	  delete [] b;
 	  s.insert(s.find("write_unsigned(")+15, "1");
-	  op1->new_body(s, FALSE);
+	  op1->new_body(s, 'j');
 	}
 	set_user_id(0);
 	break;
@@ -2540,18 +2741,30 @@ BrowserClass * BrowserClass::read(char * & st, char * k,
       result = new BrowserClass(id);
     return result;
   }
-  else if (!strcmp(k, "class")) {
+  else if (!strcmp(k, "class") || !strcmp(k, "actor")) {
+    bool actor = (*k == 'a');
+
     id = read_id(st);
     
     QString s = read_string(st);
     
     if ((result = all[id]) == 0)
       result = new BrowserClass(s, parent, new ClassData, id);
+    else if (result->is_defined) {
+      BrowserClass * already_exist = result;
+
+      result = new BrowserClass(s, parent, new ClassData, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("class", result);
+    }
     else {
       result->set_parent(parent);
       result->set_name(s);
     }
     
+    result->is_defined = TRUE;
+
     result->is_read_only = (!in_import() && read_only_file()) ||
       (user_id() != 0) && result->is_api_base();
     
@@ -2561,7 +2774,8 @@ BrowserClass * BrowserClass::read(char * & st, char * k,
     
     k = read_keyword(st);
     
-    result->def->read(st, k);	// updates k
+    if (! actor)
+      result->def->read(st, k);	// updates k
     result->def->set_browser_node(result);
     
     if (!strcmp(k, "associated_class_diagram")) {
@@ -2607,6 +2821,9 @@ BrowserClass * BrowserClass::read(char * & st, char * k,
       if (strcmp(k, "end"))
 	wrong_keyword(k, "end");
     }
+
+    if (actor)
+      result->def->set_stereotype("actor");
     
     return result;
   }
