@@ -56,7 +56,7 @@ BrowserClassDiagram::BrowserClassDiagram(QString s, BrowserNode * p, int id)
 }
 
 BrowserClassDiagram::BrowserClassDiagram(BrowserClassDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -115,6 +115,17 @@ void BrowserClassDiagram::make() {
   package_color = UmlDefaultColor;
 }
 
+BrowserClassDiagram * BrowserClassDiagram::add_class_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter class diagram's name : ",
+				      UmlClassDiagram, TRUE, FALSE))
+    return new BrowserClassDiagram(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserClassDiagram::set_name(const char * s) {
   BrowserDiagram::set_name(s);
   if (window != 0)
@@ -226,11 +237,13 @@ void BrowserClassDiagram::exec_menu_choice(int rank) {
     return;
   case 3:
     {
-      BrowserClassDiagram * cd =
-	new BrowserClassDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      cd->select_in_browser();
-      cd->edit("Class diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter class diagram's name : ",
+						      UmlClassDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -321,6 +334,10 @@ UmlCode BrowserClassDiagram::get_type() const {
   return UmlClassDiagram;
 }
 
+int BrowserClassDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserClassDiagram::get_classdiagramsettings(ClassDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_classdiagramsettings(r);
@@ -431,14 +448,6 @@ bool BrowserClassDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
-void BrowserClassDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserClassDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserClassDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -527,11 +536,21 @@ BrowserClassDiagram * BrowserClassDiagram::read(char * & st, char * k,
     
     if ((r = (BrowserClassDiagram *) all[id]) == 0)
       r = new BrowserClassDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserClassDiagram * already_exist = r;
+
+      r = new BrowserClassDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("class diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = (!in_import() && read_only_file()) || 
       (user_id() != 0) && r->is_api_base();
     
