@@ -38,6 +38,7 @@
 #include "BrowserObjectDiagram.h"
 #include "BrowserDeploymentView.h"
 #include "BrowserClass.h"
+#include "BrowserClassInstance.h"
 #include "BrowserState.h"
 #include "BrowserActivity.h"
 #include "BrowserActivityAction.h"
@@ -53,7 +54,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserClassView> BrowserClassView::all;
+IdDict<BrowserClassView> BrowserClassView::all(__FILE__);
 QStringList BrowserClassView::its_default_stereotypes;	// unicode
 
 BrowserClassView::BrowserClassView(QString s, BrowserNode * p, int id)
@@ -116,16 +117,33 @@ BrowserNode * BrowserClassView::duplicate(BrowserNode * p, QString name) {
   return result;
 }
 
+BrowserClassView* BrowserClassView::add_class_view(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter class view's name : ",
+				      UmlClassView, TRUE, FALSE))
+    return new BrowserClassView(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserClassView::clear(bool old)
 {
   all.clear(old);
   BrowserClass::clear(old);
+  BrowserState::clear(old);
+  BrowserActivity::clear(old);
+  BrowserClassInstance::clear(old);
 }
 
 void BrowserClassView::update_idmax_for_root()
 {
   all.update_idmax_for_root();
   BrowserClass::update_idmax_for_root();
+  BrowserState::update_idmax_for_root();
+  BrowserActivity::update_idmax_for_root();
+  BrowserClassInstance::update_idmax_for_root();
 }
     
 void BrowserClassView::renumber(int phase) {
@@ -170,6 +188,8 @@ void BrowserClassView::menu() {
 		     "to add a <em>object diagram</em>");
       m.setWhatsThis(m.insertItem("New class", 3),
 		     "to add a <em>class</em>");
+      m.setWhatsThis(m.insertItem("New class instance", 17),
+		     "to add a <em>class instance</em>");
       m.setWhatsThis(m.insertItem("New state machine", 4),
 		     "to add a <em>state machine</em>");
       m.setWhatsThis(m.insertItem("New activity", 16),
@@ -198,6 +218,7 @@ Note that you can undelete them after");
     m.insertItem("Generate", &subm);
     subm.insertItem("C++", 11);
     subm.insertItem("Java", 12);
+    subm.insertItem("Php", 22);
     subm.insertItem("Idl", 13);
     
     if ((edition_number == 0) &&
@@ -226,13 +247,34 @@ Do not undelete its sub items");
 void BrowserClassView::exec_menu_choice(int rank) {
   switch (rank) {
   case 0:
-    add_class_diagram();
+    {
+      BrowserClassDiagram * d = 
+	BrowserClassDiagram::add_class_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 1:
-    add_sequence_diagram();
+    {
+      BrowserSeqDiagram * d = 
+	BrowserSeqDiagram::add_sequence_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 2:
-    add_collaboration_diagram();
+    {
+      BrowserColDiagram * d = 
+	BrowserColDiagram::add_collaboration_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 3:
     {
@@ -262,7 +304,7 @@ void BrowserClassView::exec_menu_choice(int rank) {
   case 7:
     {
       QArray<StateSpec> st;
-      QArray<ColorSpec> co(11);
+      QArray<ColorSpec> co(12);
       
       classdiagram_settings.complete(st, UmlClassView);
       sequencediagram_settings.complete(st, FALSE);
@@ -280,9 +322,9 @@ void BrowserClassView::exec_menu_choice(int rank) {
       co[6].set("state color", &state_color);
       co[7].set("state action color", &stateaction_color);
       co[8].set("activity color", &activity_color);
-      co[8].set("activity region color", &activityregion_color);
-      co[9].set("activity action color", &activityaction_color);
-      co[10].set("parameter and pin color", &parameterpin_color);
+      co[9].set("activity region color", &activityregion_color);
+      co[10].set("activity action color", &activityaction_color);
+      co[11].set("parameter and pin color", &parameterpin_color);
 
       SettingsDialog dialog(&st, &co, FALSE, FALSE);
       
@@ -321,6 +363,16 @@ void BrowserClassView::exec_menu_choice(int rank) {
 		   this);
     }
     return;
+  case 22:
+    {
+      bool preserve = preserve_bodies();
+      
+      ToolCom::run((verbose_generation()) 
+		   ? ((preserve) ? "php_generator -v -p" : "php_generator -v")
+		   : ((preserve) ? "php_generator -p" : "php_generator"), 
+		   this);
+    }
+    return;
   case 13:
     ToolCom::run((verbose_generation()) ? "idl_generator -v" : "idl_generator", this);
     return;
@@ -328,7 +380,14 @@ void BrowserClassView::exec_menu_choice(int rank) {
     associated_deployment_view->select_in_browser();
     return;
   case 15:
-    add_object_diagram();
+    {
+      BrowserObjectDiagram * d = 
+	BrowserObjectDiagram::add_object_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 16:
     {
@@ -338,6 +397,15 @@ void BrowserClassView::exec_menu_choice(int rank) {
 	a->select_in_browser();
     }
     return; // package_modified called
+  case 17:
+    {
+      BrowserClassInstance * c = 
+	BrowserClassInstance::add_classinstance(this);
+      
+      if (c != 0)
+	c->select_in_browser();
+    }
+    return; // package_modified called
   default:
     if (rank >= 100)
       ToolCom::run(Tool::command(rank - 100), this);
@@ -364,6 +432,8 @@ void BrowserClassView::apply_shortcut(QString s) {
 	choice = 15;
       else if (s == "New class")
 	choice = 3;
+      else if (s == "New class instance")
+	choice = 17;
       else if (s == "New state machine")
 	choice = 4;
       else if (s == "New activity")
@@ -390,6 +460,8 @@ void BrowserClassView::apply_shortcut(QString s) {
       choice = 11;
     else if (s == "Generate Java")
       choice = 12;
+    else if (s == "Generate Php")
+      choice = 22;
     else if (s == "Generate Idl")
       choice = 13;
   }
@@ -413,29 +485,14 @@ void BrowserClassView::open(bool) {
     (new ClassViewDialog(get_data()))->show();
 }
 
-void BrowserClassView::add_class_diagram() {
-  (new BrowserClassDiagram(child_random_name("Diagram"), this))->select_in_browser();
-}
-
-void BrowserClassView::add_object_diagram() {
-  (new BrowserObjectDiagram(child_random_name("Object Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserClassView::add_sequence_diagram() {
-  (new BrowserSeqDiagram(child_random_name("Sequence Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserClassView::add_collaboration_diagram() {
-  (new BrowserColDiagram(child_random_name("Collaboration Diagram"), this))
-    ->select_in_browser();
-}
-
 UmlCode BrowserClassView::get_type() const {
   return UmlClassView;
 }
 
+int BrowserClassView::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserClassView::get_data() const {
   return def;
 }
@@ -654,12 +711,12 @@ bool BrowserClassView::get_auto_label_position(UmlCode who) const {
   case UmlClassDiagram:
     v = classdiagram_settings.auto_label_position;
     break;
-  case UmlStateDiagram:
-    v = statediagram_settings.auto_label_position;
-    break;
   case UmlObjectDiagram:
     v = objectdiagram_settings.auto_label_position;
     break;
+  case UmlStateDiagram:
+    v = statediagram_settings.auto_label_position;
+    break;
   case UmlActivityDiagram:
     v = activitydiagram_settings.auto_label_position;
     break;
@@ -807,6 +864,9 @@ bool BrowserClassView::tool_cmd(ToolCom * com, const char * args) {
 	  else
 	    (BrowserClass::add_class(this, args))->write_id(com);
 	  break;
+	case UmlClassInstance:
+	  BrowserClassInstance::add_from_tool(this, com, args);
+	  break;
 	case UmlState:
 	  if (wrong_child_name(args, UmlState, TRUE, FALSE))
 	    ok = FALSE;
@@ -847,14 +907,13 @@ bool BrowserClassView::tool_cmd(ToolCom * com, const char * args) {
 
 void BrowserClassView::DragMoveEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, UmlClass) ||
+      UmlDrag::canDecode(e, UmlClassInstance) ||
       UmlDrag::canDecode(e, UmlClassDiagram) ||
       UmlDrag::canDecode(e, UmlSeqDiagram) ||
       UmlDrag::canDecode(e, UmlColDiagram) ||
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
       UmlDrag::canDecode(e, UmlState) ||
-      UmlDrag::canDecode(e, UmlStateDiagram) ||
-      UmlDrag::canDecode(e, UmlActivity) ||
-      UmlDrag::canDecode(e, UmlActivityDiagram))
+      UmlDrag::canDecode(e, UmlActivity))
     e->accept();
   else
     ((BrowserNode *) parent())->DragMoveInsideEvent(e);
@@ -862,14 +921,13 @@ void BrowserClassView::DragMoveEvent(QDragMoveEvent * e) {
 
 void BrowserClassView::DragMoveInsideEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, UmlClass) ||
+      UmlDrag::canDecode(e, UmlClassInstance) ||
       UmlDrag::canDecode(e, UmlClassDiagram) ||
       UmlDrag::canDecode(e, UmlSeqDiagram) ||
       UmlDrag::canDecode(e, UmlColDiagram) ||
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
       UmlDrag::canDecode(e, UmlState) ||
-      UmlDrag::canDecode(e, UmlStateDiagram) ||
-      UmlDrag::canDecode(e, UmlActivity) ||
-      UmlDrag::canDecode(e, UmlActivityDiagram))
+      UmlDrag::canDecode(e, UmlActivity))
     e->accept();
   else
     e->ignore();
@@ -882,14 +940,13 @@ bool BrowserClassView::may_contains_them(const QList<BrowserNode> & l,
   for (; it.current(); ++it) {
     switch (it.current()->get_type()) {
     case UmlClass:
+    case UmlClassInstance:
     case UmlClassDiagram:
     case UmlSeqDiagram:
     case UmlColDiagram:
     case UmlObjectDiagram:
     case UmlState:
-    case UmlStateDiagram:
     case UmlActivity:
-    case UmlActivityDiagram:
       if (! may_contains(it.current(), FALSE))
 	return FALSE;
       break;
@@ -916,12 +973,13 @@ void BrowserClassView::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
   BrowserNode * bn;
   
   if ((((bn = UmlDrag::decode(e, UmlClass)) != 0) ||
-      (((bn = UmlDrag::decode(e, UmlClassDiagram)) != 0)) ||
-      (((bn = UmlDrag::decode(e, UmlSeqDiagram)) != 0)) ||
-      (((bn = UmlDrag::decode(e, UmlColDiagram)) != 0)) ||
-      (((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0)) ||
-      (((bn = UmlDrag::decode(e, UmlState)) != 0)) ||
-      (((bn = UmlDrag::decode(e, UmlActivity)) != 0))) &&
+       ((bn = UmlDrag::decode(e, UmlClassInstance)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlClassDiagram)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlSeqDiagram)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlColDiagram)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlState)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlActivity)) != 0)) &&
       (bn != after) && (bn != this)) {
     if (may_contains(bn, FALSE)) {
       BrowserNode * old_parent = (BrowserNode *) bn->parent();
@@ -933,8 +991,10 @@ void BrowserClassView::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
 	insertItem(bn);
       }
       package_modified();
-      if (old_parent != this)
+      if (old_parent != this) {
 	old_parent->package_modified();
+	bn->modified();
+      }
     }
     else {
       msg_critical("Error", "Forbiden");
@@ -999,7 +1059,6 @@ void BrowserClassView::save(QTextStream & st, bool ref, QString & warning) {
     nl_indent(st);
     st << "//activity diagram settings";
     activitydiagram_settings.save(st);
-    nl_indent(st);
     
     bool nl = FALSE;
     
@@ -1064,79 +1123,76 @@ BrowserClassView * BrowserClassView::read(char * & st, char * k,
 {
   if (!strcmp(k, "classview")) {
     int id = read_id(st);
-    
-    BrowserClassView * r = all[id];
-    
-    if (r != 0) {
-      msg_critical("Error", "reload not yet implemented");
+    BrowserClassView * already_exist = all[id];
+    BrowserClassView * r = new BrowserClassView(read_string(st), parent, id);
       
-      throw 0;
+    if (already_exist != 0) {
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("class views", r);
     }
-    else {
-      r = new BrowserClassView(read_string(st), parent, id);
+
+    r->is_read_only = !in_import() && read_only_file() || 
+      (user_id() != 0) && r->is_api_base();
       
-      r->is_read_only = !in_import() && read_only_file() || 
-	(user_id() != 0) && r->is_api_base();
+    k = read_keyword(st);
       
+    r->def->read(st, k);					// updates k
+    r->classdiagram_settings.read(st, k);			// updates k
+    if (read_file_format() >= 6) {
+      r->collaborationdiagram_settings.read(st, k);		// updates k
+      if (read_file_format() >= 25)
+	r->objectdiagram_settings.read(st, k);		// updates k
+      r->sequencediagram_settings.read(st, k);		// updates k
+    }
+    if (read_file_format() >= 21)
+      r->statediagram_settings.read(st, k);			// updates k
+    r->class_settings.read(st, k);				// updates k
+    if (read_file_format() >= 26)
+      r->activitydiagram_settings.read(st, k);		// updates k
+    read_color(st, "class_color", r->class_color, k);		// updates k
+    read_color(st, "note_color", r->note_color, k);		// updates k
+    read_color(st, "package_color", r->package_color, k);	// updates k
+    read_color(st, "fragment_color", r->fragment_color, k);	// updates k
+    read_color(st, "duration", r->duration_color, k);	// old, updates k
+    read_color(st, "duration_color", r->duration_color, k);	// updates k
+    read_color(st, "continuation_color", r->continuation_color, k);	// updates k
+    read_color(st, "state_color", r->state_color, k);		// updates k
+    read_color(st, "stateaction_color", r->stateaction_color, k);		// updates k
+    read_color(st, "activity_color", r->activity_color, k);		// updates k
+    read_color(st, "activityregion_color", r->activityregion_color, k);		// updates k
+    read_color(st, "activityaction_color", r->activityaction_color, k);		// updates k
+    read_color(st, "parameterpin_color", r->parameterpin_color, k);		// updates k
+    if ((read_file_format() < 20) && !strcmp(k, "associated_componentview")) {
+      // old format
+      r->set_associated_deploymentview(BrowserDeploymentView::read_ref(st, "deploymentview_ref"),
+				       TRUE);
       k = read_keyword(st);
-      
-      r->def->read(st, k);					// updates k
-      r->classdiagram_settings.read(st, k);			// updates k
-      if (read_file_format() >= 6) {
-	r->collaborationdiagram_settings.read(st, k);		// updates k
-	if (read_file_format() >= 25)
-	  r->objectdiagram_settings.read(st, k);		// updates k
-	r->sequencediagram_settings.read(st, k);		// updates k
-      }
-      if (read_file_format() >= 21)
-	r->statediagram_settings.read(st, k);			// updates k
-      r->class_settings.read(st, k);				// updates k
-      if (read_file_format() >= 26)
-	r->activitydiagram_settings.read(st, k);		// updates k
-      read_color(st, "class_color", r->class_color, k);		// updates k
-      read_color(st, "note_color", r->note_color, k);		// updates k
-      read_color(st, "package_color", r->package_color, k);	// updates k
-      read_color(st, "fragment_color", r->fragment_color, k);	// updates k
-      read_color(st, "duration_color", r->duration_color, k);	// updates k
-      read_color(st, "continuation_color", r->continuation_color, k);	// updates k
-      read_color(st, "state_color", r->state_color, k);		// updates k
-      read_color(st, "stateaction_color", r->stateaction_color, k);		// updates k
-      read_color(st, "activity_color", r->activity_color, k);		// updates k
-      read_color(st, "activityregion_color", r->activityregion_color, k);		// updates k
-      read_color(st, "activityaction_color", r->activityaction_color, k);		// updates k
-      read_color(st, "parameterpin_color", r->parameterpin_color, k);		// updates k
-      if ((read_file_format() < 20) && !strcmp(k, "associated_componentview")) {
-	// old format
-	r->set_associated_deploymentview(BrowserDeploymentView::read_ref(st, "deploymentview_ref"),
-					 TRUE);
-	k = read_keyword(st);
-      }
-      else if (!strcmp(k, "associated_deployment_view") ||
-	       ((read_file_format() < 20) && !strcmp(k, "associated_component_view"))) {
-	r->set_associated_deploymentview(BrowserDeploymentView::read_ref(st, read_keyword(st)),
-					 TRUE);
-	k = read_keyword(st);
-      }
-      
-      r->BrowserNode::read(st, k);				// updates k
-      
-      if (strcmp(k, "end")) {
-	while (BrowserClassDiagram::read(st, k, r) ||
-	       BrowserColDiagram::read(st, k, r) ||
-	       BrowserSeqDiagram::read(st, k, r) ||
-	       BrowserObjectDiagram::read(st, k, r) ||
-	       BrowserClass::read(st, k, r) ||
-	       BrowserState::read(st, k, r) ||
-	       BrowserActivity::read(st, k, r) ||
-	       BrowserActivityAction::read(st, k, r))
-	  k = read_keyword(st);
-	
-	if (strcmp(k, "end"))
-	  wrong_keyword(k, "end");
-      }
-      
-      return r;
     }
+    else if (!strcmp(k, "associated_deployment_view") ||
+	     ((read_file_format() < 20) && !strcmp(k, "associated_component_view"))) {
+      r->set_associated_deploymentview(BrowserDeploymentView::read_ref(st, read_keyword(st)),
+				       TRUE);
+      k = read_keyword(st);
+    }
+    
+    r->BrowserNode::read(st, k);				// updates k
+    
+    if (strcmp(k, "end")) {
+      while (BrowserClassDiagram::read(st, k, r) ||
+	     BrowserColDiagram::read(st, k, r) ||
+	     BrowserSeqDiagram::read(st, k, r) ||
+	     BrowserObjectDiagram::read(st, k, r) ||
+	     BrowserClass::read(st, k, r) ||
+	     BrowserClassInstance::read(st, k, r) ||
+	     BrowserState::read(st, k, r) ||
+	     BrowserActivity::read(st, k, r))
+	k = read_keyword(st);
+	
+      if (strcmp(k, "end"))
+	wrong_keyword(k, "end");
+    }
+    
+    return r;
   }
   else
     return 0;
