@@ -51,7 +51,7 @@
 #include "ComponentDialog.h"
 #include "DialogUtil.h"
 
-IdDict<BrowserComponent> BrowserComponent::all(257);
+IdDict<BrowserComponent> BrowserComponent::all(257, __FILE__);
 QStringList BrowserComponent::its_default_stereotypes;	// unicode
 
 BrowserComponent::BrowserComponent(QString s, BrowserNode * p, int id)
@@ -116,7 +116,7 @@ void BrowserComponent::compute_referenced_by(QList<BrowserNode> & l,
   
   while (it.current()) {
     if (!it.current()->deletedp() &&
-	((it.current()->realized_classes.findIndex(target) != -1) ||
+	((it.current()->realizing_classes.findIndex(target) != -1) ||
 	 (it.current()->provided_classes.findIndex(target) != -1) ||
 	 (it.current()->required_classes.findIndex(target) != -1)))
       l.append(it.current());
@@ -318,7 +318,7 @@ nexted <em>components</em> and <em>relations</em> \
   
   make_clsubm(m, rqsubm, required_classes, 9999, need_sep, "required");
   make_clsubm(m, prsubm, provided_classes, 19999, need_sep, "provided");
-  make_clsubm(m, rzsubm, realized_classes, 29999, need_sep, "realized");
+  make_clsubm(m, rzsubm, realizing_classes, 29999, need_sep, "realization");
     
   exec_menu_choice(m.exec(QCursor::pos()), item_above);
 }
@@ -363,7 +363,7 @@ void BrowserComponent::exec_menu_choice(int rank,
     }
     break;
   default:
-    if (select_associated(rank, 29999, realized_classes) ||
+    if (select_associated(rank, 29999, realizing_classes) ||
 	select_associated(rank, 19999, provided_classes) ||
 	select_associated(rank, 9999, required_classes))
       return;
@@ -427,10 +427,21 @@ void BrowserComponent::open(bool force_edit) {
   }
 }
 
+void BrowserComponent::modified() {
+  repaint();
+  ((BrowserNode *) parent())->modified();
+  // to update regions drawing
+  def->modified();
+}
+
 UmlCode BrowserComponent::get_type() const {
   return UmlComponent;
 }
 
+int BrowserComponent::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserComponent::DragMoveEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, BrowserSimpleRelation::drag_key(this)))
     e->accept();
@@ -467,6 +478,7 @@ void BrowserComponent::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
       if (old != this) {
 	old->modified();
 	old->package_modified();
+	bn->modified();
       }
       
       modified();
@@ -575,10 +587,10 @@ void BrowserComponent::on_delete() {
   
   QValueList<BrowserClass *>::Iterator it;
   
-  it = realized_classes.begin();
-  while (it != realized_classes.end()) {
+  it = realizing_classes.begin();
+  while (it != realizing_classes.end()) {
     if ((*it)->deletedp())
-      it = realized_classes.remove(it);
+      it = realizing_classes.remove(it);
     else
       ++it;
   }
@@ -603,8 +615,8 @@ void BrowserComponent::on_delete() {
 void BrowserComponent::remove_associated_class(BrowserClass * c) {
   QValueList<BrowserClass *>::Iterator it;
   
-  if ((it = realized_classes.find(c)) != realized_classes.end()) {
-    realized_classes.remove(it);
+  if ((it = realizing_classes.find(c)) != realizing_classes.end()) {
+    realizing_classes.remove(it);
     QObject::disconnect(c->get_data(), SIGNAL(deleted()),
 			def, SLOT(on_delete()));
     package_modified();
@@ -623,14 +635,29 @@ void BrowserComponent::remove_associated_class(BrowserClass * c) {
   }
 }
 
-QValueList<BrowserClass *> 
-BrowserComponent::get_all_provided_classes(bool sorted) const {
-  QValueList<BrowserClass *> r = provided_classes;
+void BrowserComponent::get_all_provided_classes(QValueList<BrowserClass *> & r) const {
+  QValueList<BrowserClass *>::ConstIterator it;
+
+  for (it = provided_classes.begin() ; it != provided_classes.end(); it++)
+    if (r.findIndex(*it) == -1)
+      r.append(*it);
+  
   QListViewItem * child;
     
   for (child = firstChild(); child != 0; child = child->nextSibling())
     if (((BrowserNode *) child)->get_type() == UmlComponent)
-      r += ((BrowserComponent *) child)->provided_classes;
+      ((BrowserComponent *) child)->get_all_provided_classes(r);
+}
+
+void BrowserComponent::get_all_provided_classes(QValueList<BrowserClass *> & r,
+						bool sorted) const {
+  r = provided_classes;
+  
+  QListViewItem * child;
+    
+  for (child = firstChild(); child != 0; child = child->nextSibling())
+    if (((BrowserNode *) child)->get_type() == UmlComponent)
+      ((BrowserComponent *) child)->get_all_provided_classes(r);
 
   if (sorted) {
     BrowserNodeList nl;
@@ -645,28 +672,38 @@ BrowserComponent::get_all_provided_classes(bool sorted) const {
     while (! nl.isEmpty())
       r.append((BrowserClass *) nl.take(0));
   }
-
-  return r;
 }
 
-QValueList<BrowserClass *>
-BrowserComponent::get_all_required_classes(bool sorted) const {
-  QValueList<BrowserClass *> pr = get_all_provided_classes(FALSE);
-  QValueList<BrowserClass *> r = required_classes;
+void BrowserComponent::get_all_required_classes(QValueList<BrowserClass *> & r) const {
+  QValueList<BrowserClass *>::ConstIterator it;
+
+  for (it = required_classes.begin() ; it != required_classes.end(); it++)
+    if (r.findIndex(*it) == -1)
+      r.append(*it);
+  
   QListViewItem * child;
     
   for (child = firstChild(); child != 0; child = child->nextSibling())
     if (((BrowserNode *) child)->get_type() == UmlComponent)
-      r += ((BrowserComponent *) child)->required_classes;
-
-  QValueList<BrowserClass *>::Iterator it;
+      ((BrowserComponent *) child)->get_all_required_classes(r);
+}
 
+void BrowserComponent::get_all_required_classes(QValueList<BrowserClass *> & r,
+						bool sorted) const {
+  r = required_classes;
+  
+  QListViewItem * child;
+    
+  for (child = firstChild(); child != 0; child = child->nextSibling())
+    if (((BrowserNode *) child)->get_type() == UmlComponent)
+      ((BrowserComponent *) child)->get_all_required_classes(r);
+  
   if (sorted) {
     BrowserNodeList nl;
+    QValueList<BrowserClass *>::Iterator it;
 
     for (it = r.begin() ; it != r.end(); it++)
-      if (pr.findIndex(*it) == -1)
-	nl.append(*it);
+      nl.append(*it);
 
     r.clear();
     nl.sort();
@@ -674,18 +711,6 @@ BrowserComponent::get_all_required_classes(bool sorted) const {
     while (! nl.isEmpty())
       r.append((BrowserClass *) nl.take(0));
   }
-  else {
-    it = r.begin();
-
-    while (it != r.end()) {
-      if (pr.findIndex(*it) == -1)
-	it++;
-      else
-	it = r.remove(it);
-    }
-  }
-
-  return r;
 }
 
 void BrowserComponent::set_associated_classes(const QValueList<BrowserClass *> & rz,
@@ -696,21 +721,14 @@ void BrowserComponent::set_associated_classes(const QValueList<BrowserClass *> &
     
   if (! on_read) {
     // manage removed classes
-    it = realized_classes.begin();
+    it = realizing_classes.begin();
     
-    while (it != realized_classes.end()) {
+    while (it != realizing_classes.end()) {
       if (rz.findIndex(*it) == -1) {
-	// stop to be realized
-	BrowserClass * cl = *it;
-	
-	it = realized_classes.remove(it);
-	if (pr.findIndex(*it) == -1) {
-	  cl->remove_associated_component(this);
-	  
-	  if (rq.findIndex(*it) == -1)
-	    QObject::disconnect(cl->get_data(), SIGNAL(deleted()),
-				def, SLOT(on_delete()));
-	}
+	// stop to be realizing
+	QObject::disconnect((*it)->get_data(), SIGNAL(deleted()),
+			    def, SLOT(on_delete()));
+	it = realizing_classes.remove(it);
       }
       else
 	++it;
@@ -723,14 +741,11 @@ void BrowserComponent::set_associated_classes(const QValueList<BrowserClass *> &
 	// stop to be provided
 	BrowserClass * cl = *it;
 	
-	it = provided_classes.remove(it);
-	if (rz.findIndex(*it) == -1) {
-	  cl->remove_associated_component(this);
+	cl->remove_associated_component(this);
 	  
-	  if (rq.findIndex(*it) == -1)
-	    QObject::disconnect(cl->get_data(), SIGNAL(deleted()),
-				def, SLOT(on_delete()));
-	}
+	QObject::disconnect(cl->get_data(), SIGNAL(deleted()),
+			    def, SLOT(on_delete()));
+	it = provided_classes.remove(it);
       }
       else
 	++it;
@@ -741,12 +756,9 @@ void BrowserComponent::set_associated_classes(const QValueList<BrowserClass *> &
     while (it != required_classes.end()) {
       if (rq.findIndex(*it) == -1) {
 	// stop to be required
-	BrowserClass * cl = *it;
-	
+	QObject::disconnect((*it)->get_data(), SIGNAL(deleted()),
+			    def, SLOT(on_delete()));
 	it = required_classes.remove(it);
-	if ((rz.findIndex(*it) == -1) && (pr.findIndex(*it) == -1))
-	  QObject::disconnect(cl->get_data(), SIGNAL(deleted()),
-			      def, SLOT(on_delete()));
       }
       else
 	++it;
@@ -754,33 +766,35 @@ void BrowserComponent::set_associated_classes(const QValueList<BrowserClass *> &
   }
   
   // manage added classes
-  realized_classes = rz;
-  for (it = realized_classes.begin(); it != realized_classes.end(); ++it) {
-    BrowserClass * c = *it;
+  realizing_classes = rz;
+  for (it = realizing_classes.begin(); it != realizing_classes.end(); ++it) {
+    BrowserClass * cl = *it;
     
-    if (c->get_associated_components().findIndex(this) == -1) {
-      c->add_associated_component(this);
-      QObject::connect(c->get_data(), SIGNAL(deleted()),
+    if ((pr.findIndex(cl) == -1) && (rq.findIndex(cl) == -1)) {
+      QObject::connect(cl->get_data(), SIGNAL(deleted()),
 		       def, SLOT(on_delete()));
     }
   }
   provided_classes = pr;
   for (it = provided_classes.begin(); it != provided_classes.end(); ++it) {
-    BrowserClass * c = *it;
+    BrowserClass * cl = *it;
     
-    if (c->get_associated_components().findIndex(this) == -1) {
-      c->add_associated_component(this);
-      QObject::connect(c->get_data(), SIGNAL(deleted()),
+    if (cl->get_associated_components().findIndex(this) == -1)
+      cl->add_associated_component(this);
+    
+    if ((rz.findIndex(cl) == -1) && (rq.findIndex(cl) == -1)) {
+      QObject::connect(cl->get_data(), SIGNAL(deleted()),
 		       def, SLOT(on_delete()));
     }
   }
   required_classes = rq;
   for (it = required_classes.begin(); it != required_classes.end(); ++it) {
-    BrowserClass * c = *it;
+    BrowserClass * cl = *it;
     
-    if (c->get_associated_components().findIndex(this) == -1)
-      QObject::connect(c->get_data(), SIGNAL(deleted()),
+    if ((rz.findIndex(cl) == -1) && (pr.findIndex(cl) == -1)) {
+      QObject::connect(cl->get_data(), SIGNAL(deleted()),
 		       def, SLOT(on_delete()));
+    }
   }
   
   if (!on_read)
@@ -810,14 +824,15 @@ bool BrowserComponent::tool_cmd(ToolCom * com, const char * args) {
   case getUmlDefCmd:
   case getCppDefCmd:
   case getJavaDefCmd:
+  case getPhpDefCmd:
   case getIdlDefCmd:
     BrowserNode::tool_cmd(com, args);
     if (com->api_format() > 13) {
       QValueList<BrowserClass *>::Iterator it;
       
-      com->write_unsigned(realized_classes.count());
+      com->write_unsigned(realizing_classes.count());
       
-      for (it = realized_classes.begin(); it != realized_classes.end(); ++it)
+      for (it = realizing_classes.begin(); it != realizing_classes.end(); ++it)
 	(*it)->write_id(com);
       
       com->write_unsigned(provided_classes.count());
@@ -1000,12 +1015,12 @@ void BrowserComponent::save(QTextStream & st, bool ref, QString & warning) {
     
     QValueList<BrowserClass *>::ConstIterator it;
 
-    if (! realized_classes.isEmpty()) {
+    if (! realizing_classes.isEmpty()) {
       nl_indent(st);
-      st << "realized_classes";
+      st << "realizing_classes";
       indent(+1);
       
-      for (it = realized_classes.begin(); it != realized_classes.end(); ++it) {
+      for (it = realizing_classes.begin(); it != realizing_classes.end(); ++it) {
 	nl_indent(st);
 	(*it)->save(st, TRUE, warning);
       }
@@ -1108,12 +1123,23 @@ BrowserComponent * BrowserComponent::read(char * & st, char * k,
       result = new BrowserComponent(s, parent, id);
       result->def->read(st, k);	// updates k
     }
+    else if (result->is_defined) {
+      BrowserComponent * already_exist = result;
+
+      result = new BrowserComponent(s, parent, id);
+      result->def->read(st, k);	// updates k
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("component", result);
+    }
     else {
       result->def->read(st, k);	// updates k
       result->set_parent(parent);
       result->set_name(s);
     }
     
+    result->is_defined = TRUE;
+
     result->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && result->is_api_base();
     
@@ -1127,7 +1153,7 @@ BrowserComponent * BrowserComponent::read(char * & st, char * k,
     QValueList<BrowserClass *> pr;
     QValueList<BrowserClass *> rq;
     
-    if (!strcmp(k, "realized_classes")) {
+    if (!strcmp(k, "realized_classes") || !strcmp(k, "realizing_classes")) {
       while (strcmp((k = read_keyword(st)), "end"))
 	rz.append(BrowserClass::read_ref(st, k));
       
