@@ -63,7 +63,7 @@ BrowserComponentDiagram::BrowserComponentDiagram(int id)
 }
 
 BrowserComponentDiagram::BrowserComponentDiagram(BrowserComponentDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -115,6 +115,17 @@ void BrowserComponentDiagram::make() {
   fragment_color = UmlDefaultColor;
 }
 
+BrowserComponentDiagram * BrowserComponentDiagram::add_component_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter component diagram's name : ",
+				      UmlComponentDiagram, TRUE, FALSE))
+    return new BrowserComponentDiagram(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserComponentDiagram::set_name(const char * s) {
   BrowserDiagram::set_name(s);
   if (window != 0)
@@ -231,11 +242,13 @@ void BrowserComponentDiagram::exec_menu_choice(int rank) {
     return;
   case 3:
     {
-      BrowserComponentDiagram * cd =
-	new BrowserComponentDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      cd->select_in_browser();
-      cd->edit("Component diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter component diagram's name : ",
+						      UmlComponentDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -326,6 +339,10 @@ UmlCode BrowserComponentDiagram::get_type() const {
   return UmlComponentDiagram;
 }
 
+int BrowserComponentDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserComponentDiagram::get_componentdiagramsettings(ComponentDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_componentdiagramsettings(r);
@@ -440,14 +457,6 @@ bool BrowserComponentDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
-void BrowserComponentDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserComponentDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserComponentDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -538,11 +547,21 @@ BrowserComponentDiagram *
     
     if ((r = (BrowserComponentDiagram *) all[id]) == 0)
       r = new BrowserComponentDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserComponentDiagram * already_exist = r;
+
+      r = new BrowserComponentDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("component diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && r->is_api_base();
     
