@@ -63,7 +63,7 @@ BrowserDeploymentDiagram::BrowserDeploymentDiagram(int id)
 }
 
 BrowserDeploymentDiagram::BrowserDeploymentDiagram(BrowserDeploymentDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -119,6 +119,17 @@ void BrowserDeploymentDiagram::make() {
   fragment_color = UmlDefaultColor;
 }
 
+BrowserDeploymentDiagram * BrowserDeploymentDiagram::add_deployment_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter deployment diagram's name : ",
+				      UmlDeploymentDiagram, TRUE, FALSE))
+    return new BrowserDeploymentDiagram(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserDeploymentDiagram::set_name(const char * s) {
   BrowserDiagram::set_name(s);
   if (window != 0)
@@ -234,11 +245,13 @@ void BrowserDeploymentDiagram::exec_menu_choice(int rank) {
     return;
   case 3:
     {
-      BrowserDeploymentDiagram * cd =
-	new BrowserDeploymentDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      cd->select_in_browser();
-      cd->edit("Deployment diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter deployment diagram's name : ",
+						      UmlDeploymentDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -331,6 +344,10 @@ UmlCode BrowserDeploymentDiagram::get_type() const {
   return UmlDeploymentDiagram;
 }
 
+int BrowserDeploymentDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserDeploymentDiagram::get_deploymentdiagramsettings(DeploymentDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_deploymentdiagramsettings(r);
@@ -452,14 +469,6 @@ bool BrowserDeploymentDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
-void BrowserDeploymentDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserDeploymentDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserDeploymentDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -559,11 +568,21 @@ BrowserDeploymentDiagram *
     
     if ((r = (BrowserDeploymentDiagram *) all[id]) == 0)
       r = new BrowserDeploymentDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserDeploymentDiagram * already_exist = r;
+
+      r = new BrowserDeploymentDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("deployment diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && r->is_api_base();
     
