@@ -45,7 +45,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserDeploymentView> BrowserDeploymentView::all;
+IdDict<BrowserDeploymentView> BrowserDeploymentView::all(__FILE__);
 QStringList BrowserDeploymentView::its_default_stereotypes;	// unicode
 
 BrowserDeploymentView::BrowserDeploymentView(QString s, BrowserNode * p, int id)
@@ -100,6 +100,17 @@ void BrowserDeploymentView::make() {
   note_color = UmlDefaultColor;
 }
 
+BrowserDeploymentView * BrowserDeploymentView::add_deployment_view(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter deployment view's name : ",
+				      UmlDeploymentView, TRUE, FALSE))
+    return new BrowserDeploymentView(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserDeploymentView::clear(bool old)
 {
   all.clear(old);
@@ -177,7 +188,8 @@ Note that you can undelete them after");
     m.insertItem("Generate", &subm);
     subm.insertItem("C++", 10);
     subm.insertItem("Java", 11);
-    subm.insertItem("Idl", 12);
+    subm.insertItem("Php", 12);
+    subm.insertItem("Idl", 13);
     
     if (edition_number == 0) {
       if (preserve_bodies()) {
@@ -185,6 +197,7 @@ Note that you can undelete them after");
 	
 	roundtripm.insertItem("C++", 30);
 	roundtripm.insertItem("Java", 31);
+	roundtripm.insertItem("Php", 32);
       }
       
       if (Tool::menu_insert(&toolm, get_type(), 100)) {
@@ -207,7 +220,14 @@ Do not undelete its sub items");
 void BrowserDeploymentView::exec_menu_choice(int rank) {
   switch (rank) {
   case 0:
-    add_deployment_diagram();
+    {
+      BrowserDeploymentDiagram * d =
+	BrowserDeploymentDiagram::add_deployment_diagram(this);
+      
+      if (d != 0)
+	d->select_in_browser();
+    }
+    break;
     break;
   case 1:
     {
@@ -284,6 +304,16 @@ void BrowserDeploymentView::exec_menu_choice(int rank) {
     }
     return;
   case 12:
+    {
+      bool preserve = preserve_bodies();
+      
+      ToolCom::run((verbose_generation()) 
+		   ? ((preserve) ? "php_generator -v -p" : "php_generator -v")
+		   : ((preserve) ? "php_generator -p" : "php_generator"), 
+		   this);
+    }
+    return;
+  case 13:
     ToolCom::run((verbose_generation()) ? "idl_generator -v" : "idl_generator", this);
     return;
   case 30:
@@ -292,6 +322,9 @@ void BrowserDeploymentView::exec_menu_choice(int rank) {
   case 31:
     ToolCom::run((verbose_generation()) ? "roundtrip_body -v java" : "roundtrip_body java", this);
     return;
+  case 32:
+    ToolCom::run((verbose_generation()) ? "roundtrip_body -v php" : "roundtrip_body php", this);
+    return;
   default:
     if (rank >= 100)
       ToolCom::run(Tool::command(rank - 100), this);
@@ -315,29 +348,49 @@ void BrowserDeploymentView::apply_shortcut(QString s) {
       else if (s == "New artifact")
 	choice = 2;
     }
-    if (!is_edited) {
-      if (s == "Edit")
-	choice = 3;
-      if (!is_read_only) {
-	//m.setWhatsThis(m.insertItem("Edit node settings", 4),
-	//		   "to set the sub node's settings");
-	if (s == "Edit drawing settings")
-	  choice = 5;
-	if (edition_number == 0) {
-	  if (s == "Delete")
-	    choice = 6;
+   
+    if (choice == -1) {
+      if (!is_edited) {
+	if (s == "Edit")
+	  choice = 3;
+	else if (!is_read_only) {
+	  //m.setWhatsThis(m.insertItem("Edit node settings", 4),
+	  //		   "to set the sub node's settings");
+	  if (s == "Edit drawing settings")
+	    choice = 5;
+	  if (edition_number == 0) {
+	    if (s == "Delete")
+	      choice = 6;
+	  }
+	}
+      }
+      if (choice == -1) {
+	if (preserve_bodies()) {
+	  if (s == "Roundtrip C++ operation body")
+	    choice = 30;
+	  else if (s == "Roundtrip Java operation body")
+	    choice = 31;
+	  else if (s == "Roundtrip Php operation body")
+	    choice = 32;
+	}
+	
+	if (choice == -1)
+	  mark_shortcut(s, choice, 90);
+	
+	if (choice == -1) {
+	  if (edition_number == 0)
+	    Tool::shortcut(s, choice, get_type(), 100);
+	  if (s == "Generate C++")
+	    choice = 10;
+	  else if (s == "Generate Java")
+	    choice = 11;
+	  else if (s == "Generate Php")
+	    choice = 12;
+	  else if (s == "Generate Idl")
+	    choice = 13;
 	}
       }
     }
-    mark_shortcut(s, choice, 90);
-    if (edition_number == 0)
-      Tool::shortcut(s, choice, get_type(), 100);
-    if (s == "Generate C++")
-      choice = 10;
-    else if (s == "Generate Java")
-      choice = 11;
-    else if (s == "Generate Idl")
-      choice = 12;
   }
   else if (!is_read_only && (edition_number == 0)) {
     if (s == "Undelete")
@@ -354,15 +407,14 @@ void BrowserDeploymentView::open(bool) {
     edit("Deployment view", its_default_stereotypes);
 }
 
-void BrowserDeploymentView::add_deployment_diagram() {
-  (new BrowserDeploymentDiagram(child_random_name("Diagram"), this))
-    ->select_in_browser();
-}
-
 UmlCode BrowserDeploymentView::get_type() const {
   return UmlDeploymentView;
 }
 
+int BrowserDeploymentView::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserDeploymentView::get_data() const {
   return def;
 }
@@ -574,8 +626,10 @@ void BrowserDeploymentView::DropAfterEvent(QDropEvent * e, BrowserNode * after)
 	insertItem(bn);
       }
       package_modified();
-      if (old_parent != this)
+      if (old_parent != this) {
 	old_parent->package_modified();
+	bn->modified();
+      }
     }
     else {
       msg_critical("Error", "Forbiden");
@@ -694,11 +748,21 @@ BrowserDeploymentView * BrowserDeploymentView::read(char * & st, char * k,
     
     if (r == 0)
       r = new BrowserDeploymentView(s, parent, id);
+    else if (r->is_defined) {
+      BrowserDeploymentView * already_exist = r;
+
+      r = new BrowserDeploymentView(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("deployment view", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && r->is_api_base();
     
