@@ -43,7 +43,7 @@
 #include "myio.h"
 
 // use the same extension for all the diagrams : must share dict
-IdDict<BrowserDiagram> BrowserDiagram::all(257);
+IdDict<BrowserDiagram> BrowserDiagram::all(257, __FILE__);
 
 BrowserDiagram::BrowserDiagram(QString s, BrowserNode * parent, int id)
     : BrowserNode(s, parent), Labeled<BrowserDiagram>(all, id) {
@@ -69,8 +69,10 @@ BrowserDiagram::~BrowserDiagram() {
 }
 
 void BrowserDiagram::package_modified() {
-  is_modified = TRUE;
-  BrowserNode::package_modified();
+  if (!on_load_diagram()) {
+    is_modified = TRUE;
+    BrowserNode::package_modified();
+  }
 }
 
 bool BrowserDiagram::get_auto_label_position(UmlCode) const {
