@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -48,6 +48,10 @@ class BrowserDiagram : public BrowserNode, public Labeled<BrowserDiagram> {
     
     virtual bool get_auto_label_position(UmlCode who) const;
     virtual bool get_shadow() const = 0;
+    virtual bool get_draw_all_relations() const = 0;
+    virtual void dont_draw_all_relations() = 0;
+
+    virtual void draw_svg() const = 0;
     
     CanvasFormat get_format() const { return canvas_size; }
     void set_format(CanvasFormat c) { canvas_size = c; }
