@@ -64,12 +64,14 @@ class BrowserOperation : public BrowserNode, public Labeled<BrowserOperation> {
     void set_get_of(BrowserNode * o);
     void set_set_of(BrowserNode * o);
     void update_get_of(const QString & attr_name, const QString & cpp_decl,
-		       const QString & java_decl, const QString & idl_decl,
+		       const QString & java_decl, const QString & php_decl, 
+		       const QString & idl_decl,
 		       bool is_const, bool is_class_member,
 		       const AType & cl, QString multiplicity = QString::null,
 		       QString stereotype = QString::null);
     void update_set_of(const QString & attr_name, const QString & cpp_decl,
-		       const QString & java_decl, const QString & idl_decl,
+		       const QString & java_decl, const QString & php_decl, 
+		       const QString & idl_decl,
 		       bool is_const, bool is_class_member,
 		       const AType & cl, QString multiplicity = QString::null,
 		       QString stereotype = QString::null);
@@ -84,6 +86,7 @@ class BrowserOperation : public BrowserNode, public Labeled<BrowserOperation> {
     virtual void apply_shortcut(QString s);
     virtual void open(bool);
     virtual UmlCode get_type() const;
+    virtual int get_identifier() const;
     virtual void modified();
     virtual BasicData * get_data() const;
     virtual bool allow_spaces() const;
@@ -111,8 +114,6 @@ class BrowserOperation : public BrowserNode, public Labeled<BrowserOperation> {
     
     static QString drag_key(BrowserNode * p);
     virtual QString drag_key() const;
-    virtual void DragMoveEvent(QDragMoveEvent * e);
-    virtual void DropEvent(QDropEvent * e);
     virtual void DropAfterEvent(QDropEvent * e, BrowserNode * after);
     
     static const QStringList & default_stereotypes();
