@@ -78,7 +78,7 @@
 #include "SaveProgress.h"
 #include "DialogUtil.h"
 
-IdDict<BrowserPackage> BrowserPackage::all;
+IdDict<BrowserPackage> BrowserPackage::all(__FILE__);
 QList<BrowserPackage> BrowserPackage::removed;
 
 QStringList BrowserPackage::its_default_stereotypes;	// unicode
@@ -98,6 +98,7 @@ BrowserPackage::BrowserPackage(QString s, BrowserView * parent, int id)
   classdiagram_settings.hide_operations = UmlNo;
   classdiagram_settings.show_full_members_definition = UmlNo;
   classdiagram_settings.show_members_visibility = UmlNo;
+  classdiagram_settings.show_members_stereotype = UmlNo;
   classdiagram_settings.show_parameter_dir = UmlYes;
   classdiagram_settings.show_parameter_name = UmlYes;
   classdiagram_settings.package_name_in_tab = UmlNo;
@@ -120,6 +121,7 @@ BrowserPackage::BrowserPackage(QString s, BrowserView * parent, int id)
   
   sequencediagram_settings.show_full_operations_definition = UmlNo;
   sequencediagram_settings.write_horizontally = UmlYes;
+  sequencediagram_settings.instances_drawing_mode = Natural;
   sequencediagram_settings.drawing_language = UmlView;
   sequencediagram_settings.draw_all_relations = UmlYes;
   sequencediagram_settings.shadow = UmlYes;
@@ -320,6 +322,9 @@ void BrowserPackage::clear(bool old)
   BrowserComponentView::clear(old);
   BrowserDeploymentView::clear(old);
   BrowserDiagram::clear(old);
+  
+  if (! old)
+    check_ids_cleared();
 }
 
 void BrowserPackage::update_idmax_for_root()
@@ -444,7 +449,8 @@ through a relation");
     m.insertItem("Generate", &genm);
     genm.insertItem("C++", 20);
     genm.insertItem("Java", 21);
-    genm.insertItem("Idl", 22);
+    genm.insertItem("Php", 22);
+    genm.insertItem("Idl", 23);
     
     if (edition_number == 0) {
       if (!is_read_only) {
@@ -453,12 +459,14 @@ through a relation");
 	revm.insertItem("C++", 24);
 	revm.insertItem("Java", 25);
 	revm.insertItem("Java Catalog", 26);
+	revm.insertItem("Php", 32);
 	
 	if (preserve_bodies()) {
 	  m.insertItem("Roundtrip body", &roundtripm);
 	
 	  roundtripm.insertItem("C++", 30);
 	  roundtripm.insertItem("Java", 31);
+	  roundtripm.insertItem("Php", 33);
 	}
       }
 
@@ -509,16 +517,44 @@ void BrowserPackage::exec_menu_choice(int rank) {
     add_package();
     break;
   case 1:
-    add_use_case_view();
+    {
+      BrowserUseCaseView * v = 
+	BrowserUseCaseView::add_use_case_view(this);
+      
+      if (v == 0)
+	return;
+      v->select_in_browser();
+    }
     break;
   case 2:
-    add_class_view();
+    {
+      BrowserClassView * v = 
+	BrowserClassView::add_class_view(this);
+      
+      if (v == 0)
+	return;
+      v->select_in_browser();
+    }
     break;
   case 3:
-    add_component_view();
+    {
+      BrowserComponentView * v = 
+	BrowserComponentView::add_component_view(this);
+      
+      if (v == 0)
+	return;
+      v->select_in_browser();
+    }
     break;
   case 4:
-    add_deployment_view();
+    {
+      BrowserDeploymentView * v = 
+	BrowserDeploymentView::add_deployment_view(this);
+      
+      if (v == 0)
+	return;
+      v->select_in_browser();
+    }
     break;
   case 14:
     import_project();
@@ -568,6 +604,14 @@ void BrowserPackage::exec_menu_choice(int rank) {
     }
     return;
   case 22:
+    {
+      ToolCom::run((verbose_generation()) 
+		   ? ((preserve) ? "php_generator -v -p" : "php_generator -v")
+		   : ((preserve) ? "php_generator -p" : "php_generator"), 
+		   this);
+    }
+    return;
+  case 23:
     ToolCom::run((verbose_generation()) ? "idl_generator -v" : "idl_generator", this);
     return;
   case 24:
@@ -579,9 +623,13 @@ void BrowserPackage::exec_menu_choice(int rank) {
   case 26:
     ToolCom::run("java_catalog", this);
     return;
+  case 32:
+    ToolCom::run("php_reverse", this);
+    return;
   case 27:
     renumber(phase_renumerotation++);
     if (phase_renumerotation == 2) {
+
       renumber(2);
 #if 0
       ???
@@ -606,6 +654,9 @@ void BrowserPackage::exec_menu_choice(int rank) {
   case 31:
     ToolCom::run((verbose_generation()) ? "roundtrip_body -v java" : "roundtrip_body java", this);
     return;
+  case 33:
+    ToolCom::run((verbose_generation()) ? "roundtrip_body -v php" : "roundtrip_body php", this);
+    return;
   default:
     if (rank >= 100)
       ToolCom::run(Tool::command(rank - 100), this);
@@ -655,16 +706,30 @@ void BrowserPackage::apply_shortcut(QString s) {
     }
     if (s == "Referenced by")
       choice = 13;
-    mark_shortcut(s, choice, 90);
-    if (s == "Generate C++")
-      choice = 20;
-    else if (s == "Generate Java")
-      choice = 21;
-    else if (s == "Generate Idl")
-      choice = 22;
+    else
+      mark_shortcut(s, choice, 90);
     
-    if (edition_number == 0)
-      Tool::shortcut(s, choice, get_type(), 100);
+    if (choice == -1) {
+      if (s == "Generate C++")
+	choice = 20;
+      else if (s == "Generate Java")
+	choice = 21;
+      else if (s == "Generate Php")
+	choice = 22;
+      else if (s == "Generate Idl")
+	choice = 23;
+      else if (preserve_bodies()) {
+	if (s == "Roundtrip C++ operation body")
+	  choice = 30;
+	else if (s == "Roundtrip Java operation body")
+	  choice = 31;
+	else if (s == "Roundtrip Php operation body")
+	  choice = 32;
+      }    
+      
+      if ((choice == -1) && (edition_number == 0))
+	Tool::shortcut(s, choice, get_type(), 100);
+    }
   }
   else if (!is_read_only && (edition_number == 0)) {
     if (s == "Undelete")
@@ -769,49 +834,37 @@ BrowserPackage * BrowserPackage::get_package()
 }
 
 void BrowserPackage::add_package() {
-  BrowserPackage * p =
-    new BrowserPackage(child_random_name("Package"), this);
+  QString name;
   
-  p->select_in_browser();
-  if ((owner != -1) &&
-      (msg_warning("Bouml",
-		   "Do you want to be the owner of this new package ?\n"
-		   "(other users can't modify it while you are the owner)",
-		   QMessageBox::Yes, QMessageBox::No )
-       == QMessageBox::Yes))
-    p->owner = owner;
-}
-
-void BrowserPackage::add_use_case_view() {
-  (new BrowserUseCaseView(child_random_name("Use Case View"), this))
-    ->select_in_browser();
-}
-
-void BrowserPackage::add_class_view() {
-  (new BrowserClassView(child_random_name("Class view"), this))
-    ->select_in_browser();
-}
-
-void BrowserPackage::add_component_view() {
-  (new BrowserComponentView(child_random_name("Component view"), this))
-    ->select_in_browser();
-}
-
-void BrowserPackage::add_deployment_view() {
-  (new BrowserDeploymentView(child_random_name("Deployment view"), this))
-    ->select_in_browser();
+  if (enter_child_name(name, "enter package's name : ",
+		       UmlPackage, TRUE, FALSE)) {
+    BrowserPackage * p = new BrowserPackage(name, this);
+    
+    p->select_in_browser();
+    if ((owner != -1) &&
+	(msg_warning("Bouml",
+		     "Do you want to be the owner of this new package ?\n"
+		     "(other users can't modify it while you are the owner)",
+		     QMessageBox::Yes, QMessageBox::No )
+	 == QMessageBox::Yes))
+      p->owner = owner;
+  }
 }
 
 void BrowserPackage::import_project() {
-  QString fn = QFileDialog::getOpenFileName(QString::null, "*.prj");
+  QString fn = QFileDialog::getOpenFileName(last_used_directory(), "*.prj");
   
   if (!fn.isEmpty()) {
+    set_last_used_directory(fn);
+    
     QFileInfo fi(fn);
   
     if (!fi.exists())
       return;
 
-    if (wrong_child_name(fi.baseName(), UmlPackage, TRUE, FALSE)) {
+    QString bname = my_baseName(fi);
+    
+    if (wrong_child_name(bname, UmlPackage, TRUE, FALSE)) {
       msg_critical("Error", "illegal name or already used");
       return;
     }
@@ -819,7 +872,7 @@ void BrowserPackage::import_project() {
     QApplication::setOverrideCursor(Qt::waitCursor);
     QDir di(fi.dirPath(TRUE));
     
-    BrowserPackage * p = new BrowserPackage(fi.baseName(), this);
+    BrowserPackage * p = new BrowserPackage(bname, this);
     bool err = FALSE;
     
     set_in_import(TRUE);
@@ -832,6 +885,7 @@ void BrowserPackage::import_project() {
       err = TRUE;
     }
     BrowserNode::post_load();
+    idmax_add_margin();
     BrowserDiagram::import();
     BrowserView::set_imported_project(di, 0);
     set_in_import(FALSE);
@@ -849,6 +903,10 @@ UmlCode BrowserPackage::get_type() const {
   return UmlPackage;
 }
 
+int BrowserPackage::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserPackage::get_data() const {
   return def;
 }
@@ -1586,8 +1644,10 @@ void BrowserPackage::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
 	x->insertItem(bn);
       }
       x->package_modified();
-      if (old_parent != x)
+      if (old_parent != x) {
 	old_parent->package_modified();
+	bn->modified();
+      }
     }
     else if ((parent() != 0) && (after == 0))
       ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -1609,7 +1669,7 @@ const QStringList & BrowserPackage::default_stereotypes()
   return its_default_stereotypes;
 }
 
-const QStringList & BrowserPackage::default_stereotypes(UmlCode) {
+const QStringList & BrowserPackage::default_stereotypes(UmlCode, const BrowserNode *) const {
   return relation_default_stereotypes;
 }
 
@@ -1734,9 +1794,13 @@ bool BrowserPackage::read_stereotypes(const char * f)
 
 bool BrowserPackage::import_stereotypes()
 {
-  QString fn = QFileDialog::getOpenFileName("stereotypes", "stereotypes");
+  QString fn = QFileDialog::getOpenFileName(last_used_directory(), "stereotypes");
       
-  return (!fn.isEmpty() && read_stereotypes((const char *) fn));
+  if (fn.isEmpty())
+    return FALSE;
+  
+  set_last_used_directory(fn);
+  return read_stereotypes((const char *) fn);
 }
 
 // save / restore
@@ -2276,6 +2340,7 @@ To change its format : load this project and save it.");
 	}
 	
 	read_color(st, "class_color", class_color, k);	// updates k
+	read_color(st, "duration", duration_color, k);	// old, updates k
 	read_color(st, "duration_color", duration_color, k);	// updates k
 	read_color(st, "continuation_color", continuation_color, k);	// updates k
 	read_color(st, "note_color", note_color, k);	// updates k
