@@ -36,6 +36,8 @@
 #include "BrowserOperation.h"
 #include "BrowserClass.h"
 #include "RelationData.h"
+#include "BrowserClassInstance.h"
+#include "ReferenceDialog.h"
 #include "UmlPixmap.h"
 #include "UmlGlobal.h"
 #include "myio.h"
@@ -46,7 +48,8 @@
 #include "strutil.h"
 #include "mu.h"
 
-IdDict<BrowserRelation> BrowserRelation::all(1023);
+IdDict<BrowserRelation> BrowserRelation::all(1023, __FILE__);
+static QList<BrowserRelation> Unconsistent;
 
 BrowserRelation::BrowserRelation(BrowserNode * p, RelationData * d, int id)
     : BrowserNode(d->get_name(), p), Labeled<BrowserRelation>(all, id),
@@ -96,8 +99,17 @@ void BrowserRelation::delete_it() {
   
   BrowserNode::delete_it();
   
-  def->get_start()->modified();
-  def->get_end_class()->modified();  
+  // warning, def may be 0 in case the relation was a
+  // temporary unvalidated
+  if (def != 0) {
+    BrowserRelation * b = def->get_start();
+
+    if (b != 0) {
+      b->modified();
+      def->get_start_class()->modified();  
+      def->get_end_class()->modified();  
+    }
+  }
 }
 
 bool BrowserRelation::undelete(bool, QString & warning, QString & renamed) {
@@ -172,6 +184,11 @@ void BrowserRelation::compute_referenced_by(QList<BrowserNode> & l,
   }
 }
 
+void BrowserRelation::referenced_by(QList<BrowserNode> & l) {
+  BrowserNode::referenced_by(l);
+  BrowserClassInstance::compute_referenced_by(l, this);
+}
+
 BrowserRelation * BrowserRelation::temporary(RelationData * d) {
   BrowserRelation * result = new BrowserRelation(0);
   
@@ -218,7 +235,9 @@ void BrowserRelation::update_get_oper() {
   if (def->is_a(this)) {
     t.type = def->get_end_class();
     get_oper->update_get_of(def->get_role_a(), def->get_cppdecl_a(),
-			    def->get_javadecl_a(), def->get_idldecl_a(), 
+			    def->get_javadecl_a(), 
+			    def->get_phpdecl_a(), 
+			    def->get_idldecl_a(), 
 			    def->get_isa_const_relation_a(),
 			    def->get_isa_class_relation_a(),
 			    t, def->get_multiplicity_a(),
@@ -227,7 +246,9 @@ void BrowserRelation::update_get_oper() {
   else {
     t.type = def->get_start_class();
     get_oper->update_get_of(def->get_role_b(), def->get_cppdecl_b(),
-			    def->get_javadecl_b(), def->get_idldecl_b(), 
+			    def->get_javadecl_b(),
+			    def->get_phpdecl_b(),
+			    def->get_idldecl_b(), 
 			    def->get_isa_const_relation_b(),
 			    def->get_isa_class_relation_b(),
 			    t, def->get_multiplicity_b(),
@@ -241,7 +262,9 @@ void BrowserRelation::update_set_oper() {
   if (def->is_a(this)) {
     t.type = def->get_end_class();
     set_oper->update_set_of(def->get_role_a(), def->get_cppdecl_a(),
-			    def->get_javadecl_a(), def->get_idldecl_a(), 
+			    def->get_javadecl_a(), 
+			    def->get_phpdecl_a(), 
+			    def->get_idldecl_a(), 
 			    def->get_isa_const_relation_a(),
 			    def->get_isa_class_relation_a(),
 			    t, def->get_multiplicity_a(),
@@ -250,7 +273,9 @@ void BrowserRelation::update_set_oper() {
   else {
     t.type = def->get_start_class();
     set_oper->update_set_of(def->get_role_b(), def->get_cppdecl_b(),
-			    def->get_javadecl_b(), def->get_idldecl_b(), 
+			    def->get_javadecl_b(),
+			    def->get_phpdecl_b(),
+			    def->get_idldecl_b(), 
 			    def->get_isa_const_relation_b(),
 			    def->get_isa_class_relation_b(),
 			    t, def->get_multiplicity_b(),
@@ -373,6 +398,13 @@ a double click with the left mouse button does the same thing");
 			   "to auto define the <em>get</em> and <em>set operation</em>s");
 	  m.insertSeparator();
 	}
+      }
+  
+      m.setWhatsThis(m.insertItem("Referenced by", 8),
+		     "to know who reference the <i>relation</i>");
+      m.insertSeparator();
+      
+      if (!is_read_only && (edition_number == 0)) {
 	m.setWhatsThis(m.insertItem("Delete", 2),
 		       "to delete the <em>relation</em>. \
 Note that you can undelete it after");
@@ -392,10 +424,14 @@ Note that you can undelete it after");
       m.insertItem("Tool", &toolm);
     }
   }
-  else if (!is_read_only && (edition_number == 0))
+  else if (!is_read_only && (edition_number == 0)) {
     m.setWhatsThis(m.insertItem("Undelete", 3),
 		   "undelete the <em>relation</em> \
 (except if the class at the other side is also deleted)");
+    if (def->get_start_class()->deletedp() ||
+	def->get_end_class()->deletedp())
+      m.setItemEnabled(3, FALSE);
+  }
   
   exec_menu_choice(m.exec(QCursor::pos()));
 }
@@ -427,6 +463,9 @@ void BrowserRelation::exec_menu_choice(int rank) {
     ((def->is_a(this)) ? def->get_end_class()
 		       : def->get_start_class())->select_in_browser();
     return;
+  case 8:
+    ReferenceDialog::show(this);
+    return;
   default:
     if (rank >= 100)
       ToolCom::run(Tool::command(rank - 100), this);
@@ -465,6 +504,10 @@ void BrowserRelation::apply_shortcut(QString s) {
     }
     if (s == "Select target")
       choice = 7;
+  
+    if (s == "Referenced by")
+      choice = 8;
+    
     mark_shortcut(s, choice, 90);
     if (edition_number == 0)
       Tool::shortcut(s, choice, get_type(), 100);
@@ -505,6 +548,10 @@ UmlCode BrowserRelation::get_type() const {
     : def->get_type();
 }
 
+int BrowserRelation::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserRelation::get_data() const {
   return def;
 }
@@ -591,14 +638,6 @@ bool BrowserRelation::tool_cmd(ToolCom * com, const char * args) {
   return TRUE;
 }
 
-void BrowserRelation::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserRelation::DropEvent(QDropEvent * e) {
-  DropAfterEvent(e, 0);
-}
-
 void BrowserRelation::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
   if (after == 0)
     ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -684,7 +723,7 @@ void BrowserRelation::save(QTextStream & st, bool ref, QString & warning) {
   }
 }
 
-void BrowserRelation::post_load(QList<BrowserRelation> & wrong)
+void BrowserRelation::post_load()
 {
   IdIterator<BrowserRelation> it(all);
   BrowserRelation * br;
@@ -698,23 +737,27 @@ void BrowserRelation::post_load(QList<BrowserRelation> & wrong)
       
       if ((d = br->def) == 0)
 	// end -> start relation, not read
-	wrong.append(br);
-      else if (d->get_type() == UmlRelations)
+	br->must_be_deleted();
+      else if (d->get_start() == 0) {
+	// end -> start read, but start -> end not read
+	d->set_unconsistent();
+      }
+      else if (d->get_type() == UmlRelations) {
 	// temporary data, rel not read,
-	wrong.append(br);
+	br->delete_it();	// to update diagrams
+	d->set_unconsistent();
+      }
       else if ((d->get_start() == br) &&
 	       ((endrel = d->get_end()) != 0) &&
 	       (endrel->def != d)) {
 	// br is the 'forward' direction of a bi dir relation,
 	// endrel is the 'backward' direction not read
 	if (endrel->def != 0)
-	  delete endrel->def;
-	endrel->def = br->def; // to help delete
-	
-	wrong.append(br);
+	  endrel->def->set_unconsistent();
+	d->set_unconsistent();
       }
       else {
-	// not a temporary relation
+	// not a temporary or unconsistent relation
 	if ((it.current()->get_oper != 0) && 
 	    (((BrowserNode *) it.current()->get_oper->parent())->get_type() != UmlClass))
 	  // operation was deleted
@@ -727,7 +770,34 @@ void BrowserRelation::post_load(QList<BrowserRelation> & wrong)
     }
     ++it;
   }
+  
+  while (!Unconsistent.isEmpty()) {
+    br = Unconsistent.take(0);
+    br->def->set_unconsistent();
+    br->def = 0;
+    br->must_be_deleted();
+  }
+  
+  if (RelationData::has_unconsistencies()) {
+    IdIterator<BrowserRelation> it2(all);
+  
+    while ((br = it2.current()) != 0) {
+      if ((br->def != 0) && br->def->unconsistentp()) {
+	br->def = 0; // to not call garbage
+	br->must_be_deleted();
+      }
+      ++it2;
+    }
+  
+    RelationData::delete_unconsistent();
+  }
+}
 
+// Created for a relation data ref read before its definition
+// The browser node become useless when the data definition
+// is read
+void BrowserRelation::unvalidate() {
+  def = 0;
 }
 
 BrowserRelation * BrowserRelation::read_ref(char * & st)
@@ -765,10 +835,17 @@ BrowserRelation * BrowserRelation::read(char * & st, char * k,
     id = read_id(st);
     k = read_keyword(st);
     
-    RelationData * d = RelationData::read(st, k);	// updates k
+    BrowserRelation * unconsistent;   // data id shared
+    RelationData * d = RelationData::read(st, k, unconsistent);	// updates k
+    BrowserRelation * already_exist = 0;
     
     if ((result = all[id]) == 0)
       result = new BrowserRelation(parent, d, id);
+    else if (result->is_defined) {
+      already_exist = result;
+      result = new BrowserRelation(parent, d, id);
+      d->set_unconsistent();
+    }
     else {
       if (result->def != 0)
 	// re-load
@@ -777,7 +854,16 @@ BrowserRelation * BrowserRelation::read(char * & st, char * k,
       result->set_parent(parent);
       result->set_name(d->get_name(result));
     }
-    
+
+    if (already_exist != 0) {
+      result->unconsistent_removed("relation", already_exist);
+      Unconsistent.append(already_exist);
+    }
+
+    if ((unconsistent != 0) && (unconsistent != already_exist))
+      result->unconsistent_removed("relation", unconsistent);
+
+    result->is_defined = TRUE;
     result->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && result->is_api_base();
     
