@@ -34,7 +34,7 @@
 #include "BrowserSeqDiagram.h"
 #include "SimpleData.h"
 #include "SeqDiagramWindow.h"
-#include "DiagramView.h"
+#include "SeqDiagramView.h"
 #include "UmlPixmap.h"
 #include "SettingsDialog.h"
 #include "myio.h"
@@ -63,7 +63,7 @@ BrowserSeqDiagram::BrowserSeqDiagram(int id)
 }
 
 BrowserSeqDiagram::BrowserSeqDiagram(BrowserSeqDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -73,6 +73,7 @@ BrowserSeqDiagram::BrowserSeqDiagram(BrowserSeqDiagram * model, BrowserNode * p)
   duration_color = model->duration_color;
   continuation_color = model->continuation_color;
   class_instance_color = model->class_instance_color;
+  overlapping_bars = model->overlapping_bars;
   canvas_size = model->canvas_size;
   is_modified = TRUE;
   
@@ -115,6 +116,18 @@ void BrowserSeqDiagram::make() {
   duration_color = UmlDefaultColor;
   continuation_color = UmlDefaultColor;
   class_instance_color = UmlDefaultColor;
+  overlapping_bars = TRUE;
+}
+
+BrowserSeqDiagram * BrowserSeqDiagram::add_sequence_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter sequence diagram's name : ",
+				      UmlSeqDiagram, TRUE, FALSE))
+    return new BrowserSeqDiagram(name, future_parent);
+  else
+    return 0;
 }
 
 void BrowserSeqDiagram::set_name(const char * s) {
@@ -268,11 +281,13 @@ void BrowserSeqDiagram::exec_menu_choice(int rank,
     return;
   case 3:
     {
-      BrowserSeqDiagram * sd = 
-	new BrowserSeqDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      sd->select_in_browser();
-      sd->edit("Sequence diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter sequence diagram's name : ",
+						      UmlSeqDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -371,6 +386,10 @@ UmlCode BrowserSeqDiagram::get_type() const {
   return UmlSeqDiagram;
 }
 
+int BrowserSeqDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserSeqDiagram::get_sequencediagramsettings(SequenceDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_sequencediagramsettings(r);
@@ -465,20 +484,23 @@ bool BrowserSeqDiagram::tool_cmd(ToolCom * com, const char * args) {
       w->close(TRUE);
     }
     return TRUE;
+  case sideCmd:
+    if (window != 0)
+      ((SeqDiagramView *) window->get_view())->send(com);
+    else {
+      SeqDiagramWindow * w = new SeqDiagramWindow(full_name(), this);
+      
+      ((SeqDiagramView *) w->get_view())->send(com);
+      w->dont_save();
+      w->close(TRUE);
+    }
+    return TRUE;
   default:
     return (def->tool_cmd(com, args, this, comment) ||
 	    BrowserNode::tool_cmd(com, args));
   }
 }
 
-void BrowserSeqDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserSeqDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserSeqDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -525,6 +547,8 @@ void BrowserSeqDiagram::save(QTextStream & st, bool ref, QString & warning) {
     }
     
     nl_indent(st);
+    if (overlapping_bars)
+      st << "overlapping_bars ";
     st << "size " << stringify(get_format());
     
     indent(-1);
@@ -555,11 +579,21 @@ BrowserSeqDiagram * BrowserSeqDiagram::read(char * & st, char * k,
     
     if ((r = (BrowserSeqDiagram *) all[id]) == 0)
       r = new BrowserSeqDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserSeqDiagram * already_exist = r;
+
+      r = new BrowserSeqDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("sequence diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && r->is_api_base();
     
@@ -571,6 +605,7 @@ BrowserSeqDiagram * BrowserSeqDiagram::read(char * & st, char * k,
     
     r->def->read(st, k);				// updates k
     r->settings.read(st, k);				// updates k
+    read_color(st, "duration", r->duration_color, k);	// old, updates k
     read_color(st, "duration_color", r->duration_color, k);	 // updates k
     read_color(st, "continuation_color", r->continuation_color, k);	 // updates k
     read_color(st, "note_color", r->note_color, k);	// updates k
@@ -579,6 +614,12 @@ BrowserSeqDiagram * BrowserSeqDiagram::read(char * & st, char * k,
     read_color(st, "fragment_color", r->fragment_color, k);	// updates k
     r->BrowserNode::read(st, k);			// updates k
     
+    if (!strcmp(k, "overlapping_bars"))
+      // constructor set overlapping_bars to TRUE
+      k = read_keyword(st);
+    else
+      r->overlapping_bars = FALSE;
+    
     if (!strcmp(k, "size")) {
       r->set_format(canvas_format(read_keyword(st)));
       k = read_keyword(st);
