@@ -51,7 +51,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserState> BrowserState::all(257);
+IdDict<BrowserState> BrowserState::all(257, __FILE__);
 QStringList BrowserState::its_default_stereotypes;	// unicode
 
 BrowserState::BrowserState(QString s, BrowserNode * p, StateData * d, int id)
@@ -90,11 +90,19 @@ BrowserState::~BrowserState() {
 void BrowserState::clear(bool old)
 {
   all.clear(old);
+  BrowserTransition::clear(old);
+  BrowserRegion::clear(old);
+  BrowserPseudoState::clear(old);
+  BrowserStateAction::clear(old);
 }
 
 void BrowserState::update_idmax_for_root()
 {
   all.update_idmax_for_root();
+  BrowserTransition::update_idmax_for_root();
+  BrowserRegion::update_idmax_for_root();
+  BrowserPseudoState::update_idmax_for_root();
+  BrowserStateAction::update_idmax_for_root();
 }
     
 void BrowserState::referenced_by(QList<BrowserNode> & l) {
@@ -113,7 +121,10 @@ const QPixmap* BrowserState::pixmap(int) const {
 }
 
 BrowserTransition * BrowserState::add_transition(BrowserNode * end) {
-  return new BrowserTransition(this, end);
+  BrowserTransition * result = new BrowserTransition(this, end);
+  
+  modified();
+  return result;
 }
 
 BasicData * BrowserState::add_relation(UmlCode, BrowserNode * end) {
@@ -176,10 +187,6 @@ BrowserState * BrowserState::get_state(BrowserNode * parent)
   return r;
 }
 
-void BrowserState::add_state_diagram() {
-  (new BrowserStateDiagram(child_random_name("Diagram"), this))->select_in_browser();
-}
-
 static bool is_machine(const BrowserState * s)
 {
   switch (((BrowserNode *) s->parent())->get_type()) {
@@ -303,7 +310,14 @@ void BrowserState::exec_menu_choice(int rank,
 				    bool mach) {
   switch (rank) {
   case 0:
-    add_state_diagram();
+    {
+      BrowserStateDiagram * d = 
+	BrowserStateDiagram::add_state_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 1:
     add_state(this, (bool) TRUE);
@@ -441,6 +455,10 @@ UmlCode BrowserState::get_type() const {
   return UmlState;
 }
 
+int BrowserState::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserState::get_data() const {
   return def;
 }
@@ -452,24 +470,6 @@ QString BrowserState::full_name(bool rev, bool) const {
 	       : p + "::" + name;
 }
 
-bool BrowserState::sub_state_of(BrowserState * st) const {
-  const BrowserNode * p = this;
-  
-  for (;;) {
-    p = (BrowserNode *) p->parent();
-    if (p == st)
-      return TRUE;
-    
-    switch (p->get_type()) {
-    case UmlState:
-    case UmlRegion:
-      break;
-    default:
-      return FALSE;
-    }
-  }
-}
-
 BrowserNodeList & BrowserState::instances(BrowserNodeList & result, bool sort)
 {
   IdIterator<BrowserState> it(all);
@@ -806,11 +806,20 @@ BrowserState * BrowserState::read(char * & st, char * k,
     
     if (result == 0)
       result = new BrowserState(read_string(st), parent, new StateData, id);
+    else if (result->is_defined) {
+      BrowserState * already_exist = result;
+
+      result = new BrowserState(read_string(st), parent, new StateData, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("state", result);
+    }
     else {
       result->set_parent(parent);
       result->set_name(read_string(st));
     }
     
+    result->is_defined = TRUE;
     k = read_keyword(st);
     result->def->read(st, k);
 
