@@ -56,7 +56,7 @@ BrowserStateDiagram::BrowserStateDiagram(QString s, BrowserNode * p, int id)
 }
 
 BrowserStateDiagram::BrowserStateDiagram(BrowserStateDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -117,6 +117,17 @@ void BrowserStateDiagram::make() {
   package_color = UmlDefaultColor;
 }
 
+BrowserStateDiagram * BrowserStateDiagram::add_state_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter state diagram's name : ",
+				      UmlStateDiagram, TRUE, FALSE))
+    return new BrowserStateDiagram(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserStateDiagram::set_name(const char * s) {
   BrowserDiagram::set_name(s);
   if (window != 0)
@@ -236,11 +247,13 @@ void BrowserStateDiagram::exec_menu_choice(int rank) {
     return;
   case 3:
     {
-      BrowserStateDiagram * cd =
-	new BrowserStateDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      cd->select_in_browser();
-      cd->edit("State diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter state diagram's name : ",
+						      UmlStateDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -332,6 +345,10 @@ UmlCode BrowserStateDiagram::get_type() const {
   return UmlStateDiagram;
 }
 
+int BrowserStateDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserStateDiagram::get_statediagramsettings(StateDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_statediagramsettings(r);
@@ -488,14 +505,6 @@ bool BrowserStateDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
-void BrowserStateDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserStateDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserStateDiagram::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
   if (after == 0)
     ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -611,11 +620,21 @@ BrowserStateDiagram * BrowserStateDiagram::read(char * & st, char * k,
     
     if ((r = (BrowserStateDiagram *) all[id]) == 0)
       r = new BrowserStateDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserStateDiagram * already_exist = r;
+
+      r = new BrowserStateDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("state diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = (!in_import() && read_only_file()) || 
       (user_id() != 0) && r->is_api_base();
     
