@@ -62,6 +62,7 @@ class BrowserTransition : public BrowserNode, public Labeled<BrowserTransition>
     virtual void open(bool);
     virtual bool in_edition() const;
     virtual UmlCode get_type() const;
+    virtual int get_identifier() const;
     virtual void modified();
     virtual BasicData * get_data() const;
     virtual QString full_name(bool rev = FALSE, bool itself = TRUE) const;
@@ -85,8 +86,6 @@ class BrowserTransition : public BrowserNode, public Labeled<BrowserTransition>
     static QString drag_key(BrowserNode * p);
     virtual QString drag_key() const;
     virtual QString drag_postfix() const;
-    virtual void DragMoveEvent(QDragMoveEvent * e);
-    virtual void DropEvent(QDropEvent * e);
     virtual void DropAfterEvent(QDropEvent * e, BrowserNode * after);
 };
 
