@@ -41,6 +41,9 @@
 #include "UmlPixmap.h"
 #include "UmlDrag.h"
 #include "BrowserClass.h"
+#include "BrowserClassInstance.h"
+#include "BrowserState.h"
+#include "BrowserActivity.h"
 #include "SettingsDialog.h"
 #include "myio.h"
 #include "ToolCom.h"
@@ -50,7 +53,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserUseCase> BrowserUseCase::all(257);
+IdDict<BrowserUseCase> BrowserUseCase::all(257, __FILE__);
 QStringList BrowserUseCase::its_default_stereotypes;	// unicode
 QStringList BrowserUseCase::relations_default_stereotypes[UmlRelations];	// unicode
 
@@ -77,6 +80,12 @@ BrowserUseCase::BrowserUseCase(const BrowserUseCase * model,
   continuation_color = model->continuation_color;
   usecase_color = model->usecase_color;
   package_color = model->package_color;
+  state_color = model->state_color;
+  stateaction_color = model->stateaction_color;
+  activity_color = model->activity_color;
+  activityregion_color = model->activityregion_color;
+  activityaction_color = model->activityaction_color;
+  parameterpin_color = model->parameterpin_color;
   
   is_modified = true;
 }
@@ -104,7 +113,13 @@ void BrowserUseCase::make() {
   usecase_color = UmlDefaultColor;
   package_color = UmlDefaultColor;
   fragment_color = UmlDefaultColor;
-  subject_color = UmlDefaultColor;	   
+  subject_color = UmlDefaultColor;
+  state_color = UmlDefaultColor;
+  stateaction_color = UmlDefaultColor;
+  activity_color = UmlDefaultColor;
+  activityregion_color = UmlDefaultColor;
+  activityaction_color = UmlDefaultColor;
+  parameterpin_color = UmlDefaultColor;   
 }
 
 BrowserNode * BrowserUseCase::duplicate(BrowserNode * p, QString name) {
@@ -173,6 +188,12 @@ void BrowserUseCase::menu() {
 		     "to add an <em>actor</em>");
       m.setWhatsThis(m.insertItem("New class", 5),
 		     "to add a <em>class</em>");
+      m.setWhatsThis(m.insertItem("New class instance", 14),
+		     "to add a <em>class instance</em>");
+      m.setWhatsThis(m.insertItem("New state machine", 15),
+		     "to add a <em>state machine</em>");
+      m.setWhatsThis(m.insertItem("New activity", 16),
+		     "to add an <em>activity</em>");
       m.insertSeparator();
     }
     if (!is_edited) {
@@ -243,13 +264,34 @@ Do not undelete its sub items");
 void BrowserUseCase::exec_menu_choice(int rank, BrowserNode * item_above) {
   switch (rank) {
   case 0:
-    add_use_case_diagram();
+    {
+      BrowserUseCaseDiagram * d = 
+	BrowserUseCaseDiagram::add_use_case_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 1:
-    add_sequence_diagram();
+    {
+      BrowserSeqDiagram * d = 
+	BrowserSeqDiagram::add_sequence_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 2:
-    add_collaboration_diagram();
+    {
+      BrowserColDiagram * d = 
+	BrowserColDiagram::add_collaboration_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 3:
     {
@@ -284,12 +326,14 @@ void BrowserUseCase::exec_menu_choice(int rank, BrowserNode * item_above) {
   case 7:
     {
       QArray<StateSpec> st;
-      QArray<ColorSpec> co(7);
+      QArray<ColorSpec> co(13);
       
       usecasediagram_settings.complete(st, FALSE);
       sequencediagram_settings.complete(st, FALSE);
       collaborationdiagram_settings.complete(st, FALSE);
       objectdiagram_settings.complete(st, FALSE);
+      statediagram_settings.complete(st, FALSE);
+      activitydiagram_settings.complete(st, FALSE);
       
       co[0].set("note color", &note_color);
       co[1].set("use case color", &usecase_color);
@@ -298,6 +342,12 @@ void BrowserUseCase::exec_menu_choice(int rank, BrowserNode * item_above) {
       co[4].set("subject color", &subject_color);
       co[5].set("duration color", &duration_color);
       co[6].set("continuation color", &continuation_color);
+      co[7].set("state color", &state_color);
+      co[8].set("state action color", &stateaction_color);
+      co[9].set("activity color", &activity_color);
+      co[10].set("activity region color", &activityregion_color);
+      co[11].set("activity action color", &activityaction_color);
+      co[12].set("parameter and pin color", &parameterpin_color);
 
       SettingsDialog dialog(&st, &co, FALSE, FALSE);
       
@@ -325,8 +375,41 @@ void BrowserUseCase::exec_menu_choice(int rank, BrowserNode * item_above) {
     ReferenceDialog::show(this);
     return;
   case 13:
-    add_object_diagram();
+    {
+      BrowserObjectDiagram * d = 
+	BrowserObjectDiagram::add_object_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
+  case 14:
+    {
+      BrowserClassInstance * c = 
+	BrowserClassInstance::add_classinstance(this);
+      
+      if (c != 0)
+	c->select_in_browser();
+    }
+    return; // package_modified called
+  case 15:
+    {
+      BrowserState * st = BrowserState::add_state(this, (bool) TRUE);
+      
+      if (st == 0)
+	return;
+      st->select_in_browser();
+    }
+    break;
+  case 16:
+    {
+      BrowserActivity * a = BrowserActivity::add_activity(this);
+      
+      if (a != 0)
+	a->select_in_browser();
+    }
+    return; // package_modified called
   default:
     if (rank >= 100)
       ToolCom::run(Tool::command(rank - 100), this);
@@ -357,6 +440,12 @@ void BrowserUseCase::apply_shortcut(QString s) {
 	choice = 4;
       else if (s == "New class")
 	choice = 5;
+      else if (s == "New class instance")
+	choice = 14;
+      else if (s == "New state machine")
+	choice = 15;
+      else if (s == "New activity")
+	choice = 16;
     }
     if (!is_edited) {
       if (s == "Edit")
@@ -386,26 +475,6 @@ void BrowserUseCase::apply_shortcut(QString s) {
   exec_menu_choice(choice, 0);
 }
 
-void BrowserUseCase::add_sequence_diagram() {
-  (new BrowserSeqDiagram(child_random_name("Sequence Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCase::add_collaboration_diagram() {
-  (new BrowserColDiagram(child_random_name("Collaboration Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCase::add_object_diagram() {
-  (new BrowserObjectDiagram(child_random_name("Object Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCase::add_use_case_diagram() {
-  (new BrowserUseCaseDiagram(child_random_name("Use Case Diagram"), this))
-    ->select_in_browser();
-}
-
 BrowserNodeList & BrowserUseCase::instances(BrowserNodeList & result) {
   IdIterator<BrowserUseCase> it(all);
   
@@ -460,10 +529,21 @@ void BrowserUseCase::open(bool force_edit) {
     def->edit();
 }
 
+void BrowserUseCase::modified() {
+  repaint();
+  ((BrowserNode *) parent())->modified();
+  // to update drawing
+  def->modified();
+}
+
 UmlCode BrowserUseCase::get_type() const {
   return UmlUseCase;
 }
 
+int BrowserUseCase::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserUseCase::get_data() const {
   return def;
 }
@@ -488,6 +568,16 @@ void BrowserUseCase::get_usecasediagramsettings(UseCaseDiagramSettings & r) cons
     ((BrowserNode *) parent())->get_usecasediagramsettings(r);
 }
 
+void BrowserUseCase::get_statediagramsettings(StateDiagramSettings & r) const {
+  if (! statediagram_settings.complete(r))
+    ((BrowserNode *) parent())->get_statediagramsettings(r);
+}
+
+void BrowserUseCase::get_activitydiagramsettings(ActivityDiagramSettings & r) const {
+  if (! activitydiagram_settings.complete(r))
+    ((BrowserNode *) parent())->get_activitydiagramsettings(r);
+}
+
 UmlColor BrowserUseCase::get_color(UmlCode who) const {
   UmlColor c;
   
@@ -510,6 +600,27 @@ UmlColor BrowserUseCase::get_color(UmlCode who) const {
   case UmlContinuation:
     c = continuation_color;
     break;
+  case UmlState:
+    c = state_color;
+    break;
+  case UmlStateAction:
+    c = stateaction_color;
+    break;
+  case UmlActivity:
+    c = activity_color;
+    break;
+  case UmlInterruptibleActivityRegion:
+  case UmlExpansionRegion:
+    c = activityregion_color;
+    break;
+  case UmlActivityAction:
+    c = activityaction_color;
+    break;
+  case UmlExpansionNode:
+  case UmlParameter:
+  case UmlActivityPin:
+    c = parameterpin_color;
+    break;
   default:	// UmlDuration
     c = duration_color;
   }
@@ -529,6 +640,12 @@ bool BrowserUseCase::get_auto_label_position(UmlCode who) const {
   case UmlObjectDiagram:
     v = objectdiagram_settings.auto_label_position;
     break;
+  case UmlStateDiagram:
+    v = statediagram_settings.auto_label_position;
+    break;
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.auto_label_position;
+    break;
   default:
     // error
     return FALSE;
@@ -544,6 +661,97 @@ bool BrowserUseCase::get_auto_label_position(UmlCode who) const {
   }
 }
 
+
+
+bool BrowserUseCase::get_write_label_horizontally(UmlCode who) const {
+  Uml3States v;
+  
+  switch (who) {
+  case UmlStateDiagram:
+    v = statediagram_settings.write_label_horizontally;
+    break;
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.write_label_horizontally;
+    break;
+  default:
+    // error
+    return FALSE;
+  }
+  
+  switch (v) {
+  case UmlYes:
+    return TRUE;
+  case UmlNo:
+    return FALSE;
+  default:
+    return ((BrowserNode *) parent())->get_write_label_horizontally(who);
+  }
+}
+
+bool BrowserUseCase::get_show_trans_definition(UmlCode who) const {
+  Uml3States v;
+  
+  switch (who) {
+  case UmlStateDiagram:
+    v = statediagram_settings.show_trans_definition;
+    break;
+  default:
+    // error
+    return FALSE;
+  }
+  
+  switch (v) {
+  case UmlYes:
+    return TRUE;
+  case UmlNo:
+    return FALSE;
+  default:
+    return ((BrowserNode *) parent())->get_show_trans_definition(who);
+  }
+}
+
+bool BrowserUseCase::get_show_opaque_action_definition(UmlCode who) const {
+  Uml3States v;
+  
+  switch (who) {
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.show_opaque_action_definition;
+    break;
+  default:
+    // error
+    return FALSE;
+  }
+  
+  switch (v) {
+  case UmlYes:
+    return TRUE;
+  case UmlNo:
+    return FALSE;
+  default:
+    return ((BrowserNode *) parent())->get_show_opaque_action_definition(who);
+  }
+}
+
+DrawingLanguage BrowserUseCase::get_language(UmlCode who) const {
+  DrawingLanguage v;
+  
+  switch (who) {
+  case UmlStateDiagram:
+    v = statediagram_settings.statedrawingsettings.drawing_language;
+    break;
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.activitydrawingsettings.drawing_language;
+    break;
+  default:
+    // error
+    return UmlView;
+  }
+  
+  return (v != DefaultDrawingLanguage)
+    ? v
+    : ((BrowserNode *) parent())->get_language(who);
+}
+
 BrowserNode * BrowserUseCase::get_associated() const {
   return associated_diagram;
 }
@@ -614,6 +822,9 @@ bool BrowserUseCase::tool_cmd(ToolCom * com, const char * args) {
 	  else
 	    (BrowserClass::add_class(this, args))->write_id(com);
 	  break;
+	case UmlClassInstance:
+	  BrowserClassInstance::add_from_tool(this, com, args);
+	  break;
 	case UmlSimpleRelations:
 	  {
 	    UmlCode c;
@@ -640,6 +851,18 @@ bool BrowserUseCase::tool_cmd(ToolCom * com, const char * args) {
 	    }
 	  }
 	  break;
+	case UmlState:
+	  if (wrong_child_name(args, UmlState, TRUE, FALSE))
+	    ok = FALSE;
+	  else
+	    (BrowserState::add_state(this, args))->write_id(com);
+	  break;
+	case UmlActivity:
+	  if (wrong_child_name(args, UmlActivity, TRUE, FALSE))
+	    ok = FALSE;
+	  else
+	    (BrowserActivity::add_activity(this, args))->write_id(com);
+	  break;
 	default:
 	  ok = FALSE;
 	}
@@ -672,6 +895,8 @@ void BrowserUseCase::DragMoveEvent(QDragMoveEvent * e) {
       UmlDrag::canDecode(e, UmlSeqDiagram) ||
       UmlDrag::canDecode(e, UmlColDiagram) ||
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
+      UmlDrag::canDecode(e, UmlState) ||
+      UmlDrag::canDecode(e, UmlActivity) ||
       UmlDrag::canDecode(e, BrowserSimpleRelation::drag_key(this)))
     e->accept();
   else
@@ -685,6 +910,9 @@ void BrowserUseCase::DragMoveInsideEvent(QDragMoveEvent * e) {
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
       UmlDrag::canDecode(e, UmlUseCase) ||
       UmlDrag::canDecode(e, UmlClass) ||
+      UmlDrag::canDecode(e, UmlClassInstance) ||
+      UmlDrag::canDecode(e, UmlState) ||
+      UmlDrag::canDecode(e, UmlActivity) ||
       UmlDrag::canDecode(e, BrowserSimpleRelation::drag_key(this)))
     e->accept();
   else
@@ -703,6 +931,9 @@ bool BrowserUseCase::may_contains_them(const QList<BrowserNode> & l,
     case UmlObjectDiagram:
     case UmlUseCase:
     case UmlClass:
+    case UmlClassInstance:
+    case UmlState:
+    case UmlActivity:
       break;
     default:
       if (!IsaSimpleRelation(it.current()->get_type()) ||
@@ -733,7 +964,10 @@ void BrowserUseCase::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
        ((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlUseCase)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlClass)) != 0) ||
-       ((bn = UmlDrag::decode(e, BrowserSimpleRelation::drag_key(this))) != 0)) &&
+       ((bn = UmlDrag::decode(e, UmlClassInstance)) != 0) ||
+       ((bn = UmlDrag::decode(e, BrowserSimpleRelation::drag_key(this))) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlState)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlActivity)) != 0)) &&
       (bn != after) && (bn != this)) {
     if (may_contains(bn, bn->get_type() == UmlUseCase)) {
       BrowserNode * x = this;
@@ -771,8 +1005,10 @@ void BrowserUseCase::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
 	x->insertItem(bn);
       }
       x->package_modified();
-      if (old_parent != x)
+      if (old_parent != x) {
 	old_parent->package_modified();
+	bn->modified();
+      }
     }
     else if (after == 0)
       ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -792,7 +1028,7 @@ const QStringList & BrowserUseCase::default_stereotypes() {
   return its_default_stereotypes;
 }
 
-const QStringList & BrowserUseCase::default_stereotypes(UmlCode arrow) {
+const QStringList & BrowserUseCase::default_stereotypes(UmlCode arrow, const BrowserNode *) const {
   if (IsaRelation(arrow))
     return relations_default_stereotypes[arrow];
   
@@ -823,6 +1059,11 @@ void BrowserUseCase::init()
   
   relations_default_stereotypes[UmlDependency].append("include");
   relations_default_stereotypes[UmlDependency].append("extend");
+  
+  relations_default_stereotypes[UmlGeneralisation].append("{complete,disjoint}");
+  relations_default_stereotypes[UmlGeneralisation].append("{incomplete,disjoint}");
+  relations_default_stereotypes[UmlGeneralisation].append("{complete,overlapping}");
+  relations_default_stereotypes[UmlGeneralisation].append("{incomplete,overlapping}");
 }
 
 void BrowserUseCase::save_stereotypes(QTextStream & st)
@@ -843,6 +1084,13 @@ void BrowserUseCase::read_stereotypes(char * & st, char * & k)
   }
   else
     init();
+  
+  if (!strcmp(k, "actor_stereotypes")) {
+    // old release
+    QStringList dummy;
+    read_unicode_string_list(dummy, st);
+    k = read_keyword(st);
+  }
 }
 
 void BrowserUseCase::save(QTextStream & st, bool ref, QString & warning) {
@@ -866,6 +1114,12 @@ void BrowserUseCase::save(QTextStream & st, bool ref, QString & warning) {
     nl_indent(st);
     st << "//object diagram settings";
     objectdiagram_settings.save(st);
+    nl_indent(st);
+    st << "//state diagram settings";
+    statediagram_settings.save(st);
+    nl_indent(st);
+    st << "//activity diagram settings";
+    activitydiagram_settings.save(st);
   
     bool nl = FALSE;
     
@@ -876,6 +1130,12 @@ void BrowserUseCase::save(QTextStream & st, bool ref, QString & warning) {
     save_color(st, "package_color", package_color, nl);
     save_color(st, "fragment_color", fragment_color, nl);
     save_color(st, "subject_color", subject_color, nl);
+    save_color(st, "state_color", state_color, nl);
+    save_color(st, "stateaction_color", stateaction_color, nl);
+    save_color(st, "activity_color", activity_color, nl);
+    save_color(st, "activityregion_color", activityregion_color, nl);
+    save_color(st, "activityaction_color", activityaction_color, nl);
+    save_color(st, "parameterpin_color", parameterpin_color, nl);
   
     if (associated_diagram != 0) {
       nl_indent(st);
@@ -945,13 +1205,22 @@ BrowserUseCase * BrowserUseCase::read(char * & st, char * k,
     result = all[id];
     
     if (result == 0)
-      result = 
-	new BrowserUseCase(read_string(st), parent, id);
+      result = new BrowserUseCase(read_string(st), parent, id);
+    else if (result->is_defined) {
+      BrowserUseCase * already_exist = result;
+
+      result = new BrowserUseCase(read_string(st), parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("use case", result);
+    }
     else {
       result->set_parent(parent);
       result->set_name(read_string(st));
     }
     
+    result->is_defined = TRUE;
+
     result->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && result->is_api_base();
     
@@ -962,6 +1231,11 @@ BrowserUseCase * BrowserUseCase::read(char * & st, char * k,
     result->collaborationdiagram_settings.read(st, k);	// updates k
     if (read_file_format() >= 25)
       result->objectdiagram_settings.read(st, k);	// updates k
+    if (read_file_format() >= 43) {
+      result->statediagram_settings.read(st, k);		// updates k
+      result->activitydiagram_settings.read(st, k);		// updates k
+    }
+    read_color(st, "duration", result->duration_color, k);	// old, updates k
     read_color(st, "duration_color", result->duration_color, k);	// updates k
     read_color(st, "continuation_color", result->continuation_color, k);	// updates k
     read_color(st, "note_color", result->note_color, k);	// updates k
@@ -969,7 +1243,14 @@ BrowserUseCase * BrowserUseCase::read(char * & st, char * k,
     read_color(st, "package_color", result->package_color, k);	// updates k
     read_color(st, "fragment_color", result->fragment_color, k);	// updates k
     read_color(st, "subject_color", result->subject_color, k);	// updates k
-    
+    if (read_file_format() >= 43) {
+      read_color(st, "state_color", result->state_color, k);		// updates k
+      read_color(st, "stateaction_color", result->stateaction_color, k);		// updates k
+      read_color(st, "activity_color", result->activity_color, k);		// updates k
+      read_color(st, "activityregion_color", result->activityregion_color, k);		// updates k
+      read_color(st, "activityaction_color", result->activityaction_color, k);		// updates k
+      read_color(st, "parameterpin_color", result->parameterpin_color, k);		// updates k
+    }    
     if (!strcmp(k, "associated_usecase_diagram")) {
       // old format
       result->set_associated_diagram(BrowserUseCaseDiagram::read_ref(st, "usecasediagram_ref"),
@@ -987,11 +1268,14 @@ BrowserUseCase * BrowserUseCase::read(char * & st, char * k,
     if (strcmp(k, "end")) {
       while (BrowserUseCase::read(st, k, result) ||
 	     BrowserClass::read(st, k, result) ||
+	     BrowserClassInstance::read(st, k, result) ||
 	     BrowserObjectDiagram::read(st, k, result) ||
 	     BrowserSeqDiagram::read(st, k, result) ||
 	     BrowserColDiagram::read(st, k, result) ||
 	     BrowserUseCaseDiagram::read(st, k, result) ||
-	     BrowserSimpleRelation::read(st, k, result))
+	     BrowserSimpleRelation::read(st, k, result) ||
+	     BrowserState::read(st, k, result) ||
+	     BrowserActivity::read(st, k, result))
 	k = read_keyword(st);
       
       if (strcmp(k, "end"))
@@ -1012,6 +1296,7 @@ BrowserNode * BrowserUseCase::get_it(const char * k, int id)
   BrowserNode * r;
   
   if ((r = BrowserUseCaseDiagram::get_it(k, id)) == 0)
+    // state and activity managed in class view
     r = BrowserSimpleRelation::get_it(k, id);
   
   return r;
