@@ -49,6 +49,8 @@ class BrowserUseCase : public BrowserNode, public Labeled<BrowserUseCase> {
     SequenceDiagramSettings sequencediagram_settings;
     CollaborationDiagramSettings collaborationdiagram_settings;
     ObjectDiagramSettings objectdiagram_settings;
+    StateDiagramSettings statediagram_settings;
+    ActivityDiagramSettings activitydiagram_settings;
     // note : does not have class settings because classes defined under
     //	      a use case cannot be generated
     UmlColor note_color;
@@ -58,6 +60,12 @@ class BrowserUseCase : public BrowserNode, public Labeled<BrowserUseCase> {
     UmlColor continuation_color;
     UmlColor usecase_color;
     UmlColor package_color;
+    UmlColor state_color;
+    UmlColor stateaction_color;
+    UmlColor activity_color;
+    UmlColor activityregion_color;
+    UmlColor activityaction_color;
+    UmlColor parameterpin_color;
   
     BrowserUseCase(int id);
     void make();
@@ -68,11 +76,6 @@ class BrowserUseCase : public BrowserNode, public Labeled<BrowserUseCase> {
     BrowserUseCase(const BrowserUseCase * model, BrowserNode * p);
     virtual ~BrowserUseCase();
     
-    void add_sequence_diagram();
-    void add_collaboration_diagram();
-    void add_use_case_diagram();
-    void add_object_diagram();
-    
     virtual BrowserNode * get_associated() const;
     void set_associated_diagram(BrowserUseCaseDiagram *, bool on_read = FALSE);
     
@@ -83,6 +86,8 @@ class BrowserUseCase : public BrowserNode, public Labeled<BrowserUseCase> {
     virtual void apply_shortcut(QString s);
     virtual void open(bool force_edit);
     virtual UmlCode get_type() const;
+    virtual int get_identifier() const;
+    virtual void modified();
     virtual bool may_contains_them(const QList<BrowserNode> &,
 				   bool & duplicable) const;
     virtual BasicData * get_data() const;
@@ -90,9 +95,15 @@ class BrowserUseCase : public BrowserNode, public Labeled<BrowserUseCase> {
     virtual void get_sequencediagramsettings(SequenceDiagramSettings &) const;
     virtual void get_collaborationdiagramsettings(CollaborationDiagramSettings &) const;
     virtual void get_objectdiagramsettings(ObjectDiagramSettings &) const;
+    virtual void get_statediagramsettings(StateDiagramSettings &) const;
+    virtual void get_activitydiagramsettings(ActivityDiagramSettings &) const;
     virtual UmlColor get_color(UmlCode) const;
     virtual bool get_auto_label_position(UmlCode who) const;
-    virtual const QStringList & default_stereotypes(UmlCode arrow);
+    virtual bool get_write_label_horizontally(UmlCode who) const;
+    virtual bool get_show_trans_definition(UmlCode who) const;
+    virtual bool get_show_opaque_action_definition(UmlCode who) const;
+    virtual DrawingLanguage get_language(UmlCode who) const;
+    virtual const QStringList & default_stereotypes(UmlCode, const BrowserNode *) const; // non class rel
     virtual void on_delete();
     virtual bool tool_cmd(ToolCom * com, const char * args);
     virtual void save(QTextStream &, bool ref, QString & warning);
