@@ -35,7 +35,7 @@
 #include "SimpleData.h"
 #include "UseCaseDiagramWindow.h"
 #include "BrowserUseCaseView.h"
-#include "DiagramView.h"
+#include "UseCaseDiagramView.h"
 #include "UmlPixmap.h"
 #include "SettingsDialog.h"
 #include "myio.h"
@@ -63,7 +63,7 @@ BrowserUseCaseDiagram::BrowserUseCaseDiagram(int id)
 }
 
 BrowserUseCaseDiagram::BrowserUseCaseDiagram(BrowserUseCaseDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->child_random_name("Diagram"), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -117,6 +117,17 @@ void BrowserUseCaseDiagram::make() {
   subject_color = UmlDefaultColor;
 }
 
+BrowserUseCaseDiagram * BrowserUseCaseDiagram::add_use_case_diagram(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter use case diagram's name : ",
+				      UmlUseCaseDiagram, TRUE, FALSE))
+    return new BrowserUseCaseDiagram(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserUseCaseDiagram::set_name(const char * s) {
   BrowserDiagram::set_name(s);
   if (window != 0)
@@ -263,11 +274,13 @@ void BrowserUseCaseDiagram::exec_menu_choice(int rank,
     return;
   case 3:
     {
-      BrowserUseCaseDiagram * ucd
-	= new BrowserUseCaseDiagram(this, (BrowserNode *) parent());
+      QString name;
       
-      ucd->select_in_browser();
-      ucd->edit("Use Case diagram", its_default_stereotypes);
+      if (((BrowserNode *)parent())->enter_child_name(name, "enter use case diagram's name : ",
+						      UmlUseCaseDiagram, TRUE, FALSE))
+	duplicate((BrowserNode *) parent(), name)->select_in_browser();
+      else
+	return;
     }
     break;
   case 4:
@@ -364,6 +377,10 @@ UmlCode BrowserUseCaseDiagram::get_type() const {
   return UmlUseCaseDiagram;
 }
 
+int BrowserUseCaseDiagram::get_identifier() const {
+  return get_ident();
+}
+
 void BrowserUseCaseDiagram::get_usecasediagramsettings(UseCaseDiagramSettings & r) const {
   if (! settings.complete(r))
     ((BrowserNode *) parent())->get_usecasediagramsettings(r);
@@ -471,20 +488,23 @@ bool BrowserUseCaseDiagram::tool_cmd(ToolCom * com, const char * args) {
       w->close(TRUE);
     }
     return TRUE;
+  case sideCmd:
+    if (window != 0)
+      ((UseCaseDiagramView *) window->get_view())->send(com);
+    else {
+      UseCaseDiagramWindow * w = new UseCaseDiagramWindow(full_name(), this);
+      
+      ((UseCaseDiagramView *) w->get_view())->send(com);
+      w->dont_save();
+      w->close(TRUE);
+    }
+    return TRUE;
   default:
     return (def->tool_cmd(com, args, this, comment) ||
 	    BrowserNode::tool_cmd(com, args));
   }
 }
 
-void BrowserUseCaseDiagram::DragMoveEvent(QDragMoveEvent * e) {
-  ((BrowserNode *) parent())->DragMoveInsideEvent(e);
-}
-
-void BrowserUseCaseDiagram::DropEvent(QDropEvent * e) {
-  ((BrowserNode *) parent())->DropAfterEvent(e, this);
-}
-
 void BrowserUseCaseDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -576,11 +596,21 @@ BrowserUseCaseDiagram * BrowserUseCaseDiagram::read(char * & st, char * k,
     
     if ((r = (BrowserUseCaseDiagram *) all[id]) == 0)
       r = new BrowserUseCaseDiagram(s, parent, id);
+    else if (r->is_defined) {
+      BrowserUseCaseDiagram * already_exist = r;
+
+      r = new BrowserUseCaseDiagram(s, parent, id);
+
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("use case diagram", r);
+    }
     else {
       r->set_parent(parent);
       r->set_name(s);
     }
     
+    r->is_defined = TRUE;
+
     r->is_read_only = !in_import() && read_only_file() || 
       (user_id() != 0) && r->is_api_base();
     
