@@ -38,6 +38,9 @@
 #include "BrowserUseCaseDiagram.h"
 #include "BrowserObjectDiagram.h"
 #include "BrowserClass.h"
+#include "BrowserClassInstance.h"
+#include "BrowserState.h"
+#include "BrowserActivity.h"
 #include "SettingsDialog.h"
 #include "UmlPixmap.h"
 #include "UmlDrag.h"
@@ -48,7 +51,7 @@
 #include "DialogUtil.h"
 #include "mu.h"
 
-IdDict<BrowserUseCaseView> BrowserUseCaseView::all;
+IdDict<BrowserUseCaseView> BrowserUseCaseView::all(__FILE__);
 QStringList BrowserUseCaseView::its_default_stereotypes;	// unicode
 
 BrowserUseCaseView::BrowserUseCaseView(QString s, BrowserNode * p, int id)
@@ -64,6 +67,12 @@ BrowserUseCaseView::BrowserUseCaseView(QString s, BrowserNode * p, int id)
   fragment_color = UmlDefaultColor;
   subject_color = UmlDefaultColor;
   classinstance_color = UmlDefaultColor;
+  state_color = UmlDefaultColor;
+  stateaction_color = UmlDefaultColor;
+  activity_color = UmlDefaultColor;
+  activityregion_color = UmlDefaultColor;
+  activityaction_color = UmlDefaultColor;
+  parameterpin_color = UmlDefaultColor;
 }
 
 BrowserUseCaseView::BrowserUseCaseView(const BrowserUseCaseView * model,
@@ -84,6 +93,12 @@ BrowserUseCaseView::BrowserUseCaseView(const BrowserUseCaseView * model,
   usecase_color = model->usecase_color;
   package_color = model->package_color;
   classinstance_color = model->classinstance_color;
+  state_color = model->state_color;
+  stateaction_color = model->stateaction_color;
+  activity_color = model->activity_color;
+  activityregion_color = model->activityregion_color;
+  activityaction_color = model->activityaction_color;
+  parameterpin_color = model->parameterpin_color;
 }
 
 BrowserUseCaseView::~BrowserUseCaseView() {
@@ -99,6 +114,17 @@ BrowserNode * BrowserUseCaseView::duplicate(BrowserNode * p, QString name) {
   return result;
 }
 
+BrowserUseCaseView* BrowserUseCaseView::add_use_case_view(BrowserNode * future_parent)
+{
+  QString name;
+  
+  if (future_parent->enter_child_name(name, "enter use case view's name : ",
+				      UmlUseCaseView, TRUE, FALSE))
+    return new BrowserUseCaseView(name, future_parent);
+  else
+    return 0;
+}
+
 void BrowserUseCaseView::clear(bool old)
 {
   all.clear(old);
@@ -143,6 +169,12 @@ void BrowserUseCaseView::menu() {
 		     "to add an <em>actor</em>");
       m.setWhatsThis(m.insertItem("New class", 5),
 		     "to add a <em>class</em>");
+      m.setWhatsThis(m.insertItem("New class instance", 14),
+		     "to add a <em>class instance</em>");
+      m.setWhatsThis(m.insertItem("New state machine", 15),
+		     "to add a <em>state machine</em>");
+      m.setWhatsThis(m.insertItem("New activity", 16),
+		     "to add an <em>activity</em>");
       m.insertSeparator();
       m.setWhatsThis(m.insertItem("New sub use case view", 6),
 		     "to add a sub <em>use case view</em>");
@@ -184,20 +216,42 @@ Do not undelete its sub items");
 void BrowserUseCaseView::exec_menu_choice(int rank) {
   switch (rank) {
   case 0:
-    add_use_case_diagram();
+    {
+      BrowserUseCaseDiagram * d = 
+	BrowserUseCaseDiagram::add_use_case_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 1:
-    add_sequence_diagram();
+    {
+      BrowserSeqDiagram * d = 
+	BrowserSeqDiagram::add_sequence_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 2:
-    add_collaboration_diagram();
+    {
+      BrowserColDiagram * d = 
+	BrowserColDiagram::add_collaboration_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
   case 3:
     {
       BrowserUseCase *  uc = BrowserUseCase::add_use_case(this);
       
-      if (uc != 0)
-	uc->select_in_browser();
+      if (uc == 0)
+	return;
+      uc->select_in_browser();
     }
     break;
   case 4:
@@ -220,7 +274,14 @@ void BrowserUseCaseView::exec_menu_choice(int rank) {
     }
     break;
   case 6:
-    add_use_cases_list();
+    {
+      BrowserUseCaseView * v = 
+	BrowserUseCaseView::add_use_case_view(this);
+      
+      if (v == 0)
+	return;
+      v->select_in_browser();
+    }
     break;
   case 8:
     edit("Use case view", its_default_stereotypes);
@@ -228,12 +289,14 @@ void BrowserUseCaseView::exec_menu_choice(int rank) {
   case 9:
     {
       QArray<StateSpec> st;
-      QArray<ColorSpec> co(8);
+      QArray<ColorSpec> co(14);
       
       usecasediagram_settings.complete(st, FALSE);
       objectdiagram_settings.complete(st, FALSE);
       sequencediagram_settings.complete(st, FALSE);
       collaborationdiagram_settings.complete(st, FALSE);
+      statediagram_settings.complete(st, FALSE);
+      activitydiagram_settings.complete(st, FALSE);
       
       co[0].set("note color", &note_color);
       co[1].set("use case color", &usecase_color);
@@ -243,6 +306,12 @@ void BrowserUseCaseView::exec_menu_choice(int rank) {
       co[5].set("duration color", &duration_color);
       co[6].set("continuation color", &continuation_color);
       co[7].set("class instance color", &usecase_color);
+      co[8].set("state color", &state_color);
+      co[9].set("state action color", &stateaction_color);
+      co[10].set("activity color", &activity_color);
+      co[11].set("activity region color", &activityregion_color);
+      co[12].set("activity action color", &activityaction_color);
+      co[13].set("parameter and pin color", &parameterpin_color);
 
       SettingsDialog dialog(&st, &co, FALSE, FALSE);
       
@@ -262,8 +331,41 @@ void BrowserUseCaseView::exec_menu_choice(int rank) {
     undelete(TRUE);
     break;
   case 13:
-    add_object_diagram();
+    {
+      BrowserObjectDiagram * d = 
+	BrowserObjectDiagram::add_object_diagram(this);
+      
+      if (d == 0)
+	return;
+      d->select_in_browser();
+    }
     break;
+  case 14:
+    {
+      BrowserClassInstance * c = 
+	BrowserClassInstance::add_classinstance(this);
+      
+      if (c != 0)
+	c->select_in_browser();
+    }
+    return; // package_modified called
+  case 15:
+    {
+      BrowserState * st = BrowserState::add_state(this, (bool) TRUE);
+      
+      if (st == 0)
+	return;
+      st->select_in_browser();
+    }
+    break;
+  case 16:
+    {
+      BrowserActivity * a = BrowserActivity::add_activity(this);
+      
+      if (a != 0)
+	a->select_in_browser();
+    }
+    return; // package_modified called
   default:
     if (rank >= 100)
       ToolCom::run(Tool::command(rank - 100), this);
@@ -294,8 +396,14 @@ void BrowserUseCaseView::apply_shortcut(QString s) {
 	choice = 4;
       else if (s == "New class")
 	choice = 5;
+      else if (s == "New class instance")
+	choice = 14;
       else if (s == "New use case view")
 	choice = 6;
+      else if (s == "New state machine")
+	choice = 15;
+      else if (s == "New activity")
+	choice = 16;
     }
     if (!is_edited) {
       if (s == "Edit")
@@ -323,35 +431,19 @@ void BrowserUseCaseView::apply_shortcut(QString s) {
   exec_menu_choice(choice);
 }
 
-void BrowserUseCaseView::add_sequence_diagram() {
-  (new BrowserSeqDiagram(child_random_name("Sequence Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCaseView::add_collaboration_diagram() {
-  (new BrowserColDiagram(child_random_name("Collaboration Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCaseView::add_object_diagram() {
-  (new BrowserObjectDiagram(child_random_name("Object Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCaseView::add_use_case_diagram() {
-  (new BrowserUseCaseDiagram(child_random_name("Use Case Diagram"), this))
-    ->select_in_browser();
-}
-
-void BrowserUseCaseView::add_use_cases_list() {
-  (new BrowserUseCaseView(child_random_name("Use Case View"), this))
-    ->select_in_browser();
+void BrowserUseCaseView::open(bool) {
+  if (!is_edited)
+    edit("Use Case view", its_default_stereotypes);
 }
 
 UmlCode BrowserUseCaseView::get_type() const {
   return UmlUseCaseView;
 }
 
+int BrowserUseCaseView::get_identifier() const {
+  return get_ident();
+}
+
 BasicData * BrowserUseCaseView::get_data() const {
   return def;
 }
@@ -376,6 +468,26 @@ void BrowserUseCaseView::get_objectdiagramsettings(ObjectDiagramSettings & r) co
     ((BrowserNode *) parent())->get_objectdiagramsettings(r);
 }
 
+void BrowserUseCaseView::get_statediagramsettings(StateDiagramSettings & r) const {
+  if (! statediagram_settings.complete(r))
+    ((BrowserNode *) parent())->get_statediagramsettings(r);
+}
+
+void BrowserUseCaseView::get_statedrawingsettings(StateDrawingSettings & result) const {
+  if (!statediagram_settings.statedrawingsettings.complete(result))
+    ((BrowserNode *) parent())->get_statedrawingsettings(result);
+}
+
+void BrowserUseCaseView::get_activitydiagramsettings(ActivityDiagramSettings & r) const {
+  if (! activitydiagram_settings.complete(r))
+    ((BrowserNode *) parent())->get_activitydiagramsettings(r);
+}
+
+void BrowserUseCaseView::get_activitydrawingsettings(ActivityDrawingSettings & result) const {
+  if (!activitydiagram_settings.activitydrawingsettings.complete(result))
+    ((BrowserNode *) parent())->get_activitydrawingsettings(result);
+}
+
 UmlColor BrowserUseCaseView::get_color(UmlCode who) const {
   UmlColor c;
   
@@ -401,6 +513,27 @@ UmlColor BrowserUseCaseView::get_color(UmlCode who) const {
   case UmlClass:	// instance
     c = classinstance_color;
     break;
+  case UmlState:
+    c = state_color;
+    break;
+  case UmlStateAction:
+    c = stateaction_color;
+    break;
+  case UmlActivity:
+    c = activity_color;
+    break;
+  case UmlInterruptibleActivityRegion:
+  case UmlExpansionRegion:
+    c = activityregion_color;
+    break;
+  case UmlActivityAction:
+    c = activityaction_color;
+    break;
+  case UmlExpansionNode:
+  case UmlParameter:
+  case UmlActivityPin:
+    c = parameterpin_color;
+    break;
   default:	// UmlDuration
     c = duration_color;
   }
@@ -423,10 +556,16 @@ bool BrowserUseCaseView::get_shadow(UmlCode who) const {
   case UmlColDiagram:
     v = collaborationdiagram_settings.shadow;
     break;
-  default:
-    // UmlObjectDiagram
+  case UmlObjectDiagram:
     v = objectdiagram_settings.shadow;
     break;
+  case UmlStateDiagram:
+    v = statediagram_settings.shadow;
+    break;
+  default:
+    //UmlActivityDiagram
+    v = activitydiagram_settings.shadow;
+    break;
   }
   
   switch (v) {
@@ -452,10 +591,16 @@ bool BrowserUseCaseView::get_draw_all_relations(UmlCode who) const {
   case UmlColDiagram:
     v = collaborationdiagram_settings.draw_all_relations;
     break;
-  default:
-    // UmlObjectDiagram
+  case UmlObjectDiagram:
     v = objectdiagram_settings.draw_all_relations;
     break;
+  case UmlStateDiagram:
+    v = statediagram_settings.draw_all_relations;
+    break;
+  default:
+    //UmlActivityDiagram
+    v = activitydiagram_settings.draw_all_relations;
+    break;
   }
   
   switch (v) {
@@ -499,6 +644,12 @@ bool BrowserUseCaseView::get_auto_label_position(UmlCode who) const {
   case UmlObjectDiagram:
     v = objectdiagram_settings.auto_label_position;
     break;
+  case UmlStateDiagram:
+    v = statediagram_settings.auto_label_position;
+    break;
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.auto_label_position;
+    break;
   default:
     // error
     return FALSE;
@@ -514,6 +665,95 @@ bool BrowserUseCaseView::get_auto_label_position(UmlCode who) const {
   }
 }
 
+bool BrowserUseCaseView::get_write_label_horizontally(UmlCode who) const {
+  Uml3States v;
+  
+  switch (who) {
+  case UmlStateDiagram:
+    v = statediagram_settings.write_label_horizontally;
+    break;
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.write_label_horizontally;
+    break;
+  default:
+    // error
+    return FALSE;
+  }
+  
+  switch (v) {
+  case UmlYes:
+    return TRUE;
+  case UmlNo:
+    return FALSE;
+  default:
+    return ((BrowserNode *) parent())->get_write_label_horizontally(who);
+  }
+}
+
+bool BrowserUseCaseView::get_show_trans_definition(UmlCode who) const {
+  Uml3States v;
+  
+  switch (who) {
+  case UmlStateDiagram:
+    v = statediagram_settings.show_trans_definition;
+    break;
+  default:
+    // error
+    return FALSE;
+  }
+  
+  switch (v) {
+  case UmlYes:
+    return TRUE;
+  case UmlNo:
+    return FALSE;
+  default:
+    return ((BrowserNode *) parent())->get_show_trans_definition(who);
+  }
+}
+
+bool BrowserUseCaseView::get_show_opaque_action_definition(UmlCode who) const {
+  Uml3States v;
+  
+  switch (who) {
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.show_opaque_action_definition;
+    break;
+  default:
+    // error
+    return FALSE;
+  }
+  
+  switch (v) {
+  case UmlYes:
+    return TRUE;
+  case UmlNo:
+    return FALSE;
+  default:
+    return ((BrowserNode *) parent())->get_show_opaque_action_definition(who);
+  }
+}
+
+DrawingLanguage BrowserUseCaseView::get_language(UmlCode who) const {
+  DrawingLanguage v;
+  
+  switch (who) {
+  case UmlStateDiagram:
+    v = statediagram_settings.statedrawingsettings.drawing_language;
+    break;
+  case UmlActivityDiagram:
+    v = activitydiagram_settings.activitydrawingsettings.drawing_language;
+    break;
+  default:
+    // error
+    return UmlView;
+  }
+  
+  return (v != DefaultDrawingLanguage)
+    ? v
+    : ((BrowserNode *) parent())->get_language(who);
+}
+
 bool BrowserUseCaseView::tool_cmd(ToolCom * com, const char * args) {
   switch ((unsigned char) args[-1]) {
   case createCmd:
@@ -566,6 +806,21 @@ bool BrowserUseCaseView::tool_cmd(ToolCom * com, const char * args) {
 	  else
 	    (BrowserClass::add_class(this, args))->write_id(com);
 	  break;
+	case UmlClassInstance:
+	  BrowserClassInstance::add_from_tool(this, com, args);
+	  break;
+	case UmlState:
+	  if (wrong_child_name(args, UmlState, TRUE, FALSE))
+	    ok = FALSE;
+	  else
+	    (BrowserState::add_state(this, args))->write_id(com);
+	  break;
+	case UmlActivity:
+	  if (wrong_child_name(args, UmlActivity, TRUE, FALSE))
+	    ok = FALSE;
+	  else
+	    (BrowserActivity::add_activity(this, args))->write_id(com);
+	  break;
 	default:
 	  ok = FALSE;
 	}
@@ -587,12 +842,15 @@ bool BrowserUseCaseView::tool_cmd(ToolCom * com, const char * args) {
 
 void BrowserUseCaseView::DragMoveEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, UmlClass) ||
+      UmlDrag::canDecode(e, UmlClassInstance) ||
       UmlDrag::canDecode(e, UmlUseCase) ||
       UmlDrag::canDecode(e, UmlUseCaseDiagram) ||
       UmlDrag::canDecode(e, UmlSeqDiagram) ||
       UmlDrag::canDecode(e, UmlColDiagram) ||
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
-      UmlDrag::canDecode(e, UmlUseCaseView))
+      UmlDrag::canDecode(e, UmlUseCaseView) ||
+      UmlDrag::canDecode(e, UmlState) ||
+      UmlDrag::canDecode(e, UmlActivity))
     e->accept();
   else
     ((BrowserNode *) parent())->DragMoveInsideEvent(e);
@@ -600,12 +858,15 @@ void BrowserUseCaseView::DragMoveEvent(QDragMoveEvent * e) {
 
 void BrowserUseCaseView::DragMoveInsideEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, UmlClass) ||
+      UmlDrag::canDecode(e, UmlClassInstance) ||
       UmlDrag::canDecode(e, UmlUseCase) ||
       UmlDrag::canDecode(e, UmlUseCaseDiagram) ||
       UmlDrag::canDecode(e, UmlSeqDiagram) ||
       UmlDrag::canDecode(e, UmlColDiagram) ||
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
-      UmlDrag::canDecode(e, UmlUseCaseView))
+      UmlDrag::canDecode(e, UmlUseCaseView) ||
+      UmlDrag::canDecode(e, UmlState) ||
+      UmlDrag::canDecode(e, UmlActivity))
     e->accept();
   else
     e->ignore();
@@ -621,11 +882,14 @@ bool BrowserUseCaseView::may_contains_them(const QList<BrowserNode> & l,
       duplicable = FALSE;
       // no break
     case UmlClass:
+    case UmlClassInstance:
     case UmlUseCase:
     case UmlUseCaseDiagram:
     case UmlSeqDiagram:
     case UmlColDiagram:
     case UmlObjectDiagram:
+    case UmlState:
+    case UmlActivity:
       if (! may_contains(it.current(), FALSE))
 	return FALSE;
       break;
@@ -654,12 +918,15 @@ void BrowserUseCaseView::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
   BrowserNode * bn;
   
   if ((((bn = UmlDrag::decode(e, UmlClass)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlClassInstance)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlUseCase)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlSeqDiagram)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlColDiagram)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlUseCaseDiagram)) != 0) ||
-       ((bn = UmlDrag::decode(e, UmlUseCaseView)) != 0)) &&
+       ((bn = UmlDrag::decode(e, UmlUseCaseView)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlState)) != 0) ||
+       ((bn = UmlDrag::decode(e, UmlActivity)) != 0)) &&
       (bn != this) && (bn != this)) {
     if (may_contains(bn, bn->get_type() == UmlUseCaseView)) {
       BrowserNode * x = this;
@@ -697,8 +964,10 @@ void BrowserUseCaseView::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
 	x->insertItem(bn);
       }
       x->package_modified();
-      if (old_parent != x)
+      if (old_parent != x) {
 	old_parent->package_modified();
+	bn->modified();
+      }
     }
     else if (after == 0)
       ((BrowserNode *) parent())->DropAfterEvent(e, this);
@@ -750,6 +1019,12 @@ void BrowserUseCaseView::save(QTextStream & st, bool ref, QString & warning) {
     nl_indent(st);
     st << "//object diagram settings";
     objectdiagram_settings.save(st);
+    nl_indent(st);
+    st << "//state diagram settings";
+    statediagram_settings.save(st);
+    nl_indent(st);
+    st << "//activity diagram settings";
+    activitydiagram_settings.save(st);
     
     bool nl = FALSE;
     
@@ -761,6 +1036,12 @@ void BrowserUseCaseView::save(QTextStream & st, bool ref, QString & warning) {
     save_color(st, "fragment_color", fragment_color, nl);
     save_color(st, "subject_color", subject_color, nl);
     save_color(st, "classinstance_color", classinstance_color, nl);
+    save_color(st, "state_color", state_color, nl);
+    save_color(st, "stateaction_color", stateaction_color, nl);
+    save_color(st, "activity_color", activity_color, nl);
+    save_color(st, "activityregion_color", activityregion_color, nl);
+    save_color(st, "activityaction_color", activityaction_color, nl);
+    save_color(st, "parameterpin_color", parameterpin_color, nl);
     
     BrowserNode::save(st);
     
@@ -803,53 +1084,66 @@ BrowserUseCaseView * BrowserUseCaseView::read(char * & st, char * k,
 {
   if (!strcmp(k, "usecaseview")) {
     int id = read_id(st);
-    BrowserUseCaseView * result = all[id];
-    
-    if (result != 0) {
-      msg_critical("Error", "reload not yet implemented");
-      
-      throw 0;
+    BrowserUseCaseView * already_exist = all[id];
+    BrowserUseCaseView * result =
+      new BrowserUseCaseView(read_string(st), parent, id);
+
+    if (already_exist != 0) {
+      already_exist->must_change_id(all);
+      already_exist->unconsistent_fixed("use case views", result);
     }
-    else {
-      result = new BrowserUseCaseView(read_string(st), parent, id);
     
-      result->is_read_only = !in_import() && read_only_file() || 
-	(user_id() != 0) && result->is_api_base();
+    result->is_read_only = !in_import() && read_only_file() || 
+      (user_id() != 0) && result->is_api_base();
       
-      k = read_keyword(st);
+    k = read_keyword(st);
       
-      result->def->read(st, k);						// updates k
-      result->usecasediagram_settings.read(st, k);			// updates k
-      result->sequencediagram_settings.read(st, k);			// updates k
-      result->collaborationdiagram_settings.read(st, k);		// updates k
-      if (read_file_format() >= 25)
-	result->objectdiagram_settings.read(st, k);		// updates k
-      read_color(st, "duration_color", result->duration_color, k);	// updates k
-      read_color(st, "continuation_color", result->continuation_color, k);	// updates k
-      read_color(st, "note_color", result->note_color, k);		// updates k
-      read_color(st, "usecase_color", result->usecase_color, k);	// updates k
-      read_color(st, "package_color", result->package_color, k);	// updates k
-      read_color(st, "fragment_color", result->fragment_color, k);	// updates k
-      read_color(st, "subject_color", result->subject_color, k);	// updates k
-      read_color(st, "classinstance_color", result->classinstance_color, k);	// updates k
-      result->BrowserNode::read(st, k);	// updates k
+    result->def->read(st, k);						// updates k
+    result->usecasediagram_settings.read(st, k);			// updates k
+    result->sequencediagram_settings.read(st, k);			// updates k
+    result->collaborationdiagram_settings.read(st, k);		// updates k
+    if (read_file_format() >= 25)
+      result->objectdiagram_settings.read(st, k);		// updates k
+    if (read_file_format() >= 43) {
+      result->statediagram_settings.read(st, k);		// updates k
+      result->activitydiagram_settings.read(st, k);		// updates k
+    }
+    read_color(st, "duration", result->duration_color, k);	// old, updates k
+    read_color(st, "duration_color", result->duration_color, k);	// updates k
+    read_color(st, "continuation_color", result->continuation_color, k);	// updates k
+    read_color(st, "note_color", result->note_color, k);		// updates k
+    read_color(st, "usecase_color", result->usecase_color, k);	// updates k
+    read_color(st, "package_color", result->package_color, k);	// updates k
+    read_color(st, "fragment_color", result->fragment_color, k);	// updates k
+    read_color(st, "subject_color", result->subject_color, k);	// updates k
+    read_color(st, "classinstance_color", result->classinstance_color, k);	// updates k
+    if (read_file_format() >= 43) {
+      read_color(st, "state_color", result->state_color, k);		// updates k
+      read_color(st, "stateaction_color", result->stateaction_color, k);		// updates k
+      read_color(st, "activity_color", result->activity_color, k);		// updates k
+      read_color(st, "activityregion_color", result->activityregion_color, k);		// updates k
+      read_color(st, "activityaction_color", result->activityaction_color, k);		// updates k
+      read_color(st, "parameterpin_color", result->parameterpin_color, k);		// updates k
+    }    
+    result->BrowserNode::read(st, k);	// updates k
       
-      if (strcmp(k, "end")) {
-	while (BrowserUseCaseDiagram::read(st, k, result) ||
-	       BrowserUseCase::read(st, k, result) ||
-	       BrowserUseCaseView::read(st, k, result, recursive) ||
-	       BrowserClass::read(st, k, result) ||
-	       BrowserSeqDiagram::read(st, k, result) ||
-	       BrowserColDiagram::read(st, k, result) ||
-	       BrowserObjectDiagram::read(st, k, result))
-	  k = read_keyword(st);
+    if (strcmp(k, "end")) {
+      while (BrowserUseCaseDiagram::read(st, k, result) ||
+	     BrowserUseCase::read(st, k, result) ||
+	     BrowserUseCaseView::read(st, k, result, recursive) ||
+	     BrowserClass::read(st, k, result) ||
+	     BrowserClassInstance::read(st, k, result) ||
+	     BrowserSeqDiagram::read(st, k, result) ||
+	     BrowserColDiagram::read(st, k, result) ||
+	     BrowserObjectDiagram::read(st, k, result) ||
+	     BrowserState::read(st, k, result) ||
+	     BrowserActivity::read(st, k, result))
+	k = read_keyword(st);
 	
-	if (strcmp(k, "end"))
-	  wrong_keyword(k, "end");
-      }
-      
-      return result;
+      if (strcmp(k, "end"))
+	wrong_keyword(k, "end");
     }
+    return result;
   }
   else
     return 0;
@@ -863,7 +1157,7 @@ BrowserNode * BrowserUseCaseView::get_it(const char * k, int id)
   BrowserNode * r;
   
   if ((r = BrowserUseCaseDiagram::get_it(k, id)) == 0)
-    // class, seq diag, col diag managed in class view
+    // class, seq diag, col diag, state and activity managed in class view
     r = BrowserUseCase::get_it(k, id);
   
   return r;
