@@ -50,6 +50,7 @@ ArtifactData::ArtifactData(ArtifactData * model, BrowserNode * bn)
   cpp_h = model->cpp_h;
   cpp_src = model->cpp_src;
   java_src = model->java_src;
+  php_src = model->php_src;
   idl_src = model->idl_src;
   
 #if 0
@@ -91,6 +92,10 @@ void ArtifactData::use_default_java_src() {
   java_src = GenerationSettings::java_default_source_content();
 }
 
+void ArtifactData::use_default_php_src() {
+  php_src = GenerationSettings::php_default_source_content();
+}
+
 void ArtifactData::use_default_idl_src() {
   idl_src = GenerationSettings::idl_default_source_content();
 }
@@ -155,6 +160,10 @@ void ArtifactData::send_java_def(ToolCom * com) {
   com->write_string(java_src);
 }
 
+void ArtifactData::send_php_def(ToolCom * com) {
+  com->write_string(php_src);
+}
+
 void ArtifactData::send_idl_def(ToolCom * com) {
   com->write_string(idl_src);
 }
@@ -176,6 +185,9 @@ bool ArtifactData::tool_cmd(ToolCom * com, const char * args,
       case setJavaSrcCmd:
 	java_src = args;
 	break;
+      case setPhpSrcCmd:
+	php_src = args;
+	break;
       case setIdlSrcCmd:
 	idl_src = args;
 	break;
@@ -243,6 +255,15 @@ void ArtifactData::on_delete() {
   
   browser_node->on_delete();
 }
+
+bool ArtifactData::decldefbody_contain(const QString & s, bool cs,
+				       BrowserNode *) {
+  return ((QString(get_cpp_h()).find(s, 0, cs) != -1) ||
+	  (QString(get_cpp_src()).find(s, 0, cs) != -1) ||
+	  (QString(get_java_src()).find(s, 0, cs) != -1) ||
+	  (QString(get_php_src()).find(s, 0, cs) != -1) ||
+	  (QString(get_idl_src()).find(s, 0, cs) != -1));
+}
   
 void ArtifactData::associate(BrowserArtifact * other) {
   if ((associated == 0) || (associated->find(other) == 0)) {
@@ -317,8 +338,10 @@ void ArtifactData::convert_add_include_artifact() {
 void ArtifactData::save(QTextStream & st, QString & warning) const {
   BasicData::save(st, warning);
   
-  if (stereotype == "source") {
-    if (!cpp_h.isEmpty()) {
+  bool a_text = (stereotype == "text");
+  
+  if (a_text || (stereotype == "source")) {
+    if (!a_text && !cpp_h.isEmpty()) {
       nl_indent(st);
       st << "cpp_h ";
       save_string(cpp_h, st);
@@ -333,6 +356,11 @@ void ArtifactData::save(QTextStream & st, QString & warning) const {
       st << "java_src ";
       save_string(java_src, st);
     }
+    if (!php_src.isEmpty()) {
+      nl_indent(st);
+      st << "php_src ";
+      save_string(php_src, st);
+    }
     if (!idl_src.isEmpty()) {
       nl_indent(st);
       st << "idl_src ";
@@ -364,9 +392,13 @@ void ArtifactData::read(char * & st, char * & k) {
   cpp_h = QString::null;
   cpp_src = QString::null;
   java_src = QString::null;
+  php_src = QString::null;
   idl_src = QString::null;
   
-  if (stereotype == "source") {
+  if ((stereotype == "text") && (read_file_format() < 42))
+    stereotype = "_text";
+  
+  if ((stereotype == "source") || (stereotype == "text")) {
     if (!strcmp(k, "cpp_h")) {
       cpp_h = read_string(st);
       k = read_keyword(st);
@@ -390,6 +422,11 @@ void ArtifactData::read(char * & st, char * & k) {
       k = read_keyword(st);
     }
     
+    if (!strcmp(k, "php_src")) {
+      php_src = read_string(st);
+      k = read_keyword(st);
+    }
+    
     if (!strcmp(k, "idl_src")) {
       idl_src = read_string(st);
       k = read_keyword(st);
