@@ -50,6 +50,9 @@ class ArtifactData : public SimpleData {
     // Java
     SharedStr java_src;
     
+    // PHP
+    SharedStr php_src;
+    
     // Idl
     SharedStr idl_src;
     
@@ -57,6 +60,7 @@ class ArtifactData : public SimpleData {
 			      const QString & comment);
     virtual void send_cpp_def(ToolCom * com);
     virtual void send_java_def(ToolCom * com);
+    virtual void send_php_def(ToolCom * com);
     virtual void send_idl_def(ToolCom * com);
     
     void update_associated(QPtrDict<BrowserArtifact> & d);
@@ -73,6 +77,9 @@ class ArtifactData : public SimpleData {
         
     const char * get_java_src() const { return java_src; };
     void use_default_java_src();
+        
+    const char * get_php_src() const { return php_src; };
+    void use_default_php_src();
     
     const char * get_idl_src() const { return idl_src; };
     void use_default_idl_src();
@@ -81,6 +88,8 @@ class ArtifactData : public SimpleData {
     void associate(BrowserArtifact *);
     void unassociate(BrowserArtifact *);
     
+    virtual bool decldefbody_contain(const QString & s, bool cs, BrowserNode *);
+    
     void edit();
     
     virtual bool tool_cmd(ToolCom * com, const char * args,
