@@ -53,6 +53,7 @@ AttributeData::AttributeData()
 
 AttributeData::AttributeData(const AttributeData * model, BrowserNode * bn)
     : ClassMemberData(model),
+      multiplicity(model->multiplicity),
       init_value(model->init_value), 
       constraint(model->constraint), 
       is_deleted(FALSE), isa_class_attribute(model->isa_class_attribute),
@@ -63,7 +64,7 @@ AttributeData::AttributeData(const AttributeData * model, BrowserNode * bn)
       uml_visibility(model->uml_visibility),
       cpp_visibility(model->cpp_visibility), cpp_decl(model->cpp_decl), 
       java_decl(model->java_decl), java_annotation(model->java_annotation),
-      idl_decl(model->idl_decl) {
+      php_decl(model->php_decl), idl_decl(model->idl_decl) {
   browser_node = bn;
   idl_case = 0;	// set_idlcase look at current value
   set_type(model->type);
@@ -86,6 +87,9 @@ void AttributeData::set_browser_node(BrowserAttribute * a, bool update,
   BasicData::set_browser_node(a);
   
   if (update) {
+    if (enum_item)
+      type.type = (BrowserClass *) a->parent();
+    
     if (uml_visibility == UmlDefaultVisibility)
       uml_visibility = 
 	((BrowserNode *) browser_node->parent())->get_visibility(UmlAttribute);
@@ -95,7 +99,7 @@ void AttributeData::set_browser_node(BrowserAttribute * a, bool update,
   
     if (GenerationSettings::cpp_get_default_defs()) {
       if (ClassDialog::cpp_stereotype(cl_stereotype) != "enum")
-	cpp_decl = GenerationSettings::cpp_default_attr_decl();
+	cpp_decl = GenerationSettings::cpp_default_attr_decl(multiplicity);
       else
 	cpp_decl = (enum_item) ? GenerationSettings::cpp_default_enum_item_decl() : "";
     }
@@ -104,7 +108,7 @@ void AttributeData::set_browser_node(BrowserAttribute * a, bool update,
       bool enump = ClassDialog::java_stereotype(cl_stereotype) == "enum";
       
       if (!enum_item) {
-	java_decl = GenerationSettings::java_default_attr_decl();
+	java_decl = GenerationSettings::java_default_attr_decl(multiplicity);
 	if (enump)
 	  set_stereotype("attribute");
       }
@@ -114,17 +118,23 @@ void AttributeData::set_browser_node(BrowserAttribute * a, bool update,
 	  : GenerationSettings::java_default_enum_pattern_item_decl();
     }
     
+    if (GenerationSettings::php_get_default_defs()) {
+      php_decl = (enum_item)
+	? GenerationSettings::php_default_enum_item_decl()
+	: GenerationSettings::php_default_attr_decl();
+    }
+    
     QString idl_st = ClassDialog::idl_stereotype(cl_stereotype);
     
     if (GenerationSettings::idl_get_default_defs()) {
       if (idl_st == "enum")
 	idl_decl =  (enum_item) ? GenerationSettings::idl_default_enum_item_decl() :  "";
       else if (idl_st == "union")
-	idl_decl = GenerationSettings::idl_default_union_item_decl();
+	idl_decl = GenerationSettings::idl_default_union_item_decl(multiplicity);
       else if (idl_st == "valuetype")
-	idl_decl = GenerationSettings::idl_default_valuetype_attr_decl();
+	idl_decl = GenerationSettings::idl_default_valuetype_attr_decl(multiplicity);
       else
-	idl_decl = GenerationSettings::idl_default_attr_decl();
+	idl_decl = GenerationSettings::idl_default_attr_decl(multiplicity);
     }
   }
 }
@@ -155,6 +165,13 @@ QString AttributeData::definition(bool full, DrawingLanguage language) const {
       return definition(FALSE);
     else
       return QString::null;
+  case PhpView:
+    if (full)
+      return AttributeDialog::php_decl((BrowserAttribute *) browser_node);
+    else if (!php_decl.isEmpty())
+      return definition(FALSE);
+    else
+      return QString::null;
   default:
     if (full)
       return AttributeDialog::idl_decl((BrowserAttribute *) browser_node);
@@ -165,6 +182,14 @@ QString AttributeData::definition(bool full, DrawingLanguage language) const {
   }	
 }
 
+bool AttributeData::decldefbody_contain(const QString & s, bool cs,
+					BrowserNode *) {
+  return ((QString(get_cppdecl()).find(s, 0, cs) != -1) ||
+	  (QString(get_javadecl()).find(s, 0, cs) != -1) ||
+	  (QString(get_phpdecl()).find(s, 0, cs) != -1) ||
+	  (QString(get_idldecl()).find(s, 0, cs) != -1));
+}
+
 UmlVisibility AttributeData::get_visibility(BrowserNode *) {
   return uml_visibility;
 }
@@ -259,6 +284,10 @@ void AttributeData::send_uml_def(ToolCom * com, BrowserNode * bn,
     com->write_string(constraint);
   
   type.send_def(com);
+
+  if (api >= 32)
+    com->write_string(multiplicity);
+
   com->write_string(init_value);
   com->write_bool(isa_const_attribute);
   
@@ -303,6 +332,10 @@ void AttributeData::send_java_def(ToolCom * com) {
     com->write_string(java_annotation);
 }
 
+void AttributeData::send_php_def(ToolCom * com) {
+  com->write_string(php_decl);
+}
+
 void AttributeData::send_idl_def(ToolCom * com) {
   com->write_string(idl_decl);
   if (idl_case == 0) {
@@ -338,6 +371,10 @@ bool AttributeData::tool_cmd(ToolCom * com, const char * args,
 	  else
 	    uml_visibility = v;
 	}
+	break;
+      case setMultiplicityCmd:
+	multiplicity = args;
+	break;
       case setConstraintCmd:
 	constraint = args;
 	break;
@@ -357,6 +394,9 @@ bool AttributeData::tool_cmd(ToolCom * com, const char * args,
 	  java_annotation = s;
 	}
 	break;
+      case setPhpDeclCmd:
+	php_decl = args;
+	break;
       case setIdlDeclCmd:
 	idl_decl = args;
 	break;
@@ -427,6 +467,11 @@ void AttributeData::save(QTextStream & st, QString & warning) const {
     st << "const_attribute ";
   st << stringify(uml_visibility);
   type.save(st, warning, " type ", " explicit_type ");
+  if (! multiplicity.isEmpty()) {
+    nl_indent(st);
+    st << "multiplicity ";
+    save_string(multiplicity, st);
+  }
   if (!init_value.isEmpty()) {
     nl_indent(st);
     st << "init_value ";
@@ -459,6 +504,10 @@ void AttributeData::save(QTextStream & st, QString & warning) const {
     save_string(java_annotation, st);
   }
   
+  nl_indent(st);
+  st << "php_decl ";
+  save_string(php_decl, st);  
+  
   nl_indent(st);
   if (idl_case != 0) {
     st << "idl_case ";
@@ -508,6 +557,13 @@ void AttributeData::read(char * & st, char * & k) {
   set_type(t);
   
   k = read_keyword(st);
+  if (!strcmp(k, "multiplicity")) {
+    multiplicity = read_string(st);
+    k = read_keyword(st);
+  }
+  else
+    multiplicity = 0;
+  
   if (!strcmp(k, "init_value")) {
     init_value = read_string(st);
     k = read_keyword(st);
@@ -572,6 +628,13 @@ void AttributeData::read(char * & st, char * & k) {
   else
     java_annotation = QString::null;
     
+  if (!strcmp(k, "php_decl")) {
+    php_decl = read_string(st);
+    k = read_keyword(st);
+  }
+  else
+    php_decl = QString::null;
+    
   if (!strcmp(k, "idl_case")) {
     set_idlcase(BrowserAttribute::read_ref(st), "");
     k = read_keyword(st);
