@@ -38,6 +38,7 @@ class AttributeData : public ClassMemberData {
   
   protected:
     AType type;
+    SharedStr multiplicity;
     MyStr init_value;
     MyStr constraint;
     bool is_deleted : 1;
@@ -57,6 +58,9 @@ class AttributeData : public ClassMemberData {
     SharedStr java_decl;
     SharedStr java_annotation;
     
+    // Php
+    SharedStr php_decl;
+    
     // Idl
     BrowserAttribute * idl_case;	// exclusive with idl_explicit_case
     MyStr idl_explicit_case;
@@ -66,6 +70,7 @@ class AttributeData : public ClassMemberData {
 			      const QString & comment);
     virtual void send_cpp_def(ToolCom * com);
     virtual void send_java_def(ToolCom * com);
+    virtual void send_php_def(ToolCom * com);
     virtual void send_idl_def(ToolCom * com);
     
   public:
@@ -81,8 +86,12 @@ class AttributeData : public ClassMemberData {
     virtual QString definition(bool full) const;
     QString definition(bool full, DrawingLanguage language) const;
     
+    virtual bool decldefbody_contain(const QString & s, bool cs, BrowserNode *);
+    
     const char * get_constraint() const { return constraint; };
     
+    const char * get_multiplicity() const { return multiplicity; };
+
     bool get_isa_class_attribute() const { return isa_class_attribute; };
     
     bool get_isa_const_attribute() const { return isa_const_attribute; };
@@ -100,6 +109,8 @@ class AttributeData : public ClassMemberData {
     const char * get_cppdecl() const { return cpp_decl; };
         
     const char * get_javadecl() const { return java_decl; };
+        
+    const char * get_phpdecl() const { return php_decl; };
     
     const char * get_idldecl() const { return idl_decl; };
     const char * get_idlcase() const;
