@@ -117,6 +117,11 @@ QString BasicData::definition(bool) const {
   return browser_node->get_name();
 }
 
+bool BasicData::decldefbody_contain(const QString &, bool,
+				    BrowserNode *) {
+  return FALSE;
+}
+
 void BasicData::save(QTextStream & st, QString &) const {
   if (! stereotype.isEmpty()) {
     nl_indent(st);
@@ -153,8 +158,14 @@ void BasicData::send_uml_def(ToolCom * com, BrowserNode * bn,
   com->write_string(stereotype);
   ((HaveKeyValueData *) bn)->send_def(com);
   com->write_string(comment);
-  if (com->api_format() > 11)
+  
+  int api = com->api_format();
+  
+  if (api > 11) {
     com->write_bool(bn->markedp());
+    if (api > 30)
+      com->write_unsigned(bn->get_identifier());
+  }
 }
 
 void BasicData::send_cpp_def(ToolCom *) {
@@ -163,6 +174,9 @@ void BasicData::send_cpp_def(ToolCom *) {
 void BasicData::send_java_def(ToolCom *) {
 }
 
+void BasicData::send_php_def(ToolCom *) {
+}
+
 void BasicData::send_idl_def(ToolCom *) {
 }
 
@@ -183,6 +197,8 @@ bool BasicData::tool_cmd(ToolCom * com, const char * args,
     send_uml_def(com, bn, comment);
     send_cpp_def(com);
     send_java_def(com);
+    if (com->api_format() >= 34)
+      send_php_def(com);
     send_idl_def(com);
     break;
   case getUmlDefCmd:
@@ -196,6 +212,10 @@ bool BasicData::tool_cmd(ToolCom * com, const char * args,
     send_uml_def(com, bn, comment);
     send_java_def(com);
     break;
+  case getPhpDefCmd:
+    send_uml_def(com, bn, comment);
+    send_php_def(com);
+    break;
   case getIdlDefCmd:
     send_uml_def(com, bn, comment);
     send_idl_def(com);
