@@ -85,6 +85,9 @@ class BasicData : public QObject {
     BrowserNode * get_browser_node() const { return browser_node; };
     
     virtual QString definition(bool full) const;
+    
+    virtual bool decldefbody_contain(const QString & s, bool cs,
+				     BrowserNode *);
         
     void save(QTextStream &, QString & warning) const;
     void read(char * & st, char * & k);
@@ -96,6 +99,7 @@ class BasicData : public QObject {
 			      const QString & comment);
     virtual void send_cpp_def(ToolCom * com);
     virtual void send_java_def(ToolCom * com);
+    virtual void send_php_def(ToolCom * com);
     virtual void send_idl_def(ToolCom * com);
     
   protected slots:
