@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -41,8 +41,7 @@ class ExceptionData {
 
     void send_uml_def(ToolCom * com);
     
-    void save(QTextStream &, QString & warning, 
-	      const QString & cl_oper_name, unsigned rank) const;
+    void save(QTextStream &, QString & warning) const;
     void read(char * &, char * &);
 };
 
