@@ -46,11 +46,15 @@ class ExtraMemberData : public BasicData {
     // Java
     MyStr java_decl;
     
+    // Php
+    MyStr php_decl;
+    
     // Idl
     MyStr idl_decl;
     
     virtual void send_cpp_def(ToolCom * com);
     virtual void send_java_def(ToolCom * com);
+    virtual void send_php_def(ToolCom * com);
     virtual void send_idl_def(ToolCom * com);
     
   public:
@@ -68,6 +72,8 @@ class ExtraMemberData : public BasicData {
     bool get_cpp_inline() const { return cpp_inline; };
             
     const char * get_java_decl() const { return java_decl; };
+            
+    const char * get_php_decl() const { return php_decl; };
     
     const char * get_idl_decl() const { return idl_decl; };
     
