@@ -44,6 +44,7 @@ PackageData::PackageData(PackageData * model)
   cpp_src_dir = model->cpp_src_dir;
   cpp_h_dir = model->cpp_h_dir;
   java_dir = model->java_dir;
+  php_dir = model->php_dir;
   idl_dir = model->idl_dir;
   cpp_namespace = model->cpp_namespace;
   java_package = model->java_package;
@@ -72,6 +73,10 @@ void PackageData::send_java_def(ToolCom * com) {
   com->write_string(java_package);
 }
 
+void PackageData::send_php_def(ToolCom * com) {
+  com->write_string(php_dir);
+}
+
 void PackageData::send_idl_def(ToolCom * com) {
   com->write_string(idl_dir);
   com->write_string(idl_module);
@@ -102,6 +107,9 @@ bool PackageData::tool_cmd(ToolCom * com, const char * args,
       case setJavaPackageCmd:
 	java_package = args;
 	break;
+      case setPhpDirCmd:
+	php_dir = args;
+	break;
       case setIdlDirCmd:
 	idl_dir = args;
 	break;
@@ -154,6 +162,11 @@ void PackageData::save(QTextStream & st, QString & warning) const {
     st << "java_package ";
     save_string(java_package, st);
   }
+  if (!php_dir.isEmpty()) {
+    nl_indent(st);
+    st << "php_dir ";
+    save_string(php_dir, st);
+  }
   if (!idl_dir.isEmpty()) {
     nl_indent(st);
     st << "idl_dir ";
@@ -189,6 +202,10 @@ void PackageData::read(char * & st, char * & k) {
     java_package = read_string(st);
     k = read_keyword(st);
   }
+  if (!strcmp(k, "php_dir")) {
+    php_dir = read_string(st);
+    k = read_keyword(st);
+  }
   if (!strcmp(k, "idl_dir")) {
     idl_dir = read_string(st);
     k = read_keyword(st);
