@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -31,6 +31,8 @@
 #include "PinParamData.h"
 
 class ParameterData : public SimpleData, public PinParamData {
+  Q_OBJECT
+    
   friend class ParameterDialog;
   
   protected:
@@ -40,6 +42,9 @@ class ParameterData : public SimpleData, public PinParamData {
     ParameterData();
     ParameterData(ParameterData * model, BrowserNode * bn);
   
+    virtual void do_connect(BrowserClass * c);
+    virtual void do_disconnect(BrowserClass * c);
+    
     const char * get_default_value() const { return default_value; };
 
     void edit();
@@ -51,8 +56,11 @@ class ParameterData : public SimpleData, public PinParamData {
     virtual void send_cpp_def(ToolCom * com);
     virtual void send_java_def(ToolCom * com);
     
-    void save(QTextStream &, QString & warning, QString what) const;
+    void save(QTextStream &, QString & warning) const;
     void read(char * &, char * &);
+    
+  protected slots:
+    void on_delete();
 };
 
 #endif
