@@ -43,10 +43,11 @@
 #include "GenerationSettings.h"
 #include "mu.h"
 
-IdDict<RelationData> RelationData::all(1023);
+IdDict<RelationData> RelationData::all(1023, __FILE__);
+QList<RelationData> RelationData::Unconsistent;
 
 RelationData::RelationData(UmlCode e, int id)
-    : Labeled<RelationData>(all, id), is_deleted(FALSE), type(e) {
+    : Labeled<RelationData>(all, id), is_deleted(FALSE), is_unconsistent(FALSE), type(e) {
   a.isa_class_relation = b.isa_class_relation = FALSE; 
   a.isa_volatile_relation = b.isa_volatile_relation = FALSE;
   a.isa_const_relation = b.isa_const_relation = FALSE; 
@@ -66,7 +67,7 @@ RelationData::RelationData(UmlCode e, int id)
 RelationData::RelationData(const BrowserRelation * model,
 			   BrowserRelation * r)
     : ClassMemberData((ClassMemberData *) model->get_data()),
-      Labeled<RelationData>(all, 0), is_deleted(FALSE) {
+      Labeled<RelationData>(all, 0), is_deleted(FALSE), is_unconsistent(FALSE) {
   RelationData * md = (RelationData *) model->get_data();
   
   type = md->type;
@@ -129,6 +130,9 @@ void RelationData::garbage(BrowserRelation * r) {
 	(end->get_data() == 0)) // bi-dir rel, reverse rel wasn't read
       delete this;
   }
+  else if (start == 0)
+    // end -> start read, but start -> end not read
+    delete this; 
   else {
     end = 0;
     if (start == 0)
@@ -221,6 +225,9 @@ static void default_decls(RoleData & r, UmlCode type, QString cl_stereotype)
   if (GenerationSettings::java_get_default_defs())
     r.java_decl = GenerationSettings::java_default_rel_decl(QString::null);
   
+  if (GenerationSettings::php_get_default_defs())
+    r.php_decl = GenerationSettings::php_default_rel_decl();
+  
   if (GenerationSettings::idl_get_default_defs()) {
     QString idl_st = ClassDialog::idl_stereotype(cl_stereotype);
     
@@ -257,13 +264,15 @@ BrowserRelation * RelationData::set_start_end(BrowserRelation * s, BrowserClass
 	  ? "${type}" : "";
       if (GenerationSettings::java_get_default_defs())
 	a.java_decl = "${type}";
+      if (GenerationSettings::php_get_default_defs())
+	a.php_decl = "${type}";
       if (GenerationSettings::idl_get_default_defs())
 	a.idl_decl = (ClassDialog::idl_stereotype(st) != "enum")
 	  ? "${type}" : "";
       break;
     case UmlDependency:
       if (GenerationSettings::cpp_get_default_defs())
-	a.cpp_decl = "Generated";
+	a.cpp_decl = "#include in source";
       break;
     default:
       a.uml_visibility =
@@ -369,6 +378,19 @@ void RelationData::set_stereotype(const char * s) {
     end->update_stereotype();
 }
 
+bool RelationData::decldefbody_contain(const QString & s, bool cs,
+				       BrowserNode * br) {
+  return (is_a((BrowserRelation *) br))
+    ? ((QString(get_cppdecl_a()).find(s, 0, cs) != -1) ||
+       (QString(get_javadecl_a()).find(s, 0, cs) != -1) ||
+       (QString(get_phpdecl_a()).find(s, 0, cs) != -1) ||
+       (QString(get_idldecl_a()).find(s, 0, cs) != -1))
+    : ((QString(get_cppdecl_b()).find(s, 0, cs) != -1) ||
+       (QString(get_javadecl_b()).find(s, 0, cs) != -1) ||
+       (QString(get_phpdecl_b()).find(s, 0, cs) != -1) ||
+       (QString(get_idldecl_b()).find(s, 0, cs) != -1));
+}
+
 bool RelationData::wrong_role_a_name(const QString & s) {
   return (!s.isEmpty() &&
 	  (s != a.role) &&
@@ -653,6 +675,15 @@ void RelationData::send_java_def(ToolCom * com, BrowserRelation * rel) {
   }
 }
 
+void RelationData::send_php_def(ToolCom * com, BrowserRelation * rel) {
+  if (rel == start) {
+    com->write_string(a.php_decl);
+  }
+  else {
+    com->write_string(b.php_decl);
+  }
+}
+
 void RelationData::send_idl_def(ToolCom * com, BrowserRelation * rel) {
   if (rel == start) {
     com->write_string(a.idl_decl);
@@ -764,6 +795,9 @@ bool RelationData::tool_cmd(ToolCom * com, BrowserRelation * rel,
 	  r.java_annotation = s;
 	}
 	break;
+      case setPhpDeclCmd:
+	r.php_decl = args;
+	break;
       case setIdlDeclCmd:
 	r.idl_decl = args;
 	break;
@@ -835,6 +869,8 @@ bool RelationData::tool_cmd(ToolCom * com, BrowserRelation * rel,
       send_uml_def(com, rel);
       send_cpp_def(com, rel);
       send_java_def(com, rel);
+      if (com->api_format() >= 34)
+	send_php_def(com, rel);
       send_idl_def(com, rel);
       break;
     case getUmlDefCmd:
@@ -848,6 +884,10 @@ bool RelationData::tool_cmd(ToolCom * com, BrowserRelation * rel,
       send_uml_def(com, rel);
       send_java_def(com, rel);
       break;
+    case getPhpDefCmd:
+      send_uml_def(com, rel);
+      send_php_def(com, rel);
+      break;
     case getIdlDefCmd:
       send_uml_def(com, rel);
       send_idl_def(com, rel);
@@ -884,8 +924,10 @@ static void save_role(const RoleData & role, bool assoc, QTextStream & st,
   if (assoc) {
     st << "role_name ";
     save_string(role.role, st);
-    st << " multiplicity ";
-    save_string(role.multiplicity, st);
+    if (!role.multiplicity.isEmpty()) {
+      st << " multiplicity ";
+      save_string(role.multiplicity, st);
+    }
     if (!role.init_value.isEmpty()) {
       st << " init_value ";
       save_string(role.init_value, st);
@@ -936,6 +978,12 @@ static void save_role(const RoleData & role, bool assoc, QTextStream & st,
     save_string(role.java_annotation, st);
   }
   
+  if (!role.php_decl.isEmpty()) {
+    nl_indent(st);
+    st << "php ";
+    save_string(role.php_decl, st);
+  }
+  
   if (role.idl_case != 0) {
     nl_indent(st);
     st << "idl_case ";
@@ -986,14 +1034,17 @@ void RelationData::save(QTextStream & st, bool ref, QString & warning) const {
     
     nl_indent(st);
     indent(+1);
-    st << "b ";
+    st << "b";
     if (!RelationData::uni_directional(type)) {
+      st << ' ';
       save_role(b, assoc, st, warning);
       end->save(st, TRUE, warning);
     }
     else {
-      st << "multiplicity ";
-      save_string(b.multiplicity, st);
+      if (!b.multiplicity.isEmpty()) {
+        st << " multiplicity ";
+	save_string(b.multiplicity, st);
+      }
       st << " parent ";
       end_removed_from->save(st, TRUE, warning);
     }
@@ -1007,9 +1058,13 @@ void RelationData::save(QTextStream & st, bool ref, QString & warning) const {
   }
 }
 
-RelationData * RelationData::read_ref(char * & st, bool complete)
+RelationData * RelationData::read_ref(char * & st, bool complete,
+				      const char * k)
 {
-  read_keyword(st, "relation_ref");
+  if (k == 0)
+    read_keyword(st, "relation_ref");
+  else if (strcmp(k, "relation_ref"))
+    wrong_keyword(k, "relation_ref");
   
   int id = read_id(st);
   RelationData * result = all[id];
@@ -1033,11 +1088,15 @@ static void read_role(RoleData & role, bool assoc,
   if (assoc) {
     read_keyword(st, "role_name");
     role.role = read_string(st);
-    read_keyword(st, "multiplicity");
-    role.multiplicity = read_string(st);
-    
+
     k = read_keyword(st);
     
+    if (!strcmp(k, "multiplicity")) {
+      role.multiplicity = read_string(st);
+      k = read_keyword(st);
+    }
+    else
+      role.multiplicity = 0;
     if (!strcmp(k, "init_value")) {
       role.init_value = read_string(st);
       k = read_keyword(st);
@@ -1126,6 +1185,13 @@ static void read_role(RoleData & role, bool assoc,
   else
     role.java_annotation = QString::null;
   
+  if (!strcmp(k, "php")) {
+    role.php_decl = read_string(st);
+    k = read_keyword(st);
+  }
+  else
+    role.php_decl = QString::null;
+
   if (!strcmp(k, "idl_case")) {
     rd->set_idlcase(role, BrowserAttribute::read_ref(st), "");
     k = read_keyword(st);
@@ -1152,8 +1218,11 @@ static void read_role(RoleData & role, bool assoc,
     role.idl_truncatable_inheritance = FALSE;
 }
 
-RelationData * RelationData::read(char * & st, char * & k)
+RelationData * RelationData::read(char * & st, char * & k,
+				  BrowserRelation *& unconsistent)
 {
+  unconsistent = 0;
+
   RelationData * result;
   int id;
   
@@ -1166,9 +1235,23 @@ RelationData * RelationData::read(char * & st, char * & k)
   else if (!strcmp(k, "relation")) {
     if ((result = all[id = read_id(st)]) == 0)
       result = new RelationData(relation_type(read_keyword(st)), id);
+    else if (result->type != UmlRelations) {
+      // shared identifier
+      result->set_unconsistent();
+      unconsistent = result->start;
+      result = new RelationData(relation_type(read_keyword(st)), id);      
+      result->set_unconsistent();
+    }
     else {
       result->type = relation_type(read_keyword(st));
       result->name = default_name(result->type);
+      if (result->start != 0) {
+	// Created by RelationData::read_ref()
+	// unvalidate start/end to not delete result
+	// when start/end will be deleted
+	result->start->unvalidate();
+	result->end->unvalidate();
+      }
     }
     
     k = read_keyword(st);
@@ -1197,10 +1280,11 @@ RelationData * RelationData::read(char * & st, char * & k)
     else {
       k = read_keyword(st);
       if (!strcmp(k, "multiplicity")) {
-	// new
 	result->b.multiplicity = read_string(st);
 	k = read_keyword(st);
       }
+      else
+	result->b.multiplicity = 0;
       if (strcmp(k, "parent"))
 	wrong_keyword(k, "parent");
       result->end_removed_from = BrowserClass::read_ref(st);
@@ -1251,3 +1335,29 @@ RelationData * RelationData::read(char * & st, char * & k)
   else
     return 0;
 }
+
+//
+
+bool RelationData::has_unconsistencies()
+{ 
+  return !Unconsistent.isEmpty();
+}
+
+void RelationData::set_unconsistent() {
+  if (! is_unconsistent) {
+    is_unconsistent = TRUE;
+    Unconsistent.append(this);
+  }
+}
+
+void RelationData::delete_unconsistent()
+{
+  while (! Unconsistent.isEmpty()) {
+    RelationData * d = Unconsistent.take(0);
+   
+    d->start = d->end = 0;
+    if (!d->deletedp())
+      d->BasicData::delete_it();
+    delete d;
+  }
+}
