@@ -61,6 +61,9 @@ struct RoleData {
   SharedStr java_decl;
   SharedStr java_annotation;
   
+  // Php
+  SharedStr php_decl;
+  
   // Idl
   BrowserAttribute * idl_case;	// exclusive with idl_explicit_case
   SharedStr idl_explicit_case;
@@ -74,9 +77,11 @@ class RelationData : public ClassMemberData, public Labeled<RelationData> {
   
   protected:
     static IdDict<RelationData> all;
+    static QList<RelationData> Unconsistent;
   
     // Uml
     bool is_deleted : 8;	// 1 useless here, 8 faster than 1 ?
+    bool is_unconsistent : 8;	// 1 useless here, 8 faster than 1 ?
     UmlCode type: 8;		// 1 useless here, 8 faster than 1 ?
     MyStr name;
     RoleData a;
@@ -89,6 +94,7 @@ class RelationData : public ClassMemberData, public Labeled<RelationData> {
     virtual void send_uml_def(ToolCom * com, BrowserRelation * rel);
     virtual void send_cpp_def(ToolCom * com, BrowserRelation * rel);
     virtual void send_java_def(ToolCom * com, BrowserRelation * rel);
+    virtual void send_php_def(ToolCom * com, BrowserRelation * rel);
     virtual void send_idl_def(ToolCom * com, BrowserRelation * rel);
     
   public:
@@ -126,6 +132,8 @@ class RelationData : public ClassMemberData, public Labeled<RelationData> {
     virtual void set_stereotype(const QCString &);
     virtual void set_stereotype(const char *);
     
+    virtual bool decldefbody_contain(const QString & s, bool cs, BrowserNode *);
+    
     bool is_a(const BrowserRelation * br) const { return br == start; };
     
     const char * get_role_a() const { return a.role; };
@@ -170,6 +178,9 @@ class RelationData : public ClassMemberData, public Labeled<RelationData> {
     const char * get_javadecl_a() const { return a.java_decl; };
     const char * get_javadecl_b() const { return b.java_decl; };
     
+    const char * get_phpdecl_a() const { return a.php_decl; };
+    const char * get_phpdecl_b() const { return b.php_decl; };
+    
     const char * get_idldecl_a() const { return a.idl_decl; };
     const char * get_idldecl_b() const { return b.idl_decl; };
     static const char * get_idlcase(const RoleData & role);
@@ -179,8 +190,9 @@ class RelationData : public ClassMemberData, public Labeled<RelationData> {
     void select_in_browser(bool prefer_start) const;
     
     void save(QTextStream &, bool ref, QString & warning) const;
-    static RelationData * read_ref(char * &, bool complete = FALSE);
-    static RelationData * read(char * &, char * &);
+    static RelationData * read_ref(char * &, bool complete = FALSE,
+				   const char * k = 0);
+    static RelationData * read(char * &, char * &, BrowserRelation *& unconsistent);
     
     static bool uni_directional(UmlCode);
     static const QString & default_name(UmlCode e);
@@ -192,6 +204,11 @@ class RelationData : public ClassMemberData, public Labeled<RelationData> {
     static void update_idmax_for_root();
     void renumber(int phase, BrowserRelation * br);
     bool is_writable(const BrowserRelation * br) const;
+
+    bool unconsistentp() const { return is_unconsistent; }
+    void set_unconsistent();
+    static bool has_unconsistencies();
+    static void delete_unconsistent();
     
   protected slots:
     void end_deleted();
