@@ -254,6 +254,14 @@ void ActivityActionCanvas::modified() {
   package_modified();
 }
 
+void ActivityActionCanvas::post_loaded() {
+  force_pins_arround();
+  if (the_canvas()->must_draw_all_relations()) {
+    draw_all_flows();
+    draw_all_simple_relations();
+  }
+}
+
 aCorner ActivityActionCanvas::on_resize_point(const QPoint & p) {
   return ::on_resize_point(p, rect());
 }
@@ -396,7 +404,8 @@ void ActivityActionCanvas::check_conditions() {
 
 	pre->move((cdx < 0) ? 0 : cdx, (cdy < 0) ? 0 : cdy);
 	pre->show();
-	(new ArrowCanvas(the_canvas(), this, pre, UmlAnchor, 0, FALSE))->show();
+	(new ArrowCanvas(the_canvas(), this, pre, UmlAnchor, 0, FALSE, -1.0, -1.0))
+	  ->show();
       }
       else
 	pre->set(s);
@@ -418,7 +427,8 @@ void ActivityActionCanvas::check_conditions() {
 	post->upper();
 	post->move(x() + margin, y() + height() + margin);
 	post->show();
-	(new ArrowCanvas(the_canvas(), this, post, UmlAnchor, 0, FALSE))->show();
+	(new ArrowCanvas(the_canvas(), this, post, UmlAnchor, 0, FALSE, -1.0, -1.0))
+	  ->show();
       }
       else
 	post->set(s);
@@ -520,7 +530,7 @@ void ActivityActionCanvas::draw(QPainter & p) {
 	p.drawPolyline(a);
 
 	if (fp != 0)
-	  draw_poly(fp, a, "none");
+	  draw_poly(fp, a, UmlTransparent);
       }
       else {
 	if (shadow != 0) {
@@ -538,14 +548,14 @@ void ActivityActionCanvas::draw(QPainter & p) {
 	  p.setPen(QObject::SolidLine);
 
 	  if (fp != 0)
-	    draw_poly(fp, b, QObject::darkGray);
+	    draw_shadow(fp, b);
 	}
 	
 	p.setBrush(co);
 	p.drawPolygon(a, TRUE, 0, 5);
 
 	if (fp != 0)
-	  draw_poly(fp, a, co);
+	  draw_poly(fp, a, used_color);
       }
       r.setLeft(r.left() + margin);
       margin = (int) (6 * the_canvas()->zoom());
@@ -573,7 +583,7 @@ void ActivityActionCanvas::draw(QPainter & p) {
       if (used_color == UmlTransparent) {
 	p.drawPolyline(a);
 	if (fp != 0)
-	  draw_poly(fp, a, "none");
+	  draw_poly(fp, a, UmlTransparent);
       }
       else {
 	if (shadow != 0) {
@@ -591,14 +601,14 @@ void ActivityActionCanvas::draw(QPainter & p) {
 	  p.setPen(QObject::SolidLine);
 
 	  if (fp != 0)
-	    draw_poly(fp, b, QObject::darkGray, FALSE);
+	    draw_shadow(fp, b);
 	}
       
 	p.setBrush(co);
 	p.drawPolygon(a, TRUE, 0, 5);
 
 	if (fp != 0)
-	  draw_poly(fp, a, co);
+	  draw_poly(fp, a, used_color);
       }
       r.setWidth(r.width() - margin);
       margin = (int) (6 * the_canvas()->zoom());
@@ -627,9 +637,9 @@ void ActivityActionCanvas::draw(QPainter & p) {
     p.drawRoundRect(r);
 
     if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"10\" />\n",
-	      co.rgb()&0xffffff,
+	      svg_color(used_color),
 	      r.left(), r.top(), r.width() - 1, r.height() - 1);
     
     if (data->get_action_kind() == UmlCallBehaviorAction) {
@@ -925,11 +935,11 @@ void ActivityActionCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * a;
   
   if (action == UmlFlow)
-    a = new FlowCanvas(the_canvas(), this, dest, 0, 0);
+    a = new FlowCanvas(the_canvas(), this, dest, 0, 0, -1.0, -1.0);
   else if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
