@@ -58,6 +58,7 @@
 #include "BrowserActivityAction.h"
 #include "ActivityActionData.h"
 #include "ActivityObjectData.h"
+#include "ClassInstanceData.h"
 #include "BrowserPin.h"
 #include "PinCanvas.h"
 #include "MenuTitle.h"
@@ -328,7 +329,6 @@ void ActivityDiagramView::dragEnterEvent(QDragEnterEvent * e) {
        UmlDrag::canDecode(e, UmlColDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlObjectDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlComponentDiagram, FALSE, TRUE) ||
-       UmlDrag::canDecode(e, UmlActivityDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlDeploymentDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlActivityDiagram, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlStateDiagram, TRUE, TRUE) ||
@@ -339,6 +339,7 @@ void ActivityDiagramView::dragEnterEvent(QDragEnterEvent * e) {
        UmlDrag::canDecode(e, UmlAttribute, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlRelations, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlClass, FALSE, TRUE) ||
+       UmlDrag::canDecode(e, UmlClassInstance, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlState, FALSE, TRUE)))
     e->accept();
   else
@@ -553,8 +554,8 @@ void ActivityDiagramView::dropEvent(QDropEvent * e) {
 	   ((bn = UmlDrag::decode(e, UmlActivityDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlComponentDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlDeploymentDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlStateDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlActivityDiagram)) != 0)) {
+	   ((bn = UmlDrag::decode(e, UmlStateDiagram, TRUE)) != 0) ||
+	   ((bn = UmlDrag::decode(e, UmlActivityDiagram, TRUE)) != 0)) {
     history_save();
     
     IconCanvas * ic = new IconCanvas(bn, the_canvas(), p.x(), p.y(), 0);
@@ -664,6 +665,33 @@ void ActivityDiagramView::dropEvent(QDropEvent * e) {
       window()->package_modified();
     }
   }
+  else if ((bn = UmlDrag::decode(e, UmlClassInstance)) != 0) {
+    history_protected = FALSE;
+
+    BrowserActivityObject * obj = 
+      BrowserActivityObject::add_activityobject(container(p), bn->get_name());
+
+    if (obj != 0) {
+      ((ActivityObjectData *) obj->get_data())
+	->set_type(((ClassInstanceData *) bn->get_data())->get_class());
+
+      history_save();
+      history_protected = TRUE;
+	  
+      ActivityObjectCanvas * c = 
+	new ActivityObjectCanvas(obj, the_canvas(), p.x(), p.y());
+      
+      history_protected = TRUE;
+      c->show();
+      c->force_inside();
+      history_protected = TRUE;
+      c->upper();
+    
+      canvas()->update();
+      history_protected = FALSE;
+      window()->package_modified();
+    }
+  }
   else if ((bn = UmlDrag::decode(e, UmlState, TRUE)) != 0) {
     history_protected = FALSE;
 
