@@ -177,7 +177,7 @@ void ActivityObjectCanvas::compute_size() {
     he = min;
   
   // force odd width and height for line alignment
-  setSize(wi | 1, he | 1);
+  DiagramCanvas::resize(wi | 1, he | 1);
   
   if (data->get_multiplicity()[0] != 0) {
     s = QString("{bounds=") + data->get_multiplicity();
@@ -257,7 +257,9 @@ void ActivityObjectCanvas::check_selection() {
 
 	selection->move((cdx < 0) ? 0 : cdx, (cdy < 0) ? 0 : cdy);
 	selection->show();
-	(new ArrowCanvas(the_canvas(), this, selection, UmlAnchor, 0, FALSE))->show();
+	(new ArrowCanvas(the_canvas(), this, selection, UmlAnchor,
+			 0, FALSE, -1.0, -1.0))
+	  ->show();
       }
       else
 	selection->set(s);
@@ -285,6 +287,14 @@ void ActivityObjectCanvas::modified() {
   package_modified();
 }
 
+void ActivityObjectCanvas::post_loaded() {
+  force_self_rel_visible();
+  if (the_canvas()->must_draw_all_relations()) {
+    draw_all_simple_relations();
+    draw_all_flows();
+  }
+}
+
 void ActivityObjectCanvas::moveBy(double dx, double dy) {
   DiagramCanvas::moveBy(dx, dy);
 
@@ -348,9 +358,9 @@ void ActivityObjectCanvas::draw(QPainter & p) {
       p.fillRect(r, co);
 
       if (fp != 0)
-	fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+	fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 		" x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-		co.rgb()&0xffffff, 
+		svg_color(used_color), 
 		r.x(), r.y(), r.width() - 1, r.height() - 1);
     }
     else if (fp != 0)
@@ -578,11 +588,11 @@ void ActivityObjectCanvas::connexion(UmlCode action, DiagramItem * dest, const Q
   ArrowCanvas * a;
   
   if (action == UmlFlow)
-    a = new FlowCanvas(the_canvas(), this, dest, 0, 0);
+    a = new FlowCanvas(the_canvas(), this, dest, 0, 0, -1.0, -1.0);
   else if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
