@@ -38,11 +38,16 @@
 #include "UmlCanvas.h"
 #include "LabelCanvas.h"
 #include "CdClassCanvas.h"
+#include "TransitionData.h"
 #include "BrowserDiagram.h"
 #include "StereotypeDialog.h"
+#include "DialogUtil.h"
 #include "myio.h"
 #include "MenuTitle.h"
 #include "DiagramView.h"
+#include "UmlPixmap.h"
+#include "ToolCom.h"
+#include "strutil.h"
 
 #include "geometry_hv.xpm"
 #include "geometry_vh.xpm"
@@ -50,11 +55,13 @@
 #include "geometry_vhv.xpm"
 
 ArrowCanvas::ArrowCanvas(UmlCanvas * canvas, DiagramItem * b,
-			 DiagramItem * e, UmlCode t, int id,
-			 bool own_brk)
+			 DiagramItem * e, UmlCode t,
+			 int id, bool own_brk, float dbegin, float dend)
       : QCanvasPolygon(canvas), DiagramItem(id, canvas), begin(b), end(e),
         itstype(t), geometry(NoGeometry), fixed_geometry(FALSE),
-  	label(0), stereotype(0) {
+  	label(0), stereotype(0), decenter_begin(dbegin), decenter_end(dend) {
+  boundings.resize(4);
+  
   double bz = begin->get_z();
   double ez = end->get_z();
   
@@ -69,7 +76,6 @@ ArrowCanvas::ArrowCanvas(UmlCanvas * canvas, DiagramItem * b,
   // the first time the canvas must not have a label nor stereotype  
   auto_pos = canvas->browser_diagram()
     ->get_auto_label_position(canvas->browser_diagram()->get_type());
-  boundings.resize(4);
   update_pos();
   
   connect(DrawingSettings::instance(), SIGNAL(changed()),
@@ -122,6 +128,11 @@ UmlCanvas * ArrowCanvas::the_canvas() const {
   return ((UmlCanvas *) canvas());
 }
 
+static int iabs(int v)
+{
+  return (v >= 0) ? v : -v;
+}
+
 void ArrowCanvas::update_pos() {
   hide_lines();
   
@@ -131,8 +142,24 @@ void ArrowCanvas::update_pos() {
   // calcul beginp & endp pour ne plus etre dans b_rct & e_rct
   if (begin->type() == UmlArrowPoint)
     beginp = begin->center();
-  else
+  else {
     begin->shift(beginp, end->center());
+    if ((geometry == NoGeometry) && (decenter_begin >= 0)) {
+      QRect r = begin->rect();
+      
+      if ((iabs(beginp.y() - r.top()) < 2) ||
+	  (iabs(beginp.y() - r.bottom()) < 2)) {
+	// on top or botton
+	beginp.setX((int) (r.left() + r.width()*decenter_begin));
+      }
+      else if ((iabs(beginp.x() - r.left()) < 2) ||
+	       (iabs(beginp.x() - r.right()) < 2)) {
+	// on left or right
+	beginp.setY((int) (r.top() + r.height()*decenter_begin));
+      }
+      // else end point inside 'start'
+    }
+  }
   
   switch (end->type()) {
   case UmlArrowPoint:
@@ -141,15 +168,29 @@ void ArrowCanvas::update_pos() {
     break;
   default:
     end->shift(endp, beginp);
+    if ((geometry == NoGeometry) && (decenter_end >= 0)) {
+      QRect r = end->rect();
+      
+      if ((iabs(endp.y() - r.top()) < 2) ||
+	  (iabs(endp.y() - r.bottom()) < 2)) {
+	// on top or botton
+	endp.setX((int) (r.left() + r.width()*decenter_end));
+      }
+      else if ((iabs(endp.x() - r.left()) < 2) ||
+	       (iabs(endp.x() - r.right()) < 2)) {
+	// on left or right
+	endp.setY((int) (r.top() + r.height()*decenter_end));
+      }
+      // else start point inside 'end'
+    }
   }
-  
+    
   // calcul rectangle englobant
   
   const int dx = endp.x() - beginp.x();
   const int dy = beginp.y() - endp.y();
   
   if ((dx == 0) && (dy == 0)) {
-    // to not produce crash inside Qt (?)
     boundings.setPoint(0, beginp.x(), beginp.y());
     boundings.setPoint(1, beginp.x() + 1, beginp.y());
     boundings.setPoint(2, beginp.x() + 1, beginp.y() + 1);
@@ -203,12 +244,20 @@ void ArrowCanvas::update_pos() {
     }
     break;
   default:
-    arrow[0].setX((int) (endp.x() - deltax - deltay));
-    arrow[0].setY((int) (endp.y() - deltay + deltax));
-    arrow[1].setX((int) (endp.x() + deltax - deltay));
-    arrow[1].setY((int) (endp.y() + deltay + deltax));
-    arrow[2].setX((int) (endp.x() - deltay));	// pour generalize
-    arrow[2].setY((int) (endp.y() + deltax));
+    if (itstype == UmlInner) {
+      arrow[0].setX((int) (endp.x() - deltay));
+      arrow[0].setY((int) (endp.y() + deltax));
+      arrow[1].setX((int) (endp.x() - deltay - deltay));
+      arrow[1].setY((int) (endp.y() + deltax + deltax));
+    }
+    else {
+      arrow[0].setX((int) (endp.x() - deltax - deltay));
+      arrow[0].setY((int) (endp.y() - deltay + deltax));
+      arrow[1].setX((int) (endp.x() + deltax - deltay));
+      arrow[1].setY((int) (endp.y() + deltay + deltax));
+      arrow[2].setX((int) (endp.x() - deltay));
+      arrow[2].setY((int) (endp.y() + deltax));
+    }
   }
   
   if (begin->type() != UmlArrowPoint) {  
@@ -338,7 +387,10 @@ void ArrowCanvas::drawShape(QPainter & p) {
   if (fp != 0)
     fputs("<g>\n", fp);
   
-  switch (itstype) {
+  switch (((itstype == UmlTransition) &&
+	   (get_data() != 0) && // not under construction
+	   ((TransitionData *) get_data())->internal())
+	  ? UmlDependency : itstype) {
   case UmlDirectionalAggregation:
   case UmlDirectionalAggregationByValue:
   case UmlDirectionalAssociation:
@@ -382,7 +434,7 @@ void ArrowCanvas::drawShape(QPainter & p) {
 	  p.setBrush(brsh);
 
 	  if (fp != 0)
-	    draw_poly(fp, poly, "white");
+	    draw_poly(fp, poly, UmlWhite);
 	}
 	break;
       case UmlAggregationByValue:
@@ -395,7 +447,7 @@ void ArrowCanvas::drawShape(QPainter & p) {
 	  p.setBrush(brsh);
 
 	  if (fp != 0)
-	    draw_poly(fp, poly, "black");
+	    draw_poly(fp, poly, UmlBlack);
 	}      
 	break;
       default:	// to avoid compiler warning
@@ -489,9 +541,41 @@ void ArrowCanvas::drawShape(QPainter & p) {
       if (fp != 0)
 	fprintf(fp, "\t<line stroke=\"black\" stroke-opacity=\"1\""
 		" x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n",
-		beginp.x(), beginp.y(),endp.x(), endp.y());
+		beginp.x(), beginp.y(), endp.x(), endp.y());
     }
     break;
+  case UmlInner:
+    if (end->type() != UmlArrowPoint) {
+      p.drawLine(beginp, arrow[1]);
+      p.drawPixmap(QPoint(arrow[0].x() - ARROW_LENGTH/2,
+			  arrow[0].y() - ARROW_LENGTH/2),
+		   *innerPixmap);
+
+      if (fp != 0) {
+	fprintf(fp, "\t<line stroke=\"black\" stroke-opacity=\"1\""
+		" x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n",
+		beginp.x(), beginp.y(), arrow[1].x(), arrow[1].y());
+	fprintf(fp, "<ellipse fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\" cx=\"%d\" cy=\"%d\" rx=\"%d\" ry=\"%d\" />\n",
+		arrow[0].x(), arrow[0].y(),
+		ARROW_LENGTH/2, ARROW_LENGTH/2);
+	fprintf(fp, "\t<line stroke=\"black\" stroke-opacity=\"1\""
+		" x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n",
+		arrow[0].x() - ARROW_LENGTH/2, arrow[0].y(),
+		arrow[0].x() + ARROW_LENGTH/2, arrow[0].y());
+	fprintf(fp, "\t<line stroke=\"black\" stroke-opacity=\"1\""
+		" x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n",
+		arrow[0].x(), arrow[0].y() - ARROW_LENGTH/2,
+		arrow[0].x(), arrow[0].y() + ARROW_LENGTH/2);
+      }
+    }
+    else {
+      p.drawLine(beginp, endp);
+
+      if (fp != 0)
+	fprintf(fp, "\t<line stroke=\"black\" stroke-opacity=\"1\""
+		" x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n",
+		beginp.x(), beginp.y(),endp.x(), endp.y());
+    }
   default:	// to avoid compiler warning
     break;
   }
@@ -526,6 +610,37 @@ void ArrowCanvas::delete_available(bool &, bool & out_model) const {
   out_model |= TRUE;
 }
 
+void ArrowCanvas::remove(bool) {
+  if (itstype == UmlInner) {
+    if (the_canvas()->must_draw_all_relations()) {
+      const ArrowCanvas * a = this;
+  
+      while (a->begin->type() == UmlArrowPoint) {
+	a = ((ArrowPointCanvas *) a->begin)->get_other(a);
+	if (a == 0)
+	  break;
+      }
+
+      if (a && !a->begin->isSelected() && !a->begin->get_bn()->deletedp()) {
+	a = this;
+  
+	while (a->end->type() == UmlArrowPoint) {
+	  a = ((ArrowPointCanvas *) a->end)->get_other(a);
+	  if (a == 0)
+	    break;
+	}
+  
+	if (a && !a->end->isSelected() && !a->end->get_bn()->deletedp()) {
+	  msg_warning("Bouml", "<i>Draw all relations</i> forced to <i>no</i>");
+	  the_canvas()->dont_draw_all_relations();
+	}
+      }
+    }
+  }
+
+  delete_it();
+}
+
 QRect ArrowCanvas::rect() const {
   QRect r(beginp, endp);
   
@@ -556,7 +671,7 @@ const char * ArrowCanvas::may_connect(UmlCode & l, const DiagramItem * dest) con
 void ArrowCanvas::connexion(UmlCode action, DiagramItem * dest,
 			    const QPoint &, const QPoint &) {
   ArrowCanvas * a =
-    new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
@@ -645,7 +760,8 @@ void ArrowCanvas::open() {
     BrowserNode * bn = get_start()->get_bn();
 
     if ((bn != 0) &&
-	edit(bn->default_stereotypes(itstype), plabel, pstereotype)) {
+	edit(bn->default_stereotypes(itstype, get_end()->get_bn()),
+	     plabel, pstereotype)) {
       canvas()->update();
       package_modified();
     }
@@ -732,12 +848,16 @@ void ArrowCanvas::menu(const QPoint&) {
     default_stereotype_position();
     break;
   case 6:
-    delete_it();
+    remove(FALSE);
     break;
   default:
     if (choice >= 10) {
       choice -= 10;
-      if (choice != (int) geometry)
+      if (choice == RecenterBegin)
+	set_decenter(-1.0, decenter_end);
+      else if (choice == RecenterEnd)
+	set_decenter(decenter_begin, -1.0);
+      else if (choice != (int) geometry)
 	set_geometry((LineGeometry) choice, TRUE);
       else
 	return;
@@ -783,6 +903,11 @@ void ArrowCanvas::init_geometry_menu(QPopupMenu & m, int first) {
     // VHVGeometry
     m.setItemChecked(first + VHVGeometry, TRUE);
   }
+  
+  if (decenter_begin >= 0)
+    m.insertItem("Recenter begin", first + RecenterBegin);
+  if (decenter_end >= 0)
+    m.insertItem("Recenter end", first + RecenterEnd);
 }
 
 DiagramItem * ArrowCanvas::get_start() const {
@@ -807,6 +932,7 @@ void ArrowCanvas::extremities(DiagramItem *& b, DiagramItem *& e) const {
   b = begin;
   e = end;
 }
+
 // search label & stereotype supports
 void ArrowCanvas::search_supports(ArrowCanvas *& plabel, 
 				  ArrowCanvas *& pstereotype) const {
@@ -836,6 +962,31 @@ void ArrowCanvas::search_supports(ArrowCanvas *& plabel,
   if (p->stereotype != 0) pstereotype = (ArrowCanvas *) p;
 }
 
+// reverse line, warning caller must call update_pos() after
+void ArrowCanvas::reverse() {
+  ArrowCanvas * a = this;
+  
+  while (a->begin->type() == UmlArrowPoint)
+    a = (ArrowCanvas *) ((ArrowPointCanvas *) a->begin)->get_other(a);
+    
+  for (;;) {
+    DiagramItem * di = a->begin;
+	
+    a->begin = a->end;
+    a->end = di;
+    
+    float decenter = a->decenter_begin;
+    
+    a->decenter_begin = a->decenter_end;
+    a->decenter_end = decenter;
+    
+    if (a->begin->type() != UmlArrowPoint)
+      break;
+    
+    a = (ArrowCanvas *) ((ArrowPointCanvas *) a->begin)->get_other(a);
+  } 
+}
+
 ArrowPointCanvas * ArrowCanvas::brk(const QPoint & p) {
   ArrowPointCanvas * ap =
     new ArrowPointCanvas(the_canvas(), p.x(), p.y());
@@ -843,7 +994,8 @@ ArrowPointCanvas * ArrowCanvas::brk(const QPoint & p) {
   ap->setZ(z() + 1);	// + 1 else point can't be selected
   
   ArrowCanvas * other =
-    new ArrowCanvas(the_canvas(), ap, end, itstype, 0, FALSE);
+    new ArrowCanvas(the_canvas(), ap, end, itstype, 0, FALSE, 
+		    decenter_begin, decenter_end);
 
   ap->add_line(this);
   end->remove_line(this);
@@ -964,7 +1116,7 @@ ArrowCanvas * ArrowCanvas::join(ArrowCanvas * other, ArrowPointCanvas * ap) {
     return other->ArrowCanvas::join(this, ap);
 }
 
-// the arrow is teh alone selected element and there is
+// the arrow is the alone selected element and there is
 // a mouse move event, what is must be done ?
 
 bool ArrowCanvas::cut_on_move(ArrowPointCanvas *& ap) const {
@@ -987,7 +1139,121 @@ bool ArrowCanvas::cut_on_move(ArrowPointCanvas *& ap) const {
     return FALSE;
   }
 }
+
+bool ArrowCanvas::is_decenter(QPoint mousePressPos,
+			      bool & start, bool & horiz) const {
+  switch (end->type()) {
+  case UmlArrowPoint:
+  case UmlArrowJunction:
+    break;
+  default:
+    if ((iabs(endp.x() - mousePressPos.x()) < 10) &&
+	(iabs(endp.y() - mousePressPos.y()) < 10)) {
+      // near end of line
+      start = FALSE;
+      
+      QRect r = end->rect();
+      QPoint p = endp;
+      
+      end->shift(p, beginp);
+      horiz = (iabs(endp.y() - r.top()) < 2) ||
+	(iabs(endp.y() - r.bottom()) < 2);
+      
+      return TRUE;
+    }
+  }
   
+  if ((begin->type() != UmlArrowPoint) &&
+      (iabs(beginp.x() - mousePressPos.x()) < 10) &&
+      (iabs(beginp.y() - mousePressPos.y()) < 10)) {
+    // near start of line
+    start = TRUE;
+      
+    QRect r = begin->rect();
+    QPoint p = beginp;
+    
+    begin->shift(p, end->center());
+
+    horiz = (iabs(beginp.y() - r.top()) < 2) ||
+      (iabs(beginp.y() - r.bottom()) < 2);
+      
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+
+void ArrowCanvas::decenter(QPoint p, bool start, bool horiz) {
+  float & decenter_it = (start) ? decenter_begin : decenter_end;
+  QRect r = (start) ? begin->rect() : end->rect();
+  int margin =
+    // force the position to have more than 3 point to the edge
+    // to not be < 2 and change the edge
+    (int) (9 * the_canvas()->zoom());
+  
+  if (horiz) {
+    // on top or botton
+    int nx = p.x();
+    
+    if (nx < (r.left() + margin))
+      decenter_it = margin / ((float) r.width());
+    else if (nx > (r.right() - margin))
+      decenter_it = (r.width() - margin) / ((float) r.width());
+    else
+      decenter_it = (nx - r.left()) / ((float) r.width());
+  }
+  else  {
+    // on left or right
+    int ny = p.y();
+    
+    if (ny < (r.top() + margin))
+      decenter_it = margin / ((float) r.height());
+    else if (ny > (r.bottom() - margin))
+      decenter_it = (r.height() - margin) / ((float) r.height());
+    else
+      decenter_it = (ny - r.top()) / ((float) r.height());
+  }
+  
+  decenter_it = 
+    // to have the same error after reloading
+    ((int) (decenter_it* 1000)) / 1000.0;
+  
+  propag_decenter(decenter_begin, decenter_end);
+  
+  QCanvasPolygon::setVisible(FALSE);
+  update_pos();
+  QCanvasPolygon::setVisible(TRUE);
+}
+
+void ArrowCanvas::propag_decenter(float db, float de) {
+  decenter_begin = db;
+  decenter_end = de;
+  
+  ArrowCanvas * a = this;
+  
+  while (a->begin->type() == UmlArrowPoint) {
+    a = ((ArrowPointCanvas *) a->begin)->get_other(a);
+    a->decenter_begin = db;
+    a->decenter_end = de;
+  }
+  
+  a = this;
+  
+  while (a->end->type() == UmlArrowPoint) {
+    a = ((ArrowPointCanvas *) a->end)->get_other(a);
+    a->decenter_begin = db;
+    a->decenter_end = de;
+  }
+}
+
+void ArrowCanvas::set_decenter(float db, float de) {
+  propag_decenter(db, de);
+  
+  QCanvasPolygon::setVisible(FALSE);
+  update_pos();
+  QCanvasPolygon::setVisible(TRUE);  
+}
+
 void ArrowCanvas::propag_geometry(LineGeometry geo, bool fixed) {
   geometry = geo;
   fixed_geometry = fixed;
@@ -1049,6 +1315,8 @@ ArrowCanvas * ArrowCanvas::set_geometry(LineGeometry geo, bool fixed) {
     break;
   }
   
+  ar->propag_decenter(-1.0, -1.0);
+  
   switch (geo) {
   case HVGeometry:
   case VHrGeometry:
@@ -1104,7 +1372,8 @@ ArrowCanvas * ArrowCanvas::set_geometry(LineGeometry geo, bool fixed) {
   view->protect_history(TRUE);
   return ar;
 }
-// To move te arrow point to nx,ny but 'may be not exactly'
+
+// To move the arrow point to nx,ny but 'may be not exactly'
 // to not go in an infinite loop between ArrowCanvas::update_geometry()
 // and DiagramItem::update_show_lines() because when a extremity
 // is moved this imply an update of the line attached to this point
@@ -1292,6 +1561,15 @@ void ArrowCanvas::save(QTextStream & st, bool ref, QString & warning) const {
       if (!fixed_geometry)
 	st << " unfixed";
     }
+    if (decenter_begin >= 0) {
+      // float output/input bugged
+      st << " decenter_begin " << ((int) (decenter_begin * 1000));
+    }
+    if (decenter_end >= 0) {
+      // float output/input bugged
+      st << " decenter_end " << ((int) (decenter_end * 1000));
+    }
+
     indent(+1);
     save_lines(st, TRUE, TRUE, warning);
     indent(-1);
@@ -1340,16 +1618,17 @@ const ArrowCanvas * ArrowCanvas::save_lines(QTextStream & st, bool with_label,
 }
 
 static ArrowCanvas * make(UmlCanvas * canvas, DiagramItem * b,
-			  DiagramItem * e, UmlCode t, int id)
+			  DiagramItem * e, UmlCode t, float dbegin, float dend, int id)
 {
-  return new ArrowCanvas(canvas, b, e, t, id, FALSE);
+  return new ArrowCanvas(canvas, b, e, t, id, FALSE, dbegin, dend);
 }
 
 ArrowCanvas * ArrowCanvas::read_list(char * & st, UmlCanvas * canvas,
 				     UmlCode t, LineGeometry geo,
-				     bool fixed, int id,
+				     bool fixed, float dbegin, float dend, int id,
 				     ArrowCanvas * (*pf)(UmlCanvas * canvas, DiagramItem * b,
-							 DiagramItem * e, UmlCode t, int id))
+							 DiagramItem * e, UmlCode t, 
+							 float dbegin, float dend, int id))
 {
   char * k;
     
@@ -1410,7 +1689,7 @@ ArrowCanvas * ArrowCanvas::read_list(char * & st, UmlCanvas * canvas,
 	unknown_keyword(k);
     }
     
-    result = pf(canvas, bi, di, t, id);
+    result = pf(canvas, bi, di, t, dbegin, dend, id);
     result->geometry = geo;
     result->fixed_geometry = fixed;
     result->set_z(z);
@@ -1444,20 +1723,38 @@ ArrowCanvas * ArrowCanvas::read(char * & st, UmlCanvas * canvas, char * k)
     if (! strcmp(k, "geometry")) {
       geo = line_geometry(read_keyword(st));
       k = read_keyword(st);
-      if (! strcmp(k, "unfixed"))
+      if (! strcmp(k, "unfixed")) {
+	k = read_keyword(st);
 	fixed = FALSE;
-      else {
-	fixed = TRUE;
-	unread_keyword(k, st);
       }
+      else
+	fixed = TRUE;
     }
     else {
-      unread_keyword(k, st);
       geo = NoGeometry;
       fixed = FALSE;
     }
     
-    return read_list(st, canvas, t, geo, fixed, id, &make);
+    float dbegin;
+    float dend;
+
+    if (! strcmp(k, "decenter_begin")) {
+      dbegin = read_double(st) / 1000;
+      k = read_keyword(st);
+    }
+    else
+      dbegin = -1;
+
+    if (! strcmp(k, "decenter_end")) {
+      dend = read_double(st) / 1000;
+      k = read_keyword(st);
+    }
+    else
+      dend = -1;
+    
+    unread_keyword(k, st);
+    
+    return read_list(st, canvas, t, geo, fixed, dbegin, dend, id, &make);
   }
   else
     return 0;
@@ -1477,6 +1774,8 @@ void ArrowCanvas::history_save(QBuffer & b) const {
   int v = geometry;
   
   ::save((fixed_geometry) ? v : -v, b);
+  ::save(decenter_begin, b);
+  ::save(decenter_end, b);
   ::save(label, b);
   ::save(stereotype, b);
   ::save(beginp, b);
@@ -1504,6 +1803,8 @@ void ArrowCanvas::history_load(QBuffer & b) {
     fixed_geometry = TRUE;
     geometry = (LineGeometry) geo;
   }
+  ::load(decenter_begin, b);
+  ::load(decenter_end, b);
   
   label = (LabelCanvas *) load_item(b);
   stereotype = (LabelCanvas *) load_item(b);
@@ -1565,3 +1866,47 @@ void ArrowCanvas::remove_redondant_rels()
   
   RelsToCheck.clear();
 }
+
+// for plug-out
+
+void ArrowCanvas::write_uc_rel(ToolCom * com) const {
+  // between use case an actor
+  DiagramItem * from = get_start();
+  DiagramItem * to = get_end();
+  
+  if (from->type() == UmlUseCase) {
+    com->write_unsigned((unsigned) from->get_ident());
+    to->get_bn()->write_id(com);
+    com->write_bool((itstype == UmlDirectionalAssociation)
+		    ? FALSE
+		    : (to->rect().center().x() < from->rect().center().x()));
+  }
+  else {
+    com->write_unsigned((unsigned) to->get_ident());
+    from->get_bn()->write_id(com);
+    com->write_bool((itstype == UmlDirectionalAssociation)
+		    ? TRUE
+		    : (to->rect().center().x() > from->rect().center().x()));
+  }
+  
+  ArrowCanvas * plabel;
+  ArrowCanvas * pstereotype;
+    
+  search_supports(plabel, pstereotype);
+  
+  QCString s;
+  
+  if (plabel == 0)
+    com->write_string("");
+  else {
+    s = fromUnicode(plabel->label->text());
+    com->write_string(s);
+  }
+  
+  if (pstereotype == 0)
+    com->write_string("");
+  else {
+    s = fromUnicode(pstereotype->stereotype->text());
+    com->write_string(s);
+  }
+}
