@@ -59,8 +59,10 @@ ArtifactCanvas::ArtifactCanvas(BrowserNode * bn, UmlCanvas * canvas,
   connect(bn->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
   connect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
 
-  if (canvas->must_draw_all_relations())
+  if (canvas->must_draw_all_relations()) {
     draw_all_relations();
+    draw_all_simple_relations();
+  }
 }
 
 ArtifactCanvas::ArtifactCanvas(UmlCanvas * canvas, int id)
@@ -130,7 +132,7 @@ void ArtifactCanvas::compute_size() {
   }
   
   // force odd width and height for line alignment
-  setSize(wi | 1, he | 1);
+  DiagramCanvas::resize(wi | 1, he | 1);
 }
 
 void ArtifactCanvas::change_scale() {  
@@ -146,22 +148,31 @@ void ArtifactCanvas::modified() {
   compute_size();
   show();
   update_show_lines();
-  if (the_canvas()->must_draw_all_relations())
+  if (the_canvas()->must_draw_all_relations()) {
     draw_all_relations();
+    draw_all_simple_relations();
+  }
   canvas()->update();
   package_modified();
 }
 
+void ArtifactCanvas::post_loaded() {
+  if (the_canvas()->must_draw_all_relations()) {
+    draw_all_relations();
+    draw_all_simple_relations();
+  }
+}
+
 void ArtifactCanvas::connexion(UmlCode action, DiagramItem * dest,
 				const QPoint &, const QPoint &) {
   ArrowCanvas * a;
   
   if (action == UmlContain)
-    a = new AssocContainCanvas(the_canvas(), this, dest, 0);
+    a = new AssocContainCanvas(the_canvas(), this, dest, 0, -1.0, -1.0);
   else if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
@@ -191,7 +202,8 @@ void ArtifactCanvas::draw_all_relations() {
   if (strcmp(browser_node->get_stereotype(), "source") != 0)
     // may start association
     update_relations();
-  else if (!DrawingSettings::just_modified()) {
+  else if (!DrawingSettings::just_modified() &&
+	   !on_load_diagram()) {
     // remove all association starting from 'this'
     QListIterator<ArrowCanvas> it(lines);
     
@@ -242,7 +254,8 @@ void ArtifactCanvas::update_relations(ArtifactCanvas * other) {
   // association not yet exist
   
   if (association_must_exist)
-    (new AssocContainCanvas(the_canvas(), this, other, 0))->show();
+    (new AssocContainCanvas(the_canvas(), this, other, 0, -1.0, -1.0))
+      ->show();
 }
 
 void ArtifactCanvas::update_relations() {
@@ -295,7 +308,8 @@ void ArtifactCanvas::update_relations() {
 		(adi->type() == UmlArtifact) &&
 		(((ArtifactCanvas *) adi)->browser_node == it.current())) {
 	      // find
-	      (new AssocContainCanvas(the_canvas(), this, adi, 0))->show();
+	      (new AssocContainCanvas(the_canvas(), this, adi, 0, -1.0, -1.0))
+		->show();
 	      break;
 	    }
 	  }
@@ -365,9 +379,9 @@ void ArtifactCanvas::draw(QPainter & p) {
     p.fillRect(r, co);
 
     if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff, 
+	      svg_color(used_color), 
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
   }
   else if (fp != 0)
@@ -424,7 +438,7 @@ void ArtifactCanvas::draw(QPainter & p) {
   p.lineTo(re.right(), re.top() + corner_size);
 
   if (fp != 0) {
-    draw_poly(fp, a, "none");
+    draw_poly(fp, a, UmlTransparent);
     fprintf(fp, "\t<line stroke=\"black\" stroke-opacity=\"1\""
 	    " x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n"
 	    "</g>\n",
