@@ -71,7 +71,6 @@ CdClassCanvas::CdClassCanvas(BrowserNode * bn, UmlCanvas * canvas,
   constraint = 0;
   
   compute_size();	// update used_settings
-  set_center100();
   check_constraint();
   
   subscribe(bn->get_data());	// = TRUE
@@ -201,6 +200,7 @@ void CdClassCanvas::compute_size() {
   int noper = 0;
   bool full_members = (used_settings.show_full_members_definition == UmlYes);
   bool show_visibility = (used_settings.show_members_visibility == UmlYes);
+  bool show_stereotype = (used_settings.show_members_stereotype == UmlYes);
   bool show_dir = (used_settings.show_parameter_dir == UmlYes);
   bool show_name = (used_settings.show_parameter_name == UmlYes);
   bool hide_attrs = (used_settings.hide_attributes == UmlYes);
@@ -303,11 +303,19 @@ void CdClassCanvas::compute_size() {
 	  if (subscribe(child_data))
 	    connect(child_data, SIGNAL(changed()), this, SLOT(modified()));
 	}
-	else if (full_members) {
-	  if (subscribe(child_data))
-	    connect(child_data, SIGNAL(changed()), this, SLOT(modified()));
+	
+	if (show_stereotype) {
+	  QString st = child_data->get_stereotype();
+	  
+	  if (! st.isEmpty()) 
+	    wa += fm.width("<<" + st + ">>_");
 	}
 	
+	if ((full_members || show_visibility || show_stereotype) &&
+	    subscribe(child_data))
+	  connect(child_data, SIGNAL(changed()), this, SLOT(modified()));
+	
+	
 	if (wa > wi)
 	  wi = wa;
       }
@@ -381,7 +389,7 @@ void CdClassCanvas::compute_size() {
   }
   
   // force odd width and height for line alignment
-  setSize(wi | 1, he | 1);
+  DiagramCanvas::resize(wi | 1, he | 1);
     
   if (tmpl) {
     if (templ == 0) {
@@ -407,6 +415,7 @@ void CdClassCanvas::modified() {
     show();
     update_show_lines();
     force_self_rel_visible();
+    check_inner();
     if (the_canvas()->must_draw_all_relations()) {
       draw_all_relations();    
       draw_all_simple_relations();
@@ -417,6 +426,34 @@ void CdClassCanvas::modified() {
   }
 }
 
+void CdClassCanvas::post_loaded() {
+  force_self_rel_visible();
+  check_inner();
+  if (the_canvas()->must_draw_all_relations()) {
+    draw_all_relations();    
+    draw_all_simple_relations();
+  }
+  check_constraint();
+}
+
+void CdClassCanvas::check_inner() {
+  QListIterator<ArrowCanvas> it(lines);
+	
+  while (it.current()) {
+    if (it.current()->type() == UmlInner) {
+      DiagramItem * b = ((ArrowCanvas *) it.current())->get_start();
+      DiagramItem * e = ((ArrowCanvas *) it.current())->get_end();
+      
+      if (((BrowserNode *) b->get_bn()->parent()) != e->get_bn())
+	it.current()->delete_it();
+      else
+	++it;
+    }
+    else
+      ++it;
+  }  
+}
+
 bool CdClassCanvas::has_relation(BasicData * def) const {
   QListIterator<ArrowCanvas> it(lines);
 	
@@ -430,52 +467,85 @@ bool CdClassCanvas::has_relation(BasicData * def) const {
   return FALSE;
 }
 
+bool CdClassCanvas::has_inner(DiagramItem * end) const {
+  QListIterator<ArrowCanvas> it(lines);
+	
+  while (it.current()) {
+    if ((it.current()->type() == UmlInner) &&
+	(((ArrowCanvas *) it.current())->get_end() == end))
+      return TRUE;
+    ++it;
+  }
+  
+  return FALSE;
+}
+
 void CdClassCanvas::draw_all_relations(CdClassCanvas * end) {
   QListViewItem * child;
   QCanvasItemList all = canvas()->allItems();
   QCanvasItemList::Iterator cit;
   
   for (child = browser_node->firstChild(); child; child = child->nextSibling()) {
-    if (IsaRelation(((BrowserNode *) child)->get_type()) &&
-	!((BrowserNode *) child)->deletedp()) {
-      RelationData * def =
-	((RelationData *) ((BrowserNode *) child)->get_data());
-
-      if ((def->get_start_class() == browser_node) && 	// rel begins by this
-	  ((end == 0) || (def->get_end_class() == end->browser_node)) &&
-	  !has_relation(def)) {
-	// adds it in case the other class is drawed
-	BrowserClass * end_class = 
-	  ((BrowserClass *) def->get_end_class());
-	DiagramItem * di;
+    if (!((BrowserNode *) child)->deletedp()) {
+      UmlCode k = ((BrowserNode *) child)->get_type();
+      
+      if (IsaRelation(k)) {
+	RelationData * def =
+	  ((RelationData *) ((BrowserNode *) child)->get_data());
 	
-	if (end_class == browser_node)
-	  di = this;
-	else {	
-	  di = 0;
-	  for (cit = all.begin(); cit != all.end(); ++cit) {
-	    DiagramItem * adi = QCanvasItemToDiagramItem(*cit);
-	    
-	    if ((adi != 0) &&		// an uml canvas item
-		(adi->type() == UmlClass) &&
-		(((CdClassCanvas *) adi)->browser_node == end_class) &&
-		((((CdClassCanvas *) adi) == end) || (*cit)->visible())) {
-	      // other class canvas find
-	      di = adi;
-	      break;
+	if ((def->get_start_class() == browser_node) && 	// rel begins by this
+	    ((end == 0) || (def->get_end_class() == end->browser_node)) &&
+	    !has_relation(def)) {
+	  // adds it in case the other class is drawed
+	  BrowserClass * end_class = 
+	    ((BrowserClass *) def->get_end_class());
+	  DiagramItem * di;
+	  
+	  if (end_class == browser_node)
+	    di = this;
+	  else {	
+	    di = 0;
+	    for (cit = all.begin(); cit != all.end(); ++cit) {
+	      DiagramItem * adi = QCanvasItemToDiagramItem(*cit);
+	      
+	      if ((adi != 0) &&		// an uml canvas item
+		  (adi->type() == UmlClass) &&
+		  (((CdClassCanvas *) adi)->browser_node == end_class) &&
+		  ((((CdClassCanvas *) adi) == end) || (*cit)->visible())) {
+		// other class canvas find
+		di = adi;
+		break;
+	      }
 	    }
 	  }
+	  
+	  if (di != 0)
+	    (new RelationCanvas(the_canvas(), this, di,
+				((BrowserClass *) browser_node), 
+				def->get_type(), 0, -1.0, -1.0, def))->show();
+	}
+      }
+      else if (k == UmlClass) {
+	for (cit = all.begin(); cit != all.end(); ++cit) {
+	  DiagramItem * adi = QCanvasItemToDiagramItem(*cit);
+	  
+	  if ((adi != 0) &&		// an uml canvas item
+	      (adi->type() == UmlClass) &&
+	      (((CdClassCanvas *) adi)->browser_node == child) &&
+	      ((((CdClassCanvas *) adi) == end) || (*cit)->visible())) {
+	    if (! ((CdClassCanvas *) adi)->has_inner(this))
+	      (new ArrowCanvas(the_canvas(), adi, this, UmlInner, 0, FALSE, -1.0, -1.0))
+		->show();
+	    break;
+	  }
 	}
-	
-	if (di != 0)
-	  (new RelationCanvas(the_canvas(), this, di,
-			      ((BrowserClass *) browser_node), 
-			      def->get_type(), 0, def))->show();
       }
     }
   }
   
-  if ((end == 0) && !DrawingSettings::just_modified()) {
+  if ((end == 0) &&
+      !DrawingSettings::just_modified() &&
+      !on_load_diagram()) {
     for (cit = all.begin(); cit != all.end(); ++cit) {
       DiagramItem * di = QCanvasItemToDiagramItem(*cit);
       
@@ -483,8 +553,9 @@ void CdClassCanvas::draw_all_relations(CdClassCanvas * end) {
 	  (di->type() == UmlClass) &&
 	  (((CdClassCanvas *) di) != this) &&
 	  !((CdClassCanvas *) di)->browser_node->deletedp() &&
-	  ((CdClassCanvas *) di)->visible())
+	  ((CdClassCanvas *) di)->visible()) {
 	((CdClassCanvas *) di)->draw_all_relations(this);
+      }
     }
   }
 }
@@ -531,7 +602,7 @@ void CdClassCanvas::check_constraint() {
       constraint->move((int) (x() + width() + the_canvas()->zoom() * 20),
 		       (int) y() + height());
       constraint->show();
-      (new ArrowCanvas(the_canvas(), this, constraint, UmlAnchor, 0, FALSE))->show();
+      (new ArrowCanvas(the_canvas(), this, constraint, UmlAnchor, 0, FALSE, -1.0, -1.0))->show();
     }
   }
   else if (constraint != 0) {
@@ -571,6 +642,7 @@ void CdClassCanvas::draw(QPainter & p) {
   if (! visible()) return;
   
   QRect r = rect();
+  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
   QFontMetrics fbm(the_canvas()->get_font(UmlNormalBoldFont));
   QFontMetrics fbim(the_canvas()->get_font(UmlNormalBoldItalicFont));
   QColor bckgrnd = p.backgroundColor();
@@ -619,9 +691,9 @@ void CdClassCanvas::draw(QPainter & p) {
       p.fillRect(r, co);
 
       if (fp != 0)
-	fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+	fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 		" x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-		co.rgb()&0xffffff, 
+		svg_color(used_color), 
 		r.x(), r.y(), r.width() - 1, r.height() - 1);
     }
     else if (fp != 0)
@@ -704,9 +776,11 @@ void CdClassCanvas::draw(QPainter & p) {
   int left0 = r.left();
   int left1 = left0 + (int) (4 * zoom);
   int left2 = left1 + fbim.width("#_");
+  int space = fm.width("_");
   QListViewItem * child;
   bool full_members = (used_settings.show_full_members_definition == UmlYes);
   bool show_visibility = (used_settings.show_members_visibility == UmlYes);
+  bool show_stereotype = (used_settings.show_members_stereotype == UmlYes);
   bool show_dir = (used_settings.show_parameter_dir == UmlYes);
   bool show_name = (used_settings.show_parameter_name == UmlYes);
   
@@ -743,6 +817,18 @@ void CdClassCanvas::draw(QPainter & p) {
 	    draw_text(r, QObject::AlignLeft + QObject::AlignTop, v[vi], p.font(), fp);
 	  r.setLeft(left2);
 	}
+	if (show_stereotype) {
+	  QString st = data->get_stereotype();
+	  
+	  if (! st.isEmpty()) {
+	    st = "<<" + st + ">>";
+	    p.setFont(the_canvas()->get_font(UmlNormalFont));
+	    p.drawText(r, QObject::AlignLeft + QObject::AlignTop, st);
+	    if (fp != 0)
+	      draw_text(r, QObject::AlignLeft + QObject::AlignTop, st, p.font(), fp);
+	    r.setLeft(r.left() + fm.width(st) + space);
+	  }
+	}
 	p.setFont((data->get_isa_class_attribute()) ? the_canvas()->get_font(UmlNormalUnderlinedFont)
 						    : the_canvas()->get_font(UmlNormalFont));
 	p.drawText(r, QObject::AlignLeft + QObject::AlignTop, s);
@@ -799,6 +885,18 @@ void CdClassCanvas::draw(QPainter & p) {
 	      draw_text(r, QObject::AlignLeft + QObject::AlignTop, v[vi], p.font(), fp);
 	    r.setLeft(left2);
 	  }
+	  if (show_stereotype) {
+	    QString st = data->get_stereotype();
+	    
+	    if (! st.isEmpty()) {
+	      st = "<<" + st + ">>";
+	      p.setFont(the_canvas()->get_font(UmlNormalFont));
+	      p.drawText(r, QObject::AlignLeft + QObject::AlignTop, st);
+	      if (fp != 0)
+		draw_text(r, QObject::AlignLeft + QObject::AlignTop, st, p.font(), fp);
+	      r.setLeft(r.left() + fm.width(st) + space);
+	    }
+	  }
 	  if (data->get_isa_class_operation())
 	    p.setFont(the_canvas()->get_font(UmlNormalUnderlinedFont));
 	  else if (data->get_is_abstract())
@@ -1198,11 +1296,11 @@ void CdClassCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * a;
   
   if (IsaRelation(action))
-    a = new RelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new RelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
