@@ -81,14 +81,14 @@ void ClassDiagramView::menu(const QPoint& p) {
 	  (((BrowserNode *) child)->get_type() == UmlClass) &&
 	  (drawn[(BrowserNode *) child] == 0)) {
 	// not already shown
-	m.insertItem("Add classes of the selected class view", 19);
+	m.insertItem("Add classes of the selected class view", 29);
 	m.insertSeparator();
 	break;
       }
     }
   }
   
-  switch (default_menu(m, 20)) {
+  switch (default_menu(m, 30)) {
   case EDIT_DRAWING_SETTING_CMD:
     ((BrowserClassDiagram *) ((UmlCanvas *) canvas())->browser_diagram())->edit_settings();
     break;
@@ -99,7 +99,7 @@ void ClassDiagramView::menu(const QPoint& p) {
     load("Class");
     window()->package_modified();
     break;
-  case 19:
+  case 29:
     add_classview_classes(bn, p, drawn);
     break;
   }
@@ -198,10 +198,8 @@ void ClassDiagramView::dragEnterEvent(QDragEnterEvent * e) {
        UmlDrag::canDecode(e, UmlObjectDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlComponentDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlDeploymentDiagram, FALSE, TRUE) ||
-       UmlDrag::canDecode(e, UmlActivityDiagram, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlStateDiagram, TRUE, TRUE) ||
-       UmlDrag::canDecode(e, UmlActivityDiagram, TRUE, TRUE) ||
-       UmlDrag::canDecode(e, UmlStateDiagram, TRUE, TRUE)))
+       UmlDrag::canDecode(e, UmlActivityDiagram, TRUE, TRUE)))
     e->accept();
   else
     e->ignore();
@@ -306,10 +304,14 @@ void ClassDiagramView::save(QTextStream & st, QString & warning,
 
   // then saves relations
   
-  for (di = items.first(); di != 0; di = items.next())
-    if ((!copy || di->copyable()) &&
-	(IsaRelation(di->type()) || IsaSimpleRelation(di->type())))
-      di->save(st, FALSE, warning);
+  for (di = items.first(); di != 0; di = items.next()) {
+    if (!copy || di->copyable()) {
+      UmlCode k = di->type();
+      
+      if (IsaRelation(k) || IsaSimpleRelation(k) || (k == UmlInner))
+	di->save(st, FALSE, warning);
+    }
+  }
   
   // then saves anchors
   
