@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -205,14 +205,21 @@ void CodSelfLinkCanvas::moveBy(double dx, double dy) {
 void CodSelfLinkCanvas::draw(QPainter & p) {
   if (! visible()) return;
   
+  QPoint ce = center();
+
   p.save();
-  p.translate(center().x(), center().y());
-  
+  p.translate(ce.x(), ce.y());
   p.rotate(angle);
   
   QRect r(-width() / 2, -width() / 4, width() - 1, width() / 2);
   
   p.drawEllipse(r);
+
+  FILE * fp = svg();
+
+  if (fp != 0)
+    fprintf(fp, "<ellipse fill=\"none\" stroke=\"black\" stroke-opacity=\"1\" transform=\"translate(%d %d) rotate(%d)\" rx=\"%d\" ry=\"%d\" />\n",
+	    (int) ce.x(), (int) ce.y(), (int) angle, width() / 2, width() / 4);
   
   if (selected())
     show_mark(p, r);
@@ -288,14 +295,8 @@ const char * CodSelfLinkCanvas::may_start(UmlCode & l) const {
   return (l == UmlAnchor) ? 0 : "illegal";
 }
 
-const char * CodSelfLinkCanvas::may_connect(UmlCode & l, const DiagramItem * other) const {
-  switch (other->type()) {
-  case UmlNote:
-  case UmlIcon:
-    return (l == UmlAnchor) ? 0 : "illegal";
-  default:
-    return "illegal";
-  }
+const char * CodSelfLinkCanvas::may_connect(UmlCode & l, const DiagramItem * dest) const {
+  return (l == UmlAnchor) ? dest->may_start(l) : "illegal";
 }
     
 bool CodSelfLinkCanvas::copyable() const {
