@@ -40,6 +40,7 @@ class ColDiagramWindow : public DiagramWindow {
   protected:
     ColDiagramView * view;
     QToolButton * addClass;
+    QToolButton * addClassInstance;
     QToolButton * addPackage;
     QToolButton * addFragment;
     QToolButton * addLink;
@@ -58,6 +59,7 @@ class ColDiagramWindow : public DiagramWindow {
 
   protected slots:
     void hit_class();
+    void hit_classinstance();
     void hit_package();
     void hit_fragment();
     void hit_link();
