@@ -173,7 +173,7 @@ void ComponentCanvas::compute_size() {
     BrowserComponent * comp = (BrowserComponent *) browser_node;
     const QValueList<BrowserClass *> & pr = comp->get_provided_classes();
     const QValueList<BrowserClass *> & rq = comp->get_required_classes();
-    const QValueList<BrowserClass *> & rz = comp->get_realized_classes();
+    const QValueList<BrowserClass *> & rz = comp->get_realizing_classes();
     int lh = fm.height() + four;	// line height
     
     if (req_prov && (!pr.isEmpty() || !rq.isEmpty())) {
@@ -272,18 +272,20 @@ void ComponentCanvas::change_scale() {
 }
 
 bool ComponentCanvas::valid(ArrowCanvas * a) const {
+  QValueList<BrowserClass *> l;
+  
   switch (a->type()) {
   case UmlRequired:
-    return (((BrowserComponent *) browser_node)->get_all_required_classes(FALSE)
-	    .findIndex(((ArrowJunctionCanvas *) a->get_end())->get_interface())
-	    != -1);
+    ((BrowserComponent *) browser_node)->get_all_required_classes(l, FALSE);
+    break;
   case UmlProvided: 
-    return (((BrowserComponent *) browser_node)->get_all_provided_classes(FALSE)
-	    .findIndex(((ArrowJunctionCanvas *) a->get_end())->get_interface())
-	    != -1);
+    ((BrowserComponent *) browser_node)->get_all_provided_classes(l, FALSE);
+    break;
   default:
     return TRUE;
   }
+  
+  return (l.findIndex(((ArrowJunctionCanvas *) a->get_end())->get_interface()) != -1);
 }
 
 void ComponentCanvas::modified() {
@@ -313,6 +315,12 @@ void ComponentCanvas::modified() {
   package_modified();
 }
 
+void ComponentCanvas::post_loaded() {
+  force_self_rel_visible();
+  if (the_canvas()->must_draw_all_relations())
+    draw_all_simple_relations();
+}
+
 void ComponentCanvas::check_line(ArrowCanvas * l) {
   if (!valid(l))
     Undefined.append(l);
@@ -320,9 +328,13 @@ void ComponentCanvas::check_line(ArrowCanvas * l) {
 
 bool ComponentCanvas::connexion(UmlCode action, const QPoint &, const QPoint & p) {
   BrowserComponent * comp = (BrowserComponent *) browser_node;
-  QValueList<BrowserClass *> l = 
-    (action == UmlProvided) ? comp->get_all_provided_classes(TRUE)
-			    : comp->get_all_required_classes(TRUE);
+  QValueList<BrowserClass *> l;
+   
+  if (action == UmlProvided) 
+    comp->get_all_provided_classes(l, TRUE);
+  else
+    comp->get_all_required_classes(l, TRUE);
+  
   ClassListDialog dialog("Choose class", l);
     
   if (dialog.exec() != QDialog::Accepted)
@@ -333,9 +345,10 @@ bool ComponentCanvas::connexion(UmlCode action, const QPoint &, const QPoint & p
     new ArrowJunctionCanvas(the_canvas(), p.x(), p.y(), *it, 0); 
   
   aj->show();
+  aj->upper();
   
   ArrowCanvas * a = 
-    new ArrowCanvas(the_canvas(), this, aj, action, 0, FALSE);
+    new ArrowCanvas(the_canvas(), this, aj, action, 0, FALSE, -1.0, -1.0);
   
   a->show(); 
   the_canvas()->select(a);
@@ -348,9 +361,9 @@ void ComponentCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * a;
   
   if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
@@ -367,6 +380,29 @@ void ComponentCanvas::resize(aCorner c, int dx, int dy) {
     DiagramCanvas::resize(c, dx, dy, min_width, min_height);
 }
 
+void ComponentCanvas::prepare_for_move(bool on_resize) {
+  if (! on_resize) {
+    DiagramCanvas::prepare_for_move(on_resize);
+    
+    QCanvasItemList l = collisions(TRUE);
+    QCanvasItemList::ConstIterator it;
+    QCanvasItemList::ConstIterator end = l.end();
+    DiagramItem * di;
+    BrowserNode * p = get_bn();
+  
+    for (it = l.begin(); it != end; ++it) {
+      if ((*it)->visible() && // at least not deleted
+	  !(*it)->selected() &&
+	  ((di = QCanvasItemToDiagramItem(*it)) != 0) &&
+	  (di->type() == UmlComponent) &&
+	  (di->get_bn()->parent() == p)) {
+	the_canvas()->select(*it);
+	di->prepare_for_move(FALSE);
+      }
+    }
+  }
+}
+
 bool ComponentCanvas::move_with_its_package() const {
   return TRUE;
 }
@@ -394,18 +430,13 @@ void ComponentCanvas::draw(QPainter & p) {
     
     r.setLeft(r.left() + (he * 5)/(13*2));
 
-    if (used_color != UmlTransparent) {
+    if (used_color != UmlTransparent)
       p.fillRect(r, co);
       
-      if (fp != 0)
-	fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
-		" x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-		co.rgb()&0xffffff,
-		r.x(), r.y(), r.width() - 1, r.height() - 1);
-    }
-    else if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+    if (fp != 0)
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	      svg_color(used_color),
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
 
     p.drawRect(r);
@@ -438,18 +469,18 @@ void ComponentCanvas::draw(QPainter & p) {
     p.fillRect(r, co);
     p.drawRect(r);
     if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff,
+	      svg_color((used_color != UmlTransparent) ? used_color : UmlWhite),
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
     
     r.moveBy(0, (he * 4)/13);
     p.fillRect(r, co);
     p.drawRect(r);
     if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff,
+	      svg_color((used_color != UmlTransparent) ? used_color : UmlWhite),
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
   }
   else {
@@ -492,18 +523,12 @@ void ComponentCanvas::draw(QPainter & p) {
   
     QRect re = r;
   
-    if (used_color != UmlTransparent) {
+    if (used_color != UmlTransparent)
       p.fillRect(r, co);
-
-      if (fp != 0)
-	fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
-		" x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-		co.rgb()&0xffffff, 
-		r.x(), r.y(), r.width() - 1, r.height() - 1);
-    }
-    else if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+    if (fp != 0)
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	      svg_color(used_color), 
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
 
     p.drawRect(r);
@@ -558,18 +583,18 @@ void ComponentCanvas::draw(QPainter & p) {
     p.fillRect(r2, co);
     p.drawRect(r2);
     if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff, 
+	      svg_color(used_color), 
 	      r2.x(), r2.y(), r2.width() - 1, r2.height() - 1);
     
     r2.moveBy(0, r.width() >> 1);
     p.fillRect(r2, co);
     p.drawRect(r2);
     if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff, 
+	      svg_color(used_color), 
 	      r2.x(), r2.y(), r2.width() - 1, r2.height() - 1);
     
     // compartments
@@ -577,7 +602,7 @@ void ComponentCanvas::draw(QPainter & p) {
     BrowserComponent * comp = (BrowserComponent *) browser_node;
     const QValueList<BrowserClass *> & pr = comp->get_provided_classes();
     const QValueList<BrowserClass *> & rq = comp->get_required_classes();
-    const QValueList<BrowserClass *> & rz = comp->get_realized_classes();
+    const QValueList<BrowserClass *> & rz = comp->get_realizing_classes();
     int lh = fm.height() + four;	// line height
     
     r = re;
@@ -859,12 +884,20 @@ const char * ComponentCanvas::may_start(UmlCode & l) const {
     l = UmlInherit;
     return (browser_node->is_writable()) ? 0 : "read only";
   case UmlProvided:
-    return (!((BrowserComponent *) browser_node)->get_all_provided_classes(FALSE).isEmpty())
-      ? 0 : "no provided interfaces";
+    {
+      QValueList<BrowserClass *> ll;
+      
+      ((BrowserComponent *) browser_node)->get_all_provided_classes(ll, FALSE);
+      return (!ll.isEmpty()) ? 0 : "no provided interfaces";
+    }
     break;
   case UmlRequired:
-    return (!((BrowserComponent *) browser_node)->get_all_required_classes(FALSE).isEmpty())
-      ? 0 : "no required interfaces";
+    {
+      QValueList<BrowserClass *> ll;
+      
+      ((BrowserComponent *) browser_node)->get_all_required_classes(ll, FALSE);
+      return (!ll.isEmpty()) ? 0 : "no required interfaces";
+    }
     break;
   case UmlAnchor:
     return 0;
@@ -900,15 +933,21 @@ const char * ComponentCanvas::may_connect(UmlCode & l, const DiagramItem * dest)
   case UmlArrowJunction:
     switch (l) {
     case UmlRequired:
-      return (((BrowserComponent *) browser_node)->get_all_required_classes(FALSE)
-	      .findIndex(((ArrowJunctionCanvas *) dest)->get_interface())
-	      != -1)
+    {
+      QValueList<BrowserClass *> ll;
+      
+      ((BrowserComponent *) browser_node)->get_all_required_classes(ll, FALSE);
+      return (ll.findIndex(((ArrowJunctionCanvas *) dest)->get_interface()) != -1)
 	? 0 : "not required";
+    }
     case UmlProvided:
-      return (((BrowserComponent *) browser_node)->get_all_provided_classes(FALSE)
-	      .findIndex(((ArrowJunctionCanvas *) dest)->get_interface())
-	      != -1)
+    {
+      QValueList<BrowserClass *> ll;
+      
+      ((BrowserComponent *) browser_node)->get_all_provided_classes(ll, FALSE);
+      return (ll.findIndex(((ArrowJunctionCanvas *) dest)->get_interface()) != -1)
 	? 0 : "not provided";
+    }
     default:
       return "illegal";
     }
