@@ -74,7 +74,7 @@ ConstraintCanvas::ConstraintCanvas(UmlCanvas * canvas, CdClassCanvas * a, QStrin
   if (h < height())
     h = height();
 
-  setSize(w, h);
+  DiagramCanvas::resize(w, h);
   setZ(a->get_z());
 
   width_scale100 = w;
@@ -359,7 +359,7 @@ void ConstraintCanvas::history_hide() {
 }
 
 void ConstraintCanvas::history_load(QBuffer & b) {
-  DiagramCanvas::history_load(b);
+  NoteCanvas::history_load(b);
   connect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(update()));
   
   QValueList<BasicData *>::Iterator iter;
