@@ -73,7 +73,7 @@ void DeploymentNodeCanvas::deleted() {
   canvas()->update();
 }
 
-const QString & DeploymentNodeCanvas::get_name() const {
+QString DeploymentNodeCanvas::get_name() const {
   return iname;
 }
 
@@ -242,7 +242,7 @@ void DeploymentNodeCanvas::draw(QPainter & p) {
       fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
-      draw_poly(fp, a, "none");
+      draw_poly(fp, a, UmlTransparent);
     }
   }
   else {
@@ -254,11 +254,11 @@ void DeploymentNodeCanvas::draw(QPainter & p) {
     p.setBackgroundColor(co);
 
     if (fp != 0) {
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff,
+	      svg_color(c),
 	      r.x(), r.y(), r.width() - 1, r.height() - 1);
-      draw_poly(fp, a, co);
+      draw_poly(fp, a, c);
     }
   }
 
@@ -513,9 +513,9 @@ void DeploymentNodeCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * a;
   
   if (action == UmlDependOn)
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
