@@ -38,13 +38,14 @@
 #include <qpaintdevicemetrics.h>
 #include <qfiledialog.h>
 #include <qbuffer.h>
-#ifdef WIN32
+#if defined(WIN32) || (QT_VERSION != 230)
 #include <qtimer.h>
 #endif
 
 #include "DiagramView.h"
 #include "DiagramWindow.h"
 #include "DiagramCanvas.h"
+#include "BrowserView.h"
 #include "RelationCanvas.h"
 #include "SimpleRelationCanvas.h"
 #include "TransitionCanvas.h"
@@ -80,10 +81,12 @@ UmlCode DiagramView::copied_from;
 
 DiagramView::DiagramView(QWidget * parent, UmlCanvas * canvas, int i)
     : QCanvasView(canvas, parent), id(i), selectArea(0), start(0),
-      line(0), draw_line(FALSE), arrowBeginning(0), do_resize(NoCorner),
-      preferred_zoom(0), history_protected(FALSE), history_frozen(FALSE),
-      first_move(FALSE), history_index(~0u) {
+      line(0), arrowBeginning(0), preferred_zoom(0), draw_line(FALSE),
+      do_resize(NoCorner), history_protected(FALSE), history_frozen(FALSE),
+      first_move(FALSE), on_arrow_decenter(FALSE), history_index(~0u) {
   // enableClipper(TRUE); => probleme d'affichage
+  preferred_size.setWidth(0);
+  preferred_size.setHeight(0);
   setStaticBackground(TRUE);
   setAcceptDrops(TRUE);
   setDragAutoScroll(TRUE);
@@ -353,6 +356,8 @@ void DiagramView::contentsMouseReleaseEvent(QMouseEvent * e) {
   if (!window()->frozen()) {
     if (do_resize != NoCorner)
       do_resize = NoCorner;
+    else if (on_arrow_decenter)
+      on_arrow_decenter = FALSE;
     else if (line != 0) {
       history_protected = TRUE;
       the_canvas()->del(line);
@@ -378,6 +383,17 @@ void DiagramView::contentsMouseReleaseEvent(QMouseEvent * e) {
 	    unsetCursor();
 	    arrowBeginning->post_connexion(action, i);
 	  }
+	  else if (start->may_connect(action)) {
+	    // component required/provided interface
+	    if (start->connexion(action, mousePressPos, e->pos())) {
+	      window()->package_modified();
+	      temp.clear();
+	      draw_line = FALSE;
+	      unsetCursor();
+	    }
+	    else
+	      abort_line_construction();
+	  }
 	  else if (strcmp(err, "illegal")) {
 	    msg_critical("Bouml", err);
 	    abort_line_construction();
@@ -453,22 +469,22 @@ void DiagramView::add_point(QMouseEvent * e) {
   ArrowCanvas * a;
   
   if ((t == UmlClassDiagram) && IsaRelation(action))
-    a = new RelationCanvas(the_canvas(), start, ap, 0, action, 0);
+    a = new RelationCanvas(the_canvas(), start, ap, 0, action, 0, -1.0, -1.0);
   else if ((t == UmlColDiagram) && (action == UmlLink))
-    a = new CodLinkCanvas(the_canvas(), start, ap, 0);
+    a = new CodLinkCanvas(the_canvas(), start, ap, 0, -1.0, -1.0);
   else if ((t == UmlObjectDiagram) && (action == UmlObjectLink))
-    a = new ObjectLinkCanvas(the_canvas(), start, ap, 0);
+    a = new ObjectLinkCanvas(the_canvas(), start, ap, UmlObjectLink, 0, -1.0, -1.0);
   else if (((t == UmlComponentDiagram) || (t == UmlDeploymentDiagram))
 	   && (action == UmlContain))
-    a = new AssocContainCanvas(the_canvas(), start, ap, 0);
+    a = new AssocContainCanvas(the_canvas(), start, ap, 0, -1.0, -1.0);
   else if ((t == UmlStateDiagram) && (action == UmlTransition))
-    a = new TransitionCanvas(the_canvas(), start, ap, 0, 0);
+    a = new TransitionCanvas(the_canvas(), start, ap, 0, 0, -1.0, -1.0);
   else if ((t == UmlActivityDiagram) && (action == UmlFlow))
-    a = new FlowCanvas(the_canvas(), start, ap, 0, 0);
+    a = new FlowCanvas(the_canvas(), start, ap, 0, 0, -1.0, -1.0);
   else if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), start, ap, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), start, ap, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), start, ap, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), start, ap, action, 0, FALSE, -1.0, -1.0);
   
   temp.append(a);		// before the point, see abort_line_construction()
   temp.append(ap);
@@ -511,7 +527,17 @@ void DiagramView::contentsMouseMoveEvent(QMouseEvent * e) {
 	  ArrowCanvas * a = ((ArrowCanvas *) selection().first());
 	  ArrowPointCanvas * ap;
 	  
-	  if (a->cut_on_move(ap)) {
+	  if (on_arrow_decenter) {
+	    a->decenter(e->pos(), decenter_start, decenter_horiz);
+	    window()->package_modified();
+	  }
+	  else if (a->is_decenter(mousePressPos, decenter_start, decenter_horiz)) {
+	    on_arrow_decenter = TRUE;
+	    history_save();
+	    a->decenter(e->pos(), decenter_start, decenter_horiz);
+	    window()->package_modified();
+	  }
+	  else if (a->cut_on_move(ap)) {
 	    // cuts the line in two adding an ArrowPoint which
 	    // becomes the alone selected item allowing to move it
 	    history_save();
@@ -647,7 +673,7 @@ void DiagramView::relation_to_simplerelation(UmlCode k) {
     ap->upper();
   
     ArrowCanvas * a = 
-      new SimpleRelationCanvas(the_canvas(), start, ap, 0, k, 0);
+      new SimpleRelationCanvas(the_canvas(), start, ap, 0, k, 0, -1.0, -1.0);
     
     ap->show();
     a->show();
@@ -922,11 +948,14 @@ void DiagramView::moveSelected(int dx, int dy, bool first) {
 void DiagramView::keyPressEvent(QKeyEvent * e) {
   if (draw_line) {
     abort_line_construction();
+    e->ignore();
   }
   else if (!window()->frozen()) {
     QString s = Shortcut::shortcut(e->key(), e->state());
   
     if (!s.isEmpty()) {
+      e->accept();
+
       if (s == "Move left") {
 	history_protected = TRUE;
 	if (first_move) {
@@ -978,7 +1007,6 @@ void DiagramView::keyPressEvent(QKeyEvent * e) {
       else if (s == "Select all") {
 	history_protected = TRUE;
 	select_all();	// clear history_protected
-	e->ignore();
 	return;
       }
       else if (s == "Copy") {
@@ -1004,7 +1032,6 @@ void DiagramView::keyPressEvent(QKeyEvent * e) {
 	  undo();
 	else
 	  QApplication::beep();
-	e->ignore();
 	return;
       }
       else if (s == "Redo") {
@@ -1013,7 +1040,6 @@ void DiagramView::keyPressEvent(QKeyEvent * e) {
 	  redo();
 	else
 	  QApplication::beep();
-	e->ignore();
 	return;
       }
       else if (s == "Save") {
@@ -1164,8 +1190,11 @@ void DiagramView::keyPressEvent(QKeyEvent * e) {
       canvas()->update();
       history_protected = FALSE;
     }
+    else
+      e->ignore();
   }  
-  e->ignore();
+  else
+    e->ignore();
 }
 
 void DiagramView::keyReleaseEvent(QKeyEvent *) {
@@ -1275,7 +1304,7 @@ void DiagramView::do_optimal_window_size() {
   optimal_window_size();
   canvas()->update();
   // do it an other time because of the scroll bars
-#ifdef WIN32
+#if defined(WIN32) || (QT_VERSION != 230)
   QTimer::singleShot(1, this, SLOT(optimal_window_size()));
 #else
   optimal_window_size();
@@ -1294,7 +1323,8 @@ void DiagramView::optimal_window_size() {
 
 void DiagramView::preferred_size_zoom() {
   if (preferred_zoom != 0) {
-    window()->resize(preferred_size);
+    if (preferred_size.width() != 0)
+      window()->resize(preferred_size);
     window()->new_scale((int) (preferred_zoom * 100));
   }
 }
@@ -1306,16 +1336,63 @@ void DiagramView::set_format(int f) {
   }
 }
 
+static bool find_browser_element(QCanvas * canvas, QCanvasItemList & r)
+{
+  BrowserNode * bn = BrowserView::selected_item();
+  
+  if (bn == 0)
+    return FALSE;
+      
+  BasicData * d = bn->get_data();
+  UmlCode k = bn->get_type();
+  QCanvasItemList l = canvas->allItems();
+  QCanvasItemList::Iterator it;
+
+  switch (k) {
+  case UmlClassDiagram:
+  case UmlUseCaseDiagram:
+  case UmlSeqDiagram:
+  case UmlColDiagram:
+  case UmlComponentDiagram:
+  case UmlDeploymentDiagram:
+  case UmlObjectDiagram:
+  case UmlActivityDiagram:
+    k = UmlIcon;
+    break;
+  default:
+    break;
+  }
+  
+  for (it = l.begin(); it != l.end(); ++it) {
+    if ((*it)->visible()) {
+      DiagramItem * di = QCanvasItemToDiagramItem(*it);
+      
+      if ((di != 0) && (di->type() == k)) {
+	if (di->get_bn() == bn)
+	  r.append(*it);
+	else {
+	  ArrowCanvas * a = dynamic_cast<ArrowCanvas *>(di);
+	  
+	  if ((a != 0) && (a->get_data() == d))
+	    r.append(*it);
+	}
+      }
+    }
+  }
+  
+  return !r.isEmpty();
+}
+
 int DiagramView::default_menu(QPopupMenu & m, int f) {
   QPopupMenu formatm(0);
+  QCanvasItemList l;
   
   m.insertItem("Edit drawing settings", EDIT_DRAWING_SETTING_CMD);
   m.insertSeparator();
   m.insertItem("Select diagram in browser", 1);
   m.insertItem("Select all (Ctrl+a)", 2);
-  m.insertSeparator();
-  m.insertItem("Optimal scale", 7);
-  m.insertItem("Optimal window size", 8);
+  if (find_browser_element(canvas(), l))
+    m.insertItem("Find selected browser element", 19);
   m.insertSeparator();
   m.insertItem("Copy optimal picture part", 13);
   m.insertItem("Copy visible picture part", 3);
@@ -1327,9 +1404,20 @@ int DiagramView::default_menu(QPopupMenu & m, int f) {
       (copied_from == window()->browser_diagram()->get_type()))
     m.insertItem("Paste copied items (Ctrl+v)", 9);
   m.insertSeparator();
+  m.insertItem("Optimal scale", 7);
+  m.insertItem("Optimal window size", 8);
   m.insertItem("Set preferred size and scale", 4);
-  if (preferred_zoom != 0)
-    m.insertItem("Restore preferred size and scale", 5);
+  m.insertItem("Set preferred scale (size unset)", 17);
+  if (preferred_zoom != 0) {
+    if (preferred_size.width() != 0) {
+      m.insertItem("Restore preferred size and scale", 5);
+      m.insertItem("Unset preferred size and scale", 18);
+    }
+    else {
+      m.insertItem("Restore preferred scale", 5);
+      m.insertItem("Unset preferred scale", 18);
+    }
+  }
   init_format_menu(formatm, f);
   m.insertItem("Format", &formatm);
   m.insertSeparator();
@@ -1348,7 +1436,21 @@ int DiagramView::default_menu(QPopupMenu & m, int f) {
     break;
   case 2:
     select_all();
-    return choice;	// clear history_protected
+    break;
+  case 19:
+    history_protected = TRUE;
+    unselect_all();
+    
+    {
+      QCanvasItemList::Iterator it;
+  
+      for (it = l.begin(); it != l.end(); ++it)
+	select(*it);
+    }
+    
+    ensureVisible((int) l.first()->x(), (int) l.first()->y());
+    canvas()->update();
+    break;    
   case 3:
     history_protected = TRUE;
     copy_in_clipboard(FALSE, FALSE);
@@ -1364,6 +1466,20 @@ int DiagramView::default_menu(QPopupMenu & m, int f) {
     preferred_size.setHeight(window()->height());
     window()->package_modified();
     break;
+  case 17:
+    history_protected = TRUE;
+    preferred_zoom = the_canvas()->zoom();
+    preferred_size.setWidth(0);
+    preferred_size.setHeight(0);
+    window()->package_modified();
+    break;
+  case 18:
+    history_protected = TRUE;
+    preferred_zoom = 0;
+    preferred_size.setWidth(0);
+    preferred_size.setHeight(0);
+    window()->package_modified();
+    break;
   case 5:
     preferred_size_zoom();
     break;
@@ -1424,6 +1540,7 @@ void DiagramView::load(const char * pfix) {
   // for a re-load :
   unselect_all();
   preferred_zoom = 0;
+  set_on_load_diagram(TRUE);
   
   QCanvasItemList all = canvas()->allItems();
   QCanvasItemList::Iterator it;
@@ -1472,7 +1589,19 @@ void DiagramView::load(const char * pfix) {
   if (window()->get_view() != 0)
     // re-load
     preferred_size_zoom();
+  
   canvas()->update();
+  
+  all = canvas()->allItems();  
+  
+  for (it = all.begin(); it != all.end(); ++it)
+    if ((*it)->visible() &&
+	((di = QCanvasItemToDiagramItem(*it)) != 0))
+      di->post_loaded();
+  
+  canvas()->update();
+  
+  set_on_load_diagram(FALSE);
 }
 
 void DiagramView::read() {
@@ -1515,6 +1644,7 @@ void DiagramView::read() {
 
 void DiagramView::paste() {
   history_protected = TRUE;
+  set_on_load_diagram(TRUE);
   
   double old_zoom = ((UmlCanvas *) canvas())->zoom();
   char * s = new char[clipboard.length() + 1];
@@ -1597,6 +1727,17 @@ void DiagramView::paste() {
   
   set_zoom(old_zoom);
   canvas()->update();
+  
+  l = canvas()->allItems();  
+  
+  for (it = l.begin(); it != l.end(); ++it)
+    if ((*it)->visible() &&
+	((di = QCanvasItemToDiagramItem(*it)) != 0))
+      di->post_loaded();
+  
+  canvas()->update();
+
+  set_on_load_diagram(FALSE);
   history_protected = FALSE;
   window()->package_modified();
 }
@@ -1748,6 +1889,11 @@ void DiagramView::copy_in_clipboard(bool optimal, bool temporary) {
   the_canvas()->show_limits(FALSE);
   
   if (optimal) {
+    int x0 = contentsX();
+    int y0 = contentsY();
+    
+    setContentsPos(0, 0);
+    
     int maxx;
     int maxy;
     
@@ -1782,6 +1928,8 @@ void DiagramView::copy_in_clipboard(bool optimal, bool temporary) {
 #endif
       }
     }
+    if (! temporary)
+      setContentsPos(x0, y0);
   }
   else
     QApplication::clipboard()->setPixmap(QPixmap::grabWidget(viewport()));
@@ -1799,11 +1947,13 @@ void DiagramView::restore_window_size() {
 
 void DiagramView::save_picture(bool optimal, bool svg) {
   QString filename =
-    QFileDialog::getSaveFileName(QString::null, 
+    QFileDialog::getSaveFileName(last_used_directory(), 
 				 (svg) ? "SVG file (*.svg)" : "PNG file (*.png)",
 				 this);
 
   if (!filename.isNull()) {
+    set_last_used_directory(filename);
+    
     if (svg) {
       if (filename.right(4).lower() != ".svg")
 	filename += ".svg";
