@@ -343,20 +343,12 @@ void ExpansionRegionCanvas::draw(QPainter & p) {
 
   FILE * fp = svg();
 
-  if (fp != 0) {
-    if (used_color != UmlTransparent)
-      fprintf(fp, "<g>\n"
-	      "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-dasharray=\"4,4\" stroke-opacity=\"1\""
-	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"10\" />\n",
-	      co.rgb()&0xffffff,
-	      r.left(), r.top(), r.width() - 1, r.height() - 1);
-    else
-      fprintf(fp, "<g>\n"
-	      "\t<rect fill=\"none\" stroke=\"black\" stroke-dasharray=\"4,4\" stroke-opacity=\"1\""
-	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"10\" />\n",
-	      r.left(), r.top(), r.width() - 1, r.height() - 1);
-  }
-
+  if (fp != 0)
+    fprintf(fp, "<g>\n"
+	    "\t<rect fill=\"%s\" stroke=\"black\" stroke-dasharray=\"4,4\" stroke-opacity=\"1\""
+	    " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"10\" />\n",
+	    svg_color(used_color),
+	    r.left(), r.top(), r.width() - 1, r.height() - 1);
 
   int margin = (int) (6 * the_canvas()->zoom());
   QString s;
@@ -571,7 +563,7 @@ const char * ExpansionRegionCanvas::may_connect(UmlCode & l, const DiagramItem *
 void ExpansionRegionCanvas::connexion(UmlCode action, DiagramItem * dest,
 						  const QPoint &, const QPoint &) {
   ArrowCanvas * a =
-    new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
