@@ -42,6 +42,8 @@
 #include "myio.h"
 #include "MenuTitle.h"
 #include "DialogUtil.h"
+#include "ToolCom.h"
+#include "strutil.h"
 
 FragmentCanvas::FragmentCanvas(UmlCanvas * canvas, int x, int y, int id)
     : DiagramCanvas(0, canvas, x, y, FRAGMENT_CANVAS_MIN_SIZE,
@@ -84,18 +86,13 @@ void FragmentCanvas::draw(QPainter & p) {
   p.setBackgroundColor(co);
   p.setFont(the_canvas()->get_font(UmlNormalFont));
   
-  if (used_color != UmlTransparent) {
+  if (used_color != UmlTransparent)
     p.fillRect(r, co);
 
-    if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
-	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff, 
-	      r.x(), r.y(), r.width() - 1, r.height() - 1);
-  }
-  else if (fp != 0)
-    fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+  if (fp != 0)
+    fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	    " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	    svg_color(used_color), 
 	    r.x(), r.y(), r.width() - 1, r.height() - 1);
   
   p.drawRect(r);
@@ -386,6 +383,29 @@ void FragmentCanvas::resize(aCorner c, int dx, int dy) {
     it.current()->update();
 }
 
+void FragmentCanvas::prepare_for_move(bool on_resize) {
+  if (!on_resize) {
+    DiagramCanvas::prepare_for_move(on_resize);
+    
+    QRect r = rect();
+    QCanvasItemList l = collisions(TRUE);
+    QCanvasItemList::ConstIterator it;
+    QCanvasItemList::ConstIterator end = l.end();
+    DiagramItem * di;
+  
+    for (it = l.begin(); it != end; ++it) {
+      if ((*it)->visible() && // at least not deleted
+	  !(*it)->selected() &&
+	  ((di = QCanvasItemToDiagramItem(*it)) != 0) &&
+	  r.contains(di->rect(), TRUE) &&
+	  di->move_with(UmlFragment)) {
+	the_canvas()->select(*it);
+	di->prepare_for_move(FALSE);
+      }
+    }
+  }
+}
+
 void FragmentCanvas::save(QTextStream & st, bool ref, QString & warning) const {
   if (ref) {
     st << "fragment_ref " << get_ident();
@@ -493,3 +513,68 @@ void FragmentCanvas::history_hide() {
   disconnect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
 }
 
+// for plug outs
+
+void FragmentCanvas::send(ToolCom * com, QCanvasItemList & all)
+{
+  QList<FragmentCanvas> fragments;
+  QCanvasItemList::Iterator cit;
+
+  for (cit = all.begin(); cit != all.end(); ++cit) {
+    DiagramItem *di = QCanvasItemToDiagramItem(*cit);
+    
+    if ((di != 0) && (*cit)->visible() && (di->type() == UmlFragment))
+      fragments.append((FragmentCanvas *) di);
+  }
+  
+  com->write_unsigned(fragments.count());
+  
+  FragmentCanvas * f;
+  
+  for (f = fragments.first(); f != 0; f = fragments.next()) {
+    QCString s = fromUnicode(f->name);
+    
+    com->write_string((const char *) s);
+    com->write(f->rect());
+    
+    // few separators, use bubble sort
+    unsigned sz = f->separators.count();
+    
+    if (sz == 0)
+      com->write_unsigned(1);
+    else {
+      FragmentSeparatorCanvas ** v = new FragmentSeparatorCanvas *[sz];
+      unsigned index;      
+      QListIterator<FragmentSeparatorCanvas> it(f->separators);
+      
+      for (index = 0; it.current(); ++it, index += 1)
+	v[index] = it.current();
+      
+      bool modified;
+      
+      do {
+	modified = FALSE;
+	
+	for (index = 1; index < sz; index += 1) {
+	  if (v[index - 1]->y() > v[index]->y()) {
+	    modified = TRUE;
+	    
+	    FragmentSeparatorCanvas * fs = v[index - 1];
+	    
+	    v[index - 1] = v[index];
+	    v[index] = fs;
+	  }
+	}
+      } while (modified);
+      
+      com->write_unsigned(sz + 1);
+      
+      for (index = 0; index != sz; index += 1)
+	com->write_unsigned((unsigned) v[index]->y());
+      
+      delete [] v;
+    }
+      
+    com->write_unsigned((unsigned) (f->y() + f->height() - 1));
+  }
+}
