@@ -31,6 +31,7 @@
 #define FRAGMENT_CANVAS_MIN_SIZE 30
 
 class FragmentSeparatorCanvas;
+class ToolCom;
 
 class FragmentCanvas : public QObject, public DiagramCanvas {
   Q_OBJECT
@@ -66,6 +67,7 @@ class FragmentCanvas : public QObject, public DiagramCanvas {
     virtual aCorner on_resize_point(const QPoint &);
     virtual void resize(aCorner c, int dx, int dy);
     virtual void change_scale();
+    virtual void prepare_for_move(bool on_resize);
     virtual void moveBy(double dx, double dy);
     virtual void set_z(double z);	// only called by upper() & lower()
     
@@ -80,6 +82,8 @@ class FragmentCanvas : public QObject, public DiagramCanvas {
     
     virtual void apply_shortcut(QString s);
     void edit_drawing_settings();
+    
+    static void send(ToolCom * com, QCanvasItemList & all);
   
   private slots:
     void modified();
