@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -83,6 +83,15 @@ void IconCanvas::draw(QPainter & p) {
     
   if (selected())
     show_mark(p, r);
+
+  FILE * fp = svg();
+  
+  if (fp != 0) {
+    fprintf(fp, "<g transform=\"translate(%d,%d)\">\n", 
+	    r.left(), r.top());
+    ((BrowserDiagram *) browser_node)->draw_svg();
+    fputs("</g>\n", fp);
+  }
 }
 
 void IconCanvas::change_scale() {
@@ -179,15 +188,7 @@ const char * IconCanvas::may_start(UmlCode & l) const {
 }
 
 const char * IconCanvas::may_connect(UmlCode & l, const DiagramItem * dest) const {
-  switch (dest->type()) {
-  case UmlAnchor:
-  case UmlLabel:
-  case UmlArrowPoint:
-  case UmlLifeLine:
-    return "illegal";
-  default:
-    return (l == UmlAnchor) ? 0 : "illegal";
-  }
+  return (l == UmlAnchor) ? dest->may_start(l) : "illegal";
 }
 
 bool IconCanvas::alignable() const {
