@@ -34,6 +34,7 @@
 #include "ObjectDiagramView.h"
 #include "BrowserObjectDiagram.h"
 #include "BrowserClass.h"
+#include "BrowserClassInstance.h"
 #include "DiagramCanvas.h"
 #include "OdClassInstCanvas.h"
 #include "ObjectLinkCanvas.h"
@@ -82,18 +83,44 @@ void ObjectDiagramView::contentsMousePressEvent(QMouseEvent * e) {
       switch (c) {
       case UmlClass:
 	{
-	  history_protected = TRUE;
+	  history_protected = FALSE;
+	  unselect_all();
 	  window()->selectOn();
 	  history_save();
 	  
 	  BrowserNode * parent =
 	    ((BrowserNode *) window()->browser_diagram()->parent());
-	  BrowserClass * b = BrowserClass::get_class(parent);
+	  BrowserClass * b = 
+	    BrowserClass::get_class(parent);
+	  
+	  if (b != 0) {
+	    BrowserClassInstance * i =
+	      new BrowserClassInstance("", b, parent);
+	    OdClassInstCanvas * cl = 
+	      new OdClassInstCanvas(i, the_canvas(), e->x(), e->y(), 0);
+	    
+	    cl->show();
+	    cl->upper();
+	    window()->package_modified();
+	  }
+	}
+	canvas()->update();
+	break;
+      case UmlClassInstance:
+	{
+	  history_protected = TRUE;
+	  unselect_all();
+	  window()->selectOn();
+	  history_save();
+	  
+	  BrowserNode * parent =
+	    ((BrowserNode *) window()->browser_diagram()->parent());
+	  BrowserClassInstance * b = 
+	    BrowserClassInstance::get_classinstance(parent);
 	  
 	  if (b != 0) {
 	    OdClassInstCanvas * cl = 
-	      new OdClassInstCanvas(((BrowserClass *) b), the_canvas(),
-				     e->x(), e->y(), 0);
+	      new OdClassInstCanvas(b, the_canvas(), e->x(), e->y(), 0);
 	    
 	    cl->show();
 	    cl->upper();
@@ -114,6 +141,7 @@ void ObjectDiagramView::contentsMousePressEvent(QMouseEvent * e) {
 void ObjectDiagramView::dragEnterEvent(QDragEnterEvent * e) {
   if (!window()->frozen() &&
       (UmlDrag::canDecode(e, UmlClass, TRUE, TRUE) ||
+       UmlDrag::canDecode(e, UmlClassInstance, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlPackage, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlClassDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlUseCaseDiagram, FALSE, TRUE) ||
@@ -132,12 +160,29 @@ void ObjectDiagramView::dropEvent(QDropEvent * e) {
   BrowserNode * bn;
   QPoint p = viewportToContents(e->pos());
   
-  if ((bn = UmlDrag::decode(e, UmlClass)) != 0) {
+  if ((bn = UmlDrag::decode(e, UmlClassInstance)) != 0) {
     history_save();
     
     OdClassInstCanvas * i = 
-      new OdClassInstCanvas(((BrowserClass *) bn),
-			     the_canvas(), p.x(), p.y(), 0);
+      new OdClassInstCanvas((BrowserClassInstance *) bn,
+			    the_canvas(), p.x(), p.y(), 0);
+    
+    history_protected = TRUE;
+    i->show();
+    i->upper();
+    canvas()->update();
+    history_protected = FALSE;
+    window()->package_modified();
+  }
+  else if ((bn = UmlDrag::decode(e, UmlClass)) != 0) {
+    history_save();
+    
+    BrowserNode * parent =
+      ((BrowserNode *) window()->browser_diagram()->parent());
+    BrowserClassInstance * cli =
+      new BrowserClassInstance("", (BrowserClass *) bn, parent);
+    OdClassInstCanvas * i = 
+      new OdClassInstCanvas(cli, the_canvas(), p.x(), p.y(), 0);
     
     history_protected = TRUE;
     i->show();
@@ -164,8 +209,8 @@ void ObjectDiagramView::dropEvent(QDropEvent * e) {
 	   ((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlComponentDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlDeploymentDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlStateDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlActivityDiagram)) != 0)) {
+	   ((bn = UmlDrag::decode(e, UmlStateDiagram, TRUE)) != 0) ||
+	   ((bn = UmlDrag::decode(e, UmlActivityDiagram, TRUE)) != 0)) {
     history_save();
     
     IconCanvas * ic = new IconCanvas(bn, the_canvas(), p.x(), p.y(), 0);
@@ -196,7 +241,7 @@ void ObjectDiagramView::save(QTextStream & st, QString & warning,
     switch (di->type()) {
     case UmlPackage:
     case UmlFragment:
-    case UmlClass:
+    case UmlClassInstance:
     case UmlNote:
     case UmlText:
     case UmlIcon:
