@@ -216,18 +216,13 @@ void PackageCanvas::draw(QPainter & p) {
   
   r.setWidth(r.width() * 2 / 5);
   r.setHeight(he + four);
-  if (used_color != UmlTransparent) {
+  if (used_color != UmlTransparent)
     p.fillRect(r, co);
-
-    if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
-	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff, 
-	      r.x(), r.y(), r.width() - 1, r.height() - 1);
-  }
-  else if (fp != 0)
-    fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+  
+  if (fp != 0)
+    fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	    " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	    svg_color(used_color), 
 	    r.x(), r.y(), r.width() - 1, r.height() - 1);
 
   p.drawRect(r);
@@ -260,18 +255,13 @@ void PackageCanvas::draw(QPainter & p) {
   }
   
   r.setTop(r.top() + he + four - 1);
-  if (used_color != UmlTransparent) {
+  if (used_color != UmlTransparent)
     p.fillRect(r, co);
-
-    if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
-	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff,
-	      r.x(), r.y(), r.width() - 1, r.height() - 1);
-  }
-  else if (fp != 0)
-    fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+  
+  if (fp != 0)
+    fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	    " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	    svg_color(used_color),
 	    r.x(), r.y(), r.width() - 1, r.height() - 1);
 
   p.drawRect(r);
@@ -372,6 +362,12 @@ void PackageCanvas::modified() {
   package_modified();
 }
 
+void PackageCanvas::post_loaded() {
+  force_self_rel_visible();
+  if (the_canvas()->must_draw_all_relations())
+    draw_all_simple_relations();
+}
+
 void PackageCanvas::menu(const QPoint&) {
   QPopupMenu m(0);
   QPopupMenu toolm(0);
@@ -553,9 +549,9 @@ void PackageCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * r;
   
   if (IsaSimpleRelation(action))
-    r = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    r = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    r = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    r = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   r->show();
   the_canvas()->select(r);
