@@ -172,16 +172,24 @@ void PseudoStateCanvas::modified() {
   package_modified();
 }
 
+void PseudoStateCanvas::post_loaded() {
+  force_self_rel_visible();
+  if (the_canvas()->must_draw_all_relations()) {
+    //draw_all_simple_relations();
+    draw_all_transitions();
+  }
+}
+
 void PseudoStateCanvas::connexion(UmlCode action, DiagramItem * dest,
 				  const QPoint &, const QPoint &) {
   ArrowCanvas * a;
   
   if (action == UmlTransition)
-    a = new TransitionCanvas(the_canvas(), this, dest, 0, 0);
+    a = new TransitionCanvas(the_canvas(), this, dest, 0, 0, -1.0, -1.0);
   else if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
