@@ -40,6 +40,7 @@
 #include "DiagramView.h"
 #include "BrowserRelation.h"
 #include "BrowserClass.h"
+#include "ClassData.h"
 #include "RelationData.h"
 #include "UmlWindow.h"
 #include "UmlGlobal.h"
@@ -52,8 +53,9 @@
 
 RelationCanvas::RelationCanvas(UmlCanvas * canvas, DiagramItem * b,
 			       DiagramItem * e, BrowserClass * bb,
-			       UmlCode t, int id, RelationData * d)
-      : ArrowCanvas(canvas, b, e, t, id, TRUE), br_begin(bb),
+			       UmlCode t, int id, float d_start, float d_end,
+			       RelationData * d)
+      : ArrowCanvas(canvas, b, e, t, id, TRUE, d_start, d_end), br_begin(bb),
         data(d), role_a(0), role_b(0), multiplicity_a(0), multiplicity_b(0) {
   if ((e->type() != UmlArrowPoint) && (bb == 0)) {
     // end of line construction, update all the line bb & be
@@ -63,6 +65,8 @@ RelationCanvas::RelationCanvas(UmlCanvas * canvas, DiagramItem * b,
   else if (d != 0) {
     connect(d, SIGNAL(changed()), this, SLOT(modified()));
     connect(d, SIGNAL(deleted()), this, SLOT(deleted()));
+    connect(br_begin->get_data(), SIGNAL(actuals_changed()),
+	    this, SLOT(actuals_modified()));
   }
   
   // manages the case start == end
@@ -78,6 +82,8 @@ RelationCanvas::~RelationCanvas() {
 
 void RelationCanvas::delete_it() {
   disconnect(data, 0, this, 0);
+  disconnect(br_begin->get_data(), SIGNAL(actuals_changed()),
+	     this, SLOT(actuals_modified()));
   
   ArrowCanvas::delete_it();
 }
@@ -146,10 +152,13 @@ BrowserClass * RelationCanvas::update_begin(DiagramItem * cnend) {
     d = (RelationData *) begin->add_relation(type(), cnend);
   }
   
-  connect(d, SIGNAL(changed()), this, SLOT(modified()));
-  connect(d, SIGNAL(deleted()), this, SLOT(deleted()));
   data = d;
   
+  connect(data, SIGNAL(changed()), this, SLOT(modified()));
+  connect(data, SIGNAL(deleted()), this, SLOT(deleted()));
+  connect(br_begin->get_data(), SIGNAL(actuals_changed()),
+	  this, SLOT(actuals_modified()));
+  
   return br_begin;
 }
 
@@ -405,7 +414,11 @@ void RelationCanvas::menu(const QPoint & lpos) {
       }
       else if (rank >= 10) {
 	rank -= 10;
-	if (rank != (int) geometry)
+	if (rank == RecenterBegin)
+	  set_decenter(-1.0, decenter_end);
+	else if (rank == RecenterEnd)
+	  set_decenter(decenter_begin, -1.0);
+	else if (rank != (int) geometry)
 	  set_geometry((LineGeometry) rank, TRUE);
 	else
 	  return;
@@ -427,11 +440,14 @@ ArrowPointCanvas * RelationCanvas::brk(const QPoint & p) {
   
   RelationCanvas * other =
     // do not give data to not call update()
-    new RelationCanvas(the_canvas(), ap, end, br_begin, itstype, 0);
+    new RelationCanvas(the_canvas(), ap, end, br_begin, itstype, 0,
+		       decenter_begin, decenter_end);
   
   other->data = data;
   connect(data, SIGNAL(changed()), other, SLOT(modified()));
   connect(data, SIGNAL(deleted()), other, SLOT(deleted()));
+  connect(br_begin->get_data(), SIGNAL(actuals_changed()),
+	  other, SLOT(actuals_modified()));
 
   ap->add_line(this);
   end->remove_line(this);
@@ -483,6 +499,16 @@ void RelationCanvas::modified() {
   }
 }
 
+void RelationCanvas::actuals_modified() {
+  if (data->get_type() == UmlRealize) {
+    ArrowCanvas * aplabel;
+    ArrowCanvas * apstereotype;
+    
+    search_supports(aplabel, apstereotype);
+    update_actuals((RelationCanvas *) aplabel);
+  }
+}
+
 void RelationCanvas::update(bool updatepos) {
   if (data) {
     itstype = data->get_type();
@@ -507,7 +533,9 @@ void RelationCanvas::update(bool updatepos) {
     QString s = data->get_name();
     QPoint c = center();
     
-    if (unamed || s.isEmpty() || (s == RelationData::default_name(itstype))) {
+    if (itstype == UmlRealize)
+      update_actuals(plabel);
+    else if (unamed || s.isEmpty() || (s == RelationData::default_name(itstype))) {
       // relation does not have name or 'true' name
       if (plabel != 0) {
 	// removes it
@@ -661,6 +689,36 @@ void RelationCanvas::update(bool updatepos) {
     update_pos();
 }
 
+void RelationCanvas::update_actuals(RelationCanvas * plabel) {
+  if ((plabel == 0) && (begin->type() == UmlArrowPoint))
+    return;
+  
+  QString s = 
+    ((ClassData *) br_begin->get_data())->get_actuals(data->get_end_class());
+  QPoint c = center();
+  QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
+  int h = fm.height();
+  
+  if (s.isEmpty()) {
+    // remove label
+    if (plabel != 0) {
+      // removes it
+      the_canvas()->del(plabel->label);
+      plabel->label = 0;
+    }
+  }
+  else if (plabel == 0)
+    // begin->type() != UmlArrowPoint
+    // adds label
+    label = new LabelCanvas(s, the_canvas(),
+			    c.x() - fm.width(s)/2, c.y() - h);
+  else if (plabel->label->get_name() != s) {
+    // update label, position will be updated later
+    plabel->label->set_name(s);
+    plabel->label_default_position();
+  }
+}
+
 void RelationCanvas::label_default_position() const {
   default_label_position();
   package_modified();
@@ -843,7 +901,8 @@ void RelationCanvas::drop(BrowserNode * bn, UmlCanvas * canvas)
       msg_information("Bouml", "relation already drawn");
     else {
       RelationCanvas * rel = 
-	new RelationCanvas(canvas, ccfrom, ccto, from, bn->get_type(), 0, def);
+	new RelationCanvas(canvas, ccfrom, ccto, from, bn->get_type(),
+			   0, -1.0, -1.0, def);
       
       rel->hide();	// else self rel not fully shown
       rel->show();
@@ -942,7 +1001,8 @@ void RelationCanvas::show_assoc_class(CdClassCanvas * cc) {
   }
   
   // not drawn, add it
-  (new ArrowCanvas(the_canvas(), first, cc, UmlAnchor, 0, FALSE))->show();
+  (new ArrowCanvas(the_canvas(), first, cc, UmlAnchor, 0, FALSE, -1.0, -1.0))
+    ->show();
 }
 
 // the relation doesn't have association class
@@ -1031,6 +1091,16 @@ void RelationCanvas::save(QTextStream & st, bool ref, QString & warning) const {
       if (!fixed_geometry)
 	st << " unfixed";
     }
+    if (decenter_begin >= 0) {
+      // float output/input bugged
+      nl_indent(st);
+      st << "decenter_begin " << ((int) (decenter_begin * 1000));
+    }
+    if (decenter_end >= 0) {
+      // float output/input bugged
+      nl_indent(st);
+      st << "decenter_end " << ((int) (decenter_end * 1000));
+    }
     
     const RelationCanvas * last = 
       (const RelationCanvas *) ArrowCanvas::save_lines(st, TRUE, TRUE, warning);
@@ -1071,19 +1141,37 @@ RelationCanvas * RelationCanvas::read(char * & st, UmlCanvas * canvas, char * k)
     if (! strcmp(k, "geometry")) {
       geo = line_geometry(read_keyword(st));
       k = read_keyword(st);
-      if (! strcmp(k, "unfixed"))
+      if (! strcmp(k, "unfixed")) {
+	k = read_keyword(st);
 	fixed = FALSE;
-      else {
-	fixed = TRUE;
-	unread_keyword(k, st);
       }
+      else
+	fixed = TRUE;
     }
     else {
-      unread_keyword(k, st);
       geo = NoGeometry;
       fixed = FALSE;
     }
+    
+    float dbegin;
+    float dend;
 
+    if (! strcmp(k, "decenter_begin")) {
+      dbegin = read_double(st) / 1000;
+      k = read_keyword(st);
+    }
+    else
+      dbegin = -1;
+
+    if (! strcmp(k, "decenter_end")) {
+      dend = read_double(st) / 1000;
+      k = read_keyword(st);
+    }
+    else
+      dend = -1;
+
+    unread_keyword(k, st);
+    
     read_keyword(st, "from");
     read_keyword(st, "ref");
     
@@ -1165,13 +1253,15 @@ RelationCanvas * RelationCanvas::read(char * & st, UmlCanvas * canvas, char * k)
 	di = dict_get(read_id(st), "classcanvas", canvas);
 
       // do not give rd to not call update()
-      result = new RelationCanvas(canvas, bi, di, b, t, id);
+      result = new RelationCanvas(canvas, bi, di, b, t, id, dbegin, dend);
       result->geometry = geo;
       result->fixed_geometry = fixed;
       result->set_z(z);
       result->data = rd;
       connect(rd, SIGNAL(changed()), result, SLOT(modified()));
       connect(rd, SIGNAL(deleted()), result, SLOT(deleted()));
+      connect(b->get_data(), SIGNAL(actuals_changed()),
+	      result, SLOT(actuals_modified()));
 
       if (first == 0) {
 	first = result;
@@ -1302,9 +1392,13 @@ void RelationCanvas::history_load(QBuffer & b) {
   multiplicity_b = (LabelCanvas *) load_item(b);
   connect(data, SIGNAL(changed()), this, SLOT(modified()));
   connect(data, SIGNAL(deleted()), this, SLOT(deleted()));
+  connect(br_begin->get_data(), SIGNAL(actuals_changed()),
+	  this, SLOT(actuals_modified()));
 }
 
 void RelationCanvas::history_hide() {
   QCanvasItem::setVisible(FALSE);
   disconnect(data, 0, this, 0);
+  disconnect(br_begin->get_data(), SIGNAL(actuals_changed()),
+	     this, SLOT(actuals_modified()));
 }
