@@ -46,7 +46,8 @@ class RelationCanvas : public ArrowCanvas {
     
   public:
     RelationCanvas(UmlCanvas * canvas, DiagramItem * b, DiagramItem * e,
-		   BrowserClass * bb, UmlCode t, int id, RelationData * d = 0);
+		   BrowserClass * bb, UmlCode t, int id, 
+		   float d_start, float d_end, RelationData * d = 0);
     virtual ~RelationCanvas();
     
     virtual void delete_it();
@@ -95,9 +96,11 @@ class RelationCanvas : public ArrowCanvas {
   protected:
     BrowserClass * update_begin(DiagramItem * cnend);
     void update(bool updatepos);
+    void update_actuals(RelationCanvas * plabel);
     
   private slots:
     void modified();	// canvas must be updated
+    void actuals_modified();	// label may be modified
     void deleted();	// the relation is deleted
 };
 
