@@ -27,18 +27,20 @@
 #define SDLIFELINECANVAS_H
 
 #include "DiagramCanvas.h"
+#include "SdDurationSupport.h"
 
 #define LIFE_LINE_TOPOFFSET 41
 
 class SdDead;
 class SdObjCanvas;
 class SdDurationCanvas;
+class ToolCom;
 
-class SdLifeLineCanvas : public DiagramCanvas {
+class SdLifeLineCanvas : public DiagramCanvas, public SdDurationSupport {
   protected:
+    QList<SdDurationCanvas> durations;
     SdObjCanvas * obj;
     int end;
-    QList<SdDurationCanvas> durations;
   
   public:
     SdLifeLineCanvas(UmlCanvas * canvas, SdObjCanvas * o);
@@ -47,11 +49,21 @@ class SdLifeLineCanvas : public DiagramCanvas {
     virtual void delete_it();
 
     SdObjCanvas * get_obj() const { return obj; };
-    void add(SdDurationCanvas *);
-    void remove(SdDurationCanvas *);
+    virtual void add(SdDurationCanvas *);
+    virtual void remove(SdDurationCanvas *);
+    void toFlat();
+    void toOverlapping();
     double instance_max_y() const;
     void update_pos();
-    void update_instance_dead();
+    virtual void update_instance_dead();
+    virtual void update_v_to_contain(SdDurationCanvas *, bool);
+    virtual int sub_x(int sub_w) const;
+    virtual double min_y() const;
+    virtual SdLifeLineCanvas * get_line() const;
+    virtual bool isaDuration() const;
+    virtual double getZ() const;
+    
+    virtual bool is_decenter(const QPoint &, bool &) const;
     
     virtual void drawShape(QPainter & p);
     virtual void moveBy(double dx, double dy);
@@ -68,6 +80,8 @@ class SdLifeLineCanvas : public DiagramCanvas {
     virtual void save(QTextStream &, bool ref, QString & warning) const;
     virtual void history_save(QBuffer &) const;
     virtual void history_load(QBuffer &);
+    
+    static void send(ToolCom * com, const QCanvasItemList & l);
 };
 
 #endif
