@@ -43,6 +43,8 @@
 #include "BrowserOperation.h"
 #include "BrowserDiagram.h"
 #include "myio.h"
+#include "ToolCom.h"
+#include "../Tools/aMessageKind.h"
 
 SdMsgBaseCanvas::SdMsgBaseCanvas(UmlCanvas * canvas, SdDurationCanvas * d,
 				 UmlCode l, int v, int id)
@@ -86,7 +88,7 @@ QString SdMsgBaseCanvas::get_msg(bool with_args) const {
     the_canvas()->browser_diagram()->get_sequencediagramsettings(dflt);
     
     m = msg->definition(dflt.show_full_operations_definition == UmlYes,
-			dflt.drawing_language, TRUE);
+			dflt.drawing_language, TRUE, TRUE);
   }
   
   if (!with_args || args.isEmpty())
@@ -182,25 +184,36 @@ const char * SdMsgBaseCanvas::may_connect(UmlCode & l, const DiagramItem * dest)
   return (l == UmlAnchor) ? dest->may_start(l) : "illegal";
 }
 
+bool SdMsgBaseCanvas::is_decenter(const QPoint &, bool &) const {
+  return FALSE;
+}
+
 void SdMsgBaseCanvas::open() {
-  if (itsType != UmlReturnMsg) {
-    SdMsgDialog d(this);
-    
-    if (d.exec() == QDialog::Accepted)
-      modified();
-  }
-  else {
-    bool ok = FALSE;
-    QString s =
-      QInputDialog::getText("Return", "value :", QLineEdit::Normal,
-			    (const char *) explicit_msg, &ok);
-    
-    s = s.stripWhiteSpace();
-    
-    if (ok && (s != (const char *) explicit_msg)) {
-      explicit_msg = s;
-      modified();
+  switch (itsType) {
+  case UmlReturnMsg:
+  case UmlSelfReturnMsg:
+    {
+      bool ok = FALSE;
+      QString s =
+	QInputDialog::getText("Return", "value :", QLineEdit::Normal,
+			      (const char *) explicit_msg, &ok);
+      
+      s = s.stripWhiteSpace();
+      
+      if (ok && (s != (const char *) explicit_msg)) {
+	explicit_msg = s;
+	modified();
+      }
     }
+    break;
+  default:
+    {
+      SdMsgDialog d(this);
+      
+      if (d.exec() == QDialog::Accepted)
+	modified();
+    }
+    break;
   }
 }
 
@@ -219,7 +232,7 @@ void SdMsgBaseCanvas::modified() {
     }
     else if (label == 0) {
       // adds label
-      label = new LabelCanvas(s, the_canvas(), 0, 0);
+      label = new LabelCanvas(s, the_canvas(), 0, 0, FALSE, FALSE, FALSE, FALSE);
       default_label_position();
       label->show();
     }
@@ -238,8 +251,9 @@ void SdMsgBaseCanvas::modified() {
 
 void SdMsgBaseCanvas::default_label_position() const {
   QFontMetrics fm(the_canvas()->get_font(UmlNormalFont));
+  QSize sz = fm.size(0, label->get_name());
       
-  label->move(rect().center().x() - fm.width(label->get_name())/2,
+  label->move(rect().center().x() - sz.width()/2,
 	      y() - fm.height());
 }
 
@@ -287,7 +301,7 @@ void SdMsgBaseCanvas::save(QTextStream & st, QString & warning) const {
     if (msg->deletedp()) {
       warning += QString("<b>") + the_canvas()->browser_diagram()->full_name() +
 	"</b> reference the class <b>" +
-	  dest->get_line()->get_obj()->get_type()->full_name() +
+	  dest->get_line()->get_obj()->get_class()->full_name() +
 	    "</b> deleted operation <b>" + msg->definition(TRUE) + "</b><br>\n<br>\n";
       if (warning[0] == '!') {
 	st << " msg ";
@@ -326,7 +340,8 @@ void SdMsgBaseCanvas::save(QTextStream & st, QString & warning) const {
 }
 
 void SdMsgBaseCanvas::read(char * & st) {
-  if ((itsType != UmlReturnMsg) || (read_file_format() >= 4)) {
+  if (((itsType != UmlReturnMsg) && (itsType != UmlSelfReturnMsg)) ||
+      (read_file_format() >= 4)) {
     char * k = read_keyword(st);
     const OperationData * method = 0;
     QString expl;
@@ -374,7 +389,8 @@ void SdMsgBaseCanvas::read(char * & st) {
     
     label =
       new LabelCanvas(get_msg(TRUE), the_canvas(),
-		      lx, (int) read_double(st));
+		      lx, (int) read_double(st),
+		      FALSE, FALSE, FALSE, FALSE);
     label->show();
   }
 }
@@ -394,3 +410,57 @@ void SdMsgBaseCanvas::history_load(QBuffer & b) {
     connect(msg, SIGNAL(deleted()), this, SLOT(modified()));
   }
 }
+
+// for plug out
+
+void SdMsgBaseCanvas::send_implicit_return(ToolCom * com, int fromid,
+					   unsigned x, unsigned y)
+{
+  com->write_id(0);
+  com->write_string("");
+  com->write_unsigned((unsigned) fromid);
+  com->write_unsigned((unsigned) fromid);
+  
+  com->write_char(anImplicitReturn);
+  com->write_string("");
+  com->write_unsigned(x);
+  com->write_unsigned(y);
+  com->write_unsigned(y);
+}
+
+void SdMsgBaseCanvas::send(ToolCom * com, int fromid) const {
+  if (msg == 0) {
+    com->write_id(0);
+    com->write_string(explicit_msg);
+  }
+  else if (msg->deletedp()) {
+    com->write_id(0);
+    com->write_string(msg->get_browser_node()->get_name());
+  }
+  else 
+    msg->get_browser_node()->write_id(com);
+  com->write_unsigned((unsigned) fromid);
+  com->write_unsigned((unsigned) dest->get_line()->get_obj()->get_ident());
+  
+  switch (itsType) {
+  case UmlSyncMsg:
+  case UmlSyncSelfMsg:
+    com->write_char(aSynchronousCall);
+    break;
+  case UmlAsyncMsg:
+  case UmlAsyncSelfMsg:
+    com->write_char(anAsynchronousCall);
+    break;
+  default:
+    com->write_char(anExplicitReturn);
+    break;
+  }
+
+  com->write_string((const char *) args);
+  com->write_unsigned((unsigned) x());
+  
+  unsigned v = (unsigned) y() + height()/2;
+  
+  com->write_unsigned(v);
+  com->write_unsigned(v);
+}
