@@ -31,6 +31,7 @@
 
 class SdDurationCanvas;
 class OperationData;
+class ToolCom;
 
 class SdMsgBaseCanvas : public QObject, public DiagramCanvas {
   Q_OBJECT
@@ -67,9 +68,13 @@ class SdMsgBaseCanvas : public QObject, public DiagramCanvas {
     void update_after_move(SdDurationCanvas * p);
     void default_label_position() const;
     virtual void update_hpos() = 0;
+    virtual void check_vpos(const QRect &) = 0;
     virtual double min_y() const = 0;
     virtual void change_duration(SdDurationCanvas * oldone,
 				 SdDurationCanvas * newone) = 0;
+    virtual int overlap_dir(SdDurationCanvas *) const = 0;
+    
+    virtual bool is_decenter(const QPoint &, bool &) const;
 
     virtual UmlCode type() const;
     virtual const char * may_start(UmlCode &) const;
@@ -87,6 +92,10 @@ class SdMsgBaseCanvas : public QObject, public DiagramCanvas {
     void save(QTextStream & st, QString & warning) const;
     void read(char * &);
     
+    void send(ToolCom * com, int fromid) const;
+    static void send_implicit_return(ToolCom * com, int fromid,
+				     unsigned x, unsigned y);
+  
   protected slots:
     void modified();	// canvas must be updated
 };
