@@ -33,6 +33,7 @@
 #include "SeqDiagramWindow.h"
 #include "SeqDiagramView.h"
 #include "BrowserSeqDiagram.h"
+#include "BrowserClassInstance.h"
 #include "BrowserClass.h"
 #include "DiagramCanvas.h"
 #include "SdClassInstCanvas.h"
@@ -43,6 +44,7 @@
 #include "IconCanvas.h"
 #include "FragmentCanvas.h"
 #include "SdContinuationCanvas.h"
+#include "SdLifeLineCanvas.h"
 #include "NoteCanvas.h"
 #include "TextCanvas.h"
 #include "UmlPixmap.h"
@@ -59,10 +61,16 @@ SeqDiagramView::SeqDiagramView(QWidget * parent, UmlCanvas * canvas, int id)
 void SeqDiagramView::menu(const QPoint&) {
   QPopupMenu m(0);
   QPopupMenu formatm(0);
+  BrowserSeqDiagram * sd = (BrowserSeqDiagram *) window()->browser_diagram();
+  bool overlapping = sd->is_overlapping_bars();
   
   m.insertItem(new MenuTitle("Sequence diagram menu", m.font()), -1);
+  m.insertItem((overlapping) ? "Transform to flat activity bars"
+			     : "Transform to overlapping activity bars",
+	       29);
+  m.insertSeparator();
  
-  switch (default_menu(m, 20)) {
+  switch (default_menu(m, 30)) {
   case EDIT_DRAWING_SETTING_CMD:
     ((BrowserSeqDiagram *) the_canvas()->browser_diagram())->edit_settings();
     return;
@@ -73,6 +81,44 @@ void SeqDiagramView::menu(const QPoint&) {
     load("Sequence");
     window()->package_modified();
     break;
+  case 29:
+    unselect_all();
+    sd->set_overlapping_bars(!overlapping);
+    if (overlapping)
+      toFlat();
+    else
+      toOverlapping();
+    canvas()->update();
+    window()->package_modified();
+    break;
+  }
+}
+
+void SeqDiagramView::toFlat() {
+  QCanvasItemList all = canvas()->allItems();
+  QCanvasItemList::Iterator cit;
+    
+  for (cit = all.begin(); cit != all.end(); ++cit) {
+    DiagramItem * it = QCanvasItemToDiagramItem(*cit);
+      
+    if ((it != 0) && // an uml canvas item
+	(*cit)->visible() &&
+	(it->type() == UmlLifeLine))
+      ((SdLifeLineCanvas *) it)->toFlat();
+  }
+}
+
+void SeqDiagramView::toOverlapping() {
+  QCanvasItemList all = canvas()->allItems();
+  QCanvasItemList::Iterator cit;
+    
+  for (cit = all.begin(); cit != all.end(); ++cit) {
+    DiagramItem * it = QCanvasItemToDiagramItem(*cit);
+      
+    if ((it != 0) && // an uml canvas item
+	(*cit)->visible() &&
+	(it->type() == UmlLifeLine))
+      ((SdLifeLineCanvas *) it)->toOverlapping();
   }
 }
 
@@ -85,6 +131,29 @@ void SeqDiagramView::contentsMousePressEvent(QMouseEvent * e) {
       
       switch (c) {
       case UmlClass:
+	{
+	  history_protected = FALSE;
+	  unselect_all();
+	  window()->selectOn();
+	  history_save();
+	  
+	  BrowserNode * parent =
+	    ((BrowserNode *) window()->browser_diagram()->parent());
+	  BrowserClass * b = 
+	    BrowserClass::get_class(parent);
+	  
+	  if (b != 0) {
+	    SdClassInstCanvas * cli = 
+	      new SdClassInstCanvas(b, the_canvas(), e->x(), 0);
+	    
+	    cli->show();
+	    cli->moveBy(e->x() - cli->center().x(), 0);
+	    cli->upper();
+	    window()->package_modified();
+	  }
+	}
+	break;
+      case UmlClassInstance:
 	{
 	  history_protected = TRUE;
 	  unselect_all();
@@ -93,11 +162,12 @@ void SeqDiagramView::contentsMousePressEvent(QMouseEvent * e) {
 	  
 	  BrowserNode * parent =
 	    ((BrowserNode *) window()->browser_diagram()->parent());
-	  BrowserClass * b = BrowserClass::get_class(parent);
+	  BrowserClassInstance * b = 
+	    BrowserClassInstance::get_classinstance(parent);
 	  
 	  if (b != 0) {
 	    SdClassInstCanvas * cli = 
-	      new SdClassInstCanvas(((BrowserClass *) b), the_canvas(), e->x(), 0);
+	      new SdClassInstCanvas(b, the_canvas(), e->x(), 0);
 	    
 	    cli->show();
 	    cli->moveBy(e->x() - cli->center().x(), 0);
@@ -108,6 +178,7 @@ void SeqDiagramView::contentsMousePressEvent(QMouseEvent * e) {
 	break;
       case UmlSyncSelfMsg:
       case UmlAsyncSelfMsg:
+      case UmlSelfReturnMsg:
 	{
 	  history_protected = TRUE;
 	  unselect_all();
@@ -208,6 +279,7 @@ void SeqDiagramView::keyPressEvent(QKeyEvent * e) {
 void SeqDiagramView::dragEnterEvent(QDragEnterEvent * e) {
   if (!window()->frozen() &&
       (UmlDrag::canDecode(e, UmlClass, TRUE, TRUE) ||
+       UmlDrag::canDecode(e, UmlClassInstance, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlClassDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlUseCaseDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlSeqDiagram, FALSE, TRUE) ||
@@ -226,11 +298,25 @@ void SeqDiagramView::dropEvent(QDropEvent * e) {
   BrowserNode * bn;
   QPoint p = viewportToContents(e->pos());
   
-  if ((bn = UmlDrag::decode(e, UmlClass)) != 0) {
+  if ((bn = UmlDrag::decode(e, UmlClassInstance)) != 0) {
     history_save();
     
     SdClassInstCanvas * cli = 
-      new SdClassInstCanvas(((BrowserClass *) bn), the_canvas(), p.x(), 0);
+      new SdClassInstCanvas((BrowserClassInstance *) bn, 
+			    the_canvas(), p.x(), 0);
+    
+    history_protected = TRUE;
+    cli->show();
+    cli->moveBy(p.x() - cli->center().x(), 0);
+    canvas()->update();
+    history_protected = FALSE;
+    window()->package_modified();
+  }
+  else if ((bn = UmlDrag::decode(e, UmlClass)) != 0) {
+    history_save();
+    
+    SdClassInstCanvas * cli = 
+      new SdClassInstCanvas(bn, the_canvas(), p.x(), 0);
     
     history_protected = TRUE;
     cli->show();
@@ -246,8 +332,8 @@ void SeqDiagramView::dropEvent(QDropEvent * e) {
 	   ((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlComponentDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlDeploymentDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlStateDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlActivityDiagram)) != 0)) {
+	   ((bn = UmlDrag::decode(e, UmlStateDiagram, TRUE)) != 0) ||
+	   ((bn = UmlDrag::decode(e, UmlActivityDiagram, TRUE)) != 0)) {
     history_save();
     
     IconCanvas * ic = new IconCanvas(bn, the_canvas(), p.x(), p.y(), 0);
@@ -279,6 +365,7 @@ void SeqDiagramView::save(QTextStream & st, QString & warning,
     case UmlFragment:
     case UmlContinuation:
     case UmlClass:
+    case UmlClassInstance:
     case UmlNote:
     case UmlText:
     case UmlIcon:
@@ -306,6 +393,7 @@ void SeqDiagramView::save(QTextStream & st, QString & warning,
     case UmlSyncSelfMsg:
     case UmlAsyncSelfMsg:
     case UmlReturnMsg:
+    case UmlSelfReturnMsg:
       if (!copy || di->copyable())
 	di->save(st, FALSE, warning);
     default:
@@ -365,4 +453,13 @@ void SeqDiagramView::read(char * st, char * k) {
     wrong_keyword(k, "end");
 }
 
+// for plug-out
 
+void SeqDiagramView::send(ToolCom * com) {
+  QCanvasItemList l = canvas()->allItems();
+  
+  FragmentCanvas::send(com, l);
+  SdClassInstCanvas::send(com, l);
+  SdLifeLineCanvas::send(com, l);
+  TextCanvas::send(com, l);
+}
