@@ -31,6 +31,7 @@
 class SeqDiagramWindow;
 class DiagramItem;
 class UmlCanvas;
+class ToolCom;
 
 class SeqDiagramView : public DiagramView {
   public:
@@ -40,10 +41,14 @@ class SeqDiagramView : public DiagramView {
     virtual void read(char *, char * k);
     virtual void save(QTextStream & st, QString & warning, bool copy) const;
     
+    void send(ToolCom * com);
+    
   private:
     SeqDiagramWindow * window() {
       return (SeqDiagramWindow *) parent();
-    };
+    }
+    void toFlat();
+    void toOverlapping();
 
   protected:
     virtual void contentsMousePressEvent(QMouseEvent *);
