@@ -52,6 +52,7 @@ const char * asyncmsgText = "Click this button to send an <em>asynchronous messa
 const char * syncselfmsgText = "Click this button to send a <em>reflexive synchronous message</em>";
 const char * asyncselfmsgText = "Click this button to send a <em>reflexive asynchronous message</em>";
 const char * returnmsgText = "Click this button to add an <em>explicitly return</em>";
+const char * selfreturnmsgText = "Click this button to add an <em>explicit reflexive return</em>";
 const char * textText = "Click this button to add a <em>text</em>";
 const char * addcontinuationText = "Click this button to add a <em>continuation</em>";
 
@@ -76,6 +77,12 @@ SeqDiagramWindow::SeqDiagramWindow(const QString & s, BrowserSeqDiagram * b, int
   addFragment->setToggleButton(TRUE);
   QWhatsThis::add(addFragment, addfragmentText);
   
+  addClassInstance
+    = new QToolButton(*classinstanceButton, "Add Class instance", QString::null,
+		      this, SLOT(hit_classinstance()), toolbar, "add class instance");
+  addClassInstance->setToggleButton(TRUE);
+  QWhatsThis::add(addClassInstance, addclassinstanceText);
+  
   addClass
     = new QToolButton(*classButton, "Add Class instance", QString::null,
 		      this, SLOT(hit_class()), toolbar, "add class");
@@ -118,6 +125,12 @@ SeqDiagramWindow::SeqDiagramWindow(const QString & s, BrowserSeqDiagram * b, int
   returnMsg->setToggleButton(TRUE);
   QWhatsThis::add(returnMsg, returnmsgText);
   
+  selfreturnMsg =
+    new QToolButton(*selfreturnmsgButton, "Explicit reflexive return", QString::null,
+		    this, SLOT(hit_selfreturnmsg()), toolbar, "explicit reflexive return");
+  selfreturnMsg->setToggleButton(TRUE);
+  QWhatsThis::add( selfreturnMsg,  selfreturnmsgText);
+  
   note =
     new QToolButton(*noteButton, "Note", QString::null,
 		    this, SLOT(hit_note()), toolbar, "note");
@@ -188,11 +201,13 @@ void SeqDiagramWindow::hit_button(UmlCode c, QToolButton * b) {
   select->setOn(FALSE);
   addFragment->setOn(FALSE);
   addClass->setOn(FALSE);
+  addClassInstance->setOn(FALSE);
   addContinuation->setOn(FALSE);
   syncMsg->setOn(FALSE);
   asyncMsg->setOn(FALSE);
   syncSelfMsg->setOn(FALSE);
   asyncSelfMsg->setOn(FALSE);
+  selfreturnMsg->setOn(FALSE);
   returnMsg->setOn(FALSE);
   note->setOn(FALSE);
   anchor->setOn(FALSE);
@@ -206,6 +221,10 @@ void SeqDiagramWindow::hit_fragment() {
   hit_button(UmlFragment, addFragment);
 }
 
+void SeqDiagramWindow::hit_classinstance() {
+  hit_button(UmlClassInstance, addClassInstance);
+}
+
 void SeqDiagramWindow::hit_class() {
   hit_button(UmlClass, addClass);
 }
@@ -234,6 +253,10 @@ void SeqDiagramWindow::hit_returnmsg() {
   hit_button(UmlReturnMsg, returnMsg);
 }
 
+void SeqDiagramWindow::hit_selfreturnmsg() {
+  hit_button(UmlSelfReturnMsg, selfreturnMsg);
+}
+
 void SeqDiagramWindow::hit_note() {
   hit_button(UmlNote, note);
 }
