@@ -40,6 +40,7 @@ class SeqDiagramWindow : public DiagramWindow {
   protected:
     SeqDiagramView * view;
     QToolButton * addClass;
+    QToolButton * addClassInstance;
     QToolButton * addContinuation;
     QToolButton * addFragment;
     QToolButton * syncMsg;
@@ -47,6 +48,7 @@ class SeqDiagramWindow : public DiagramWindow {
     QToolButton * syncSelfMsg;
     QToolButton * asyncSelfMsg;
     QToolButton * returnMsg;
+    QToolButton * selfreturnMsg;
     QToolButton * note;
     QToolButton * anchor;
     QToolButton * text;
@@ -61,6 +63,7 @@ class SeqDiagramWindow : public DiagramWindow {
 
   protected slots:
     void hit_class();
+    void hit_classinstance();
     void hit_continuation();
     void hit_fragment();
     void hit_syncmsg();
@@ -68,6 +71,7 @@ class SeqDiagramWindow : public DiagramWindow {
     void hit_syncselfmsg();
     void hit_asyncselfmsg();
     void hit_returnmsg();
+    void hit_selfreturnmsg();
     void hit_note();
     void hit_anchor();
     void hit_text();
