@@ -238,12 +238,11 @@ void StateDiagramView::dragEnterEvent(QDragEnterEvent * e) {
        UmlDrag::canDecode(e, UmlColDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlObjectDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlComponentDiagram, FALSE, TRUE) ||
-       UmlDrag::canDecode(e, UmlStateDiagram, FALSE, TRUE) ||
+       UmlDrag::canDecode(e, UmlStateDiagram, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlDeploymentDiagram, FALSE, TRUE) ||
        UmlDrag::canDecode(e, UmlPseudoState, TRUE, TRUE) ||
        UmlDrag::canDecode(e, UmlStateAction, TRUE, TRUE) ||
-       UmlDrag::canDecode(e, UmlActivityDiagram, TRUE, TRUE) ||
-       UmlDrag::canDecode(e, UmlStateDiagram, TRUE, TRUE)))
+       UmlDrag::canDecode(e, UmlActivityDiagram, TRUE, TRUE)))
     e->accept();
   else
     e->ignore();
@@ -338,8 +337,8 @@ void StateDiagramView::dropEvent(QDropEvent * e) {
 	   ((bn = UmlDrag::decode(e, UmlStateDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlComponentDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlDeploymentDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlStateDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlActivityDiagram)) != 0)) {
+	   ((bn = UmlDrag::decode(e, UmlStateDiagram, TRUE)) != 0) ||
+	   ((bn = UmlDrag::decode(e, UmlActivityDiagram, TRUE)) != 0)) {
     history_save();
     
     IconCanvas * ic = new IconCanvas(bn, the_canvas(), p.x(), p.y(), 0);
