@@ -41,6 +41,7 @@
 #include "myio.h"
 #include "MenuTitle.h"
 #include "strutil.h"
+#include "ToolCom.h"
 
 SubjectCanvas::SubjectCanvas(UmlCanvas * canvas, int x, int y, int id)
     : DiagramCanvas(0, canvas, x, y, SUBJECT_CANVAS_MIN_SIZE,
@@ -77,18 +78,13 @@ void SubjectCanvas::draw(QPainter & p) {
   p.setBackgroundColor(co);
   p.setFont(the_canvas()->get_font(UmlNormalBoldFont));
   
-  if (used_color != UmlTransparent)  {
+  if (used_color != UmlTransparent)
     p.fillRect(r, co);
-    
-    if (fp != 0)
-      fprintf(fp, "\t<rect fill=\"#%06x\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
-	      " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
-	      co.rgb()&0xffffff, 
-	      r.x(), r.y(), r.width() - 1, r.height() - 1);
-  }
-  else if (fp != 0)
-    fprintf(fp, "\t<rect fill=\"none\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
+  
+  if (fp != 0)
+    fprintf(fp, "\t<rect fill=\"%s\" stroke=\"black\" stroke-width=\"1\" stroke-opacity=\"1\""
 	    " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	    svg_color(used_color), 
 	    r.x(), r.y(), r.width() - 1, r.height() - 1);
 
   p.drawRect(r);
@@ -302,6 +298,29 @@ void SubjectCanvas::resize(aCorner c, int dx, int dy) {
   DiagramCanvas::resize(c, dx, dy, min_width, min_height);
 }
 
+void SubjectCanvas::prepare_for_move(bool on_resize) {
+  if (!on_resize) {
+    DiagramCanvas::prepare_for_move(on_resize);
+    
+    QRect r = rect();
+    QCanvasItemList l = collisions(TRUE);
+    QCanvasItemList::ConstIterator it;
+    QCanvasItemList::ConstIterator end = l.end();
+    DiagramItem * di;
+  
+    for (it = l.begin(); it != end; ++it) {
+      if ((*it)->visible() && // at least not deleted
+	  !(*it)->selected() &&
+	  ((di = QCanvasItemToDiagramItem(*it)) != 0) &&
+	  r.contains(di->rect(), TRUE) &&
+	  di->move_with(UmlSubject)) {
+	the_canvas()->select(*it);
+	di->prepare_for_move(FALSE);
+      }
+    }
+  }
+}
+
 void SubjectCanvas::save(QTextStream & st, bool ref, QString &) const {
   if (ref) {
     st << "subject_ref " << get_ident();
@@ -378,3 +397,30 @@ void SubjectCanvas::history_hide() {
   disconnect(DrawingSettings::instance(), SIGNAL(changed()), this, SLOT(modified()));
 }
 
+// for plug outs
+
+void SubjectCanvas::send(ToolCom * com, QCanvasItemList & all)
+{
+  QList<SubjectCanvas> subjects;
+  QCanvasItemList::Iterator cit;
+
+  for (cit = all.begin(); cit != all.end(); ++cit) {
+    DiagramItem *di = QCanvasItemToDiagramItem(*cit);
+    
+    if ((di != 0) && 
+	(*cit)->visible() &&
+	(di->type() == UmlSubject))
+      subjects.append((SubjectCanvas *) di);
+  }
+  
+  com->write_unsigned(subjects.count());
+  
+  SubjectCanvas * sc;
+  
+  for (sc = subjects.first(); sc != 0; sc = subjects.next()) {
+    QCString s = fromUnicode(sc->name);
+    
+    com->write_string((const char *) s);
+    com->write(sc->rect());
+  }
+}
