@@ -28,6 +28,8 @@
 
 #include "DiagramCanvas.h"
 
+class ToolCom;
+
 #define SUBJECT_CANVAS_MIN_SIZE 30
 
 class SubjectCanvas : public QObject, public DiagramCanvas {
@@ -60,6 +62,7 @@ class SubjectCanvas : public QObject, public DiagramCanvas {
     virtual const char * may_connect(UmlCode & l, const DiagramItem * dest) const;
     virtual aCorner on_resize_point(const QPoint &);
     virtual void resize(aCorner c, int dx, int dy);
+    virtual void prepare_for_move(bool on_resize);
     virtual void change_scale();
     
     virtual bool has_drawing_settings() const;
@@ -74,6 +77,8 @@ class SubjectCanvas : public QObject, public DiagramCanvas {
     virtual void history_load(QBuffer &);
     virtual void history_hide();
     
+    static void send(ToolCom * com, QCanvasItemList & all);
+    
   private slots:
     void modified();
 };
