@@ -112,6 +112,14 @@ void UcClassCanvas::modified() {
   package_modified();
 }
 
+void UcClassCanvas::post_loaded() {
+  force_self_rel_visible();
+  if (the_canvas()->must_draw_all_relations()) {
+    draw_all_depend_gene();    
+    draw_all_simple_relations();
+  }
+}
+
 UmlCode UcClassCanvas::type() const {
   return UmlClass;
 }
@@ -312,12 +320,15 @@ void UcClassCanvas::draw_all_depend_gene(UcClassCanvas * end) {
 	if (di != 0)
 	  (new RelationCanvas(the_canvas(), this, di,
 			      ((BrowserClass *) browser_node), 
-			      def->get_type(), 0, def))->show();
+			      def->get_type(), 0, -1.0, -1.0, def))
+	    ->show();
       }
     }
   }
   
-  if ((end == 0) && !DrawingSettings::just_modified()) {
+  if ((end == 0) &&
+      !DrawingSettings::just_modified() &&
+      !on_load_diagram()) {
     for (cit = all.begin(); cit != all.end(); ++cit) {
       DiagramItem * di = QCanvasItemToDiagramItem(*cit);
       
@@ -336,9 +347,9 @@ void UcClassCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * a;
   
   if ((dest->type() == UmlClass) && IsaRelation(action))
-    a = new RelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new RelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
