@@ -32,6 +32,7 @@
 #include <qpopupmenu.h> 
 
 #include "UcUseCaseCanvas.h"
+#include "UcClassCanvas.h"
 #include "SimpleRelationCanvas.h"
 #include "BasicData.h"
 #include "BrowserUseCase.h"
@@ -83,18 +84,13 @@ void UcUseCaseCanvas::draw(QPainter & p) {
   if (! visible()) return;
   
   QRect r = rect();
-  UmlColor co;
   UseCaseData * data = (UseCaseData *) browser_node->get_data();
   
-  if (itscolor != UmlDefaultColor)
-    co = itscolor;
-  else {
-    if (used_color == UmlDefaultColor)
-      used_color = the_canvas()->browser_diagram()->get_color(UmlUseCase);
-    co = used_color;
-  }
+  used_color = (itscolor == UmlDefaultColor)
+    ? the_canvas()->browser_diagram()->get_color(UmlUseCase)
+    : itscolor;
   
-  QColor col = color(co);
+  QColor col = color(used_color);
   QBrush brsh = p.brush();
   bool realizationp =
     !strcmp(data->get_stereotype(), "realization");
@@ -105,7 +101,7 @@ void UcUseCaseCanvas::draw(QPainter & p) {
   if (fp != 0)
     fputs("<g>\n", fp);
   
-  if (co != UmlTransparent) {
+  if (used_color != UmlTransparent) {
     const int shadow = the_canvas()->shadow() - 1;
     
     if (shadow != -1) {
@@ -127,9 +123,9 @@ void UcUseCaseCanvas::draw(QPainter & p) {
     }
 
     if (fp != 0)
-      fprintf(fp, "\t<ellipse fill=\"#%06x\" stroke=\"black\"%s stroke-width=\"1\" stroke-opacity=\"1\""
+      fprintf(fp, "\t<ellipse fill=\"%s\" stroke=\"black\"%s stroke-width=\"1\" stroke-opacity=\"1\""
 	      " cx=\"%d\" cy=\"%d\" rx=\"%d\" ry=\"%d\" />\n",
-	      col.rgb()&0xffffff,
+	      svg_color(used_color),
 	      (realizationp) ? " stroke-dasharray=\"4,4\"" : "",
 	      r.left() + rx, r.top() + ry, rx, ry);
   }
@@ -139,7 +135,7 @@ void UcUseCaseCanvas::draw(QPainter & p) {
 	    (realizationp) ? " stroke-dasharray=\"4,4\"" : "",
 	    r.left() + rx, r.top() + ry, rx, ry);
   
-  p.setBackgroundMode((co == UmlTransparent) ? QObject::TransparentMode : QObject::OpaqueMode);
+  p.setBackgroundMode((used_color == UmlTransparent) ? QObject::TransparentMode : QObject::OpaqueMode);
   p.setBrush(col);
 
   if (realizationp)
@@ -224,12 +220,28 @@ void UcUseCaseCanvas::open() {
   browser_node->open(FALSE);
 }
 
+void UcUseCaseCanvas::update_name() {
+  QString oldname = label->get_name();
+  const char * st = browser_node->get_data()->get_stereotype();
+  QString newname;
+  
+  if ((*st == 0) || !strcmp(st, "realization"))
+    newname = browser_node->get_name();
+  else
+    newname = QString("<<") + st + QString(">>\n") + browser_node->get_name();
+  
+  if (newname != oldname) {
+    label->set_name(newname);
+    label->recenter();
+  }
+}
+
 void UcUseCaseCanvas::modified() {
   used_color = (itscolor == UmlDefaultColor)
     ? the_canvas()->browser_diagram()->get_color(UmlUseCase)
     : itscolor;
   
-  label->set_name(browser_node->get_name());
+  update_name();
   
   // force son reaffichage
   hide();
@@ -241,6 +253,12 @@ void UcUseCaseCanvas::modified() {
   package_modified();
 }
 
+void UcUseCaseCanvas::post_loaded() {
+  force_self_rel_visible();
+  if (the_canvas()->must_draw_all_relations())
+    draw_all_simple_relations();
+}
+
 void UcUseCaseCanvas::menu(const QPoint&) {
   QPopupMenu m(0);
   QPopupMenu toolm(0);
@@ -424,9 +442,9 @@ void UcUseCaseCanvas::connexion(UmlCode action, DiagramItem * dest,
   ArrowCanvas * a;
   
   if (IsaSimpleRelation(action))
-    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0);
+    a = new SimpleRelationCanvas(the_canvas(), this, dest, 0, action, 0, -1.0, -1.0);
   else
-    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE);
+    a = new ArrowCanvas(the_canvas(), this, dest, action, 0, FALSE, -1.0, -1.0);
   
   a->show();
   the_canvas()->select(a);
@@ -498,6 +516,7 @@ UcUseCaseCanvas * UcUseCaseCanvas::read(char * & st, UmlCanvas * canvas, char *
     else if (strcmp(k, "label_xy"))
       wrong_keyword(k, "label_xy/label_xyz");
     
+    result->update_name();
     result->show();
     return result;
   }
@@ -534,3 +553,76 @@ void UcUseCaseCanvas::history_load(QBuffer & b) {
   connect(browser_node->get_data(), SIGNAL(changed()), this, SLOT(modified()));
   connect(browser_node->get_data(), SIGNAL(deleted()), this, SLOT(deleted()));
 }
+
+// for plug outs
+
+void UcUseCaseCanvas::send(ToolCom * com, QCanvasItemList & all)
+{
+  QList<UcUseCaseCanvas> lu;
+  QList<UcClassCanvas> la;
+  QCanvasItemList::Iterator cit;
+
+  for (cit = all.begin(); cit != all.end(); ++cit) {
+    DiagramItem *di = QCanvasItemToDiagramItem(*cit);
+    
+    if ((di != 0) && (*cit)->visible()) {
+      switch (di->type()) {
+      case UmlUseCase:
+	lu.append((UcUseCaseCanvas *) di);
+	break;
+      case UmlClass:
+	la.append((UcClassCanvas *) di);
+	break;
+      default:
+	break;
+      }
+    }
+  }
+  
+  // send UC
+  
+  com->write_unsigned(lu.count());
+  
+  QListIterator<UcUseCaseCanvas> itu(lu);
+  
+  for (; itu.current(); ++itu) {
+    com->write_unsigned((unsigned) itu.current()->get_ident());
+    itu.current()->get_bn()->write_id(com);
+    com->write(itu.current()->rect());
+  }
+  
+  // send Actors
+  
+  com->write_unsigned(la.count());
+  
+  QListIterator<UcClassCanvas> ita(la);
+  
+  for (; ita.current(); ++ita)
+    ita.current()->get_bn()->write_id(com);
+  
+  // send rels
+  
+  QList<ArrowCanvas> lr;
+  
+  for (itu.toFirst(); itu.current(); ++itu) {
+    QListIterator<ArrowCanvas> itl(itu.current()->lines);
+    
+    for (; itl.current(); ++itl) {
+      ArrowCanvas * r = itl.current();
+      DiagramItem * from = r->get_start();
+      DiagramItem * to = r->get_end();
+      
+      if ((from->type() == UmlUseCase)
+	  ? (to->type() == UmlClass)
+	  : (from->type() == UmlClass))
+	lr.append(r);
+    }
+  }
+  
+  com->write_unsigned(lr.count());
+  
+  QListIterator<ArrowCanvas> itr(lr);
+  
+  for (; itr.current(); ++itr)
+    itr.current()->write_uc_rel(com);
+}
