@@ -58,11 +58,13 @@ class UmlWindow : public QMainWindow {
     QPopupMenu * projectMenu;
     QPopupMenu * windowsMenu;
     QPopupMenu * toolMenu;
+    QPopupMenu * langMenu;
     QPopupMenu * miscMenu;
     QPopupMenu * fontSizeMenu;
     QPopupMenu * formatMenu;
     int use_cpp_id;
     int use_java_id;
+    int use_php_id;
     int use_idl_id;
     int verbose_gen_id;
     int preserve_bodies_id;
@@ -86,6 +88,7 @@ class UmlWindow : public QMainWindow {
     bool can_close();
     
     static void set_commented(BrowserNode * bn);
+    static void update_comment_if_needed(BrowserNode * bn);
     static void set_message(const QString &);
     static QWorkspace * get_workspace();
     static void clear();
@@ -125,6 +128,7 @@ class UmlWindow : public QMainWindow {
     
     void use_cpp();
     void use_java();
+    void use_php();
     void use_idl();
     void verbose();
     void preserve();
@@ -145,13 +149,16 @@ class UmlWindow : public QMainWindow {
     void show_trace();
     void cpp_generate();
     void java_generate();
+    void php_generate();
     void idl_generate();
     void java_catalog();
     //void doc_generate();
     void cpp_reverse();
     void java_reverse();
+    void php_reverse();
     void cpp_roundtrip();
     void java_roundtrip();
+    void php_roundtrip();
     void run_tool(int param);
     void tool_settings();
     void import_tool_settings();
@@ -164,6 +171,8 @@ class UmlWindow : public QMainWindow {
     void projectMenuAboutToShow();
     void historicActivated(int id);
     
+    void langMenuAboutToShow();
+    
     void miscMenuAboutToShow();
     
     void fontSizeMenuAboutToShow();
@@ -174,6 +183,9 @@ class UmlWindow : public QMainWindow {
     
     void windowsMenuAboutToShow();
     void windowsMenuActivated(int id);
+
+    virtual void keyPressEvent(QKeyEvent * e);
+
 };
 
 #endif
