@@ -233,8 +233,8 @@ void UseCaseDiagramView::dropEvent(QDropEvent * e) {
 	   ((bn = UmlDrag::decode(e, UmlObjectDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlComponentDiagram)) != 0) ||
 	   ((bn = UmlDrag::decode(e, UmlDeploymentDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlStateDiagram)) != 0) ||
-	   ((bn = UmlDrag::decode(e, UmlActivityDiagram)) != 0)) {
+	   ((bn = UmlDrag::decode(e, UmlStateDiagram, TRUE)) != 0) ||
+	   ((bn = UmlDrag::decode(e, UmlActivityDiagram, TRUE)) != 0)) {
     history_save();
     
     IconCanvas * ic = new IconCanvas(bn, the_canvas(), p.x(), p.y(), 0);
@@ -333,4 +333,13 @@ void UseCaseDiagramView::read(char * st, char * k) {
     wrong_keyword(k, "end");
 }
 
+// for plug-out
+
+void UseCaseDiagramView::send(ToolCom * com) {
+  QCanvasItemList l = canvas()->allItems();
+  
+  FragmentCanvas::send(com, l);
+  SubjectCanvas::send(com, l);
+  UcUseCaseCanvas::send(com, l);
+}
 
