@@ -53,7 +53,7 @@ AboutDialog::AboutDialog() : QDialog(0, "About BOUML", TRUE) {
   lbp->setPixmap(bp);
   hbox->addWidget(lbp);
   hbox->addWidget(new QLabel("  ", this));
-  hbox->addWidget(new QLabel("BOUML release 2.26.3\n\ne-mail : bouml@free.fr\nhome : http://bouml.free.fr",
+  hbox->addWidget(new QLabel("BOUML release 3.3.4\n\ne-mail : bouml@free.fr\nhome : http://bouml.free.fr\n\nBruno Pages",
 			     this));
   
   hbox = new QHBoxLayout(vbox); 
