@@ -83,6 +83,10 @@ AttributeDialog::AttributeDialog(AttributeData * a)
   java_in_enum_pattern = !java_in_enum && (lang_st == "enum_pattern");
   java_in_typedef = !java_in_enum && (lang_st == "typedef");
   
+  lang_st = GenerationSettings::php_class_stereotype(stereotype);
+  php_in_enum = in_enum || (lang_st == "enum");
+  php_in_typedef = !php_in_enum && (lang_st == "typedef");
+  
   lang_st = GenerationSettings::idl_class_stereotype(stereotype);
   idl_in_enum = in_enum || (lang_st == "enum");
   idl_in_typedef = !idl_in_enum && (lang_st == "typedef");
@@ -154,10 +158,9 @@ AttributeDialog::AttributeDialog(AttributeData * a)
     
     edtype = new QComboBox(!visit, grid);
     edtype->insertItem(a->get_type().get_full_type());
+    BrowserClass::instances(nodes);
+    nodes.full_names(list);        
     if (!visit) {
-      BrowserClass::instances(nodes);
-      nodes.full_names(list);
-        
       edtype->insertStringList(GenerationSettings::basic_types());
       offset = edtype->count();
       edtype->insertStringList(list);
@@ -165,13 +168,27 @@ AttributeDialog::AttributeDialog(AttributeData * a)
       view = a->browser_node->container(UmlClass);
     }
     edtype->setCurrentItem(0);
-    
     edtype->setSizePolicy(sp);
 
+    new QLabel("multiplicity : ", grid);
+    htab = new QHBox(grid);
+    htab->setMargin(0);
+    multiplicity = new QComboBox(!visit, htab);
+    multiplicity->setSizePolicy(sp);
+    multiplicity->insertItem(a->get_multiplicity());
+    if (!visit) {
+      multiplicity->insertItem("1");
+      multiplicity->insertItem("0..1");
+      multiplicity->insertItem("*");
+      multiplicity->insertItem("1..*");
+    }
+
     new QLabel("initial value :", grid);
   }
-  else
+  else {
+    multiplicity = 0;
     new QLabel("value :", grid);
+  }
   
   htab = new QHBox(grid);
   edinit = new LineEdit(a->get_init_value(), htab);
@@ -291,6 +308,9 @@ AttributeDialog::AttributeDialog(AttributeData * a)
     }
     
     addTab(grid, "C++");
+  
+    if (!GenerationSettings::cpp_get_default_defs())
+      removePage(grid);
   }
   else
     cpptab = 0;
@@ -353,10 +373,53 @@ AttributeDialog::AttributeDialog(AttributeData * a)
 	    this, SLOT(java_edit_annotation()));
     
     addTab(grid, "Java");
+  
+    if (!GenerationSettings::java_get_default_defs())
+      removePage(grid);
   }
   else
     javatab = 0;
   
+  // Php
+  
+  if (! php_in_typedef) {
+    grid = new QGrid(2, this);
+    phptab = grid;
+    grid->setMargin(5);
+    grid->setSpacing(5);
+    
+    new QLabel("Declaration :", grid);
+    edphpdecl = new MultiLineEdit(grid);
+    edphpdecl->setText(a->get_phpdecl());
+    edphpdecl->setFont(font);
+    if (visit)
+      edphpdecl->setReadOnly(TRUE);
+    else
+      connect(edphpdecl, SIGNAL(textChanged()), this, SLOT(php_update()));
+    
+    new QLabel("Result after\nsubstitution :", grid);
+    showphpdecl = new MultiLineEdit(grid);
+    showphpdecl->setReadOnly(TRUE);
+    showphpdecl->setFont(font);
+    
+    new QLabel(grid);
+    htab = new QHBox(grid);
+
+    if (! visit) {
+      connect(new QPushButton("Default declaration", htab), SIGNAL(pressed ()),
+	      this, SLOT(php_default()));
+      connect(new QPushButton("Not generated in Php", htab), SIGNAL(pressed ()),
+	      this, SLOT(php_unmapped())); 
+    }
+    
+    addTab(grid, "Php");
+  
+    if (!GenerationSettings::php_get_default_defs())
+      removePage(grid);
+  }
+  else
+    phptab = 0;
+  
   // IDL
   
   if (! idl_in_typedef) {
@@ -411,14 +474,21 @@ AttributeDialog::AttributeDialog(AttributeData * a)
 					    : "Default Attribute declaration",
 			      htab),
 	      SIGNAL(pressed ()), this, SLOT(idl_default()));
-      if (!idl_in_enum && !idl_in_union && !idl_in_struct)
-	connect(new QPushButton("Default State declaration", htab), SIGNAL(pressed ()),
-		this, SLOT(idl_default_state()));
+      if (!idl_in_enum && !idl_in_union) {
+	if (!idl_in_struct)
+	  connect(new QPushButton("Default State declaration", htab), SIGNAL(pressed ()),
+		  this, SLOT(idl_default_state()));
+	connect(new QPushButton("Default constant declaration", htab), SIGNAL(pressed ()),
+		this, SLOT(idl_default_constant()));
+      }
       connect(new QPushButton("Not generated in Idl", htab), SIGNAL(pressed ()),
 	      this, SLOT(idl_unmapped()));
     }
     
     addTab(grid, "Idl");
+  
+    if (!GenerationSettings::idl_get_default_defs())
+      removePage(grid);
   }
   else
     idltab = 0;
@@ -550,6 +620,8 @@ void AttributeDialog::accept() {
     
       att->isa_const_attribute = constattribute_cb->isChecked();
 
+      att->multiplicity = multiplicity->currentText().stripWhiteSpace();
+
       att->set_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
     }
     
@@ -573,6 +645,8 @@ void AttributeDialog::accept() {
     att->java_decl = (java_in_typedef) ? QString::null : edjavadecl->text();
     att->java_annotation = javaannotation;
     
+    att->php_decl = (php_in_typedef) ? QString::null : edphpdecl->text();
+    
     att->idl_decl = (idl_in_typedef) ? QString::null : edidldecl->text();
     if (idl_in_union) {
       int index;
@@ -585,7 +659,7 @@ void AttributeDialog::accept() {
     }
     
     bn->set_comment(comment->text());
-    UmlWindow::set_commented(bn);
+    UmlWindow::update_comment_if_needed(bn);
   
     att->constraint = constraint->stripWhiteSpaceText();
     
@@ -616,6 +690,11 @@ void AttributeDialog::update_all_tabs(QWidget * w) {
     if (!visit)
       edjavadecl->setFocus();
   }
+  else if (w == phptab) {
+    php_update();
+    if (!visit)
+      edphpdecl->setFocus();
+  }
   else if (w == idltab) {
     idl_update();
     if (!visit)
@@ -658,8 +737,9 @@ void AttributeDialog::post_edit_init(AttributeDialog * d, QString s)
 }
 
 void AttributeDialog::cpp_default() {
-  edcppdecl->setText((cpp_in_enum) ? GenerationSettings::cpp_default_enum_item_decl()
-				   : GenerationSettings::cpp_default_attr_decl());
+  edcppdecl->setText((cpp_in_enum) 
+		     ? GenerationSettings::cpp_default_enum_item_decl()
+		     : GenerationSettings::cpp_default_attr_decl(multiplicity->currentText().stripWhiteSpace()));
   cpp_update();
 }
 
@@ -751,8 +831,19 @@ void AttributeDialog::cpp_update() {
     }
     else if (!strncmp(p, "${type}", 7)) {
       p += 7;
-      s += GenerationSettings::cpp_type(type(edtype->currentText().stripWhiteSpace(),
-					     list, nodes));
+      s += get_cpp_name(the_type(edtype->currentText().stripWhiteSpace(),
+				 list, nodes));
+    }
+    else if (!strncmp(p, "${multiplicity}", 15)) {
+      p += 15;
+      
+      QString m = multiplicity->currentText().stripWhiteSpace();
+      
+      s += (*m == '[') ? m : QString("[") + m + "]";
+    }
+    else if (!strncmp(p, "${stereotype}", 13)) {
+      p += 13;
+      s += GenerationSettings::cpp_relationattribute_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
     }
     else if (*p == '@')
       manage_alias(att->browser_node, p, s, kvtable);
@@ -798,7 +889,18 @@ QString AttributeDialog::cpp_decl(const BrowserAttribute * at)
       p += 11;
     else if (!strncmp(p, "${type}", 7)) {
       p += 7;
-      s += GenerationSettings::cpp_type(d->type.get_type());
+      s += get_cpp_name(d->type);
+    }
+    else if (!strncmp(p, "${multiplicity}", 15)) {
+      p += 15;
+      
+      QString m = d->get_multiplicity();
+      
+      s += (*m == '[') ? m : QString("[") + m + "]";
+    }
+    else if (!strncmp(p, "${stereotype}", 13)) {
+      p += 13;
+      s += GenerationSettings::cpp_relationattribute_stereotype(d->stereotype);
     }
     else if (*p == '\n') {
       s += ' ';
@@ -830,7 +932,7 @@ void AttributeDialog::java_default() {
   else {
     if (java_in_enum)
       edstereotype->setCurrentItem(attribute_st_rank);
-    edjavadecl->setText(GenerationSettings::java_default_attr_decl());
+    edjavadecl->setText(GenerationSettings::java_default_attr_decl(multiplicity->currentText().stripWhiteSpace()));
   }
   java_update();
 }
@@ -939,8 +1041,33 @@ void AttributeDialog::java_update() {
     }
     else if (!strncmp(p, "${type}", 7)) {
       p += 7;
-      s += GenerationSettings::java_type(type(edtype->currentText().stripWhiteSpace(),
-					      list, nodes));
+      s += get_java_name(the_type(edtype->currentText().stripWhiteSpace(),
+				  list, nodes));
+    }
+    else if (!strncmp(p, "${multiplicity}", 15)) {
+      p += 15;
+      
+      QString m = multiplicity->currentText().stripWhiteSpace();
+      
+      if (*m != '[')
+	s += "[]";
+      else {
+	for (unsigned index = 0; index != m.length(); index += 1) {
+	  switch (m.at(index).latin1()) {
+	  case '[':
+	    s += '[';
+	    break;
+	  case ']':
+	    s += ']';
+	  default:
+	    break;
+	  }
+	}
+      }
+    }
+    else if (!strncmp(p, "${stereotype}", 13)) {
+      p += 13;
+      s += GenerationSettings::java_relationattribute_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
     }
     else
       s += *p++;
@@ -991,7 +1118,32 @@ QString AttributeDialog::java_decl(const BrowserAttribute * at)
       p += 8;
     else if (!strncmp(p, "${type}", 7)) {
       p += 7;
-      s += GenerationSettings::java_type(d->type.get_type());
+      s += get_java_name(d->type);
+    }
+    else if (!strncmp(p, "${multiplicity}", 15)) {
+      p += 15;
+      
+      QString m = d->get_multiplicity();
+      
+      if (*m != '[')
+	s += "[]";
+      else {
+	for (unsigned index = 0; index != m.length(); index += 1) {
+	  switch (m.at(index).latin1()) {
+	  case '[':
+	    s += '[';
+	    break;
+	  case ']':
+	    s += ']';
+	  default:
+	    break;
+	  }
+	}
+      }
+    }
+    else if (!strncmp(p, "${stereotype}", 13)) {
+      p += 13;
+      s += GenerationSettings::java_relationattribute_stereotype(d->get_multiplicity());
     }
     else if (!strncmp(p, "${@}", 4))
       p += 4;
@@ -1019,21 +1171,187 @@ void AttributeDialog::java_edit_annotation() {
     java_update();
 }
   
+void AttributeDialog::php_default() {
+  if (php_in_enum)
+    edphpdecl->setText(GenerationSettings::php_default_enum_item_decl());
+  else
+    edphpdecl->setText(GenerationSettings::php_default_attr_decl());
+
+  php_update();
+}
+
+void AttributeDialog::php_unmapped() {
+  edphpdecl->setText(QString::null);
+  showphpdecl->setText(QString::null);
+}
+
+void AttributeDialog::php_update() {
+  // do NOT write
+  //	const char * p = edphpdecl->text();
+  // because the QString is immediatly destroyed !
+  QString def = edphpdecl->text();
+  const char * p = def;
+  const char * pp = 0;
+  QString indent = "";
+  QString s;
+
+  while ((*p == ' ') || (*p == '\t'))
+    indent += *p++;
+  
+  s = indent;
+  
+  for (;;) {
+    if (*p == 0) {
+      if (pp == 0)
+	break;
+      
+      // comment management done
+      p = pp;
+      pp = 0;
+      if (*p == 0)
+	break;
+      s += indent;
+    }
+      
+    if (!strncmp(p, "${comment}", 10))
+      manage_comment(comment->text(), p, pp, FALSE);
+    else if (!strncmp(p, "${description}", 14))
+      manage_description(comment->text(), p, pp);
+    else if (!strncmp(p, "${name}", 7)) {
+      p += 7;
+      if (!php_in_enum && !constattribute_cb->isChecked())
+	s += "$";
+      s += edname->text();
+    }
+    else if (!strncmp(p, "${value}", 8)) {
+      p += 8;
+      
+      if (!edinit->text().stripWhiteSpace().isEmpty()) {
+	s += (edinit->text().stripWhiteSpace().at(0) == QChar('='))
+	  ? " " : " = ";
+	s += edinit->text();
+      }
+      else if (php_in_enum)
+	s += " = ...";
+    }
+    else if (!strncmp(p, "${const}", 8)) {
+      p += 8;
+      if (constattribute_cb->isChecked())
+	s += "const ";
+    }
+    else if (!strncmp(p, "${var}", 6)) {
+      p += 6;
+      if (!php_in_enum && 
+	  !constattribute_cb->isChecked() &&
+	  !classattribute_cb->isChecked() &&
+	  (uml_visibility.value() == UmlPackageVisibility))
+	s += "var ";
+    }
+    else if (*p == '\n') {
+      s += *p++;
+      if (*p)
+	s += indent;
+    }
+    else if (*p == '@')
+      manage_alias(att->browser_node, p, s, kvtable);
+    else if (!strncmp(p, "${visibility}", 13)) {
+      p += 13;
+      if (uml_visibility.value() != UmlPackageVisibility)
+	s += uml_visibility.state() + ' ';
+    }
+    else if (!strncmp(p, "${static}", 9)) {
+      p += 9;
+      if (classattribute_cb->isChecked())
+	s += "static ";
+    }
+    else
+      s += *p++;
+  }
+
+  showphpdecl->setText(s);
+}
+
+QString AttributeDialog::php_decl(const BrowserAttribute * at)
+{
+  QString s;
+  AttributeData * d = (AttributeData *) at->get_data();
+  QCString decl = d->php_decl;
+  
+  remove_comments(decl);
+  
+  const char * p = decl;
+  
+  while ((*p == ' ') || (*p == '\t'))
+    p += 1;
+  
+  while (*p) {
+    if (!strncmp(p, "${comment}", 10))
+      p += 10;
+    else if (!strncmp(p, "${description}", 14))
+      p += 14;
+    else if (!strncmp(p, "${name}", 7)) {
+      p += 7;
+      
+      ClassData * cld = (ClassData *) 
+	((BrowserNode *) at->parent())->get_data();
+      QString stereotype = cld->get_stereotype();
+      bool in_enum = (stereotype == "enum");
+      QString lang_st = GenerationSettings::php_class_stereotype(stereotype);
+      
+      in_enum |= (lang_st == "enum");
+
+      if (!in_enum &&
+	  !((AttributeData *) at->get_data())->get_isa_const_attribute())
+	s += "$";
+      s += at->get_name();
+    }
+    else if (!strncmp(p, "${value}", 8))
+      break;
+    else if (!strncmp(p, "${const}", 8))
+      p += 8;
+    else if (!strncmp(p, "${visibility}", 13))
+      p += 13;
+    else if (!strncmp(p, "${static}", 9))
+      p += 9;
+    else if (!strncmp(p, "${var}", 6))
+      p += 6;
+    else if (*p == '\n') {
+      s += ' ';
+      do
+	p+= 1;
+      while ((*p == ' ') || (*p == '\t'));
+    }
+    else if (*p == ';')
+      break;
+    else if (*p == '@')
+      manage_alias(at, p, s, 0);
+    else
+      s += *p++;
+  }
+
+  return s;
+}
+
 void AttributeDialog::idl_default() {
   if (idl_in_enum)
     edidldecl->setText(GenerationSettings::idl_default_enum_item_decl());
   else if (idl_in_union)
-    edidldecl->setText(GenerationSettings::idl_default_union_item_decl());
+    edidldecl->setText(GenerationSettings::idl_default_union_item_decl(multiplicity->currentText().stripWhiteSpace()));
   else
     edidldecl->setText((constattribute_cb->isChecked() &&
 			!edinit->text().stripWhiteSpace().isEmpty())
-		       ? GenerationSettings::idl_default_const_decl()
-		       : GenerationSettings::idl_default_attr_decl());
+		       ? GenerationSettings::idl_default_const_decl(multiplicity->currentText().stripWhiteSpace())
+		       : GenerationSettings::idl_default_attr_decl(multiplicity->currentText().stripWhiteSpace()));
   idl_update();
 }
 
 void AttributeDialog::idl_default_state() {
-  edidldecl->setText(GenerationSettings::idl_default_valuetype_attr_decl());
+  edidldecl->setText(GenerationSettings::idl_default_valuetype_attr_decl(multiplicity->currentText().stripWhiteSpace()));
+  idl_update();
+}
+
+void AttributeDialog::idl_default_constant() {
+  edidldecl->setText(GenerationSettings::idl_default_const_decl(multiplicity->currentText().stripWhiteSpace()));
   idl_update();
 }
 
@@ -1123,8 +1441,16 @@ void AttributeDialog::idl_update() {
     }
     else if (!strncmp(p, "${type}", 7)) {
       p += 7;
-      s += GenerationSettings::idl_type(type(edtype->currentText().stripWhiteSpace(),
-					     list, nodes));
+      s += get_idl_name(the_type(edtype->currentText().stripWhiteSpace(),
+				 list, nodes));
+    }
+    else if (!strncmp(p, "${stereotype}", 13)) {
+      p += 13;
+      s += GenerationSettings::idl_relationattribute_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
+    }
+    else if (!strncmp(p, "${multiplicity}", 15)) {
+      p += 15;
+      s += multiplicity->currentText().stripWhiteSpace();
     }
     else if (*p == '@')
       manage_alias(att->browser_node, p, s, kvtable);
@@ -1177,7 +1503,15 @@ QString AttributeDialog::idl_decl(const BrowserAttribute * at)
       p += 11;
     else if (!strncmp(p, "${type}", 7)) {
       p += 7;
-      s += GenerationSettings::idl_type(d->type.get_type());
+      s += get_idl_name(d->type);
+    }
+    else if (!strncmp(p, "${stereotype}", 13)) {
+      p += 13;
+      s += GenerationSettings::idl_relationattribute_stereotype(d->get_stereotype());
+    }
+    else if (!strncmp(p, "${multiplicity}", 15)) {
+      p += 15;
+      s += d->get_multiplicity();
     }
     else if (*p == '\n') {
       s += ' ';
