@@ -212,7 +212,7 @@ void ClassViewDialog::accept() {
     }
     
     bn->set_comment(comment->text());
-    UmlWindow::set_commented(bn);
+    UmlWindow::update_comment_if_needed(bn);
     
     kvtable->update(bn);
   
