@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -32,12 +32,14 @@
 class QComboBox;
 class ColMsg;
 class OperationData;
+class BrowserClass;
 
 class CodChangeMsgDialog : public QDialog {
   Q_OBJECT
     
   protected:
     ColMsg * msg;
+    BrowserClass * cl;
     QStringList list;
     QValueList<const OperationData *> opers;
     QComboBox * edoper;    
