@@ -197,7 +197,7 @@ void ExpansionRegionDialog::accept() {
 
   data->set_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
   bn->set_comment(comment->text());
-  UmlWindow::set_commented(bn);
+  UmlWindow::update_comment_if_needed(bn);
     
   kvtable->update(bn);
   
