@@ -63,6 +63,9 @@ class ExtraMemberDialog : public QTabDialog {
     // java tab
     MultiLineEdit * edjava_decl;
     
+    // php tab
+    MultiLineEdit * edphp_decl;
+    
     // IDL tab
     MultiLineEdit * edidl_decl;
     
@@ -75,6 +78,7 @@ class ExtraMemberDialog : public QTabDialog {
     static void post_edit_cpp_decl(ExtraMemberDialog *, QString);
     static void post_edit_cpp_def(ExtraMemberDialog *, QString);
     static void post_edit_java_decl(ExtraMemberDialog *, QString);
+    static void post_edit_php_decl(ExtraMemberDialog *, QString);
     static void post_edit_idl_decl(ExtraMemberDialog *, QString);
     
   public:
@@ -88,6 +92,7 @@ class ExtraMemberDialog : public QTabDialog {
     void edit_cpp_decl();
     void edit_cpp_def();
     void edit_java_decl();
+    void edit_php_decl();
     void edit_idl_decl();
     void edit_description();
 };
