@@ -76,7 +76,7 @@ class GenerationSettingsDialog : public QTabDialog {
     MultiLineEdit * edcpp_union_decl;
     MultiLineEdit * edcpp_enum_decl;
     MultiLineEdit * edcpp_typedef_decl;
-    MultiLineEdit * edcpp_attr_decl;
+    MultiLineEdit * edcpp_attr_decl[3];
     MultiLineEdit * edcpp_enum_item_decl;
     MultiLineEdit * edcpp_rel_decl[2][3];
     VisibilityGroup cpp_get_visibility;
@@ -105,7 +105,7 @@ class GenerationSettingsDialog : public QTabDialog {
     MultiLineEdit * edjava_interface_decl;
     MultiLineEdit * edjava_enum_decl;
     MultiLineEdit * edjava_enum_pattern_decl;
-    MultiLineEdit * edjava_attr_decl;
+    MultiLineEdit * edjava_attr_decl[3];
     MultiLineEdit * edjava_enum_item_decl;
     MultiLineEdit * edjava_enum_pattern_item_decl;
     MultiLineEdit * edjava_enum_pattern_item_case;
@@ -122,6 +122,27 @@ class GenerationSettingsDialog : public QTabDialog {
     QCheckBox * java_set_param_final_cb;
     IncludeTable * java_import_table;
   
+    // Php specific
+    MultiLineEdit * edphp_src_content;
+    QComboBox * edphp_extension;
+    QCheckBox * php_phpdoc_cb;
+    MultiLineEdit * edphp_class_decl;
+    LineEdit * edphp_external_class_decl;
+    MultiLineEdit * edphp_interface_decl;
+    MultiLineEdit * edphp_enum_decl;
+    MultiLineEdit * edphp_attr_decl;
+    MultiLineEdit * edphp_enum_item_decl;
+    MultiLineEdit * edphp_rel_decl;
+    MultiLineEdit * edphp_oper_def;
+    VisibilityGroup php_get_visibility;
+    LineEdit * edphp_get_name;
+    QCheckBox * uml_follow_php_get_name;
+    QCheckBox * php_get_final_cb;
+    VisibilityGroup php_set_visibility;
+    LineEdit * edphp_set_name;
+    QCheckBox * uml_follow_php_set_name;
+    QCheckBox * php_set_final_cb;
+  
     // Idl specific
     MultiLineEdit * edidl_src_content;
     QComboBox * edidl_extension;
@@ -133,10 +154,10 @@ class GenerationSettingsDialog : public QTabDialog {
     MultiLineEdit * edidl_typedef_decl;
     MultiLineEdit * edidl_exception_decl;
     MultiLineEdit * edidl_enum_decl;
-    MultiLineEdit * edidl_attr_decl;
-    MultiLineEdit * edidl_valuetype_attr_decl;
-    MultiLineEdit * edidl_const_decl;
-    MultiLineEdit * edidl_union_item_decl;
+    MultiLineEdit * edidl_attr_decl[3];
+    MultiLineEdit * edidl_valuetype_attr_decl[3];
+    MultiLineEdit * edidl_const_decl[3];
+    MultiLineEdit * edidl_union_item_decl[3];
     MultiLineEdit * edidl_enum_item_decl;
     MultiLineEdit * edidl_rel_decl[3];
     MultiLineEdit * edidl_valuetype_rel_decl[3];
@@ -158,10 +179,12 @@ class GenerationSettingsDialog : public QTabDialog {
     
     // directories
     LineEdit * edcpproot;
-    LineEdit * edjavaroot;  
+    LineEdit * edjavaroot; 
+    LineEdit * edphproot;  
     LineEdit * edidlroot;
     QPushButton * cpprelbutton;
     QPushButton * javarelbutton;
+    QPushButton * phprelbutton;
     QPushButton * idlrelbutton;
     
     static QSize previous_size;
@@ -181,10 +204,14 @@ class GenerationSettingsDialog : public QTabDialog {
     void init_java1();
     void init_java2();
     void init_java3();
+    void init_java4();
+    void init_php1();
+    void init_php2();
     void init_idl1();
     void init_idl2();
     void init_idl3();
     void init_idl4();
+    void init_idl5();
     void init_descriptions();
     void init_dirs();
     void relative(LineEdit * ed, QPushButton * button);
@@ -194,14 +221,22 @@ class GenerationSettingsDialog : public QTabDialog {
     virtual void accept();
     void cpproot_browse();
     void javaroot_browse();
+    void phproot_browse();
     void idlroot_browse();
     void cpp_relative();
     void java_relative();
+    void php_relative();
     void idl_relative();
+    void java_get_visi_changed(int);
+    void java_set_visi_changed(int);
+    void php_get_visi_changed(int);
+    void php_set_visi_changed(int);
     void follow_cpp_get_name();
     void follow_cpp_set_name();
     void follow_java_get_name();
     void follow_java_set_name();
+    void follow_php_get_name();
+    void follow_php_set_name();
     void follow_idl_get_name();
     void follow_idl_set_name();
 };
@@ -217,8 +252,11 @@ class TypesTable : public StringTable {
 };
 
 class StereotypesTable : public StringTable {
+  protected:
+    bool with_php;
+  
   public:
-    StereotypesTable(QWidget * parent, int nst, Stereotype * st);
+    StereotypesTable(QWidget * parent, int nst, Stereotype * st, bool php);
   
     virtual void init_row(int r);
   
