@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -43,10 +43,10 @@
 
 QSize InstanceDialog::previous_size;
 
-InstanceDialog::InstanceDialog(Instance * i, const char * kind,
-			       const char * caption)
-    : QDialog(0, caption, TRUE), inst(i) {
-  setCaption(caption);
+InstanceDialog::InstanceDialog(Instance * i, QString w, UmlCode k)
+    : QDialog(0, w + " instance dialog", TRUE),
+      inst(i), what(w), kind(k) {
+  setCaption(what + " instance dialog");
   
   QVBoxLayout * vbox = new QVBoxLayout(this);  
   
@@ -57,11 +57,13 @@ InstanceDialog::InstanceDialog(Instance * i, const char * kind,
   vbox->addWidget(grid);
   new QLabel("name : ", grid);
   edname = new LineEdit(inst->get_name(), grid);
+  edname->setFocus();
   
   new QLabel("", grid);
   new QLabel("", grid);
   
-  SmallPushButton * b = new SmallPushButton(kind, grid);
+  SmallPushButton * b =
+    new SmallPushButton(what + " :", grid);
   
   connect(b, SIGNAL(clicked()), this, SLOT(menu_type()));
   
@@ -114,18 +116,49 @@ void InstanceDialog::menu_type() {
   
   BrowserNode * bn = BrowserView::selected_item();
     
-  if ((bn->get_type() == UmlClass) && !bn->deletedp())
-    m.insertItem("Choose class selected in browser", 1);
+  if ((bn != 0) && 
+      (bn->get_type() == kind) && !bn->deletedp())
+    m.insertItem("Choose " + what + " selected in browser", 1);
   else
     bn = 0;
   
-  if ((index != -1) || (bn != 0)) {
+  bool new_available = inst->new_type_available();
+  
+  if (new_available)
+    m.insertItem("Create " + what + " and choose it", 2);
+  
+  if (new_available || (index != -1) || (bn != 0)) {
     switch (m.exec(QCursor::pos())) {
     case 0:
       nodes.at(index)->select_in_browser();
       break;
+    case 2:
+      bn = inst->new_type();
+      if (bn == 0)
+	return;
+      bn->select_in_browser();
+      // no break
     case 1:
-      edtype->setCurrentItem(list.findIndex(bn->full_name(TRUE)));
+
+      {
+	QString s = bn->full_name(TRUE);
+	
+	if ((index = list.findIndex(s)) == -1) {
+	  // new class, may be created through an other dialog
+	  index = 0;
+	  QStringList::Iterator iter = list.begin();
+	  QStringList::Iterator iter_end = list.end();
+	  
+	  while ((iter != iter_end) && (*iter < s)) {
+	    ++iter;
+	    index += 1;
+	  }
+	  nodes.insert((unsigned) index, bn);
+	  list.insert(iter, s);
+	  edtype->insertItem(s, index);
+	}
+      }
+      edtype->setCurrentItem(index);
     }
   }
 }
