@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -29,15 +29,18 @@
 #include <qdialog.h>
 
 class MultiLineEdit;
+class QPushButton;
 
 class MLEDialog : public QDialog {
   Q_OBJECT
     
   protected:
     MultiLineEdit * e;
+    QPushButton * ok;
+    QPushButton * cancel;
     
   public:
-    MLEDialog(QString t);
+    MLEDialog(QString t, bool buttons);
     ~MLEDialog();
     
     QString text();
