@@ -154,6 +154,9 @@ ComboColor::ComboColor(QWidget * parent, UmlColor v,
     case UmlBlue:
       insertItem(* BluePixmap);
       break;
+    case UmlLightMediumBlue:
+      insertItem(* LightMediumBluePixmap);
+      break;
     case UmlMediumBlue:
       insertItem(* MediumBluePixmap);
       break;
@@ -166,6 +169,9 @@ ComboColor::ComboColor(QWidget * parent, UmlColor v,
     case UmlGreen:
       insertItem(* GreenPixmap);
       break;
+    case UmlLightMediumGreen:
+      insertItem(* LightMediumGreenPixmap);
+      break;
     case UmlMediumGreen:
       insertItem(* MediumGreenPixmap);
       break;
@@ -205,12 +211,21 @@ ComboColor::ComboColor(QWidget * parent, UmlColor v,
     case UmlDarkMagenta:
       insertItem(* DarkMagentaPixmap);
       break;
+    case UmlVeryLightGray:
+      insertItem(* VeryLightGrayPixmap);
+      break;
     case UmlLightGray:
       insertItem(* LightGrayPixmap);
       break;
     case UmlGray:
       insertItem(* GrayPixmap);
       break;
+    case UmlDarkGray:
+      insertItem(* DarkGrayPixmap);
+      break;
+    case UmlBlack:
+      insertItem(* BlackPixmap);
+      break;
     default:
       insertItem("Unknown color");
     }
@@ -227,17 +242,18 @@ ComboColor::ComboColor(QWidget * parent, UmlColor v,
 #ifdef WIN32
   setSizeLimit(34);	// yes !, don't set it to count() !
 #else
-  setSizeLimit(20);	// yes !, don't set it to count() !
+  setSizeLimit(25);	// yes !, don't set it to count() !
 #endif
 }
 
 QSize SettingsDialog::previous_size;
 
 SettingsDialog::SettingsDialog(QArray<StateSpec> * st, QArray<ColorSpec> * co,
-			       bool nodefault, bool own, bool unchanged)
-    : QTabDialog(0, "Diagram Drawing Settings dialog", TRUE),
+			       bool nodefault, bool own, bool unchanged,
+			       const char * title)
+    : QTabDialog(0, title, TRUE),
       states(st), colors(co), several(unchanged) {
-  setCaption("Diagram Drawing Settings dialog");
+  setCaption(title);
   
   QGrid * grid = 0;
   QString tabname;
