@@ -31,6 +31,7 @@
 #include <qvbox.h>
 #include <qlabel.h>
 #include <qcombobox.h> 
+#include <qcheckbox.h> 
 #include <qpushbutton.h> 
 
 #include "TransitionDialog.h"
@@ -42,6 +43,7 @@
 #include "strutil.h"
 #include "UmlDesktop.h"
 #include "BodyDialog.h"
+#include "GenerationSettings.h"
 
 QSize TransitionDialog::previous_size;
 
@@ -87,6 +89,14 @@ TransitionDialog::TransitionDialog(TransitionData * r)
   sp.setHorData(QSizePolicy::Expanding);
   edstereotype->setSizePolicy(sp);
   
+  if (r->get_start_node() != r->get_end_node())
+    internal_cb = 0;
+  else {
+    new QLabel(grid);
+    internal_cb = new QCheckBox("internal", grid);
+    internal_cb->setChecked(r->internal());
+  }
+    
   QVBox * vtab = new QVBox(grid);
   new QLabel("description :", vtab);
   if (! visit)
@@ -101,15 +111,17 @@ TransitionDialog::TransitionDialog(TransitionData * r)
 
   // UML / OCL
   init_tab(ocltab, uml, rel->uml, "Ocl", SLOT(edit_uml_trigger()),
-	   SLOT(edit_uml_guard()), SLOT(edit_uml_expr()));
+	   SLOT(edit_uml_guard()), SLOT(edit_uml_expr()), TRUE);
 
   // CPP
   init_tab(cpptab, cpp, rel->cpp, "C++", SLOT(edit_cpp_trigger()),
-	   SLOT(edit_cpp_guard()), SLOT(edit_cpp_expr()));
+	   SLOT(edit_cpp_guard()), SLOT(edit_cpp_expr()),
+	   GenerationSettings::cpp_get_default_defs());
 
   // Java
   init_tab(javatab, java, rel->java, "Java", SLOT(edit_java_trigger()),
-	   SLOT(edit_java_guard()), SLOT(edit_java_expr()));
+	   SLOT(edit_java_guard()), SLOT(edit_java_expr()),
+	   GenerationSettings::java_get_default_defs());
   
   // USER : list key - value
   
@@ -154,7 +166,8 @@ void TransitionDialog::polish() {
 
 void TransitionDialog::init_tab(QWidget *& tab, TransDialog & d, TransDef & td,
 				const char * lbl, const char * sl_trigger,
-				const char * sl_guard, const char * sl_expr) {
+				const char * sl_guard, const char * sl_expr,
+				bool enabled) {
   QGrid * grid = new QGrid(2, this);
   QVBox * vtab;
 
@@ -202,6 +215,9 @@ void TransitionDialog::init_tab(QWidget *& tab, TransDialog & d, TransDef & td,
     d.edexpr->setReadOnly(TRUE);
   
   addTab(grid, lbl);
+  
+  if (! enabled)
+    removePage(grid);
 }
 
 void TransitionDialog::edit_description() {
@@ -257,12 +273,15 @@ void TransitionDialog::accept() {
     bn->set_name(s);
     rel->set_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
     
+    if (internal_cb != 0)
+      rel->set_internal(internal_cb->isChecked());
+    
     uml.accept(rel->uml);  
     cpp.accept(rel->cpp);  
     java.accept(rel->java);    
     
     bn->set_comment(comment->text());
-    UmlWindow::set_commented(bn);
+    UmlWindow::update_comment_if_needed(bn);
     
     kvtable->update(bn);
     
