@@ -34,6 +34,7 @@
 
 class QPushButton;
 class QComboBox;
+class QCheckBox;
 
 class LineEdit;
 class MultiLineEdit;
@@ -59,6 +60,7 @@ class TransitionDialog : public QTabDialog {
     QWidget * umltab;
     TransitionData * rel;
     LineEdit * edname;
+    QCheckBox * internal_cb;
     QComboBox * edstereotype;
     MultiLineEdit * comment;
     QList<BodyDialog> edits;
@@ -94,7 +96,7 @@ class TransitionDialog : public QTabDialog {
   protected:
     void init_tab(QWidget *&, TransDialog &, TransDef & td, const char * lbl,
 		  const char * sl_trigger, const char * sl_guard,
-		  const char * sl_expr);
+		  const char * sl_expr, bool enabled);
   protected slots:
     virtual void polish();
     virtual void accept();
