@@ -159,7 +159,7 @@ void UseCaseDialog::accept() {
 
   uc->set_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
   bn->set_comment(comment->text());
-  UmlWindow::set_commented(bn);
+  UmlWindow::update_comment_if_needed(bn);
     
   kvtable->update(bn);
   
