@@ -35,6 +35,7 @@ class QButtonGroup;
 
 class VisibilityGroup {
   protected:
+    QButtonGroup * bgroup;
     QRadioButton * public_rb;
     QRadioButton * protected_rb;
     QRadioButton * private_rb;
@@ -51,6 +52,8 @@ class VisibilityGroup {
 			const char * title = 0,
 			const char * default_string = 0);
     void update_default(const VisibilityGroup & default_grp);
+    void connect(const char *, QWidget *, const char *);
+    void follow(const VisibilityGroup & other);
     UmlVisibility value();
     
     QString state();
