@@ -54,9 +54,11 @@ class Stereotype {
     QString uml;
     QString cpp;
     QString java;
+    QString php;
     QString idl;
     
-    void set(const char * u, const char * c, const char * j, const char * i);
+    void set(const char * u, const char * c, const char * j,
+	     const char * p, const char * i);
 };
 
 // on the tool API one use a QDict, it is useless here because
@@ -94,7 +96,7 @@ class GenerationSettings {
     static SharedStr cpp_union_decl;
     static SharedStr cpp_enum_decl;
     static SharedStr cpp_typedef_decl;
-    static SharedStr cpp_attr_decl;
+    static SharedStr cpp_attr_decl[3/*multiplicity*/];
     static SharedStr cpp_enum_item_decl;
     static SharedStr cpp_rel_decl[2/*relation kind*/][3/*multiplicity*/];
     static SharedStr cpp_oper_decl;
@@ -119,20 +121,35 @@ class GenerationSettings {
     static SharedStr java_enum_decl;
     static SharedStr java_enum_pattern_decl;
     static SharedStr java_interface_decl;
-    static SharedStr java_attr_decl;
+    static SharedStr java_attr_decl[3/*multiplicity*/];
     static SharedStr java_enum_item_decl;
     static SharedStr java_enum_pattern_item_decl;
     static SharedStr java_enum_pattern_item_case;
     static SharedStr java_rel_decl[3/*multiplicity*/];
     static SharedStr java_oper_def;
-    static UmlVisibility java_get_visibility;
+    static UmlVisibility javaphp_get_visibility;
     static SharedStr java_get_name;
     static bool java_get_final;
-    static UmlVisibility java_set_visibility;
+    static UmlVisibility javaphp_set_visibility;
     static SharedStr java_set_name;
     static bool java_set_final;
     static bool java_set_param_final;
     
+    static bool php_default_defs;
+    static SharedStr php_src_content;
+    static SharedStr php_class_decl;
+    static SharedStr php_external_class_decl;
+    static SharedStr php_enum_decl;
+    static SharedStr php_interface_decl;
+    static SharedStr php_attr_decl;
+    static SharedStr php_enum_item_decl;
+    static SharedStr php_rel_decl;
+    static SharedStr php_oper_def;
+    static SharedStr php_get_name;
+    static bool php_get_final;
+    static SharedStr php_set_name;
+    static bool php_set_final;
+    
     static bool idl_default_defs;
     static SharedStr idl_src_content;
     static SharedStr idl_external_class_decl;
@@ -143,11 +160,11 @@ class GenerationSettings {
     static SharedStr idl_exception_decl;
     static SharedStr idl_union_decl;
     static SharedStr idl_enum_decl;
-    static SharedStr idl_attr_decl;
-    static SharedStr idl_valuetype_attr_decl;
-    static SharedStr idl_union_item_decl;
+    static SharedStr idl_attr_decl[3/*multiplicity*/];
+    static SharedStr idl_valuetype_attr_decl[3/*multiplicity*/];
+    static SharedStr idl_union_item_decl[3/*multiplicity*/];
     static SharedStr idl_enum_item_decl;
-    static SharedStr idl_const_decl;
+    static SharedStr idl_const_decl[3/*multiplicity*/];
     static SharedStr idl_rel_decl[3/*multiplicity*/];
     static SharedStr idl_valuetype_rel_decl[3/*multiplicity*/];
     static SharedStr idl_union_rel_decl[3/*multiplicity*/];
@@ -168,11 +185,12 @@ class GenerationSettings {
     static IncludesSpec cpp_includes;
     static QString java_extension;
     static IncludesSpec java_imports;
+    static QString php_extension;
     static QString idl_extension;
     static IncludesSpec idl_includes;
     
-    static int nrelstereotypes;
-    static Stereotype * rel_stereotypes;
+    static int nrelattrstereotypes;
+    static Stereotype * relattr_stereotypes;
     
     static int nclassstereotypes;
     static Stereotype * class_stereotypes;
@@ -185,10 +203,11 @@ class GenerationSettings {
     
     static QString cpp_root_dir;
     static QString java_root_dir;
+    static QString php_root_dir;
     static QString idl_root_dir;
   
     static int find_type(const QString &);
-    static int find_relation_stereotype(const QString &);
+    static int find_relationattribute_stereotype(const QString &);
     static int find_class_stereotype(const QString &);
     static Builtin & get_type(const char * u);
     static Stereotype & get_stereotype(int & n, Stereotype * & st, const char * u);
@@ -198,6 +217,7 @@ class GenerationSettings {
     static void send_uml_def(ToolCom * com);
     static void send_cpp_def(ToolCom * com);
     static void send_java_def(ToolCom * com);
+    static void send_php_def(ToolCom * com);
     static void send_idl_def(ToolCom * com);
     
   public:
@@ -218,9 +238,9 @@ class GenerationSettings {
     static const char * cpp_default_union_decl() { return cpp_union_decl; };
     static const char * cpp_default_enum_decl() { return cpp_enum_decl; };
     static const char * cpp_default_typedef_decl() { return cpp_typedef_decl; };
-    static const char * cpp_default_attr_decl() { return cpp_attr_decl; };
+    static const char * cpp_default_attr_decl(const QString & mult);
     static const char * cpp_default_enum_item_decl() { return cpp_enum_item_decl; };
-    static QString cpp_default_rel_decl(UmlCode rel, const QString & mult);
+    static const char * cpp_default_rel_decl(UmlCode rel, const QString & mult);
     static const char * cpp_default_oper_decl() { return cpp_oper_decl; };
     static const char * cpp_default_oper_def() { return cpp_oper_def; };
     static bool cpp_oper_force_throw() { return cpp_force_throw; };
@@ -246,21 +266,37 @@ class GenerationSettings {
     static const char * java_default_interface_decl() { return java_interface_decl; };
     static const char * java_default_enum_decl() { return java_enum_decl; };
     static const char * java_default_enum_pattern_decl() { return java_enum_pattern_decl; };
-    static const char * java_default_attr_decl() { return java_attr_decl; };
+    static const char * java_default_attr_decl(const QString & mult);
     static const char * java_default_enum_item_decl() { return java_enum_item_decl; };
     static const char * java_default_enum_pattern_item_decl() { return java_enum_pattern_item_decl; };
     static const char * java_default_enum_pattern_item_case() { return java_enum_pattern_item_case; };
-    static QString java_default_rel_decl(const QString & mult);
+    static const char * java_default_rel_decl(const QString & mult);
     static const char * java_default_oper_def() { return java_oper_def; };
-    static UmlVisibility java_default_get_visibility() { return java_get_visibility; };
+    static UmlVisibility javaphp_default_get_visibility() { return javaphp_get_visibility; };
     static const char * java_default_get_name() { return java_get_name; };
     static bool java_default_get_final() { return java_get_final; };
-    static UmlVisibility java_default_set_visibility() { return java_set_visibility; };
+    static UmlVisibility javaphp_default_set_visibility() { return javaphp_set_visibility; };
     static const char * java_default_set_name() { return java_set_name; };
     static bool java_default_set_final() { return java_set_final; };
     static bool java_default_set_param_final() { return java_set_param_final; };
     static bool java_javadoc_style() { return java_javadoc_comment; }
 
+    static bool php_get_default_defs() { return php_default_defs; };
+    static bool php_set_default_defs(bool y);
+    static const char * php_default_source_content() { return php_src_content; };
+    static const char * php_default_class_decl() { return php_class_decl; };
+    static const char * php_default_external_class_decl() { return php_external_class_decl; };
+    static const char * php_default_enum_decl() { return php_enum_decl; };
+    static const char * php_default_interface_decl() { return php_interface_decl; };
+    static const char * php_default_attr_decl() { return php_attr_decl; }
+    static const char * php_default_enum_item_decl() { return php_enum_item_decl; };
+    static const char * php_default_rel_decl() { return php_rel_decl; }
+    static const char * php_default_oper_def() { return php_oper_def; };
+    static const char * php_default_get_name() { return php_get_name; };
+    static bool php_default_get_final() { return php_get_final; };
+    static const char * php_default_set_name() { return php_set_name; };
+    static bool php_default_set_final() { return php_set_final; };
+
     static bool idl_get_default_defs() { return idl_default_defs; };
     static bool idl_set_default_defs(bool y);
     static const char * idl_default_source_content() { return idl_src_content; };
@@ -272,14 +308,14 @@ class GenerationSettings {
     static const char * idl_default_exception_decl() { return idl_exception_decl; };
     static const char * idl_default_union_decl() { return idl_union_decl; };
     static const char * idl_default_enum_decl() { return idl_enum_decl; };
-    static const char * idl_default_attr_decl() { return idl_attr_decl; };
-    static const char * idl_default_valuetype_attr_decl() { return idl_valuetype_attr_decl; };
-    static const char * idl_default_const_decl() { return idl_const_decl; };
-    static const char * idl_default_union_item_decl() { return idl_union_item_decl; };
+    static const char * idl_default_attr_decl(const QString & mult);
+    static const char * idl_default_valuetype_attr_decl(const QString & mult);
+    static const char * idl_default_const_decl(const QString & mult);
+    static const char * idl_default_union_item_decl(const QString & mult);
     static const char * idl_default_enum_item_decl() { return idl_enum_item_decl; };
-    static QString idl_default_rel_decl(const QString & mult);
-    static QString idl_default_valuetype_rel_decl(const QString & mult);
-    static QString idl_default_union_rel_decl(const QString & mult);
+    static const char * idl_default_rel_decl(const QString & mult);
+    static const char * idl_default_valuetype_rel_decl(const QString & mult);
+    static const char * idl_default_union_rel_decl(const QString & mult);
     static const char * idl_default_oper_decl() { return idl_oper_decl; };
     static const char * idl_default_get_name() { return idl_get_name; };
     static const char * idl_default_set_name() { return idl_set_name; };
@@ -288,12 +324,13 @@ class GenerationSettings {
     static DrawingLanguage uml_default_get_name() { return uml_get_name; }
     static DrawingLanguage uml_default_set_name() { return uml_set_name; }
     
-    static QString cpp_relation_stereotype(const QString &);
-    static QString java_relation_stereotype(const QString &);
-    static QString idl_relation_stereotype(const QString &);
+    static QString cpp_relationattribute_stereotype(const QString &);
+    static QString java_relationattribute_stereotype(const QString &);
+    static QString idl_relationattribute_stereotype(const QString &);
     
     static QString cpp_class_stereotype(const QString &);
     static QString java_class_stereotype(const QString &);
+    static QString php_class_stereotype(const QString &);
     static QString idl_class_stereotype(const QString &);
     
     static QString default_artifact_description() { return artifact_default_description; }
@@ -304,6 +341,7 @@ class GenerationSettings {
     
     static const QString & get_cpp_root_dir() { return cpp_root_dir; };
     static const QString & get_java_root_dir() { return java_root_dir; };
+    static const QString & get_php_root_dir() { return php_root_dir; };
     static const QString & get_idl_root_dir() { return idl_root_dir; };
     
     static const QString & get_cpp_h_extension() { return cpp_h_extension; };
@@ -312,6 +350,7 @@ class GenerationSettings {
     //static IncludesSpec get_cpp_includes;
     static const QString & get_java_extension() { return java_extension; };
     //static IncludesSpec get_java_imports;
+    static const QString & get_php_extension() { return php_extension; };
     static const QString & get_idl_extension() { return idl_extension; };
 
     static bool edit();
@@ -319,6 +358,7 @@ class GenerationSettings {
     static bool tool_global_uml_cmd(ToolCom * com, const char * args);
     static bool tool_global_cpp_cmd(ToolCom * com, const char * args);
     static bool tool_global_java_cmd(ToolCom * com, const char * args);
+    static bool tool_global_php_cmd(ToolCom * com, const char * args);
     static bool tool_global_idl_cmd(ToolCom * com, const char * args);
 
     static void save_dirs(QTextStream & st);
