@@ -29,6 +29,8 @@
 
 #include <qintdict.h> 
 #include <qdatetime.h> 
+#include <qmessagebox.h>
+#include <qlist.h>
 
 #include "mu.h"
 #include "Labeled.h"
@@ -49,6 +51,23 @@ bool in_import()
   return NeedRenumber;
 }
 
+//
+
+void update_idmax_for_root(QIntDict<void> & d, int & idmax)
+{
+  QIntDictIterator<void> it(d); 
+  
+  while (it.current()) {
+    int id = it.currentKey();
+    
+    if ((((unsigned) (id & ~127)) > ((unsigned) idmax)) &&
+        ((id & 127) == 0))
+      idmax = id & ~127;
+    
+    ++it;
+  }
+}
+
 //
     
 int place(IdDict<void> & d, int id, void * x)
@@ -63,7 +82,7 @@ int place(IdDict<void> & d, int id, void * x)
     }
     else if (d.old_diagram) {
       // place id unchanged among the old ones
-      d.dict[1].insert(id, x);
+      d.dict[1].replace(id, x);
       
       if ((d.dict[1].count() / 2) >= d.dict[1].size())
 	d.dict[1].resize(d.dict[1].size() * 2 - 1);
@@ -76,7 +95,7 @@ int place(IdDict<void> & d, int id, void * x)
     }
     else if (NeedRenumber) {
       // place id unchanged among the old ones
-      d.dict[1].insert(id, x);
+      d.dict[1].replace(id, x);
       
       if ((d.dict[1].count() / 2) >= d.dict[1].size())
 	d.dict[1].resize(d.dict[1].size() * 2 - 1);
@@ -104,14 +123,14 @@ int place(IdDict<void> & d, int id, void * x)
 	    ((id & 127) == user_id()))
 	  d.idmax = id & ~127;
     }
-    else {
+    else {      
       // no renum, id unchanged
       if ((((unsigned) (id & ~127)) > ((unsigned) d.idmax)) &&
 	  ((id & 127) == user_id()))
 	d.idmax = id & ~127;
     }
     
-    d.dict[0].insert(id, x);
+    d.dict[0].replace(id, x);
     
     if ((d.dict[0].count() / 2) >= d.dict[0].size())
       d.dict[0].resize(d.dict[0].size() * 2 - 1);
@@ -123,10 +142,89 @@ int place(IdDict<void> & d, int id, void * x)
 int new_place(IdDict<void> & d, int user_id, void * x)
 {
   if (d.idmax == FIRST_ID)
-    d.idmax = FIRST_BASE_ID | user_id;
+    d.idmax = FIRST_BASE_ID;
   else
-    d.idmax = (d.idmax + 128) | user_id;
+    d.idmax += 128;
   
-  d.dict[0].insert(d.idmax, x);
-  return d.idmax;
+  d.dict[0].insert(d.idmax | user_id, x);
+  return d.idmax | user_id;
+}
+
+// to change id when two elements use the same
+
+struct NeedChange {
+  IdDict<void> & dict;
+  int & ident;
+  void * elt;
+
+  NeedChange(IdDict<void> & d, int & id, void * e) : dict(d), ident(id), elt(e) {}
+};
+
+static QList<NeedChange> MustBeRenumered;
+
+void will_change_id(IdDict<void> & d, int & id, void * x)
+{
+  MustBeRenumered.append(new NeedChange(d, id, x));
+}
+
+void do_change_shared_ids()
+{
+  int user = user_id();
+
+  while (!MustBeRenumered.isEmpty()) {
+    NeedChange * x = MustBeRenumered.take(0);
+
+    x->ident = new_place(x->dict, user, x->elt);
+    delete x;
+  }
+}
+
+//
+
+// don't use for instance a static global QList to
+// not be dependent on the initialization order made
+// before 'main' execution
+
+struct IntList {
+  int * pint;
+  const char * file;
+  IntList * next;
+};
+
+// initialized to 0 before any execution associated
+// to the initializations before 'main'
+IntList * FirstCell;
+
+void memo_idmax_loc(int & idmaxref, const char * who)
+{
+  if (who != 0) {
+    IntList * cell = new IntList;
+    
+    cell->pint = &idmaxref;
+    cell->next = FirstCell;
+    cell->file = who;
+    FirstCell = cell;
+  }
+}
+
+// add a margin of 8
+void idmax_add_margin()
+{
+  for (IntList * cell = FirstCell; cell != 0; cell = cell->next)
+    *(cell->pint) += 128*8;
+}
+
+//
+
+// to check all operation XX:clear() was called
+// ie don't forget to call the operation when a new
+// Labeled<> is added
+
+void check_ids_cleared()
+{
+  for (IntList * cell = FirstCell; cell != 0; cell = cell->next)
+    if (*(cell->pint) != FIRST_ID)
+      QMessageBox::critical(0, "Bouml", 
+			    cell->file + QString("\nclear() not called !\n"
+						 "check also update_idmax_for_root()"));
 }
