@@ -136,6 +136,7 @@ ClassDiagramSettings::ClassDiagramSettings() {
   hide_operations = UmlDefaultState;
   show_full_members_definition = UmlDefaultState;
   show_members_visibility = UmlDefaultState;
+  show_members_stereotype = UmlDefaultState;
   show_parameter_dir = UmlDefaultState;
   show_parameter_name = UmlDefaultState;
   package_name_in_tab = UmlDefaultState;
@@ -155,15 +156,16 @@ void ClassDiagramSettings::save(QTextStream & st) const {
       << " hide_operations " << stringify(hide_operations)
 	<< " show_members_full_definition " << stringify(show_full_members_definition)
 	  << " show_members_visibility " << stringify(show_members_visibility)
-	    << " show_parameter_dir " << stringify(show_parameter_dir)
-	      << " show_parameter_name " << stringify(show_parameter_name)
-		<< " package_name_in_tab " << stringify(package_name_in_tab)
-		  << " class_drawing_mode " << stringify(class_drawing_mode)
-		    << " drawing_language " << stringify(drawing_language)
-		      << " show_context_mode " << stringify(show_context_mode)
-			<< " auto_label_position " << stringify(auto_label_position)
-			  << " show_infonote " << stringify(show_infonote)
-			    << " shadow " << stringify(shadow);
+	    << " show_members_stereotype " << stringify(show_members_stereotype)
+	      << " show_parameter_dir " << stringify(show_parameter_dir)
+		<< " show_parameter_name " << stringify(show_parameter_name)
+		  << " package_name_in_tab " << stringify(package_name_in_tab)
+		    << " class_drawing_mode " << stringify(class_drawing_mode)
+		      << " drawing_language " << stringify(drawing_language)
+			<< " show_context_mode " << stringify(show_context_mode)
+			  << " auto_label_position " << stringify(auto_label_position)
+			    << " show_infonote " << stringify(show_infonote)
+			      << " shadow " << stringify(shadow);
 }
 
 void ClassDiagramSettings::read(char * & st, char * & k) {
@@ -184,6 +186,7 @@ void ClassDiagramSettings::read(char * & st, char * & k) {
     // old version
     show_full_members_definition = state(read_keyword(st));
     show_members_visibility = show_full_members_definition;
+    show_members_stereotype = UmlNo;
     show_parameter_dir = UmlYes;
     show_parameter_name = UmlYes;
     k = read_keyword(st);
@@ -199,6 +202,11 @@ void ClassDiagramSettings::read(char * & st, char * & k) {
       show_members_visibility = state(read_keyword(st));
       k = read_keyword(st);
     }
+    if (!strcmp(k, "show_members_stereotype")) {
+      // new version
+      show_members_stereotype = state(read_keyword(st));
+      k = read_keyword(st);
+    }
     if (!strcmp(k, "show_parameter_dir")) {
       // new version
       show_parameter_dir = state(read_keyword(st));
@@ -250,6 +258,7 @@ bool ClassDiagramSettings::complete(ClassDiagramSettings & result) const {
   check_default(hide_operations, UmlDefaultState);
   check_default(show_full_members_definition, UmlDefaultState);
   check_default(show_members_visibility, UmlDefaultState);
+  check_default(show_members_stereotype, UmlDefaultState);
   check_default(show_parameter_dir, UmlDefaultState);
   check_default(show_parameter_name, UmlDefaultState);
   check_default(package_name_in_tab, UmlDefaultState);
@@ -260,7 +269,7 @@ bool ClassDiagramSettings::complete(ClassDiagramSettings & result) const {
   check_default(show_infonote, UmlDefaultState);
   check_default(shadow, UmlDefaultState);
   
-  return done == 14;
+  return done == 15;
 }
 
 void ClassDiagramSettings::complete(QArray<StateSpec> & a, UmlCode who) {
@@ -269,7 +278,7 @@ void ClassDiagramSettings::complete(QArray<StateSpec> & a, UmlCode who) {
   switch (who) {
   case UmlClass:
     // order known by ClassDiagramSettings::set
-    a.resize(i + 10);
+    a.resize(i + 11);
   
     a[i].set("drawing language", &drawing_language);
     a[i + 1].set("drawing mode", &class_drawing_mode);
@@ -280,11 +289,13 @@ void ClassDiagramSettings::complete(QArray<StateSpec> & a, UmlCode who) {
 		 &show_full_members_definition);
     a[i + 6].set("show members visibility",
 		 &show_members_visibility);
-    a[i + 7].set("show parameter direction",
+    a[i + 7].set("show members stereotype",
+		 &show_members_stereotype);
+    a[i + 8].set("show parameter direction",
 		 &show_parameter_dir);
-    a[i + 8].set("show parameter name",
+    a[i + 9].set("show parameter name",
 		 &show_parameter_name);
-    a[i + 9].set("show information note",
+    a[i + 10].set("show information note",
 		 &show_infonote);
     break;
   case UmlPackage:
@@ -294,7 +305,7 @@ void ClassDiagramSettings::complete(QArray<StateSpec> & a, UmlCode who) {
     a[i + 1].set("show context", &show_context_mode);
     break;
   case UmlClassDiagram:
-    a.resize(i + 14);
+    a.resize(i + 15);
   
     a[i].set("drawing language", &drawing_language);
     a[i + 1].set("classes drawing mode", &class_drawing_mode);
@@ -304,19 +315,21 @@ void ClassDiagramSettings::complete(QArray<StateSpec> & a, UmlCode who) {
 		 &show_full_members_definition);
     a[i + 5].set("show members visibility",
 		 &show_members_visibility);
-    a[i + 6].set("show parameter direction",
+    a[i + 6].set("show members stereotype",
+		 &show_members_stereotype);
+    a[i + 7].set("show parameter direction",
 		 &show_parameter_dir);
-    a[i + 7].set("show parameter name",
+    a[i + 8].set("show parameter name",
 		 &show_parameter_name);
-    a[i + 8].set("draw all relations", &draw_all_relations);
-    a[i + 9].set("show packages name in tab", &package_name_in_tab);
-    a[i + 10].set("show classes and packages context", &show_context_mode);
-    a[i + 11].set("automatic labels position", &auto_label_position);
-    a[i + 12].set("show information note", &show_infonote);
-    a[i + 13].set("draw shadow", &shadow);
+    a[i + 9].set("draw all relations", &draw_all_relations);
+    a[i + 10].set("show packages name in tab", &package_name_in_tab);
+    a[i + 11].set("show classes and packages context", &show_context_mode);
+    a[i + 12].set("automatic labels position", &auto_label_position);
+    a[i + 13].set("show information note", &show_infonote);
+    a[i + 14].set("draw shadow", &shadow);
     break;
   default:
-    a.resize(i + 14);
+    a.resize(i + 15);
   
     a[i].set("class#drawing language", &drawing_language);
     a[i + 1].set("class#classes drawing mode", &class_drawing_mode);
@@ -326,16 +339,18 @@ void ClassDiagramSettings::complete(QArray<StateSpec> & a, UmlCode who) {
 		 &show_full_members_definition);
     a[i + 5].set("class#show classes members visibility",
 		 &show_members_visibility);
-    a[i + 6].set("class#show operation parameter direction",
+    a[i + 6].set("class#show classes members stereotype",
+		 &show_members_stereotype);
+    a[i + 7].set("class#show operation parameter direction",
 		 &show_parameter_dir);
-    a[i + 7].set("class#show operation parameter name",
+    a[i + 8].set("class#show operation parameter name",
 		 &show_parameter_name);
-    a[i + 8].set("class#draw all relations", &draw_all_relations);
-    a[i + 9].set("class#show packages name in tab", &package_name_in_tab);
-    a[i + 10].set("class#show classes and packages context", &show_context_mode);
-    a[i + 11].set("class#automatic labels position", &auto_label_position);
-    a[i + 12].set("class#show information note", &show_infonote);
-    a[i + 13].set("class#draw shadow", &shadow);
+    a[i + 9].set("class#draw all relations", &draw_all_relations);
+    a[i + 10].set("class#show packages name in tab", &package_name_in_tab);
+    a[i + 11].set("class#show classes and packages context", &show_context_mode);
+    a[i + 12].set("class#automatic labels position", &auto_label_position);
+    a[i + 13].set("class#show information note", &show_infonote);
+    a[i + 14].set("class#draw shadow", &shadow);
   }
 }
 
@@ -356,11 +371,13 @@ void ClassDiagramSettings::set(QArray<StateSpec> & a, int index) {
   if (a[index + 6].name != 0)
     show_members_visibility = (Uml3States) *((Uml3States *) a[index + 6].state);
   if (a[index + 7].name != 0)
-    show_parameter_dir = (Uml3States) *((Uml3States *) a[index + 7].state);
+    show_members_stereotype = (Uml3States) *((Uml3States *) a[index + 7].state);
   if (a[index + 8].name != 0)
-    show_parameter_name = (Uml3States) *((Uml3States *) a[index + 8].state);
+    show_parameter_dir = (Uml3States) *((Uml3States *) a[index + 8].state);
   if (a[index + 9].name != 0)
-    show_infonote = (Uml3States) *((Uml3States *) a[index + 9].state);
+    show_parameter_name = (Uml3States) *((Uml3States *) a[index + 9].state);
+  if (a[index + 10].name != 0)
+    show_infonote = (Uml3States) *((Uml3States *) a[index + 10].state);
 }
 
 
@@ -369,6 +386,7 @@ void ClassDiagramSettings::set(QArray<StateSpec> & a, int index) {
 SequenceDiagramSettings::SequenceDiagramSettings() {
   show_full_operations_definition = UmlDefaultState;
   write_horizontally = UmlDefaultState;
+  instances_drawing_mode = DefaultClassDrawingMode;
   drawing_language = DefaultDrawingLanguage;
   draw_all_relations = UmlDefaultState;
   shadow = UmlDefaultState;
@@ -378,9 +396,10 @@ void SequenceDiagramSettings::save(QTextStream & st) const {
   nl_indent(st);
   st << "show_full_operations_definition " << stringify(show_full_operations_definition)
     << " write_horizontally " << stringify(write_horizontally)
-      << " drawing_language " << stringify(drawing_language)
-	<< " draw_all_relations " << stringify(draw_all_relations)
-	  << " shadow " << stringify(shadow);
+      << " class_drawing_mode " << stringify(instances_drawing_mode)
+	<< " drawing_language " << stringify(drawing_language)
+	  << " draw_all_relations " << stringify(draw_all_relations)
+	    << " shadow " << stringify(shadow);
 }
 
 void SequenceDiagramSettings::read(char * & st, char * & k) {
@@ -393,6 +412,10 @@ void SequenceDiagramSettings::read(char * & st, char * & k) {
     write_horizontally = state(read_keyword(st));
     k = read_keyword(st);
   }
+  if (!strcmp(k, "class_drawing_mode")) {
+    instances_drawing_mode = drawing_mode(read_keyword(st));
+    k = read_keyword(st);
+  }  
   if (!strcmp(k, "drawing_language")) {
     drawing_language = ::drawing_language(read_keyword(st));
     k = read_keyword(st);
@@ -412,31 +435,34 @@ bool SequenceDiagramSettings::complete(SequenceDiagramSettings & result) const {
   
   check_default(show_full_operations_definition, UmlDefaultState);
   check_default(write_horizontally, UmlDefaultState);
+  check_default(instances_drawing_mode, DefaultClassDrawingMode);  
   check_default(drawing_language, DefaultDrawingLanguage);
   check_default(draw_all_relations, UmlDefaultState);
   check_default(shadow, UmlDefaultState);
   
-  return done == 5;
+  return done == 6;
 }
 
 void SequenceDiagramSettings::complete(QArray<StateSpec> & a, bool local) {
   int i = a.size();
   
-  a.resize(i + 5);
+  a.resize(i + 6);
 
   if (local) {
     a[i].set("drawing language", &drawing_language);
-    a[i + 1].set("show operations full definition", &show_full_operations_definition);
-    a[i + 2].set("write name:type horizontally", &write_horizontally);
-    a[i + 3].set("draw all relations", &draw_all_relations);
-    a[i + 4].set("draw shadow", &shadow);
+    a[i + 1].set("instances drawing mode", &instances_drawing_mode);
+    a[i + 2].set("show operations full definition", &show_full_operations_definition);
+    a[i + 3].set("write name:type horizontally", &write_horizontally);
+    a[i + 4].set("draw all relations", &draw_all_relations);
+    a[i + 5].set("draw shadow", &shadow);
   }
   else {
     a[i].set("sequence#drawing language", &drawing_language);
-    a[i + 1].set("sequence#show operations full definition", &show_full_operations_definition);
-    a[i + 2].set("sequence#write name:type horizontally", &write_horizontally);
-    a[i + 3].set("sequence#draw all relations", &draw_all_relations);
-    a[i + 4].set("sequence#draw shadow", &shadow);
+    a[i + 1].set("sequence#instances drawing mode", &instances_drawing_mode);
+    a[i + 2].set("sequence#show operations full definition", &show_full_operations_definition);
+    a[i + 3].set("sequence#write name:type horizontally", &write_horizontally);
+    a[i + 4].set("sequence#draw all relations", &draw_all_relations);
+    a[i + 5].set("sequence#draw shadow", &shadow);
   }
 }
 
