@@ -123,9 +123,11 @@ void Shortcut::init()
     "Edit drawing settings",
     "Edit generation settings",
     "Edit Java body",
+    "Edit Php body",
     "Generate C++",
     "Generate Idl",
     "Generate Java",
+    "Generate Php",
     "Lower",
     "Mark",
     "Menu",
@@ -141,6 +143,7 @@ void Shortcut::init()
     "New attribute",
     "New class",
     "New class diagram",
+    "New class instance",
     "New class view",
     "New collaboration diagram",
     "New component",
@@ -178,11 +181,15 @@ void Shortcut::init()
     "Referenced by",
     "Redo",
     "Remove from view",
+    "Roundtrip C++ operation body",
+    "Roundtrip Java operation body",
+    "Roundtrip Php operation body",
     "Save",
     "See C++ header file",
     "See C++ source file",
     "See Idl source file",
     "See Java source file",
+    "See Php source file",
     "Select all",
     "Select associated artifact",
     "Select associated deployment view",
