@@ -68,12 +68,14 @@ enum UmlCode {
   UmlActivityObject, UmlExpansionNode, UmlActivityPin, 
   InitialAN, FlowFinalAN, ActivityFinalAN,	// warning : ActivityNode order used
   DecisionAN, MergeAN, ForkAN, JoinAN,
+  UmlClassInstance, // api >= 31
   
   // the followings are unknown by api.h
   UmlDependOn, UmlInherit, UmlObjectLink,	// not between classes
   UmlArrowPoint,
   UmlNote, UmlAnchor, UmlLabel, UmlText,
-  UmlSyncMsg, UmlAsyncMsg, UmlSyncSelfMsg, UmlAsyncSelfMsg, UmlReturnMsg,
+  UmlSyncMsg, UmlAsyncMsg, UmlSyncSelfMsg, UmlAsyncSelfMsg,
+  UmlReturnMsg, UmlSelfReturnMsg,
   UmlLifeLine, UmlActivityDuration,
   UmlLink, UmlSelfLink, UmlLinkDirs,
   UmlContain, 
@@ -90,6 +92,7 @@ enum UmlCode {
   UmlInfo,
   UmlFragmentSeparator,
   UmlConstraint,
+  UmlInner,
 
   UmlCodeSup
 };
@@ -113,7 +116,7 @@ inline bool IsaActivityContainer(UmlCode k)
   }
 }
 
-enum aCorner { NoCorner, TopLeft, TopRight, BottomLeft, BottomRight };
+enum aCorner { NoCorner, UmlTopLeft, UmlTopRight, UmlBottomLeft, UmlBottomRight };
 
 enum UmlVisibility { UmlPublic, UmlProtected, UmlPrivate, UmlPackageVisibility, UmlDefaultVisibility };
 
@@ -148,10 +151,12 @@ enum UmlColor {
   UmlDarkYellow,
   UmlLightBlue,
   UmlBlue,
+  UmlLightMediumBlue,
   UmlMediumBlue,
   UmlDarkBlue,
   UmlLightGreen,
   UmlGreen,
+  UmlLightMediumGreen,
   UmlMediumGreen,
   UmlDarkGreen,
   UmlLightRed,
@@ -165,14 +170,17 @@ enum UmlColor {
   UmlMagenta,
   UmlMidMagenta,
   UmlDarkMagenta,
+  UmlVeryLightGray,
   UmlLightGray,
   UmlGray,
+  UmlDarkGray,
+  UmlBlack,
   UmlDefaultColor
 };
 
 enum ClassDrawingMode { asClass, asControl, asBoundary, asEntity, asActor, Natural, DefaultClassDrawingMode };
 
-enum DrawingLanguage { UmlView, CppView, JavaView, IdlView, DefaultDrawingLanguage };
+enum DrawingLanguage { UmlView, CppView, JavaView, IdlView, PhpView, DefaultDrawingLanguage };
 
 enum ShowContextMode { noContext, umlContext, namespaceContext, packageContext, moduleContext, DefaultShowContextMode };
 
@@ -203,6 +211,9 @@ enum LineGeometry {
   GeometrySup
 };
 
+#define RecenterBegin GeometrySup
+#define RecenterEnd (RecenterBegin + 1)
+
 extern const char * stringify(UmlVisibility);
 extern const char * stringify(UmlParamDirection);
 extern const char * stringify(UmlCode);	// lines only
