@@ -46,15 +46,20 @@ extern QColor UmlMediumYellowColor;
 extern QColor UmlDarkYellowColor;
 extern QColor UmlLightBlueColor;
 extern QColor UmlBlueColor;
+extern QColor UmlLightMediumBlueColor;
 extern QColor UmlMediumBlueColor;
 extern QColor UmlDarkBlueColor;
 extern QColor UmlLightGreenColor;
 extern QColor UmlGreenColor;
+extern QColor UmlLightMediumGreenColor;
 extern QColor UmlMediumGreenColor;
 extern QColor UmlDarkGreenColor;
 extern QColor UmlRedColor;
+extern QColor UmlVeryLightGrayColor;
 extern QColor UmlLightGrayColor;
 extern QColor UmlGrayColor;
+extern QColor UmlDarkGrayColor;
+extern QColor UmlBlackColor;
 
 extern QColor UmlLightRedColor;
 extern QColor UmlMidRedColor;
