@@ -51,6 +51,19 @@
 #include "strutil.h"
 #include "DialogUtil.h"
 #include "mu.h"
+#include "UmlGlobal.h"
+
+// QFile::baseName() return string before the first '.' rather than the last
+
+QString my_baseName(QFileInfo & fi)
+{
+  QString fn = fi.fileName();
+  int index = fn.findRev('.');
+  
+  return (index == -1)
+    ? fn
+    : fn.left(index);
+}
 
 static ReadContext Context;
 static bool ReadOnlyForced;
@@ -82,6 +95,19 @@ void force_read_only(bool y)
   ReadOnlyForced = y;
 }
 
+static bool OnLoadDiagram;
+
+bool on_load_diagram()
+{
+  return OnLoadDiagram;
+}
+
+void set_on_load_diagram(bool y)
+{
+  OnLoadDiagram = y;
+}
+
+
 const char * stringify(UmlVisibility v) {
   switch(v) {
   case UmlPublic:
@@ -228,6 +254,8 @@ const char * stringify(UmlCode c) {
     return "---O";
   case UmlRequired:
     return "---(";
+  case UmlInner:
+    return "---+";
 
   case InitialPS:
     return "initial";
@@ -310,6 +338,8 @@ const char * stringify(UmlColor c)
     return "lightblue";
   case UmlBlue:
     return "blue";
+  case UmlLightMediumBlue:
+    return "lightmediumblue";
   case UmlMediumBlue:
     return "mediumblue";
   case UmlDarkBlue:
@@ -318,16 +348,24 @@ const char * stringify(UmlColor c)
     return "lightgreen";
   case UmlGreen:
     return "green";
+  case UmlLightMediumGreen:
+    return "lightmediumgreen";
   case UmlMediumGreen:
     return "mediumgreen";
   case UmlDarkGreen:
     return "darkgreen";
   case UmlRed:
     return "red";
+  case UmlVeryLightGray:
+    return "verylightgray";
   case UmlLightGray:
     return "lightgray";
   case UmlGray:
     return "gray";
+  case UmlDarkGray:
+    return "darkgray";
+  case UmlBlack:
+    return "black";
     
   case UmlLightRed:
     return "lightred";
@@ -424,6 +462,8 @@ const char * stringify(DrawingLanguage l)
     return "c++";
   case JavaView:
     return "java";
+  case PhpView:
+    return "php";
   case IdlView:
     return "idl";
   case DefaultDrawingLanguage:
@@ -541,6 +581,8 @@ UmlCode arrow_type(const char * s)
     return UmlRequired;
   if (!strcmp(s, "---O"))
     return UmlProvided;
+  if (!strcmp(s, "---+"))
+    return UmlInner;
   
   msg_critical("Error",
 	       Context.filename + " : " + s + " is not a line kind");
@@ -750,6 +792,8 @@ DrawingLanguage drawing_language(const char * s)
     return CppView;
   if (!strcmp(s, "java"))
     return JavaView;
+  if (!strcmp(s, "php"))
+    return PhpView;
   if (!strcmp(s, "idl"))
     return IdlView;
   if (!strcmp(s, "default"))
@@ -796,6 +840,8 @@ UmlColor color(const char * s)
     return UmlDarkYellow;
   if (!strcmp(s, "lightblue"))
     return UmlLightBlue;
+  if (!strcmp(s, "lightmediumblue"))
+    return UmlLightMediumBlue;
   if (!strcmp(s, "mediumblue"))
     return UmlMediumBlue;
   if (!strcmp(s, "blue"))
@@ -806,6 +852,8 @@ UmlColor color(const char * s)
     return UmlLightGreen;
   if (!strcmp(s, "green"))
     return UmlGreen;
+  if (!strcmp(s, "lightmediumgreen"))
+    return UmlLightMediumGreen;
   if (!strcmp(s, "mediumgreen"))
     return UmlMediumGreen;
   if (!strcmp(s, "darkgreen"))
@@ -816,10 +864,16 @@ UmlColor color(const char * s)
     return UmlMidRed;
   if (!strcmp(s, "red"))
     return UmlRed;
+  if (!strcmp(s, "verylightgray"))
+    return UmlVeryLightGray;
   if (!strcmp(s, "lightgray"))
     return UmlLightGray;
   if (!strcmp(s, "gray"))
     return UmlGray;
+  if (!strcmp(s, "darkgray"))
+    return UmlDarkGray;
+  if (!strcmp(s, "black"))
+    return UmlBlack;
   
   if (! strcmp(s, "verylightorange"))
     return UmlVeryLightOrange;
@@ -1997,11 +2051,13 @@ void warn(const QString & s)
   QTextView * t = new QTextView(d);
   QFontMetrics fm(QApplication::font());
   //int w = e->maxLineWidth() + e->minimumSizeHint().width();
-  int maxw = (UmlWindow::get_workspace()->height() * 4)/5;
+  int maxw = (UmlWindow::get_workspace()->width() * 4)/5;
+  int maxh = (UmlWindow::get_workspace()->height() * 4)/5;
+  int he = (e->numLines() + 5) * fm.height();
   
   t->setText(s);
   t->setMinimumSize(maxw, //(w > maxw) ? maxw : w,
-		    ((e->numLines() > 15) ? 15 : e->numLines()) * fm.height());
+		    (he > maxh) ? maxh : he);
 		    
   vbox->addWidget(t);
   
@@ -2023,6 +2079,19 @@ DiagramItem * load_item(QBuffer & b)
   return di;
 }
 
+void save_ptr(const void * ptr, QBuffer & b)
+{
+  b.writeBlock((char *) &ptr, sizeof(ptr));
+}
+
+void * load_ptr(QBuffer & b)
+{
+  void * ptr;
+  
+  b.readBlock((char *) &ptr, sizeof(ptr));
+  return ptr;
+}
+
 void save(const QPoint & p, QBuffer & b)
 {
   int i;
@@ -2078,6 +2147,24 @@ double load_double(QBuffer & b)
   return d;
 }
 
+void save(float f, QBuffer & b)
+{
+  b.writeBlock((char *) &f, sizeof(f));
+}
+
+void load(float & f, QBuffer & b)
+{
+  b.readBlock((char *) &f, sizeof(f));
+}
+
+double load_float(QBuffer & b)
+{
+  float f;
+  
+  b.readBlock((char *) &f, sizeof(f));
+  return f;
+}
+
 void save(int i, QBuffer & b)
 {
   b.writeBlock((char *) &i, sizeof(i));
@@ -2130,11 +2217,30 @@ int svg_height()
   return pict_height;
 }
 
-void draw_poly(FILE * fp, QPointArray & poly, const char * color, bool stroke)
+const char * svg_color(UmlColor c)
+{
+  static const char * svg[UmlDefaultColor];
+  const char * r = svg[c];
+  
+  if (r == 0) {
+    if (c == UmlTransparent)
+      r = svg[c] = "none";
+    else {
+      QColor col = color(c);
+      
+      r = svg[c] = new char[16];
+      sprintf((char *) r, "#%06x", col.rgb()&0xffffff);
+    }
+  }
+  
+  return r;
+}
+
+void draw_poly(FILE * fp, QPointArray & poly, UmlColor color, bool stroke)
 {
   fprintf(fp, (stroke) ? "\t<polygon fill=\"%s\" stroke=\"black\" stroke-opacity=\"1\""
 		       : "\t<polygon fill=\"%s\" stroke=\"none\"",
-	  color);
+	  svg_color(color));
 
   const char * sep = " points=\"";
   int n = poly.size();
@@ -2148,11 +2254,10 @@ void draw_poly(FILE * fp, QPointArray & poly, const char * color, bool stroke)
   fputs("\" />\n", fp);
 }
 
-void draw_poly(FILE * fp, QPointArray & poly, const QColor & color, bool stroke)
+void draw_shadow(FILE * fp, QPointArray & poly)
 {
-  fprintf(fp, (stroke) ? "\t<polygon fill=\"#%06x\" stroke=\"black\" stroke-opacity=\"1\""
-		       : "\t<polygon fill=\"#%06x\" stroke=\"none\"",
-	  color.rgb()&0xffffff);
+  fprintf(fp, "\t<polygon fill=\"#%06x\" stroke=\"none\"",
+	  QObject::darkGray.rgb()&0xffffff);
 
   const char * sep = " points=\"";
   int n = poly.size();
@@ -2203,19 +2308,29 @@ static void xml_text(FILE * fp, QString s)
 }
 
 void draw_text(int x, int y, int w, int h, int align,
-	       QString s, const QFont & fn, FILE * fp)
+	       QString s, const QFont & fn, FILE * fp,
+	       UmlColor fg, UmlColor bg)
 {
   if (s.isEmpty())
     return;
 
+  if (bg != UmlTransparent)
+    fprintf(fp, "\t<rect fill=\"%s\" stroke=\"none\" stroke-opacity=\"1\""
+	    " x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" />\n",
+	    svg_color(bg), x, y, w, h);
+
+  if (fg == UmlTransparent)
+    fg = UmlBlack;
+  
   int ps = fn.pixelSize();
   int pts = fn.pointSize();
   char header[256];
 
-  sprintf(header, "\t<text font-family=\"%s\" font-size=\"%d\" fill=\"black\"",
+  sprintf(header, "\t<text font-family=\"%s\" font-size=\"%d\" fill=\"%s\" xml:space=\"preserve\"",
 	  (const char *) fn.family(),
 	  // decrease size to help to have enough area
-	  (ps == -1) ? /* Qt3 */ pts : ps - 1);
+	  (ps == -1) ? /* Qt3 */ pts : ps - 1,
+	  svg_color(fg));
 
   if (ps == -1)
     /* Qt3 */
@@ -2346,3 +2461,18 @@ void draw_text(int x, int y, int w, int h, int align,
   }
 }
 
+//
+
+static QString Last_Used_Directory;
+
+QString last_used_directory()
+{
+  return Last_Used_Directory;
+}
+
+void set_last_used_directory(QString s)
+{
+  QFileInfo fi(s);
+  
+  Last_Used_Directory = fi.dirPath();
+}
