@@ -29,7 +29,7 @@
 #include <stdio.h>
 #include "UmlEnum.h"
 
-#define FILEFORMAT 37
+#define FILEFORMAT 48
 
 class QDir;
 class QFile;
@@ -47,6 +47,8 @@ class QFont;
 
 class DiagramItem;
 
+extern QString my_baseName(QFileInfo & fi);
+
 typedef struct {
   QString filename;
   unsigned linenum;
@@ -64,6 +66,9 @@ extern bool read_only_file();
 extern void set_read_only_file();
 extern void force_read_only(bool y);
 
+extern bool on_load_diagram();
+extern void set_on_load_diagram(bool);
+
 extern void backup(QDir & d, QString fn);
 extern void delete_backup(QDir & d);
 
@@ -75,6 +80,9 @@ extern void save_if_needed(const char * filename, const char * newdef);
 extern char * read_file(QString filename);
 extern char * read_file(QString filename, int offset, int len);
 
+extern QString last_used_directory();
+extern void set_last_used_directory(QString);
+
 extern void set_read_file_format(unsigned);
 extern unsigned read_file_format();
 
@@ -130,6 +138,9 @@ extern void warn(const QString &);
 extern void save(const DiagramItem *, QBuffer &);
 extern DiagramItem * load_item(QBuffer &);
 
+extern void save_ptr(const void *, QBuffer &);
+extern void * load_ptr(QBuffer &);
+
 extern void save(const QPoint & p, QBuffer &);
 extern void load(QPoint & p, QBuffer &);
 
@@ -140,6 +151,10 @@ extern void save(double, QBuffer &);
 extern void load(double &, QBuffer &);
 extern double load_double(QBuffer &);
 
+extern void save(float, QBuffer &);
+extern void load(float &, QBuffer &);
+extern double load_float(QBuffer &);
+
 extern void save(int, QBuffer &);
 extern void load(int &, QBuffer &);
 
@@ -150,12 +165,13 @@ extern bool start_svg(const char * f, int w, int h);
 extern void end_svg();
 extern FILE * svg();
 extern int svg_height();
+extern const char * svg_color(UmlColor);
+extern void draw_shadow(FILE * fp, QPointArray & poly);
 extern void draw_poly(FILE * fp, QPointArray & poly,
-		      const char * color, bool stroke = TRUE);
-extern void draw_poly(FILE * fp, QPointArray & poly,
-		      const QColor & color, bool stroke = TRUE);
+		      UmlColor color, bool stroke = TRUE);
 extern void draw_text(const QRect & r, int align, QString s, const QFont &, FILE *);
-extern void draw_text(int x, int y, int w, int h, int align, QString s, const QFont &, FILE *);
-
+extern void draw_text(int x, int y, int w, int h, int align, QString s,
+		      const QFont &, FILE *, UmlColor fg = UmlBlack,
+		      UmlColor bg = UmlTransparent);
 
 #endif
