@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyright (C) 2004-2006 Bruno PAGES  All rights reserved.
+// Copyleft 2004-2007 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -77,4 +77,7 @@ bool verbose_generation();
 void toggle_preserve_bodies();
 bool preserve_bodies();
 
+void toggle_add_operation_profile();
+bool add_operation_profile();
+
 #endif
