@@ -479,6 +479,13 @@ void ToolCom::write_char(char c)
 #endif
 }
 
+void ToolCom::write(QRect r) {
+  write_unsigned((unsigned) r.x());
+  write_unsigned((unsigned) r.y());
+  write_unsigned((unsigned) r.width());
+  write_unsigned((unsigned) r.height());  
+}
+
 void ToolCom::write_ack(bool b)
 {
   /*if (with_ack)*/ {
@@ -550,7 +557,7 @@ void ToolCom::data_received(Socket * who) {
 	   close();
 	   return;
 	   }*/
-	else if (api_version > 30) {
+	else if (api_version > 36) {
 	  TraceDialog::add("<font color =\"red\"><b>the plug-out is incompatible with this too old version of BOUML<b></font>");
 	  TraceDialog::show_it();
 	  close();
@@ -622,6 +629,7 @@ void ToolCom::data_received(Socket * who) {
 	    break;
 	  case showTraceCmd:
 	    TraceDialog::show_it();
+	    break;
 	  case traceAutoRaiseCmd:
 	    TraceDialog::trace_auto_raise(p[1]);
 	    break;
@@ -701,7 +709,7 @@ void ToolCom::data_received(Socket * who) {
 	      switch (get_kind(p)) {
 	      case UmlOperation:
 		op = BrowserOperation::get_it("operation_ref", id);
-	        if (!op->deletedp())
+	        if ((op == 0) || !op->deletedp())
 		  break;
 		// no break
 	      default:
@@ -717,7 +725,7 @@ void ToolCom::data_received(Socket * who) {
 	    break;
 	  default:
 #ifdef DEBUGCOM
-	    cerr << "unknown MiscGlobalCmd : " << p[0] << "\n";
+	    cerr << "unknown MiscGlobalCmd : " << (unsigned) p[0] << "\n";
 #endif
 	    throw 0;
 	  }
@@ -729,7 +737,7 @@ void ToolCom::data_received(Socket * who) {
 #endif
 	  if (!GenerationSettings::tool_global_uml_cmd(this, p+1)) {
 #ifdef DEBUGCOM
-	    cerr << "unknown UmlSettingsCmd : " << p[0] << "\n";
+	    cerr << "unknown UmlSettingsCmd : " << (unsigned) p[0] << "\n";
 #endif
 	    throw 0;
 	  }
@@ -741,7 +749,7 @@ void ToolCom::data_received(Socket * who) {
 #endif
 	  if (!GenerationSettings::tool_global_cpp_cmd(this, p+1)) {
 #ifdef DEBUGCOM
-	    cerr << "unknown CppSettingsCmd : " << p[0] << "\n";
+	    cerr << "unknown CppSettingsCmd : " << (unsigned) p[0] << "\n";
 #endif
 	    throw 0;
 	  }
@@ -753,7 +761,19 @@ void ToolCom::data_received(Socket * who) {
 #endif
 	  if (!GenerationSettings::tool_global_java_cmd(this, p+1)) {
 #ifdef DEBUGCOM
-	    cerr << "unknown JavaSettingsCmd : " << p[0] << "\n";
+	    cerr << "unknown JavaSettingsCmd : " << (unsigned) p[0] << "\n";
+#endif
+	    throw 0;
+	  }
+	  break;
+	case phpSettingsCmd:
+#ifdef DEBUGCOM
+	  cerr << "ToolCom::data_received() PhpSettingsCmd "
+	    << (unsigned) p[0] << '\n';
+#endif
+	  if (!GenerationSettings::tool_global_php_cmd(this, p+1)) {
+#ifdef DEBUGCOM
+	    cerr << "unknown PhpSettingsCmd : " << (unsigned) p[0] << "\n";
 #endif
 	    throw 0;
 	  }
@@ -772,7 +792,7 @@ void ToolCom::data_received(Socket * who) {
 	  break;
 	default:
 #ifdef DEBUGCOM
-	  cerr << "unknown CmdFamily : " << p[-1] << "\n";
+	  cerr << "unknown CmdFamily : " << (unsigned) p[-1] << "\n";
 #endif
 	  throw 0;
 	}
