@@ -6,6 +6,7 @@ QPixmap * noteButton;
 QPixmap * textButton;
 QPixmap * actorButton;
 QPixmap * classButton;
+QPixmap * classinstanceButton;
 QPixmap * anchorButton;
 QPixmap * selectButton;
 QPixmap * usecaseButton;
@@ -30,6 +31,7 @@ QPixmap * syncmsgButton;
 QPixmap * syncselfmsgButton;
 QPixmap * asyncselfmsgButton;
 QPixmap * returnmsgButton;
+QPixmap * selfreturnmsgButton;
 QPixmap * selflinkButton;
 QPixmap * fitscaleButton;
 QPixmap * optwindowsizeButton;
@@ -67,6 +69,8 @@ QPixmap * ActorIcon;
 QPixmap * DeletedActorIcon;
 QPixmap * ClassIcon;
 QPixmap * DeletedClassIcon;
+QPixmap * ClassInstanceIcon;
+QPixmap * DeletedClassInstanceIcon;
 QPixmap * PublicEmbeddedClassIcon;
 QPixmap * ProtectedEmbeddedClassIcon;
 QPixmap * PackageEmbeddedClassIcon;
@@ -225,17 +229,22 @@ QPixmap * YellowPixmap;
 QPixmap * DarkYellowPixmap;
 QPixmap * LightBluePixmap;
 QPixmap * BluePixmap;
+QPixmap * LightMediumBluePixmap;
 QPixmap * MediumBluePixmap;
 QPixmap * DarkBluePixmap;
 QPixmap * LightGreenPixmap;
 QPixmap * GreenPixmap;
 QPixmap * MediumGreenPixmap;
+QPixmap * LightMediumGreenPixmap;
 QPixmap * DarkGreenPixmap;
 QPixmap * LightRedPixmap;
 QPixmap * MidRedPixmap;
 QPixmap * RedPixmap;
+QPixmap * VeryLightGrayPixmap;
 QPixmap * LightGrayPixmap;
 QPixmap * GrayPixmap;
+QPixmap * DarkGrayPixmap;
+QPixmap * BlackPixmap;
 QPixmap * VeryLightOrangePixmap;
 QPixmap * LightOrangePixmap;
 QPixmap * OrangePixmap;
@@ -257,6 +266,7 @@ QPixmap * rightUnavailablePixmap;
 
 // diagram pixmap
 
+QPixmap * innerPixmap;
 QPixmap * providedPixmap;
 QPixmap * entryPointPixmap;
 QPixmap * exitPointPixmap;
@@ -292,6 +302,7 @@ void init_pixmaps()
 #include "text_button.xpm"
 #include "actor_button.xpm"
 #include "class_button.xpm"
+#include "classinstance_button.xpm"
 #include "anchor_button.xpm"
 #include "select_button.xpm"
 #include "usecase_button.xpm"
@@ -316,6 +327,7 @@ void init_pixmaps()
 #include "syncselfmsg_button.xpm"
 #include "asyncselfmsg_button.xpm"
 #include "returnmsg_button.xpm"
+#include "selfreturnmsg_button.xpm"
 #include "selflink_button.xpm"
 #include "fitscale_button.xpm"
 #include "optwinsize_button.xpm"
@@ -351,6 +363,7 @@ void init_pixmaps()
   textButton = new QPixmap((const char **) text_button);
   actorButton = new QPixmap((const char **) actor_button);
   classButton = new QPixmap((const char **) class_button);
+  classinstanceButton = new QPixmap((const char **) classinstance_button);
   anchorButton = new QPixmap((const char **) anchor_button);
   selectButton = new QPixmap((const char **) select_button);
   usecaseButton = new QPixmap((const char **) usecase_button);
@@ -375,6 +388,7 @@ void init_pixmaps()
   syncselfmsgButton = new QPixmap((const char **) syncselfmsg_button);
   asyncselfmsgButton = new QPixmap((const char **) asyncselfmsg_button);
   returnmsgButton = new QPixmap((const char **) returnmsg_button);
+  selfreturnmsgButton = new QPixmap((const char **) selfreturnmsg_button);
   selflinkButton = new QPixmap((const char **) selflink_button);
   fitscaleButton = new QPixmap((const char **) fitscale_button);
   optwindowsizeButton = new QPixmap((const char **) optwinsize_button);
@@ -412,6 +426,8 @@ void init_pixmaps()
 #include "deletedactor_xpm.xpm"
 #include "class_xpm.xpm"
 #include "deletedclass_xpm.xpm"
+#include "deletedclassinstance_xpm.xpm"
+#include "classinstance_xpm.xpm"
 #include "publicembeddedclass_xpm.xpm"
 #include "protectedembeddedclass_xpm.xpm"
 #include "packageembeddedclass_xpm.xpm"
@@ -563,6 +579,8 @@ void init_pixmaps()
   DeletedActorIcon = new QPixmap((const char **) deletedactor_xpm);
   ClassIcon = new QPixmap((const char **) class_xpm);
   DeletedClassIcon = new QPixmap((const char **) deletedclass_xpm);
+  ClassInstanceIcon = new QPixmap((const char **) classinstance_xpm);
+  DeletedClassInstanceIcon = new QPixmap((const char **) deletedclassinstance_xpm);
   PublicEmbeddedClassIcon = new QPixmap((const char **) publicembeddedclass_xpm);
   ProtectedEmbeddedClassIcon = new QPixmap((const char **) protectedembeddedclass_xpm);
   PackageEmbeddedClassIcon = new QPixmap((const char **) packageembeddedclass_xpm);
@@ -719,10 +737,12 @@ void init_pixmaps()
 #include "darkyellow_xpm.xpm"
 #include "lightblue_xpm.xpm"
 #include "blue_xpm.xpm"
+#include "lightmediumblue_xpm.xpm"
 #include "mediumblue_xpm.xpm"
 #include "darkblue_xpm.xpm"
 #include "lightgreen_xpm.xpm"
 #include "green_xpm.xpm"
+#include "lightmediumgreen_xpm.xpm"
 #include "mediumgreen_xpm.xpm"
 #include "darkgreen_xpm.xpm"
 #include "lightred_xpm.xpm"
@@ -736,8 +756,11 @@ void init_pixmaps()
 #include "magenta_xpm.xpm"
 #include "midmagenta_xpm.xpm"
 #include "darkmagenta_xpm.xpm"
+#include "verylightgray_xpm.xpm"
 #include "lightgray_xpm.xpm"
 #include "gray_xpm.xpm"
+#include "darkgray_xpm.xpm"
+#include "black_xpm.xpm"
 
   WhitePixmap = new QPixmap((const char **) white_xpm);
   LightYellowPixmap = new QPixmap((const char **) lightyellow_xpm);
@@ -746,10 +769,12 @@ void init_pixmaps()
   DarkYellowPixmap = new QPixmap((const char **) darkyellow_xpm);
   LightBluePixmap = new QPixmap((const char **) lightblue_xpm);
   BluePixmap = new QPixmap((const char **) blue_xpm);
+  LightMediumBluePixmap = new QPixmap((const char **) lightmediumblue_xpm);
   MediumBluePixmap = new QPixmap((const char **) mediumblue_xpm);
   DarkBluePixmap = new QPixmap((const char **) darkblue_xpm);
   LightGreenPixmap = new QPixmap((const char **) lightgreen_xpm);
   GreenPixmap = new QPixmap((const char **) green_xpm);
+  LightMediumGreenPixmap = new QPixmap((const char **) lightmediumgreen_xpm);
   MediumGreenPixmap = new QPixmap((const char **) mediumgreen_xpm);
   DarkGreenPixmap = new QPixmap((const char **) darkgreen_xpm);
   LightRedPixmap = new QPixmap((const char **) lightred_xpm);
@@ -763,8 +788,11 @@ void init_pixmaps()
   MagentaPixmap = new QPixmap((const char **) magenta_xpm);
   MidMagentaPixmap = new QPixmap((const char **) midmagenta_xpm);
   DarkMagentaPixmap = new QPixmap((const char **) darkmagenta_xpm);
+  VeryLightGrayPixmap = new QPixmap((const char **) verylightgray_xpm);
   LightGrayPixmap = new QPixmap((const char **) lightgray_xpm);
   GrayPixmap = new QPixmap((const char **) gray_xpm);
+  DarkGrayPixmap = new QPixmap((const char **) darkgray_xpm);
+  BlackPixmap = new QPixmap((const char **) black_xpm);
 
   // dialog pixmap
 
@@ -785,6 +813,7 @@ void init_pixmaps()
   
   // diagram pixmap
   
+#include "inner_xpm.xpm"
 #include "provided_xpm.xpm"
 #include "entrypoint_pixmap.xpm"
 #include "exitpoint_pixmap.xpm"
@@ -811,6 +840,7 @@ void init_pixmaps()
 #include "joinfork_big_pixmap.xpm"
 #include "joinfork_horiz_big_pixmap.xpm"
   
+  innerPixmap = new QPixmap((const char **) inner_xpm);
   providedPixmap = new QPixmap((const char **) provided_xpm);
   entryPointPixmap = new QPixmap((const char **) entrypoint_pixmap);
   exitPointPixmap = new QPixmap((const char **) exitpoint_pixmap);
