@@ -11,6 +11,7 @@ extern QPixmap * noteButton;
 extern QPixmap * textButton;
 extern QPixmap * actorButton;
 extern QPixmap * classButton;
+extern QPixmap * classinstanceButton;
 extern QPixmap * anchorButton;
 extern QPixmap * selectButton;
 extern QPixmap * usecaseButton;
@@ -35,6 +36,7 @@ extern QPixmap * syncmsgButton;
 extern QPixmap * syncselfmsgButton;
 extern QPixmap * asyncselfmsgButton;
 extern QPixmap * returnmsgButton;
+extern QPixmap * selfreturnmsgButton;
 extern QPixmap * selflinkButton;
 extern QPixmap * fitscaleButton;
 extern QPixmap * optwindowsizeButton;
@@ -72,6 +74,8 @@ extern QPixmap * ActorIcon;
 extern QPixmap * DeletedActorIcon;
 extern QPixmap * ClassIcon;
 extern QPixmap * DeletedClassIcon;
+extern QPixmap * ClassInstanceIcon;
+extern QPixmap * DeletedClassInstanceIcon;
 extern QPixmap * TemplateIcon;
 extern QPixmap * DeletedTemplateIcon;
 extern QPixmap * PublicEmbeddedClassIcon;
@@ -230,17 +234,22 @@ extern QPixmap * MediumYellowPixmap;
 extern QPixmap * DarkYellowPixmap;
 extern QPixmap * LightBluePixmap;
 extern QPixmap * BluePixmap;
+extern QPixmap * LightMediumBluePixmap;
 extern QPixmap * MediumBluePixmap;
 extern QPixmap * DarkBluePixmap;
 extern QPixmap * LightGreenPixmap;
 extern QPixmap * GreenPixmap;
+extern QPixmap * LightMediumGreenPixmap;
 extern QPixmap * MediumGreenPixmap;
 extern QPixmap * DarkGreenPixmap;
 extern QPixmap * LightRedPixmap;
 extern QPixmap * MidRedPixmap;
 extern QPixmap * RedPixmap;
+extern QPixmap * VeryLightGrayPixmap;
 extern QPixmap * LightGrayPixmap;
 extern QPixmap * GrayPixmap;
+extern QPixmap * DarkGrayPixmap;
+extern QPixmap * BlackPixmap;
 
 extern QPixmap * VeryLightOrangePixmap;
 extern QPixmap * LightOrangePixmap;
@@ -263,6 +272,7 @@ extern QPixmap * rightUnavailablePixmap;
 
 // diagram pixmap
 
+extern QPixmap * innerPixmap;
 extern QPixmap * providedPixmap;
 extern QPixmap * entryPointPixmap;
 extern QPixmap * exitPointPixmap;
